/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.coarsegrainsupport;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import org.cellmicrocosmos.cm2.engine.DataPool;


/**
 * This superclass describes a control panel that is displayed in the right half of the molecule editor
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */
public abstract class ControlPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	JPanel componentPanel = new JPanel();
	JLabel logArea = new JLabel();
	JPanel logPanel = new JPanel();

	JPanel headerpanel = new JPanel();
	JLabel headerTitle = new JLabel();
	JLabel headerText = new JLabel();

	public MoleculeEditor editor;

	public ControlPanel(MoleculeEditor editor) {
		this.editor = editor;
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		this.add(headerpanel);

		this.add(componentPanel);
		componentPanel.setLayout(new GridLayout(4, 1));
		headerTitle.setFont(headerTitle.getFont().deriveFont(Font.BOLD,
				headerTitle.getFont().getSize2D() + 3f));
		headerTitle.setBorder(BorderFactory.createRaisedBevelBorder());
		logArea.setBorder(BorderFactory.createLoweredBevelBorder());
		logPanel.setLayout(new BorderLayout());

		logPanel.add(logArea);

		headerpanel.add(headerTitle, BorderLayout.NORTH);
		headerpanel.add(headerText, BorderLayout.CENTER);

	}

	public void log(String text, int cmode) {

		this.editor.logs.addElement(new Object[] { text, cmode });
		this.updateTextArea();

	}

	public void replaceLastLog(String text, int cmode) {

		if (this.editor.logs.isEmpty()) {
			this.editor.logs.addElement(new Object[] { text, cmode });
		} else {
			this.editor.logs.lastElement()[0] = text;
			this.editor.logs.lastElement()[1] = cmode;
		}
		this.updateTextArea();
	}

	public void clearLogs() {
		this.editor.logs.clear();
		this.updateTextArea();
	}

	public void updateTextArea() {

		String message = "";

		String color = null;

		for (int i = 0; i < this.editor.logs.size(); i++) {
			String text = (String) this.editor.logs.get(i)[0];

			if (((Integer) this.editor.logs.get(i)[1]).intValue() == 0) {
				color = DataPool.J3DSettings.bright_appearance ? "dark gray"
						: "gray";
			} else {
				int cmode = ((Integer) this.editor.logs.get(i)[1]).intValue();
				if (cmode == 1)
					color = "red";

			}

			String s = text.replaceAll("\n", "<br>");

			if (color != null)
				s = "<font color=\"" + color + "\">" + s + "</font>";

			message += "<li>" + s + "</li>";
			// message+=s+"<br>";
		}

		message = "<html><ul>" + message + "</ul></html>";

		logArea.setText(message);
	}

	public void doLayout() {

		// ControlPanel.this.ta.setPreferredSize(new
		// Dimension(this.instructionsPanel.getPreferredSize().width-20,this.instructionsPanel.getPreferredSize().height-10));
		// ControlPanel.this.headerText.setPreferredSize(new
		// Dimension(this.getSize().width-40,100));
		ControlPanel.this.headerpanel.setPreferredSize(new Dimension(this
				.getSize().width, this.getSize().height / 4));
		ControlPanel.this.headerpanel.setMinimumSize(new Dimension(this
				.getSize().width, this.getSize().height / 4));
		ControlPanel.this.headerpanel.setMaximumSize(new Dimension(this
				.getSize().width, this.getSize().height / 4));
		ControlPanel.this.headerText.setPreferredSize(new Dimension(this
				.getSize().width - 40, this.getSize().height / 5));
		ControlPanel.this.headerText.setMinimumSize(new Dimension(this
				.getSize().width - 40, this.getSize().height / 5));
		ControlPanel.this.headerText.setMaximumSize(new Dimension(this
				.getSize().width - 40, this.getSize().height / 5));
		// ControlPanel.this.headerlabel.setMaximumSize(new
		// Dimension(this.getSize().width,30));
		// this.setPreferredSize(new
		// Dimension(this.editor.display.canvas.getSize().width/3,this.editor.mainFrame.getHeight()));

		super.doLayout();
	}

	abstract void updateComponents();

	public class TitledPanel extends JPanel {

		private static final long serialVersionUID = 1L;

		public TitledPanel(String title) {

			this.setBorder(BorderFactory.createTitledBorder(null, title,
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, this.getFont().deriveFont(
							this.getFont().getSize2D() + 2f)));

		}
	}
}
