/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.coarsegrainsupport;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.media.j3d.AmbientLight;
import javax.media.j3d.Background;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.ImageComponent;
import javax.media.j3d.ImageComponent2D;
import javax.media.j3d.J3DGraphics2D;
import javax.media.j3d.Light;
import javax.media.j3d.PhysicalBody;
import javax.media.j3d.PointLight;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.vecmath.Color3f;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Particle;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.gui.MyGradientBufferedImage;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


import com.sun.j3d.utils.behaviors.mouse.MouseRotate;
import com.sun.j3d.utils.behaviors.mouse.MouseTranslate;
import com.sun.j3d.utils.behaviors.mouse.MouseZoom;
import com.sun.j3d.utils.geometry.Primitive;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.picking.PickCanvas;
import com.sun.j3d.utils.picking.PickResult;
import com.sun.j3d.utils.universe.SimpleUniverse;

/**
 * Describes the display of the MoleculeEditor
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */
public class Display {

	Canvas3D canvas;
	PickCanvas pickCanvas;
	SimpleUniverse universe;
	// OrbitBehavior orbit;

	BranchGroup mainBranchGroup = new BranchGroup();
	Background background = new Background();

	public final MyGradientBufferedImage BIdark = new MyGradientBufferedImage(
			new Color(53, 56, 53), new Color(5, 10, 5), 0);

	public final MyGradientBufferedImage BIbright = new MyGradientBufferedImage(
			new Color(255, 255, 255), new Color(225, 230, 230), 0);

	private ImageComponent2D ic;

	PointLight light1 = new PointLight();
	AmbientLight ambientLight = new AmbientLight();

	final BoundingSphere worldBounds = new BoundingSphere(new Point3d(0.0, 0.0,
			0.0), 100000.0);
	MoleculeEditor editor;

	Hud hud = new Hud();
	public Matrix3f initialRotationMatrix;

	public Display(MoleculeEditor editor) {
		this.editor = editor;
		this.canvas = new Canvas3D(SimpleUniverse.getPreferredConfiguration()) {

			private static final long serialVersionUID = 1L;

			@Override
			public void postRender() {

				super.postRender();

				try {
					Display.this.hud.drawHud(Display.this.canvas
							.getGraphics2D());
					Display.this.canvas.getGraphics2D().flush(true);
				} catch (Throwable e) {
					// e.printStackTrace();
					System.out.println(e);
				}

			}

		};

		// canvas.setPreferredSize(new Dimension(frameW - frameW / 3, frameW
		// - frameW / 3));

		Cm2_Toolkit.setAllCapabilitysForBranchGroup(this.mainBranchGroup);

		this.universe = new SimpleUniverse(this.canvas);

		this.background = new Background() {
			public void setImage(ImageComponent2D ic) {

				super.setColor(new Color3f(new JPanel().getBackground()));
				if (DataPool.J3DSettings.noBackground) {
					super.setImage(null);
				} else {
					super.setImage(ic);
				}
			}
		};
		this.background.setImageScaleMode(Background.SCALE_FIT_ALL);
		this.background.setCapability(Background.ALLOW_IMAGE_WRITE);

		if (this.ic != null) {
			this.background.setImage(this.ic);
			this.background.setImageScaleMode(Background.SCALE_FIT_ALL);
			this.background.setCapability(Background.ALLOW_IMAGE_WRITE);
		}

		if (DataPool.J3DSettings.bright_appearance) {
			this.ic = new ImageComponent2D(ImageComponent.FORMAT_RGB,
					this.BIbright);

			if (this.ic != null) {
				this.background.setImage(this.ic);

			}

		} else {
			this.ic = new ImageComponent2D(ImageComponent.FORMAT_RGB,
					this.BIdark);
			if (this.ic != null) {
				this.background.setImage(this.ic);

			}

		}

		this.background.setCapability(Background.ALLOW_COLOR_WRITE);
		this.background.setApplicationBounds(worldBounds);
		this.mainBranchGroup.addChild(this.background);

		final Color3f lightColor = new Color3f(1f, 1f, 1f);
		final Color3f ambientColor = new Color3f(1f, 1f, 1f);

		this.light1.setColor(lightColor);
		this.ambientLight.setColor(ambientColor);

		this.light1.setInfluencingBounds(worldBounds);
		this.ambientLight.setInfluencingBounds(worldBounds);

		this.light1.setCapability(Light.ALLOW_STATE_WRITE);
		this.light1.setCapability(PointLight.ALLOW_POSITION_WRITE);

		this.ambientLight.setCapability(Light.ALLOW_COLOR_WRITE);
		this.ambientLight.setCapability(Light.ALLOW_STATE_WRITE);

		this.light1.setEnable(true);

		final BranchGroup lbg1 = new BranchGroup();

		lbg1.addChild(this.light1);
		this.light1.setPosition(new Point3f(-10, 50, -50));

		this.mainBranchGroup.addChild(lbg1);

		this.mainBranchGroup.addChild(this.ambientLight);

		this.pickCanvas = new PickCanvas(this.canvas, this.mainBranchGroup);
		

		     pickCanvas.setTolerance(0.0f);


		this.universe.getCanvas().getView().setSceneAntialiasingEnable(
				DataPool.J3DSettings.antialiasing);

		this.universe.addBranchGraph(this.mainBranchGroup);

		

		universe.getViewer().getView().setFrontClipPolicy(
				CM2_CONSTANTS.J3D.VIEW_FRONT_CLIP_POLICY);
		universe.getViewer().getView().setBackClipPolicy(
				CM2_CONSTANTS.J3D.VIEW_BACK_CLIP_POLICY);
		universe.getViewer().getView().setFrontClipDistance(
				CM2_CONSTANTS.J3D.VIEW_FRONT_CLIP_DISTANCE);
		universe.getViewer().getView().setBackClipDistance(
				CM2_CONSTANTS.J3D.VIEW_BACK_CLIP_DISTANCE);
		universe.getViewer().getView().setSceneAntialiasingEnable(
				DataPool.J3DSettings.antialiasing);

		this.canvas
				.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

	}

	void setUpCamera() {

		TransformGroup homeTG;
		Transform3D home = new Transform3D();

		homeTG = universe.getViewingPlatform().getViewPlatformTransform();

		homeTG.getTransform(home);

		Point3f lower = this.editor.currentMolecule.getLower(false);
		Point3f upper = this.editor.currentMolecule.getUpper(false);

		float dist = lower.distance(upper);

		Vector3f v = new Vector3f(0, 0, dist * 2);

		home.setTranslation(v);

		homeTG.setTransform(home);

	}

	void startDisplayOfMolecule(Molecule_3D mol) {

		this.editor.currentMolecule = mol;
		this.initialRotationMatrix = Cm2_Toolkit.copyMatrix(mol
				.getRotationMatrix());

		this.editor.currentMolecule
				.setRenderMode(Molecule_3D.RENDER_MODE_BALLS_N_STICKS);
		this.editor.currentMolecule.getTransformGroup().removeChild(
				this.editor.currentMolecule.getShape());

		this.mainBranchGroup.addChild(this.editor.currentMolecule
				.getBranchGroup());

		final MouseTranslate mousetrans = new MouseTranslate() {
			public void transformChanged(final Transform3D arg) {
				final Vector3f vec = new Vector3f();
				arg.get(vec);
				Display.this.editor.currentMolecule
						.setPosition(new Point3f(vec));

			}
		};

		mousetrans.setPickable(false);
		mousetrans.setTransformGroup(this.editor.currentMolecule
				.getTransformGroup());
		final BranchGroup bg2 = new BranchGroup();
		bg2.addChild(mousetrans);
		this.editor.currentMolecule.getTransformGroup().addChild(bg2);

		mousetrans.setEnable(true);
		mousetrans.setSchedulingBounds(this.worldBounds);

		mousetrans.setFactor(0.005d);

		final MouseZoom mousezoom = new MouseZoom() {
			public void transformChanged(final Transform3D arg) {
				final Vector3f vec = new Vector3f();
				arg.get(vec);
				Display.this.editor.currentMolecule
						.setPosition(new Point3f(vec));

			}
		};

		mousezoom.setPickable(false);
		mousezoom.setTransformGroup(this.editor.currentMolecule
				.getTransformGroup());
		final BranchGroup bg3 = new BranchGroup();
		bg3.addChild(mousezoom);
		this.editor.currentMolecule.getTransformGroup().addChild(bg3);

		mousezoom.setEnable(true);
		mousezoom.setSchedulingBounds(this.worldBounds);

		mousezoom.setFactor(0.02d);

		final MouseRotate mouserot = new MouseRotate() {
			public void transformChanged(final Transform3D arg) {
				final Matrix3f matrix = new Matrix3f();
				arg.get(matrix);
				Display.this.editor.currentMolecule.setRotationMatrix(matrix);

			}

			public void setEnable(boolean b) {
				super.setEnable(b);
				mousetrans.setEnable(b);
				mousezoom.setEnable(b);
			}

		};

		mouserot.setPickable(false);
		mouserot.setTransformGroup(this.editor.currentMolecule
				.getTransformGroup());
		final BranchGroup bg = new BranchGroup();
		bg.addChild(mouserot);
		this.editor.currentMolecule.getTransformGroup().addChild(bg);
		this.editor.currentMolecule.setMouseBehavior(mouserot);
		mouserot.setEnable(true);
		mouserot.setSchedulingBounds(this.worldBounds);

		mouserot.setFactor(0.01d);

	}

	void makeAllPickable(boolean b) {
		Atom_3D[] atoms = this.editor.currentMolecule.atoms;
		for (int i = 0; i < atoms.length; i++) {
			if (atoms[i].object_3D instanceof Sphere) {
				Sphere sp = (Sphere) atoms[i].object_3D;
				sp.setPickable(b);
			}

		}

	}

	Sphere pick(MouseEvent arg0) {
		this.pickCanvas.setShapeLocation(arg0);
		PickResult pickresult = this.pickCanvas.pickClosest();
		if (pickresult == null) {
			return null;
		}

		Primitive prim = (Primitive) pickresult.getNode(PickResult.PRIMITIVE);
		if (!(prim instanceof Sphere)) {
			return null;
		}
		return (Sphere) prim;
	}

	Sphere[] pickRubberBand(Rectangle rubberband) {

		Vector<Sphere> ret = new Vector<Sphere>();
		int west = rubberband.x;
		int north = rubberband.y;
		int east = rubberband.x + rubberband.width;
		int south = rubberband.y + rubberband.height;
		for (int i = west; i < east; i += 4) {
			for (int j = north; j < south; j += 4) {
				this.pickCanvas.setShapeLocation(i, j);

				PickResult[] pickresult = this.pickCanvas.pickAll();
				if (pickresult == null) {
					continue;
				}

				for (int k = 0; k < pickresult.length; k++) {
					Primitive prim = (Primitive) pickresult[k]
							.getNode(PickResult.PRIMITIVE);
					if (!(prim instanceof Sphere)) {
						continue;
					}
					Sphere sp = (Sphere) prim;
					if (!ret.contains(sp)) {
						ret.addElement(sp);
					}

				}

			}
		}
		return ret.toArray(new Sphere[0]);
	}

	int cursorMode = 0;

	public void switchCursor(int c) {
		if (this.cursorMode == c)
			return;
		this.cursorMode = c;
		if (c == 0)
			this.canvas.setCursor(Cursor
					.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		else if (c == 1)
			this.canvas.setCursor(Cursor
					.getPredefinedCursor(Cursor.HAND_CURSOR));
		else
			this.canvas.setCursor(Cursor
					.getPredefinedCursor(Cursor.MOVE_CURSOR));
	}

	class Hud {

		Atom_3D atCursor = null;

		Font font1 = new Font("Arial", Font.BOLD, 12);
		Font font2 = new Font("Arial", Font.PLAIN, 12);
		Font font3 = new Font("Arial", Font.PLAIN, 10);

		Font font4 = new Font("Arial", Font.PLAIN, 9);

		public Rectangle rubberband = null;

		public void drawHud(J3DGraphics2D g) {

//			int W = Display.this.canvas.getWidth();
			int H = Display.this.canvas.getHeight();
			Color fg = DataPool.J3DSettings.bright_appearance ? Color.black
					: Color.white;

			if (rubberband != null) {
				g.setColor(Color.green);
				g.drawRect(rubberband.x, rubberband.y, rubberband.width,
						rubberband.height);
			}

			g.setColor(fg);

			// JLabel l=new JLabel("A");
			// l.setFont(font1);
			int prefH = H - 20;
			g.drawLine(0, prefH, 150, prefH);
			g.drawLine(150, prefH, 150, H);
			g.drawLine(80, prefH, 80, H);

			if (this.atCursor != null) {

				g.setFont(font1);
				g.setColor(this.atCursor.color);
				g.fillOval(8, H - 15, 10, 10);
				g.setColor(fg);

				g.setFont(font3);
				g.drawString("Name:", 85, H - 6);

				g.setFont(font2);
				g.drawString("" + atCursor.name, 118, H - 5);

				if (atCursor instanceof Particle) {

					JLabel l = new JLabel(""
							+ ((Particle) atCursor).particleIdentifier);
					l.setFont(font2);
					int ext = l.getPreferredSize().width + 29 + 150;
					g.setColor(this.atCursor.color);
					g.drawOval(5, H - 18, 16, 16);
					g.setColor(fg);

					g.drawLine(150, prefH, ext, prefH);
					g.drawLine(ext, prefH, ext, H);
					g.setFont(font1);
					g.drawString("Particle", 28, H - 5);

					g.setFont(font3);
					g.drawString("ID:", 155, H - 6);
					g.setFont(font2);
					g.drawString("" + ((Particle) atCursor).particleIdentifier,
							174, H - 5);

				} else {
					g.setFont(font1);
					g.drawString("Atom", 31, H - 5);

					JLabel l = new JLabel("" + atCursor.residueName);
					l.setFont(font2);
					int ext = l.getPreferredSize().width + 55 + 150;

					g.drawLine(150, prefH, ext, prefH);
					g.drawLine(ext, prefH, ext, H);

					g.setFont(font3);
					g.drawString("Residue:", 155, H - 6);
					g.setFont(font2);
					g.drawString("" + atCursor.residueName, 200, H - 5);

				}

			} else {
				int ext = 223;
				g.drawLine(150, prefH, ext, prefH);
				g.drawLine(ext, prefH, ext, H);
			}

			g.setColor(fg);
			int start1 = 5;
			int start2 = 100;
			int pos1 = 15;
			int pos2 = 15;

			if (editor.mode == MoleculeEditor.MODE_GRANULARITY) {

				g.setFont(font3);
				g.drawString("Left Click", start1, pos1);
				pos1 += 15;
				g.setFont(font4);
				g.drawString("Select/Deselect", start2, pos2);
				pos2 += 15;
				
				g.drawString("ALT & Move  :", start1, pos1);
				pos1 += 15;
				g.drawString("Rubberband Selection", start2, pos2);
				pos2 += 15;
			}
			// else if (editor.mode == MoleculeEditor.MODE_PROPERTIES) {
			//
			// g.setFont(font3);
			// g.drawString("Left Click", start1, 15);
			//
			// g.setFont(font4);
			// g.drawString("Select/Deselect Particle", start2, 15);
			//
			// }
			else if (editor.mode == MoleculeEditor.MODE_CONFORMATION) {

				g.setFont(font3);
				g.drawString("Left Click", start1, pos1);
				pos1 += 15;
				g.drawString("Right Click", start1, pos1);
				pos1 += 15;
				g.drawString("ALT & Move  :", start1, pos1);
				pos1 += 15;
				if (editor.currentJoint != null) {
					g.drawString("SHIFT & Move", start1, pos1);
					pos1 += 15;
					g.drawString("CTRL & Move", start1, pos1);
					pos1 += 15;
				} else {
					g.drawString("CTRL & Move", start1, pos1);
					pos1 += 15;
				}

				g.setFont(font4);
				g.drawString("Select/Deselect", start2, pos2);
				pos2 += 15;
				g.drawString("Select/Deselect Rotation Center", start2, pos2);
				pos2 += 15;
				g.drawString("Rubberband Selection", start2, pos2);
				pos2 += 15;

				if (editor.currentJoint != null) {
					g.drawString("Rotate Selection around Center Point",
							start2, pos2);
					pos2 += 15;
					g.drawString("Translate Selection", start2, pos2);
					pos2 += 15;

				} else {
					g.drawString("Translate Selection", start2, pos2);
					pos2 += 15;
				}

			}

			if (editor.currentlySelected.isEmpty() == false) {
				g.setFont(font3);
				g.drawString("Doubleclick nowhere to Deselect All", start1,
						pos1 + 10);

			}


		}

	}

	public void setRubberband(Rectangle rubberband) {
		this.hud.rubberband = rubberband;

	}

	public Point3f getCurrentEyePosition() {
		Point3d eye_pos = new Point3d();

		canvas.getCenterEyeInImagePlate(eye_pos);

		Transform3D motion = new Transform3D();
		canvas.getImagePlateToVworld(motion);
		motion.transform(eye_pos);
		return new Point3f(eye_pos);

	}

	/**
	 * Sets the eyeposition given per parameter.
	 * 
	 * 
	 * @param leftEye
	 * @param rightEye
	 */
	public void setEyePosition(Point3d leftEye, Point3d rightEye) {
		PhysicalBody pBody = this.canvas.getView().getPhysicalBody();
		pBody.setLeftEyePosition(leftEye);
		pBody.setRightEyePosition(rightEye);
	}
	
	Atom_3D currentInfo = null;

	public void updateCanvas() {

		if (this.editor.mode == MoleculeEditor.MODE_CONFORMATION) {

			if (this.editor.moving < 1) {

				if (currentInfo != null && this.hud.atCursor == currentInfo)
					return;

				currentInfo = this.hud.atCursor;
				this.editor.currentMolecule.displayAdditionalInformation(
						this.hud.atCursor, new Vector3f(this
								.getCurrentEyePosition()).length());
			} else {
				this.editor.currentMolecule.displayAdditionalInformation(
						this.hud.atCursor, new Vector3f(this
								.getCurrentEyePosition()).length());
			}
			if (this.hud.rubberband != null)
				this.canvas.postRender();
		} else {
			this.canvas.postRender();
		}
	}

}
