/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.coarsegrainsupport;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.Particle;
import org.cellmicrocosmos.cm2.engine.DataPool;


/**
 * This class represents the controlpanel used in the Coarse-Grain-mode of the molecule editor
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */
public class GranularityPanel extends ControlPanel implements ActionListener {

	TitledPanel tp1 = new TitledPanel("Selection");
	TitledPanel tp2 = new TitledPanel("Identificators");
	TitledPanel tp3 = new TitledPanel("");

	ParticlePropertiesPanel ppp;

	JButton combine = new JButton("Combine to Particle");
	JButton restore = new JButton("Suspend Particle");
	JButton editMembers = new JButton("Extend / Reduce Members");
	JButton editPropsBtn = new JButton("Edit Identifiers");
	JButton test = new JButton("Test");
	JButton membersOkBtn = new JButton("Ok");
	JButton membersCancelBtn = new JButton("Discard");

	public GranularityPanel(MoleculeEditor editor) {
		super(editor);

		ppp = new ParticlePropertiesPanel(this.editor, false, null, null, null,
				null);
		ppp.p2.add(Box.createHorizontalStrut(ppp.comp1.getPreferredSize().width
				- (ppp.comp2.getPreferredSize().width
						+ ppp.comp3.getPreferredSize().width
						+ ppp.l3.getPreferredSize().width + 30)), 3);
		this.headerTitle.setText("  Coarse Graining  ");
		this.headerText
				.setText("<html>This section enables you to modify the Granularity of the molecule. Freely define Groups of Atoms and combine them into Coarse Grain-Particles.</html>");

		this.componentPanel.add(tp1);
		this.componentPanel.add(tp2);
		this.componentPanel.add(tp3);

		tp2.add(ppp, BorderLayout.CENTER);

		tp1.setLayout(new BoxLayout(tp1, BoxLayout.Y_AXIS));
		JPanel p1 = new JPanel();

		p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));

		tp1.add(Box.createGlue());
		tp1.add(p1);
		tp1.add(Box.createGlue());

		p1.add(Box.createHorizontalGlue());
		p1.add(combine);
		p1.add(Box.createHorizontalStrut(5));
		p1.add(restore);
		p1.add(Box.createHorizontalGlue());

		tp3.setLayout(new BoxLayout(tp3, BoxLayout.Y_AXIS));
		JPanel p2 = new JPanel();

		p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));

		tp3.add(Box.createGlue());
		tp3.add(p2);
		tp3.add(Box.createGlue());

		p2.add(Box.createHorizontalGlue());
		p2.add(editMembers);
		p2.add(Box.createHorizontalStrut(5));
		p2.add(editPropsBtn);
		p2.add(membersOkBtn);
		p2.add(Box.createHorizontalStrut(5));
		p2.add(membersCancelBtn);
		p2.add(Box.createHorizontalGlue());

		// p1.add(test);

		combine.addActionListener(this);
		restore.addActionListener(this);
		editMembers.addActionListener(this);
		test.addActionListener(this);
		editPropsBtn.addActionListener(this);
		membersOkBtn.addActionListener(this);
		membersCancelBtn.addActionListener(this);

		componentPanel.add(logPanel);

	}

	private static final long serialVersionUID = 1L;

	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getSource() == combine || arg0.getSource() == restore) {

			this.editor.combineCurrentSelection();
		} else if (arg0.getSource() == editMembers) {

			this.editor.editMembers();

		} else if (arg0.getSource() == editPropsBtn) {

			Particle p = (Particle) this.editor.currentlySelected
					.firstElement();

			if (this.editor.currentlySelected.size() > 1) {
				if (!this.askForInput(p, null, null, null, null)) {
					return;
				}
			} else {
				if (!this.askForInput(p, p.particleIdentifier, p.name,
						p.element, p.color)) {
					return;
				}
			}

			String newID = p.particleIdentifier;
			Color newColor = p.color;
			String newName = p.name;
			String element = p.element;
			for (int i = 0; i < this.editor.currentlySelected.size(); i++) {
				Particle otherP = (Particle) this.editor.currentlySelected
						.get(i);
				otherP.particleIdentifier = newID;
				otherP.name = newName;
				otherP.element = element;
				otherP.color = newColor;
			}
			if (this.editor.currentlySelected.size() > 1)
				this.log("Strings applied to all", 0);
			else
				this.log("Strings applied", 0);
			this.editor.currentlySelected.clear();
			this.editor.updateSelection();

			for (int i = 0; i < this.editor.currentMolecule.currentCGParticles
					.size(); i++) {
				if (this.editor.currentMolecule.currentCGParticles.get(i).particleIdentifier
						.equals(p.particleIdentifier))
					this.editor.currentMolecule.currentCGParticles.get(i).color = p.color;
			}

			this.editor.currentMolecule.updateAtomColors();

			this.editor.setChanged(true);

		} else if (arg0.getSource() == this.membersOkBtn) {
			this.editor.recombineEditedMembers(true);
		} else if (arg0.getSource() == this.membersCancelBtn) {
			this.editor.recombineEditedMembers(false);
		}

		else if (arg0.getSource() == test) {

		}

	}

	private void fillHash(Hashtable<String, Integer> hashSample,
			Atom_3D[] included) {
		for (int i = 0; i < included.length; i++) {
			if (!hashSample.containsKey(included[i].element)) {
				hashSample.put(included[i].element, 1);
			} else {
				hashSample.put(included[i].element, hashSample
						.get(included[i].element) + 1);
			}
		}
	}

	private String getHighest(Hashtable<String, Integer> hash) {

		Enumeration<String> en = hash.keys();

		int max = -100000;
		String highest = null;
		while (en.hasMoreElements()) {
			String key = en.nextElement();
			int n = hash.get(key).intValue();
			if (n >= max) {
				highest = key;
				max = n;
			}
		}
		return highest;
	}

	void updateComponents() {

		if (this.editor.particleExtending != null) {

			if (this.editor.currentlySelected.isEmpty()) {
				this.editor.recombineEditedMembers(false);
				return;
			}
			this.membersOkBtn.setVisible(true);
			this.membersCancelBtn.setVisible(true);
			this.combine.setEnabled(false);
			this.restore.setEnabled(false);
			this.editMembers.setVisible(false);
			this.editPropsBtn.setVisible(false);
			return;
		} else {
			this.membersOkBtn.setVisible(false);
			this.membersCancelBtn.setVisible(false);
			this.editMembers.setVisible(true);
			this.editPropsBtn.setVisible(true);
		}

		((JTextField) this.ppp.comp1).setText(null);

		((JTextField) this.ppp.comp2).setText(null);

		((JTextField) this.ppp.comp3).setText(null);

		if (this.editor.currentlySelected.isEmpty()) {
			this.combine.setText("Combine to Particle");
			this.restore.setText("Suspend Particle");
			this.combine.setEnabled(false);
			this.restore.setEnabled(false);
			this.editMembers.setEnabled(false);
			this.editPropsBtn.setEnabled(false);
			// this.ppp.setEnabled(false);
		} else if (this.editor.currentlySelected.firstElement() instanceof Particle) {

			this.editPropsBtn.setEnabled(true);

			if (this.editor.currentlySelected.size() < 2) {
				this.restore.setText("Suspend Particle");
				this.restore.setEnabled(true);
				this.combine.setEnabled(false);
				this.editMembers.setEnabled(true);
				// this.ppp.setEnabled(true);

				String pID = ((Particle) this.editor.currentlySelected
						.firstElement()).particleIdentifier;

				if (pID != null && pID.length() > 0)
					((JTextField) this.ppp.comp1).setText(pID);

				if (this.editor.currentlySelected.firstElement().name != null
						&& this.editor.currentlySelected.firstElement().name
								.length() > 0)
					((JTextField) this.ppp.comp2)
							.setText(this.editor.currentlySelected
									.firstElement().name);

				if (this.editor.currentlySelected.firstElement().element != null
						&& this.editor.currentlySelected.firstElement().element
								.length() > 0)
					((JTextField) this.ppp.comp3)
							.setText(this.editor.currentlySelected
									.firstElement().element);

			} else {
				this.restore.setText("Suspend Particles");
				this.restore.setEnabled(true);
				this.combine.setEnabled(false);
				this.editMembers.setEnabled(false);
				// this.ppp.setEnabled(false);
			}

		} else {

			this.editPropsBtn.setEnabled(false);

			if (this.editor.currentlySelected.size() < 2) {
				this.combine.setText("Wrap into Particle");
				this.restore.setEnabled(false);
				this.combine.setEnabled(true);
				this.editMembers.setEnabled(false);
				this.ppp.setEnabled(false);
			} else {
				this.combine.setText("Combine to Particle");
				this.restore.setEnabled(false);
				this.combine.setEnabled(true);
				this.editMembers.setEnabled(false);
				// this.ppp.setEnabled(false);
			}
		}

	}

	public boolean askForInput(Particle particle, String ID, String name,
			String element, Color color) {

		final JDialog dia = new JDialog();
		final JButton okBtn = new JButton("Apply");
		final JButton cancelBtn = new JButton("Cancel");

		JPanel p = new JPanel();
		p.add(okBtn);
		p.add(cancelBtn);
		dia.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		final ParticlePropertiesPanel ppp = new ParticlePropertiesPanel(editor,
				true, ID, name, element, color);

		JPanel infoP = new JPanel();
		JLabel infoL = new JLabel(
				"<html>Specify the three following fields of this Particle:<br>"
						+ "<li><b><u>Particle ID:</u></b> An Identifier String for internal use only. Probably<br>"
						+ "used by membrane-generating Algorithms in order to access further<br>"
						+ "data of behavior (like force field parameters). Multiple Particles<br>"
						+ "may have the same ID in order to be grouped into one class. Using<br>"
						+ "an Element-Symbol here should make this Particle behave like Atoms<br>"
						+ "of the refering Element </li><br>"
						+ "<li><b><u>Atom Name  :</u></b> The Atom-Name of this Particle that will be written into<br>"
						+ "the particular ATOM field in the final Pdb File</li><br>"
						+ "<li><b><u>Element  :</u></b> The Element Symbol for this Particles Entry in the final <br>"
						+ "Pdb File</li>"
						+ "<br>Those three fields are mandatory!<br><br></html>");
		infoP.add(infoL, BorderLayout.CENTER);

		ppp.add(infoP, 0);
		ppp.add(Box.createVerticalStrut(10), 8);
		// ppp.validate();

		final JTextField tf1 = (JTextField) ((JComboBox) ppp.comp1).getEditor()
				.getEditorComponent();
		final JTextField tf2 = (JTextField) ((JComboBox) ppp.comp2).getEditor()
				.getEditorComponent();
		final JTextField tf3 = (JTextField) ((JComboBox) ppp.comp3).getEditor()
				.getEditorComponent();

		final JButton cBtn = ppp.cBtn;
		ActionListener ac = new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				if (arg0.getSource() == cancelBtn) {

					tf1.setText(null);
					tf2.setText(null);
					tf3.setText(null);

					dia.dispose();
				} else if (arg0.getSource() == okBtn) {
					dia.dispose();
				}

			}

		};

		CaretListener cl = new CaretListener() {
			public void caretUpdate(CaretEvent e) {

				if ((tf1.getText() != null && tf1.getText().length() > 0)
						&& (tf2.getText() != null && tf2.getText().length() > 0)
						&& (tf3.getText() != null && tf3.getText().length() > 0)) {
					okBtn.setEnabled(true);
				} else {
					okBtn.setEnabled(false);
				}

			}
		};

		okBtn.addActionListener(ac);
		cancelBtn.addActionListener(ac);

		tf1.addCaretListener(cl);
		tf2.addCaretListener(cl);
		tf3.addCaretListener(cl);

		okBtn.setEnabled(false);

		dia.getContentPane().add(ppp, BorderLayout.CENTER);
		dia.getContentPane().add(p, BorderLayout.SOUTH);

		dia.pack();

		ppp.p2.add(Box.createHorizontalStrut(25), 0);

		dia.setLocationRelativeTo(this);
		// dia.setSize(new Dimension(400,400));

		dia.setModal(true);
		dia.toFront();

		String[] names = DataPool.AtomValues
				.getAllParticleIDsOfMembraneModelSorted();

		for (int i = 0; i < names.length; i++) {

			((JComboBox) ppp.comp1).addItem(names[i]);
		}

		names = DataPool.AtomValues.getAllAtomTypesOfMembraneModelSorted();

		for (int i = 0; i < names.length; i++) {
			((JComboBox) ppp.comp2).addItem(names[i]);
		}
		names = DataPool.AtomValues.getAllElementsOfMembraneModelSorted();

		for (int i = 0; i < names.length; i++) {
			((JComboBox) ppp.comp3).addItem(names[i]);
		}

		((JComboBox) ppp.comp1).setMaximumRowCount(((JComboBox) ppp.comp1)
				.getMaximumRowCount() * 2);
		((JComboBox) ppp.comp2).setMaximumRowCount(((JComboBox) ppp.comp2)
				.getMaximumRowCount() * 2);
		((JComboBox) ppp.comp3).setMaximumRowCount(((JComboBox) ppp.comp3)
				.getMaximumRowCount() * 2);

		if (name != null)
			tf2.setText(name);
		else {

			Hashtable<String, Integer> hashSample = new Hashtable<String, Integer>();

			Atom_3D[] included = particle.includedAtoms.toArray(new Atom_3D[0]);
			this.fillHash(hashSample, included);

			String key = this.getHighest(hashSample);

			tf2.setText(key);

			tf2.selectAll();
			tf1.requestFocus();
		}
		if (element != null)
			tf3.setText(element);
		else {

			Hashtable<String, Integer> hashSample = new Hashtable<String, Integer>();

			Atom_3D[] included = particle.includedAtoms.toArray(new Atom_3D[0]);
			this.fillHash(hashSample, included);

			String key = this.getHighest(hashSample);

			tf3.setText(key);

			tf3.selectAll();
			tf1.requestFocus();
		}

		if (ID != null)
			tf1.setText(ID);
		else {
			tf1.setText("");
			// if(tf2.getText()!=null&&tf2.getText().length()>0)
			// tf1.setText(tf2.getText());
		}

		if (this.editor.currentlySelected.size() > 1)
			dia.setTitle("Assign ID's to all Selected");
		else
			dia.setTitle("Assing ID's to Selected");
		dia.setVisible(true);

		if ((tf1.getText() != null && tf1.getText().length() > 0)
				&& (tf2.getText() != null && tf2.getText().length() > 0)
				&& (tf3.getText() != null && tf3.getText().length() > 0)) {

			particle.particleIdentifier = tf1.getText().trim();
			particle.name = tf2.getText().trim();
			particle.element = tf3.getText().trim();
			particle.color = cBtn.getForeground();

			// while (true) {
			// try {
			// new Integer(particle.element);
			// break;
			// } catch (Exception e) {
			// if (particle.element.length() < 1)
			// break;
			// particle.element = particle.element.substring(1);
			// }
			// }
			// if (particle.element.length() > 1) {
			// particle.element = particle.element.substring(0, 1);
			// }
			return true;
		}

		return false;

	}

}
