/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.coarsegrainsupport;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Vector;

import javax.media.j3d.BranchGroup;
import javax.media.j3d.Node;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.TransformGroup;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.MyBounds;
import org.cellmicrocosmos.cm2.dataobjects.Particle;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.engine.Engine;
import org.cellmicrocosmos.cm2.functionality3d.Atom_Creator_3D;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.Sphere;


/**
 * This is the base class of the Molecule Editor. Manages most of its superior functionalities.
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */
public class MoleculeEditor implements MouseListener, MouseMotionListener,
		KeyListener {

	public static final int MODE_GRANULARITY = 0;
	public static final int MODE_CONFORMATION = 1;
	int mode = MODE_GRANULARITY;

	int moving = 0;

	Molecule_3D currentMolecule;
	Vector<Atom_3D> currentlySelected = new Vector<Atom_3D>();
	Atom_3D currentJoint = null;
	int frameW = (int) (Toolkit.getDefaultToolkit().getScreenSize().width * 0.6);
	int frameH = (int) (Toolkit.getDefaultToolkit().getScreenSize().height * 0.6);
	Point previousMousePos = null;

	MoleculeEditorToolbar toolbarProfile = new MoleculeEditorToolbar(
			this);
	// ToolBarForApplying toolbarApply = new ToolBarForApplying(this);
	JTabbedPane tabbed = new JTabbedPane();

	boolean canpresskey = true;
	public Point3f[] initialPoints;
	public Particle particleExtending = null;

	JFrame mainFrame = new JFrame() {

		private static final long serialVersionUID = 1L;

		public void setVisible(boolean b) {
			super.setVisible(b);
			MoleculeEditor.this.engine.membraneOperations.setAllEnabled(!b);
		}

	};

	JPanel mainPanel = new JPanel() {

		private static final long serialVersionUID = 1L;

		public void doLayout() {

//			display.canvas.setMaximumSize(new Dimension(
//					mainFrame.getSize().width - frameW / 3, mainFrame
//							.getHeight() - 60));
			display.canvas.setMinimumSize(new Dimension(
					frameW / 3, mainFrame
							.getHeight() - 60));
			display.canvas.setPreferredSize(new Dimension(
					mainFrame.getSize().width - frameW / 3, mainFrame
							.getHeight() - 60));

			super.doLayout();
		}
	};
	public Vector<Object[]> logs = new Vector<Object[]>() {

		private static final long serialVersionUID = 1L;

		public synchronized void addElement(Object[] o) {
			if (this.size() == 5) {
				this.removeElementAt(0);
			}
			super.addElement(o);
			this.trimToSize();

		}
	};
	public Vector<Point3f[]> history = new Vector<Point3f[]>() {

		private static final long serialVersionUID = 1L;

		public synchronized void addElement(Point3f[] o) {
			if (this.size() == 50) {
				this.removeElementAt(0);
			}
			super.addElement(o);
			this.trimToSize();

		}
	};

	ControlPanel controlPanel;

	Display display;
	// public boolean editing = false;
	public boolean applyAble;
	public Engine engine;
	private Rectangle rubberband = null;

	boolean changed = false;

	public MoleculeEditor(Molecule_3D mol, boolean applyable, Engine engine,
			String currentProfile, Double standardStereoValue) {

		this.engine = engine;
		this.applyAble = applyable;

		try {
			display = new Display(this);
			/* setting up standard eye distance for stereo view */
			Point3d left = new Point3d(-standardStereoValue/2, 0.0, 0.0);
			Point3d right = new Point3d(standardStereoValue/2, 0.0, 0.0);
			display.setEyePosition(left, right);
		} catch (final Throwable e) {
			JOptionPane.showMessageDialog(null,
					"The following Error occured:\\n" + e.getMessage(),
					"ERROR", JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
			return;
		}

		// this.mainFrame.getContentPane().setLayout(new
		// BoxLayout(this.mainFrame.getContentPane(),BoxLayout.Y_AXIS));
		this.mainFrame.setAlwaysOnTop(true);
		this.mainFrame.getContentPane().add(toolbarProfile, BorderLayout.NORTH);
		this.mainFrame.getContentPane().add(mainPanel, BorderLayout.CENTER);
		
		JToolBar toolbar = new JToolBar();
		toolbar.setFloatable(false);
		this.mainFrame.getContentPane().add(toolbar, BorderLayout.SOUTH);
		// this.toolbarApply.setUp(mol);

		this.toolbarProfile.applyBtn.setVisible(applyable);
		// this.toolbarProfile.cbConf.setVisible(applyable);
		// this.toolbarProfile.deactivateBtn.setVisible(applyable);

		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.X_AXIS));

		tabbed.setMinimumSize(new Dimension(
				frameW / 3, mainFrame
						.getHeight() - 60));
		tabbed.setPreferredSize(new Dimension(
				mainFrame.getSize().width - frameW / 3, mainFrame
						.getHeight() - 60));
		
		JSplitPane displayAndTabbedSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,display.canvas,tabbed);
		displayAndTabbedSplitPane.setOneTouchExpandable(true);
		displayAndTabbedSplitPane.setDividerLocation(mainFrame.getSize().width - frameW / 3);
        
		mainPanel.add(displayAndTabbedSplitPane);
//		mainPanel.add(display.canvas);
//		mainPanel.add(tabbed);

		tabbed
				.addTab("Coarse Grain", Cm2_IconDepot
						.getSpecifiedIcon("CG_Granularity"),
						new GranularityPanel(this));
		// tabbed.addTab("Particle Properties", Cm2_IconDepot
		// .getSpecifiedIcon("CG_Properties"), new PropertiesPanel(this));

		// if (mol instanceof Lipid_3D)
		tabbed.addTab("Edit Conformation", Cm2_IconDepot
				.getSpecifiedIcon("CG_Conformation"), new ConformationPanel(
				this));

		tabbed.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent arg0) {
				MoleculeEditor.this.switchToMode(tabbed.getSelectedIndex());

				if (tabbed.getSelectedIndex() == 0) {
					// currentMolecule.rotateCoarseGrainMembers();
					// currentMolecule.updateAllBonds();
				}
			}

		});

		this.mainFrame.setLocationByPlatform(true);

		this.addListeners();

		// this.mainFrame.pack();

		this.mainFrame.setSize(new Dimension(this.frameW, this.frameH));
		this.mainFrame.setIconImage(Cm2_IconDepot.getSpecifiedIcon("Cm2_Icon")
				.getImage());

		display.startDisplayOfMolecule(mol);
		
		
		mol.move();
		
		

		this.mainFrame.setVisible(true);
		display.setUpCamera();

		this.mainFrame.setTitle("Molecule Editor - " + mol.getPDBcode()
				+ " / Model " + mol.getModelSet() + " - "
				+ mol.getCurrentProfileName());

		if (currentProfile != null
				&& !currentProfile.equals(mol.getCurrentProfileName()))
			this.setProfile(currentProfile);

		this.switchToMode(this.mode);

		String[] names = Molecule_Profile_IO.getAllProfileNames(
				this.currentMolecule, this.currentMolecule.getPDBcode());

		toolbarProfile.setComboBoxData(names, this.currentMolecule
				.getCurrentProfileName());

		Point3f[] prevPoints = new Point3f[this.currentMolecule.allAtomsSorted.length];

		for (int i = 0; i < this.currentMolecule.allAtomsSorted.length; i++) {
			Atom_3D atom = this.currentMolecule.allAtomsSorted[i];
			prevPoints[i] = new Point3f(atom.position);
		}
		this.initialPoints = prevPoints;

		display.makeAllPickable(true);

		for (int i = 0; i < this.currentMolecule.activeConstraints.size(); i++) {
			this.currentMolecule.activeConstraints.get(i).update();
		}

		this.setChanged(false);
	}

	private void addListeners() {
		display.canvas.addMouseListener(this);
		display.canvas.addMouseMotionListener(this);
		display.canvas.addKeyListener(this);

	}

	private boolean mayCurrentlyPickThis(Atom_3D pickedAtom, boolean log) {

		if (mode == MoleculeEditor.MODE_GRANULARITY) {

			if (this.particleExtending != null) {
				if (pickedAtom instanceof Particle)
					return false;
				if (this.currentlySelected.size() == 1
						&& this.currentlySelected.contains(pickedAtom))
					return false;

			}

			if (this.currentlySelected.isEmpty())
				return true;

			if (this.currentlySelected.firstElement() instanceof Particle) {
				if (pickedAtom instanceof Particle) {
					return true;
				} else {
					if (log)
						this.controlPanel.log(
								"Cannot mix Particles and Atoms!", 1);
					return false;
				}
			} else {
				if (pickedAtom instanceof Particle) {
					if (log)
						this.controlPanel.log(
								"Cannot mix Particles and Atoms!", 1);
					return false;
				} else {
					return true;
				}
			}

		}

		return true;
	}

	public void updateSelection() {

		// this.display.mainBranchGroup.removeChild(this.currentMolecule.getBranchGroup());
		Atom_3D[] atoms = this.currentMolecule.atoms;
		for (int i = 0; i < atoms.length; i++) {
			if (atoms[i].branchGroup == null)
				continue;
			if (this.currentlySelected.contains(atoms[i])) {
				if (atoms[i].branchGroup.numChildren() < 2) {
					Atom_Creator_3D
							.addBalls(
									this.currentMolecule,
									new Atom_3D[] { atoms[i] },
									false,
									Color.GREEN,
									this.currentMolecule.getRenderMode() != Molecule_3D.RENDER_MODE_VAN_DER_WAALS ? 0
											: 1);
				}
			} else {

				for (int j = 0; j < atoms[i].branchGroup.numChildren(); j++) {
					TransformGroup tr = (TransformGroup) atoms[i].branchGroup
							.getChild(j);
					if (tr.getChild(0) instanceof Box) {
						// Box box = (Box)tr.getChild(0);
						// if(box.getAppearance().getPolygonAttributes().getPolygonMode()==PolygonAttributes.POLYGON_LINE)
						// {

						Node node = atoms[i].branchGroup.getParent();
						TransformGroup parent = (TransformGroup) node;
						parent.removeChild(atoms[i].branchGroup);
						atoms[i].branchGroup.removeChild(tr);
						parent.addChild(atoms[i].branchGroup);
						// }
					}
				}

			}
			// if(this.currentJoint!=null) {
			if (atoms[i] == this.currentJoint) {
				if (atoms[i].branchGroup.numChildren() < (atoms[i] instanceof Particle ? 3
						: 2)) {
					Atom_Creator_3D
							.addBalls(
									this.currentMolecule,
									new Atom_3D[] { atoms[i] },
									false,
									Color.orange,
									this.currentMolecule.getRenderMode() != Molecule_3D.RENDER_MODE_VAN_DER_WAALS ? 0
											: 1);

				}
				// }
			} else {
				for (int j = 0; j < atoms[i].branchGroup.numChildren(); j++) {
					TransformGroup tr = (TransformGroup) atoms[i].branchGroup
							.getChild(j);
					if (tr.getChild(0) instanceof Box) {

						Box box = (Box) tr.getChild(0);
						if (box.getAppearance().getPolygonAttributes()
								.getPolygonMode() == PolygonAttributes.POLYGON_FILL) {

							Node node = atoms[i].branchGroup.getParent();
							TransformGroup parent = (TransformGroup) node;
							parent.removeChild(atoms[i].branchGroup);
							atoms[i].branchGroup.removeChild(tr);
							parent.addChild(atoms[i].branchGroup);
						}
					}
				}
			}
		}
		// this.display.mainBranchGroup.addChild(this.currentMolecule.getBranchGroup());
		if (this.controlPanel != null)
			this.controlPanel.updateComponents();

	}

	void switchToMode(int mode) {

		this.mode = mode;
		this.currentlySelected.clear();
		this.currentJoint = null;
		this.updateSelection();
		((ControlPanel) tabbed.getComponentAt(mode)).updateTextArea();
		this.controlPanel = (ControlPanel) tabbed.getComponentAt(mode);

		// if (mode == MoleculeEditor.MODE_PROPERTIES)

		this.controlPanel.clearLogs();

		this.currentJoint = null;
		if (mode == MoleculeEditor.MODE_GRANULARITY) {
			this.controlPanel.log("Select Atoms to combine", 0);
		}
		// else if (mode == MoleculeEditor.MODE_PROPERTIES) {
		// this.controlPanel
		// .log(
		// "Select one or more CG Particles in order to\nchange their properties",
		// 0);
		// }
		else if (mode == MoleculeEditor.MODE_CONFORMATION) {
			this.controlPanel.log("Select Atoms to move", 0);
			this.controlPanel
					.log(
							"Select an Atom to serve as a Center Point\n if you want to use the Rotate Operation",
							0);
		}
		this.controlPanel.updateComponents();
		this.display.canvas.repaint();
	}

	public void mouseClicked(MouseEvent arg0) {

		// if (!this.editing)
		// return;
		if (arg0.getButton() == MouseEvent.BUTTON1) {

			Sphere pickedSphere = this.display.pick(arg0);

			if (pickedSphere == null && arg0.getClickCount() < 2) {

			} else if (pickedSphere == null) {
				this.currentlySelected.clear();

			} else {
				Atom_3D pickedAtom = (Atom_3D) pickedSphere.getUserData();

				if (this.currentJoint == pickedAtom)
					return;
				if (mayCurrentlyPickThis(pickedAtom, true) == false)
					return;

				if (this.currentlySelected.contains(pickedAtom)) {
					this.currentlySelected.removeElement(pickedAtom);
				} else {
					this.currentlySelected.addElement(pickedAtom);
					// if(this.currentJoint==pickedAtom) {
					// this.currentJoint=null;
					// }
				}
			}
		} else if (arg0.getButton() == MouseEvent.BUTTON3) {
			if (this.mode != MoleculeEditor.MODE_CONFORMATION)
				return;
			Sphere pickedSphere = this.display.pick(arg0);

			if (pickedSphere == null && arg0.getClickCount() < 2) {

			} else if (pickedSphere == null) {
				this.currentJoint = null;

			} else {
				Atom_3D pickedAtom = (Atom_3D) pickedSphere.getUserData();

				if (this.currentlySelected.contains(pickedAtom))
					return;

				if (this.currentJoint == pickedAtom) {
					this.currentJoint = null;
				} else {
					this.currentJoint = pickedAtom;
					// if(this.currentlySelected.contains(pickedAtom)) {
					// this.currentlySelected.removeElement(pickedAtom);
					// }
				}

			}

		}
		this.updateSelection();
	}

	public void handleRubberBandSelection() {

		Sphere[] picks = this.display.pickRubberBand(this.rubberband);

		if (picks.length < 1)
			return;

		boolean allAlreadySelected = true;
		for (int i = 0; i < picks.length; i++) {
			Atom_3D pickedAtom = (Atom_3D) picks[i].getUserData();
			if (!this.currentlySelected.contains(pickedAtom))
				allAlreadySelected = false;
		}
		if (allAlreadySelected) {
			for (int i = 0; i < picks.length; i++) {
				Atom_3D pickedAtom = (Atom_3D) picks[i].getUserData();
				this.currentlySelected.removeElement(pickedAtom);
			}
		} else {

			for (int i = 0; i < picks.length; i++) {

				Atom_3D pickedAtom = (Atom_3D) picks[i].getUserData();

				if (this.currentlySelected.isEmpty()) {
					this.currentlySelected.addElement(pickedAtom);

				} else {

					if (this.mayCurrentlyPickThis(pickedAtom, false))
						this.currentlySelected.addElement(pickedAtom);
				}
			}
		}
		this.controlPanel.updateComponents();
		this.updateSelection();
	}

	public void mouseEntered(MouseEvent arg0) {

		previousMousePos = arg0.getPoint();
		this.display.canvas.requestFocus();
	}

	public void mouseExited(MouseEvent arg0) {

	}

	public void mousePressed(MouseEvent arg0) {

	}

	public void mouseReleased(MouseEvent arg0) {
		if (arg0.getButton() != MouseEvent.BUTTON1)
			return;

	}

	public void mouseDragged(MouseEvent arg0) {

	}

	final float sensitivity = 0.5f;
	Point initRb = null;

	public void mouseMoved(MouseEvent arg0) {

		float x = arg0.getX() - this.previousMousePos.x;
		float y = arg0.getY() - this.previousMousePos.y;

		x *= sensitivity;
		y *= sensitivity;
		if (this.moving == 1) {
			this.rotateCurrent(x, y);

		} else if (this.moving == 2) {
			this.translateCurrent(x, y);
		} else if (arg0.getModifiers() == MouseEvent.ALT_MASK) {

			if (this.rubberband == null) {
				this.rubberband = new Rectangle(arg0.getX(), arg0.getY(), 0, 0);
				this.currentMolecule.getMouseBehavior().setEnable(false);
				this.initRb = new Point(this.rubberband.x, this.rubberband.y);
			}

			Point start = new Point(arg0.getX() >= initRb.x ? initRb.x
					: initRb.x - (initRb.x - arg0.getX()),
					arg0.getY() >= initRb.y ? initRb.y : initRb.y
							- (initRb.y - arg0.getY()));
			Dimension size = new Dimension(Math.abs(initRb.x - arg0.getX()),
					Math.abs(initRb.y - arg0.getY()));
			rubberband = new Rectangle(start.x, start.y, size.width,
					size.height);

			this.display.setRubberband(rubberband);
		}
		// if (this.moving > 0) {
		// display.hud.atCursor = null;
		// this.display.updateAtomInfo();
		// }else {
		//				
		// }

		previousMousePos = arg0.getPoint();

		Sphere pickedSphere = this.display.pick(arg0);

		if (pickedSphere == null) {
			if (this.moving < 1)
				display.hud.atCursor = null;

			this.display.updateCanvas();

			if (this.mode == MoleculeEditor.MODE_GRANULARITY) {
				this.display.switchCursor(0);
			}
			return;
		}

		Atom_3D pickedAtom = (Atom_3D) pickedSphere.getUserData();

		if (this.moving < 1)
			display.hud.atCursor = pickedAtom;

		this.display.updateCanvas();

		if (mayCurrentlyPickThis(pickedAtom, false) == false)
			return;
		if (this.mode == MoleculeEditor.MODE_GRANULARITY) {
			this.display.switchCursor(1);
		}

	}

	public void keyPressed(KeyEvent arg0) {
		if (!canpresskey)
			return;
		canpresskey = false;

		if (arg0.getKeyCode() == KeyEvent.VK_SHIFT) {

			this.startRotating();

		} else if (arg0.getKeyCode() == KeyEvent.VK_CONTROL) {

			this.startTranslating();

		}

	}

	public void keyReleased(KeyEvent arg0) {
		canpresskey = true;
		if (arg0.getKeyCode() == KeyEvent.VK_ALT) {
			if (this.rubberband != null) {

				this.currentMolecule.getMouseBehavior().setEnable(true);
				this.handleRubberBandSelection();
				this.rubberband = null;
				this.initRb = null;
				this.display.setRubberband(rubberband);
			}
		} else if (arg0.getKeyCode() == KeyEvent.VK_SHIFT) {

			this.finishMoving();

		} else if (arg0.getKeyCode() == KeyEvent.VK_CONTROL) {

			this.finishMoving();

		}
	}

	public void keyTyped(KeyEvent arg0) {

	}

	public void combineCurrentSelection() {

		if (this.currentlySelected.isEmpty())
			return;
		if (this.currentlySelected.firstElement() instanceof Particle) {

			for (int i = 0; i < this.currentlySelected.size(); i++) {
				this.currentMolecule
						.restoreFromCoarseGrain((Particle) this.currentlySelected
								.get(i));
			}
			this.controlPanel.log(
					this.currentlySelected.size() > 1 ? this.currentlySelected
							.size()
							+ " Particles suspended" : "Particle suspended", 0);
			this.currentlySelected.clear();
			display.makeAllPickable(true);
			this.controlPanel.updateComponents();
			this.updateSelection();

		} else {

			Atom_3D[] atoms = this.currentlySelected.toArray(new Atom_3D[0]);
			Particle newParticle = new Particle(atoms);

			if (((GranularityPanel) this.controlPanel).askForInput(newParticle,
					null, null, null, null)) {

				this.currentMolecule.coarseGrain(newParticle);
				this.currentlySelected.clear();
				display.makeAllPickable(true);
				this.controlPanel.updateComponents();
				this.updateSelection();
				this.controlPanel.log("Particle created", 0);
				for (int i = 0; i < this.currentMolecule.currentCGParticles
						.size(); i++) {
					if (this.currentMolecule.currentCGParticles.get(i).particleIdentifier
							.equals(newParticle.particleIdentifier))
						this.currentMolecule.currentCGParticles.get(i).color = newParticle.color;
				}

				this.currentMolecule.updateAtomColors();
				this.setChanged(true);
			}
		}

		this.setChanged(true);
	}

	public void treatCurrent(boolean saveAsNew, boolean apply) {

		String name = this.currentMolecule.getCurrentProfileName();

		if (saveAsNew && changed) {

			String[] choices = Molecule_Profile_IO.getAllProfileNames(
					this.currentMolecule, this.currentMolecule.getPDBcode());
			String[] exclusions = { "Default Profile" };

			name = Cm2_Toolkit.showInputDialog(this.mainFrame,
					"Enter a new Name or choose an existing one",
					"Enter a name", choices, exclusions, name);

		}

		if (name == null || name.length() < 1)
			return;

		this.currentMolecule.setCurrentProfileName(name);

		try {

			if (!name.equals("Default Profile")) {
				Molecule_Profile_IO.saveProfile(this.currentMolecule, name);
				this.controlPanel.log("Profile successfully stored", 0);
			}
			if (apply) {

				this.applyCurrent();
				return;
			}
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this.mainFrame,
					"Saving Failed!\n\nReason:\n" + e.toString(), "Error",
					JOptionPane.ERROR_MESSAGE);
			this.controlPanel.log("Saving failed!", 1);
			// e.printStackTrace();
		}

		String[] names = Molecule_Profile_IO.getAllProfileNames(
				this.currentMolecule, this.currentMolecule.getPDBcode());

		toolbarProfile.setComboBoxData(names, this.currentMolecule
				.getCurrentProfileName());

		this.mainFrame.setTitle("Molecule Editor - "
				+ this.currentMolecule.getPDBcode() + " / Model "
				+ this.currentMolecule.getModelSet() + " - "
				+ this.currentMolecule.getCurrentProfileName());

		display.canvas.repaint();
		// this.leaveEditMode();
		this.setChanged(false);
	}

	public void setProfile(String selectedProfile) {

		MoleculeProfile profile;

		try {
			if (selectedProfile.equals("Default Profile"))
				profile = Molecule_Profile_IO
						.readDefaultProfile(this.currentMolecule);

			else
				profile = Molecule_Profile_IO.readProfile(this.currentMolecule,
						selectedProfile, this.currentMolecule.getPDBcode());
		} catch (Exception e) {

			// String[] names = Molecule_Profile_IO.getAllProfileNames(
			// this.currentMolecule, this.currentMolecule.getPDBcode());
			// toolbarProfile.setComboBoxData(names, this.currentMolecule
			// .getCurrentProfileName());

			JOptionPane.showMessageDialog(this.mainFrame,
					"Loading Failed!\n\nReason:\n" + e.toString(), "Error",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
			return;
		}

		// this.toolbarProfile.deactivateBtn.setVisible(selectedProfile
		// .equals("Default Profile"));

		this.currentlySelected.clear();
		this.updateSelection();

		this.currentMolecule.applyProfile(profile);
		this.currentMolecule.updateVisualAtomPositionsOnly();
		this.mainFrame.setTitle("Molecule Editor - "
				+ this.currentMolecule.getPDBcode() + " / Model "
				+ this.currentMolecule.getModelSet() + " - "
				+ this.currentMolecule.getCurrentProfileName());

		this.setChanged(false);
	}

	public void applyCurrent() {

		MoleculeProfile profile;
		try {
			profile = Molecule_Profile_IO.readProfile(this.currentMolecule,
					this.currentMolecule.getCurrentProfileName(),
					this.currentMolecule.getPDBcode());
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this.mainFrame,
					"Operation Failed!\n\nReason:\n" + e.toString(), "Error",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
			return;
		}

		Molecule_3D[] toApplyTo;

		MembraneModel model = DataPool.getMembraneModel();

		Vector<Molecule_3D> temp = new Vector<Molecule_3D>();
		if (this.currentMolecule instanceof Protein_3D) {
			toApplyTo = model.getAllMoleculesInMembrane(this.currentMolecule
					.getDomainID(), this.currentMolecule.getLayerID());

			for (int i = 0; i < toApplyTo.length; i++) {
				if (toApplyTo[i].getPDBFile().equals(
						this.currentMolecule.getPDBFile()))
					temp.addElement(toApplyTo[i]);
			}
			toApplyTo = temp.toArray(new Molecule_3D[0]);

			int val = JOptionPane.showConfirmDialog(null,
					"Do you want to apply Conformation?", "Apply Profile",
					JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
			if (val == JOptionPane.NO_OPTION) {
				profile.positions = new Point3f[0];
			}
			asked = false;
			rotateCoarseGrainMembers = false;
			applyProfile(toApplyTo, profile);

		} else {
			Lipid_3D lip = (Lipid_3D) this.currentMolecule;

			Molecule_3D[] ec = model.getAllExampleLipidsExtraInDomain(lip
					.getDomainID(), lip.getLayerID());

			Molecule_3D[] ic = model.getAllExampleLipidsIntraInDomain(lip
					.getDomainID(), lip.getLayerID());

			Molecule_3D[] liveonesEC = model.getAllLipidsExtraInDomain(lip
					.getDomainID(), lip.getLayerID());
			Molecule_3D[] liveonesIC = model.getAllLipidsIntraInDomain(lip
					.getDomainID(), lip.getLayerID());

			for (int i = 0; i < ec.length; i++) {
				if (ec[i].getPDBFile().equals(lip.getPDBFile()))
					temp.addElement(ec[i]);
			}
			ec = temp.toArray(new Molecule_3D[0]);
			temp.clear();

			for (int i = 0; i < ic.length; i++) {
				if (ic[i].getPDBFile().equals(lip.getPDBFile()))
					temp.addElement(ic[i]);
			}
			ic = temp.toArray(new Molecule_3D[0]);
			temp.clear();

			for (int i = 0; i < liveonesEC.length; i++) {
				if (liveonesEC[i].getPDBFile().equals(lip.getPDBFile()))
					temp.addElement(liveonesEC[i]);
			}
			liveonesEC = temp.toArray(new Molecule_3D[0]);
			temp.clear();

			for (int i = 0; i < liveonesIC.length; i++) {
				if (liveonesIC[i].getPDBFile().equals(lip.getPDBFile()))
					temp.addElement(liveonesIC[i]);
			}
			liveonesIC = temp.toArray(new Molecule_3D[0]);
			temp.clear();

			JPanel panel = new JPanel();
			panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

			JCheckBox cb2 = new JCheckBox("Lipid Template");

			JCheckBox cb4 = new JCheckBox("Actual Lipids");

			JCheckBox cb5 = new JCheckBox("EXTRA");
			JCheckBox cb6 = new JCheckBox("INTRA");

			JPanel p1 = new JPanel();
			p1.add(cb2);
			p1.add(cb4);
			p1.setBorder(BorderFactory.createTitledBorder("Members"));

			JPanel p3 = new JPanel();
			p3.add(cb5);
			p3.add(cb6);
			p3.setBorder(BorderFactory.createTitledBorder("Membrane Side"));

			panel.add(p3);
			panel.add(p1);

			if (ec.length < 1 && liveonesEC.length < 1)
				cb5.setVisible(false);
			if (ic.length < 1 && liveonesIC.length < 1)
				cb6.setVisible(false);

			if (ec.length < 1 && ic.length < 1) {
				cb2.setVisible(false);
			}
			if (liveonesEC.length < 1 && liveonesIC.length < 1) {
				cb4.setVisible(false);
			}

			cb5.setSelected(true);
			cb6.setSelected(true);

			cb2.setSelected(true);

			cb4.setSelected(false);

			final JDialog dia = new JDialog();

			JPanel panel2 = new JPanel();
			JButton ok = new JButton("Ok");
			panel2.add(ok);
			ok.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent arg0) {
					dia.dispose();

				}

			});

			dia.setTitle("Apply Conformation to...");
			dia.getContentPane().add(panel, BorderLayout.NORTH);
			dia.getContentPane().add(panel2, BorderLayout.SOUTH);
			dia.setLocationRelativeTo(this.toolbarProfile.applyBtn);
			dia.setAlwaysOnTop(true);
			dia.setModal(true);
			dia.pack();
			dia.toFront();
			dia.setVisible(true);

			this.mainFrame.setVisible(false);
			this.mainFrame.dispose();

			Point3f[] backup = profile.positions;

			if (!cb5.isSelected() || !cb2.isSelected()) {
				profile.positions = new Point3f[0];
			}

			applyProfile(ec, profile);
			profile.positions = backup;

			if (!cb5.isSelected() || !cb4.isSelected()) {
				profile.positions = new Point3f[0];
			}

			applyProfile(liveonesEC, profile);
			profile.positions = backup;

			if (!cb6.isSelected() || !cb2.isSelected()) {
				profile.positions = new Point3f[0];
			}

			applyProfile(ic, profile);
			profile.positions = backup;

			if (!cb6.isSelected() || !cb4.isSelected()) {
				profile.positions = new Point3f[0];
			}

			applyProfile(liveonesIC, profile);
			profile.positions = backup;

			asked = false;
			rotateCoarseGrainMembers = false;

		}

	}

	public void cancelCurrent() {
		this.mainFrame.setVisible(false);
		this.mainFrame.dispose();

	}

	private void startRotating() {

		if (this.tabbed.getSelectedIndex() < 1)
			return;
		if (this.currentJoint == null || this.currentlySelected.isEmpty())
			return;

		Point3f[] prevPoints = new Point3f[this.currentMolecule.allAtomsSorted.length];

		for (int i = 0; i < this.currentMolecule.allAtomsSorted.length; i++) {
			Atom_3D atom = this.currentMolecule.allAtomsSorted[i];
			prevPoints[i] = new Point3f(atom.position);
		}
		this.history.addElement(prevPoints);

		this.controlPanel.updateComponents();

		this.currentMolecule.getMouseBehavior().setEnable(false);

		moving = 1;

	}

	private void startTranslating() {
		if (this.tabbed.getSelectedIndex() < 1)
			return;
		if (this.currentlySelected.isEmpty())
			return;
		Point3f[] prevPoints = new Point3f[this.currentMolecule.allAtomsSorted.length];

		for (int i = 0; i < this.currentMolecule.allAtomsSorted.length; i++) {
			Atom_3D atom = this.currentMolecule.allAtomsSorted[i];
			prevPoints[i] = new Point3f(atom.position);
		}
		this.history.addElement(prevPoints);

		this.controlPanel.updateComponents();

		this.currentMolecule.getMouseBehavior().setEnable(false);

		moving = 2;

	}

	private void finishMoving() {

		moving = 0;

		this.currentMolecule.getMouseBehavior().setEnable(true);
		this.setChanged(true);
		if (this.tabbed.getSelectedIndex() < 1)
			return;

	}

	// boolean toggle=true;
	private synchronized void rotateCurrent(float x, float y) {

		// toggle=!toggle;
		// if(toggle)return;
		Matrix3f rotationRelative = Cm2_Toolkit.createAUnitMatrix();
		Cm2_Toolkit.rotate_X_Axis_Of_CoordinateSystem(rotationRelative, y);
		Cm2_Toolkit.rotate_Y_Axis_Of_CoordinateSystem(rotationRelative, x);
		Vector<Atom_3D> toRotate = new Vector<Atom_3D>();
		for (int i = 0; i < this.currentlySelected.size(); i++) {
			Atom_3D atom = this.currentlySelected.get(i);
			toRotate.addElement(atom);
			if (atom instanceof Particle) {
				Particle p = (Particle) atom;
				for (int j = 0; j < p.includedAtoms.size(); j++) {
					toRotate.addElement(p.includedAtoms.get(j));
				}
			}
		}
		Point3f ref = new Point3f(this.currentJoint.position);
		ref = Cm2_Toolkit.transformToNewPoint(this.currentMolecule
				.getRotationMatrix(), ref);

		Point3f[] points = new Point3f[toRotate.size()];

		final Matrix3f revMatrix = Cm2_Toolkit.copyMatrix(this.currentMolecule
				.getRotationMatrix());
		revMatrix.invert();

		for (int i = 0; i < points.length; i++) {
			Atom_3D atom = toRotate.get(i);
			points[i] = new Point3f(atom.position);
			points[i] = Cm2_Toolkit.transformToNewPoint(this.currentMolecule
					.getRotationMatrix(), points[i]);
			points[i].sub(ref);
			points[i] = Cm2_Toolkit.transformToNewPoint(rotationRelative,
					points[i]);
			points[i].add(ref);

			points[i] = Cm2_Toolkit.transformToNewPoint(revMatrix, points[i]);
			atom.position = points[i];

		}

		this.currentMolecule.updateVisualAtomPositionsOnly();

	}

	private synchronized void translateCurrent(float x, float y) {

		// toggle=!toggle;
		// if(toggle)return;

		Vector<Atom_3D> toRotate = new Vector<Atom_3D>();
		for (int i = 0; i < this.currentlySelected.size(); i++) {
			Atom_3D atom = this.currentlySelected.get(i);
			toRotate.addElement(atom);
			if (atom instanceof Particle) {
				Particle p = (Particle) atom;
				for (int j = 0; j < p.includedAtoms.size(); j++) {
					toRotate.addElement(p.includedAtoms.get(j));
				}
			}
		}
		y = -y;
		if (Math.abs(y) == 0f)
			y = 0f;

		Vector3f vDirection = new Vector3f(x, y, 0);
		vDirection.normalize();
		Vector3f camPos = new Vector3f(this.display.getCurrentEyePosition());

		vDirection.scale(camPos.length() / 12f);

		if (vDirection.toString().contains("NaN")) {
			return;
		}

		final Matrix3f revMatrix = Cm2_Toolkit.copyMatrix(this.currentMolecule
				.getRotationMatrix());
		revMatrix.invert();

		Point3f[] points = new Point3f[toRotate.size()];

		for (int i = 0; i < points.length; i++) {
			Atom_3D atom = toRotate.get(i);
			points[i] = new Point3f(atom.position);
			points[i] = Cm2_Toolkit.transformToNewPoint(this.currentMolecule
					.getRotationMatrix(), points[i]);

			points[i].add(vDirection);

			points[i] = Cm2_Toolkit.transformToNewPoint(revMatrix, points[i]);

			atom.position = points[i];

		}

		this.currentMolecule.updateVisualAtomPositionsOnly();

	}

	public void undoConformation(boolean fromHistory) {
		Point3f[] prevPoints;

		if (!fromHistory) {
			prevPoints = this.initialPoints;
			this.history.clear();
		} else if (this.history.size() > 0) {

			prevPoints = this.history.lastElement();
			this.history.removeElement(prevPoints);
		} else
			return;

		this.setAtomPositions(prevPoints);
		if (!fromHistory) {
			this.controlPanel.log("Initial Conformation restored!", 0);
		}

		this.setChanged(true);
	}

	public void setDefaultConformation() {

		try {

			MoleculeProfile profile = Molecule_Profile_IO
					.readDefaultProfile(this.currentMolecule);
			this.history.clear();
			this.setAtomPositions(profile.positions);
			this.controlPanel.log("Default Conformation restored!", 0);
		} catch (Exception e) {

			JOptionPane.showMessageDialog(this.mainFrame,
					"Loading Failed!\n\nReason:\n" + e.toString(), "Error",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
			this.controlPanel.log("Loading failed!", 1);
			return;
		}
		this.setChanged(true);
	}

	private void setAtomPositions(Point3f[] prevPoints) {
		for (int i = 0; i < this.currentMolecule.allAtomsSorted.length; i++) {
			Atom_3D atom = this.currentMolecule.allAtomsSorted[i];
			atom.position = prevPoints[i];
		}

		for (int i = 0; i < this.currentMolecule.currentCGParticles.size(); i++) {
			Particle p = this.currentMolecule.currentCGParticles.get(i);
			if (p.includedAtoms.size() > 1) {
				MyBounds mb = new MyBounds(p.includedAtoms
						.toArray(new Atom_3D[0]));
				p.position = mb.center;
			} else {
				p.position = new Point3f(
						p.includedAtoms.firstElement().position);
			}
		}
		this.currentMolecule.updateVisualAtomPositionsOnly();

		this.controlPanel.updateComponents();

	}

	public void editMembers() {

		Particle p = (Particle) this.currentlySelected.firstElement();
		this.particleExtending = p;
		this.currentlySelected.clear();

		this.currentMolecule.restoreFromCoarseGrain(p);

		this.controlPanel.log("Choose new Members...", 0);

		for (int i = 0; i < p.includedAtoms.size(); i++) {
			this.currentlySelected.addElement(p.includedAtoms.get(i));
		}
		display.makeAllPickable(true);
		this.controlPanel.updateComponents();
		this.updateSelection();

		this.tabbed.setEnabled(false);
		this.toolbarProfile.saveAsBtn.setEnabled(false);
		this.toolbarProfile.applyBtn.setEnabled(false);

	}

	public void recombineEditedMembers(boolean applied) {

		if (applied) {
			this.particleExtending.set(this.currentlySelected
					.toArray(new Atom_3D[0]));
			this.setChanged(true);
			this.controlPanel.log("Particle changed", 0);
		}
		this.currentlySelected.clear();
		this.currentMolecule.coarseGrain(this.particleExtending);
		this.currentlySelected.addElement(this.particleExtending);
		this.particleExtending = null;

		display.makeAllPickable(true);
		this.controlPanel.updateComponents();
		this.updateSelection();

		this.tabbed.setEnabled(true);
		this.toolbarProfile.saveAsBtn.setEnabled(true);
		this.toolbarProfile.applyBtn.setEnabled(true);
	}

	public void setChanged(boolean b) {
		this.changed = b;

		if (b) {
			this.toolbarProfile.applyBtn.setText("Save and Apply");

		} else {
			this.toolbarProfile.applyBtn.setText("Apply Profile");

		}
		this.toolbarProfile.saveAsBtn.setVisible(b);

	}

	boolean asked = false;
	boolean rotateCoarseGrainMembers = false;

	public void applyProfile(Molecule_3D[] molsToApplyTo,
			MoleculeProfile profile) {

		if (DataPool.J3DSettings.enableFastTransitions)
			this.engine.membraneview.getMembrane3D().dockScreen(false);
		
		for (int i = 0; i < molsToApplyTo.length; i++) {
			Molecule_3D mol = molsToApplyTo[i];

			BranchGroup parent = (BranchGroup) mol.getBranchGroup().getParent();
			if (parent != null)
				parent.removeChild(mol.getBranchGroup());

			for (int j = 0; j < mol.currentCGParticles.size(); j++) {
				if ((asked == false)
						&& mol.currentCGParticles.get(j).includedAtoms.size() > 1) {
					asked = true;
					int val = JOptionPane
							.showConfirmDialog(
									null,
									"Do you want to adapt Atoms that included by Coarse-Crain-Particles\n"
											+ "to the Particle Alignment? The Group of Atoms will be rotated\n"
											+ "in order to regain the original Bond Lengths.\n\n",
									"Adapt Hidden Atoms",
									JOptionPane.YES_NO_OPTION,
									JOptionPane.QUESTION_MESSAGE);
					if (val == JOptionPane.YES_OPTION)
						rotateCoarseGrainMembers = true;
				}
			}
			try {
				if (rotateCoarseGrainMembers)
					mol.rotateCoarseGrainMembers();
			} catch (Exception e) {
				e.printStackTrace();
			}
			mol.applyProfile(profile);

			mol.updateGeometry();
			if (parent != null)
				parent.addChild(mol.getBranchGroup());

		}

		if (DataPool.J3DSettings.enableFastTransitions)
			this.engine.membraneview.getMembrane3D().dockScreen(true);
		
		engine.membraneOperations.refreshViews(true, true, false, true, true,
				true);
	}
	
	/**
	 * Returns the Display of this MoleculeEditor. Relevant for
	 * optimizing stereo view on this class by changing the stereo
	 * slider.
	 * 
	 * @return
	 */
	public Display getDisplay(){
		return this.display;
	}
	
}
