/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.constants;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.io.File;
import java.util.Hashtable;

import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.View;
import javax.swing.JProgressBar;
import javax.vecmath.Vector2f;

import org.cellmicrocosmos.cm2.io.Compatibility;
import org.cellmicrocosmos.cm2.io.FileIOHandler;


/**
 * A number of major hardcoded variables are summarized here. 
 * These are mainly standard values which can be changed by the user
 * by changing the settings of the program.
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class CM2_CONSTANTS {

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net, Sebastian Rubert, Bjoern Sommer bjoern@CELLmicrocosmos.org
	 * 
	 */
	public static class GENERAL {

		public final static String NAME_OF_THIS_PROGRAM = "CELLmicrocosmos MembraneEditor";

		public final static String SHORT_NAME_OF_THIS_PROGRAM = "CmME";

		public final static String VERSION = "Version 2.2.2_0";

		public final static String STANDARD_EDITION_NAME = "Standard";

		public final static String DEVELOPER_EDITION_NAME = "Developer";

		public final static String STANDALONE_NAME = "(Standalone)";

		public final static String WEBSTART_NAME = "(Webstart)";

		public static final String CM2_HOMEPAGE = "http://Cm2.CELLmicrocosmos.org";

		public final static String COPYRIGHT = "Copyright (C) 2005-2014 The CELLmicrocosmos 2 Development Team\n"
												+"(Bjrn Sommer, Tim Dingersen et al., Bielefeld University)\n"
												+"Released under GNU GENERAL PUBLIC LICENSE 3 (GPL3)";

		public final static String OTHERNOTES = "Includes some external source code and packages from other projects.\n"
				+ "For a detailed list and/or the complete GPL3 license just click the button below.";

	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
	 * 
	 */
	public static class GUI_SETTINGS_AND_CONTROLS {

		/**
		 * The Developer Edition comes with the PlugIn-Manager, which allows to
		 * work with user generated algorithms. The standalone version works
		 * with .JAR and .JAVA files, the webstart version only with precompiled
		 * .JAR files. For compiling this JAR file, cm2_membranepackingalgorithm.jar is
		 * needed. The Standard Edition does not provide these options.
		 */
		public static final boolean DEVELOPER_EDITION = true;

		// ########################################
		// Size of some frames relative to screensize
		public static final double MAIN_FRAME_SCALE = 1;

		public static final double PDBSEARCH_FRAME_SCALE = 0.75;

		public static final double HELP_FRAME_SCALE = 0.85;

		// ##########################################

		/**
		 * the max numbers of previous selections that remain in the combobox in
		 * the pdb search dialog
		 */
		public static final int MAX_PDB_SEARCH_HISTORY_SIZE = 15;
		
		/**
		 * the standard height for multilayers for the Add Multilayer dialog
		 */
		public static final Double STANDARD_MULTILAYER_Y_TRANSLATION = 100d;
		
		/**
		 * the standard height for multilayers for the Add Multilayer dialog
		 */
		public static final Double STANDARD_MULTILAYERY_MOLECULE_HEIGHT = 100d;

		/**
		 * the max numbers of previous selections that remain in the combobox
		 * for quickload
		 */
		public static final int MAX_QUICKLOAD_HISTORY_SIZE = 4;

		/**
		 * The Time the user has to hold down the mouse button to start
		 * DragNDrop
		 * 
		 */
		public static final int DND_TIMER_COUNTDOWN = 800;

		/**
		 * modifiers for the middle Mouse Button for Systems that only have one
		 * button (like Mac)
		 */
		public static final int MIDDLE_BUTTON_MODIFIER = 17; // SHIFT

		/**
		 * modifiers for the right Mouse Button for Systems that only have one
		 * button (like Mac)
		 */
		public static final int RIGHT_BUTTON_MODIFIER = 18; // CTRL

		/**
		 * Alternative Text Color to use
		 */
		public static final Color ALT_TEXT_COLOR = new JProgressBar()
				.getForeground().darker();

		public static final Font FONT_PDB_TEXTFIELDS = new Font("Courier",
				Font.PLAIN, 14);
		
		/**
		 * Decides if the background color of the UI is also used for the 3D universe
		 */
		public static final boolean SHOW_UI_BACKGROUND = true;

	}

	/**
	 * Priorities for some Threads. A High Priority speeds up things but slows
	 * down the response of the Progress Dialog and it's Cancel Option and vice
	 * versa
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public static class THREADS {
		public static final int PRIORITY_SEARCH_THREAD = Thread.MIN_PRIORITY;

		public static final int PRIORITY_LOAD_MEMBRANE_THREAD = Thread.NORM_PRIORITY;

		public static final int PRIORITY_GENERATE_MEMBRANE_THREAD = Thread.MAX_PRIORITY;

		public static final int PRIORITY_LIPID_FILLING_ALGORITHMS = Thread.NORM_PRIORITY;
	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
	 * 
	 */
	public static class IO {

		public static final boolean DO_BORDER_CHECK_WHEN_LOADING = false;

		// ########### Files #################################################
		public static final File CONFIG_FILE = new File(FileIOHandler.home
				+ "cm2settings" + File.separator + "config.xml");

		public static final File PLUGIN_FILE = new File(FileIOHandler.home
				+ "cm2settings" + File.separator + "plugins.txt");

		public final static File DEFAULT_DATABASE = new File("Local_Database"
				+ File.separator + "DefaultDataBase.xml");

		public static final File ENTRIES_FILE = new File(FileIOHandler.home
				+ "cm2settings" + File.separator + "rcsb_entries.idx")
				.getAbsoluteFile();

		public static final File UAMU_FILE = new File(FileIOHandler.home
				+ "cm2settings" + File.separator
				+ "unified_atomic_mass_units.txt");

		public static final File LAYOUT_FILE = new File(FileIOHandler.home
				+ "cm2settings" + File.separator + "layout.bin");

		public static final File TEMP_MEMBRANE_FILE = new File("tempfile.pdb");
		
		public static final File TEMP_APLVORO_FILE = new File("tempfile.aplvoro");
		
		public static final String OPM_FILENAME_ADDON = "_OPM";
		
		public static final boolean OPM_DUMMY_LAYERS_IN_PDBEXPORT = false;

		// ########### Download Links #######################################
		public final static String DEFAULT_DOWNLOAD_LINK = "http://www.rcsb.org/pdb/files/#PDBCODE#.pdb";
		
		public final static String DEFAULT_OPM_DOWNLOAD_LINK = "http://opm.phar.umich.edu/pdb/#PDBCODE#.pdb";

		public final static String DEFAULT_SEARCH_ENTRIES_LINK = "ftp://ftp.wwpdb.org/pub/pdb/derived_data/index/entries.idx";

		public final static String DEFAULT_PREVIEW_IMAGE_LINK = "http://www.rcsb.org/pdb/images/#PDBCODE#_bio_r_250.jpg";

		public final static String DEFAULT_RCSB_SITE_LINK = "http://www.rcsb.org/pdb/explore/explore.do?structureId=#PDBCODE#";

		public final static String HTTP_TEST_LINK = "http://www.rcsb.org";

		public final static String FTP_TEST_LINK = "ftp://ftp.rcsb.org";

		// ############ Jmol Execution Link ##################
		
		public final static String DEFAULT_JMOL_EXECUTION_LINK = "java -jar -Xmx512m \""+FileIOHandler.home+"jmol"+File.separator+"Jmol.jar\"";
		
		// ############ APL@Voro Execution Link ##################
		
		public final static String DEFAULT_APLVORO_EXECUTION_LINK = null;
				
		// ############ Directories ##################
		public final static File DEFAULT_DB_DIRECTORY = new File("Local_Database");

		public final static File DEFAULT_MEMBRANE_DIRECTORY = new File(
				"Membrane_Files");

		public final static File DEFAULT_PDB_DIRECTORY = new File("Pdb_Files");

		public final static File DEFAULT_INCOMING_DIRECTORY = new File(
				"Pdb_Files" + File.separator + "User_Files");

		public final static File TEMP_DIRECTORY = new File(FileIOHandler.home
				+ "temp");

		public static final File DEFAULT_ALGORITHM_DIRECTORY = new File(
				"Cm2_MembranePackingAlgorithms");
		
		public static final File ALGORITHM_SETTINGS=new File("Algorithm_Settings");

		public static final String RAWDATA_SEPERATOR = "(SPRT)";

		public static final File MOLECULE_PROFILES_DIR = new File("Pdb_Profiles");

		public static Compatibility[] INCLUDED_COMPATIBILITIES = {
				new Amira(), new APLVoro(), new Chimera(), new Gromacs(), new ICMBrowser(), new Jmol(), 
				new MembraneEditor(), new MOE(), new PyMOL(), new Swiss(), new ViewerLite(), new VMD() };
	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public static class MEMBRANE_SIZES {

		public static final int MAX_SIZE_OF_MEMBRANE = 10000;

		public final static int MIN_SIZE_OF_MEMBRANE = 20;

		public final static Dimension DEFAULT_SIZE_OF_MEMBRANE = new Dimension(
				200, 200);

		public static final int MAX_ATOMS_FOR_PROPER_SECONDARY_STRUCTURE = 99999;

		public static final float MAX_Y_TRANSLATION = 10000;
		
		public static final float MIN_Y_TRANSLATION = -1000;
	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public static class MOLECULE_DEFAULTS {
		/**
		 * Default Color for Proteins (used if no other can be found) Also used
		 * for table grids
		 */
		public static final Color DEFAULT_PROTEIN_COLOR = new Color(230, 245, 230);

		/**
		 * Default Color for Lipids (used if no other can be found) Also used
		 * for table grids
		 */
		public static final Color DEFAULT_LIPID_COLOR = new Color(245, 230, 230);

		/**
		 * Color for protein Highlighting when editor has bright appearance
		 */
		public static final Color HIGLIGHTING_COLOR_BRIGHT = new Color(50, 50,
				150);

		/**
		 * Color for protein Highlighting when editor has dark appearance
		 */
		public static final Color HIGLIGHTING_COLOR_DARK = new Color(200, 200,
				255);

		/**
		 * 
		 */

	}

	

	// ##################################################################

	/**
	 * @author Tim Dingersen dackelnose@gmx.net
	 */
	public static class IMAGES_AND_ICONS {

		/**
		 * Directory where the Icons are located
		 */
		public static final File ICONS_PARENT = new File(FileIOHandler.home
				+ "cm2images" + File.separator + "Icons");
		// this is for backward compatibility with Cm2_1 only
		public static final File ICONS_PARENT_CM2_1 = new File(FileIOHandler.home
				+ "cm2images" + File.separator + "icons");

		/**
		 * images that are show at startup randomly (different Sizes allowed)
		 * 
		 */
		public static final String[] STARTUP_IMAGES = {
				FileIOHandler.home + "cm2images" + File.separator + "Init1.JPG",
				FileIOHandler.home + "cm2images" + File.separator + "Init2.JPG",
				FileIOHandler.home + "cm2images" + File.separator + "Init3.JPG"};

		/**
		 * The text foreground colors for the startup dialog. Each image in the
		 * array above must have its corresponding color
		 */
		public static final Color[] STARTUP_DIALOG_TEXT_FOREGROUND = {
			null, null, null};

	}

	/**
	 * J3d values
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
	 * 
	 */
	public static class J3D {

		/**
		 * The Factor that is used to scale objects in the 3d view
		 */
		public static final float J3D_ANGSTROM_SCALE = 0.01f;

//		/**
//		 * The minimum number of atoms a pdb file must have in order to be
//		 * accepted
//		 */
//		public static final int PDB_READER_MINIMUM_ATOMS_FOR_VALIDATION_OF_PDB_FILE = 3;

//		/**
//		 * 3d View Background for bright appearance
//		 */
//		public static final Color DEFAULT_COLOR_BRIGHT = Color.white;
//
//		/**
//		 * 3d View Background for dark appearance
//		 */
//		public static final Color DEFAULT_COLOR_DARK = Color.BLACK;

		// #########################################
		// Default Appearance settings

		public static final int DETAIL_MIN_PRECISION = 4;

		public static final int DETAIL_PRECISION_MODIFIER = 8;

		public static final float DETAIL_AMBIENT_LIGHT = 0.8f;

		public static final int DETAIL_SHININESS_PROTEINS = 5;

		public static final int DETAIL_SHININESS_LIPIDS = 10;

		public static final int DETAIL_POLYGON_MODE = PolygonAttributes.POLYGON_FILL;

		// ########################################

		public static final float MOUSE_SENSITIVITY = 1f;

		public static final int DETAIL_DEFAULTVERTICES = 10;

		public static final Color DEFAULTMICRODOMAINCOLOR = Color.LIGHT_GRAY;

		// ########################################
		// Clipping Settings

		public static final boolean VIEW_UNIFORM_BACKGROUND = false; 
		public static final boolean VIEW_PARALLEL_PROJECTION = false;
		public static final boolean VIEW_ALL_MULTILAYERS = false;
		public static int VIEW_FRONT_CLIP_POLICY = View.VIRTUAL_EYE;
		public static int VIEW_BACK_CLIP_POLICY = View.VIRTUAL_EYE;
		public static double VIEW_FRONT_CLIP_DISTANCE = 0.01d;
		public static double VIEW_BACK_CLIP_DISTANCE = 50.0d;
		public static double VIEW_BACK_CLIP_DISTANCE_PARALLEL = 200.0d;
		public static final boolean VIEW_ANTIALIASING = false;

		// ########################################
		// StereoView Options

		public static final int STEREOVIEW_MIN = -30;

		public static final int STEREOVIEW_MAX = 30;

		public static final int STEREOVIEW_DEFAULT = 0;

		public static final double STEREOVIEW_SCALE = 10000;

		// ########################################
		// Transitions for refreshing the MembraneView, should be turned on for
		// slower graphicards
		public static final boolean FAST_TRANSITIONS = true;
		
		public static final boolean PPE_AUTOMATIC = false;

		public static final int CRITICAL_ATOM_COUNT = 200000;//20000;

		public static final Color COLOR_COORDINATEBARS_DARK = new Color(200,
				205, 200);

		public static final Color COLOR_COORDINATEBARS_BRIGHT = new Color(40,
				75, 45);

		public static final Color COLOR_COORDINATEBARS_DARK_WET = new Color(
				170, 180, 220);

		public static final Color COLOR_COORDINATEBARS_BRIGHT_WET = new Color(
				40, 45, 130);

	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public static class DEBUG {

		/**
		 * if true, all Errors will be printed into a file instead of the System
		 * Tray
		 */
		public static final boolean REDIRECT_ERRORS_TO_LOGFILE = false;

		/**
		 * if true, lets JMol printout Logs
		 * 
		 */
		public static final boolean PRINTOUT_JMOL_LOGS = false;

		public static final boolean READ_HELPFILES_DIRECTLY_FROM_MAIN_FOLDER = false;

	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public static class MISC {

		/**
		 * number of retries when a Protein gets randomly added
		 */
		public static final int RETRIES_AT_RANDOM_PROTEIN_PLACING = 200;

		public static final boolean ALLOW_ATOM_MODE = true;

		public static final boolean ALLOW_MULTIPLE_ENTRIES_IN_DB = false;

		// For each decimal place a '0' is added as a Dividend
		public static final int LIPID_PERCENTAGE_DIVISOR = 0;
		
		// This is the standard value for the computation mode, if false, percentages are used
		public static final boolean COMPUTE_ABSOLUTE_VALUES = false;
	
	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
	 * 
	 */
	public static class ALGORITHMS {

		public static final String[] INCLUDED_ALGORITHMS = {
				"org.cellmicrocosmos.cm2.membranepackingalgorithms.AdvancedRandomPlacing",	
				"org.cellmicrocosmos.cm2.membranepackingalgorithms.atomlevelminimizer.Atom_Level_Minimizer",
				"org.cellmicrocosmos.cm2.membranepackingalgorithms.Distributor",
				"org.cellmicrocosmos.cm2.membranepackingalgorithms.IntersectionDeleter",
				"org.cellmicrocosmos.cm2.membranepackingalgorithms.LinearPlacing",
				"org.cellmicrocosmos.cm2.membranepackingalgorithms.MembraneShifter",
				"org.cellmicrocosmos.cm2.membranepackingalgorithms.RandomPlacing",
				"org.cellmicrocosmos.cm2.membranepackingalgorithms.SimulatedAnnealing",
				"org.cellmicrocosmos.cm2.membranepackingalgorithms.TheWandererRookie"
		};
		
		public static final boolean DEFAULT_START_IN_BACKGROUND = false;
		
		public static final boolean DEFAULT_SHOW_PROTOCOL = false;
		
		public static final boolean DEFAULT_UNIQUE_RANDOM = false;
		
		public static final long DEFAULT_UNIQUE_RANDOM_SEED = 0;

	}

	public static class  ATOMVALUES{

		public static final float DEFAULT_COVALENT_RADIUS = 0.6f;
		public static final float DEFAULT_VANDERWAALS_RADIUS = 1.5f;

		//Values are angstrom; x=covalent,y=van der waals
		public static void feedWithRadii(
				Hashtable<String, Vector2f> radii) {



			radii.put("H",new Vector2f(0.320f,1.20f));
			radii.put("C",new Vector2f(0.720f,1.70f));
			radii.put("N",new Vector2f(0.680f,1.55f));
			radii.put("O",new Vector2f(0.680f,1.52f));
			radii.put("P",new Vector2f(1.036f,1.800f));
			radii.put("S",new Vector2f(1.020f,1.800f));
			radii.put("CA",new Vector2f(1.76f,2.31f));
//			radii.put("FE",new Vector2f(1.420f,1.948f));
			radii.put("ZN",new Vector2f(1.22f,1.39f));
			radii.put("CD",new Vector2f(1.44f,1.58f));
			radii.put("I",new Vector2f(1.400f,1.98f));
			radii.put("CL",new Vector2f(1.02f,1.75f));
			radii.put("F",new Vector2f(0.71f,1.47f));
			radii.put("SE",new Vector2f(1.20f,1.90f));
		}
		
	}
	
	/**
	 * The compatibility list associated with the PDB Output Settings
	 * and affecting the PDBWriter. 
	 * 
	 * This has to be constantly improved and extended.
	 * 
	 * @author Bjoern Sommer bjoern@CELLmicrocosmos.org
	 *
	 */
	
	private static class Amira extends Compatibility {

		public String getApplicationName() {
			return "amira (Visage Imaging)";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return null;
		}

		public Boolean atomserials_All99999() {
			return null;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return null;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}

		public Integer requiredAtomserials() {
			return null;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return null;
		}

		public Integer shiftY() {
			return null;
		}

		public Boolean sortByType() {
			return null;
		}

		public Boolean swapYZ() {
			return null;
		}

		public Boolean writeReverseParsingInfo() {
			return null;
		}
		
		public String remark() {
			return "No restrictions known";
		}
		
	}
	
	private static class APLVoro extends Compatibility {

		public String getApplicationName() {
			return "APL@Voro";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return true;
		}

		public Boolean atomserials_All99999() {
			return false;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return null;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}

		public Integer requiredAtomserials() {
			return null;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return Compatibility.TRANSFORM_NONE;
		}

		public Integer shiftY() {
			return null;
		}

		public Boolean sortByType() {
			return true;
		}

		public Boolean swapYZ() {
			return true;
		}

		public Boolean writeReverseParsingInfo() {
			return null;
		}
		
		public String remark() {
			return null;
		}
		
	}

	private static class Chimera extends Compatibility {

		public String getApplicationName() {
			return "Chimera (UCSF)";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return null;
		}

		public Boolean atomserials_All99999() {
			return null;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return null;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}
		
		public Integer requiredAtomserials() {
			return Compatibility.ATOMSERIALS_PROTSONLY;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return null;
		}

		public Integer shiftY() {
			return null;
		}

		public Boolean sortByType() {
			return null;
		}

		public Boolean swapYZ() {
			return null;
		}

		public Boolean writeReverseParsingInfo() {
			return null;
		}
		
		public String remark() {
			return null;
		}
		
	}
	
	private static class Gromacs extends Compatibility {

		public String getApplicationName() {
			return "Gromacs";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return true;
		}

		public Boolean atomserials_All99999() {
			return false;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return true;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}

		public Integer requiredAtomserials() {
			return null;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return Compatibility.TRANSFORM_NONE;
		}

		public Integer shiftY() {
			return Compatibility.TRANSFORM_BOTTOM;
		}

		public Boolean sortByType() {
			return true;
		}

		public Boolean swapYZ() {
			return true;
		}

		public Boolean writeReverseParsingInfo() {
			return null;
		}
		
		public String remark() {
			return null;
		}
		
	}
	
	private static class ICMBrowser extends Compatibility {

		public String getApplicationName() {
			return "ICMBrowser (Molsoft)";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return null;
		}

		public Boolean atomserials_All99999() {
			return null;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return null;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}
		
		public Integer requiredAtomserials() {
			return null;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return null;
		}

		public Integer shiftY() {
			return null;
		}

		public Boolean sortByType() {
			return null;
		}

		public Boolean swapYZ() {
			return null;
		}

		public Boolean writeReverseParsingInfo() {
			return null;
		}
		
		public String remark() {
			return "May have problems with multiple layers";
		}
		
	}
	
	private static class Jmol extends Compatibility { // no restrictions known

		public String getApplicationName() {
			return "Jmol";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return null;
		}

		public Boolean atomserials_All99999() {
			return null;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return null;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}
		
		public Integer requiredAtomserials() {
			return null;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return null;
		}

		public Integer shiftY() {
			return null;
		}

		public Boolean sortByType() {
			return null;
		}

		public Boolean swapYZ() {
			return null;
		}

		public Boolean writeReverseParsingInfo() {
			return null;
		}
		
		public String remark() {
			return "No restrictions known";
		}
		
	}
	
	private static class MembraneEditor extends Compatibility {

		public String getApplicationName() {
			return "MembraneEditor (CELLmicrocosmos)";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return null;
		}

		public Boolean atomserials_All99999() {
			return null;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return null;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}
		
		public Integer requiredAtomserials() {
			return null;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return null;
		}

		public Integer shiftY() {
			return null;
		}

		public Boolean sortByType() {
			return null;
		}

		public Boolean swapYZ() {
			return null;
		}

		public Boolean writeReverseParsingInfo() {
			return true;
		}
		
		public String remark() {
			return null;
		}
		
	}
	
	private static class MOE extends Compatibility {

		public String getApplicationName() {
			return "MOE (Chemical Computing Group)";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return null;
		}

		public Boolean atomserials_All99999() {
			return null;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return null;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}
		
		public Integer requiredAtomserials() {
			return null;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return null;
		}

		public Integer shiftY() {
			return null;
		}

		public Boolean sortByType() {
			return null;
		}

		public Boolean swapYZ() {
			return null;
		}

		public Boolean writeReverseParsingInfo() {
			return false;
		}
		
		public String remark() {
			return null;
		}
		
	}

	private static class PyMOL extends Compatibility {

		public String getApplicationName() {
			return "PyMOL (Schrdinger)";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return null;
		}

		public Boolean atomserials_All99999() {
			return null;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return null;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}
		
		public Integer requiredAtomserials() {
			return null;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return null;
		}

		public Integer shiftY() {
			return null;
		}

		public Boolean sortByType() {
			return null;
		}

		public Boolean swapYZ() {
			return null;
		}

		public Boolean writeReverseParsingInfo() {
			return null;
		}
		
		public String remark() {
			return "No restrictions known";
		}
		
	}
	
	private static class Swiss extends Compatibility { // no restrictions known

		public String getApplicationName() {
			return "Swiss-PDBViewer (GlaxoSmithKline)";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return null;
		}

		public Boolean atomserials_All99999() {
			return null;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return null;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}
		
		public Integer requiredAtomserials() {
			return null;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return null;
		}

		public Integer shiftY() {
			return null;
		}

		public Boolean sortByType() {
			return null;
		}

		public Boolean swapYZ() {
			return null;
		}

		public Boolean writeReverseParsingInfo() {
			return null;
		}

		public String remark() {
			return "Problems with larger membranes";
		}
		
	}

	private static class ViewerLite extends Compatibility {

		public String getApplicationName() {
			return "ViewerLite (accelrys)";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return false;
		}

		public Boolean atomserials_All99999() {
			return null;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return null;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}

		public Integer requiredAtomserials() {
			return null;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return null;
		}

		public Integer shiftY() {
			return null;
		}

		public Boolean sortByType() {
			return null;
		}

		public Boolean swapYZ() {
			return null;
		}
		
		public Boolean writeReverseParsingInfo() {
			return null;
		}

		public String remark() {
			return null;
		}
		
	}
	
	private static class VMD extends Compatibility {

		public String getApplicationName() {
			return "VMD";
		}

		public Boolean adoptTitleSections() {
			return null;
		}

		public Boolean assignResSeqNumbers() {
			return null;
		}

		public Boolean atomserials_All99999() {
			return null;
		}
		
		public Boolean atomserials_RestartNumberingWith0() {
			return null;
		}

		public Boolean caseSensitiveChainIds() {
			return null;
		}
		
		public Integer requiredAtomserials() {
			return null;
		}

		public Integer requiredChains() {
			return null;
		}

		public Integer shiftXZ() {
			return null;
		}

		public Integer shiftY() {
			return null;
		}

		public Boolean sortByType() {
			return null;
		}

		public Boolean swapYZ() {
			return null;
		}

		public Boolean writeReverseParsingInfo() {
			return false;
		}
		
		public String remark() {
			return null;
		}
		
	}

}
