/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.dataobjects;

import java.awt.Color;
import java.util.Vector;

import javax.media.j3d.BranchGroup;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;


import com.sun.j3d.utils.geometry.Primitive;

/**
 * A class that represents an Atom, very similar to a line in the coordinate
 * section of a pdb file.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class Atom_3D implements Comparable {

	public Primitive object_3D = null;

	public BranchGroup branchGroup = null;

	public String recordName = "";

	public Point3f position;

	public String name = "";

	public String residueName = "";

	public String occupancy = "";

	public String tempFactor = "";

	public String element = "";

	public String charge = "";
	
	public String chainID="";

	public Color color = Color.black;

	public float radiusCov = CM2_CONSTANTS.ATOMVALUES.DEFAULT_COVALENT_RADIUS;

	public float radiusVDW = CM2_CONSTANTS.ATOMVALUES.DEFAULT_VANDERWAALS_RADIUS;

	public Vector<Atom_3D> currentNeighbors = new Vector<Atom_3D>();

	public Integer indexForPDBWriter = null;

	public int comparator = -1;

	public float massValue=0;

	/**
	 * Creates an empty instance
	 * 
	 */
	public Atom_3D() {

	}

	/**
	 * Creates an instance using the passed atom entry string of a pdb file in
	 * order to set up the varialbles. If any fields in the string are out of
	 * their expected range, those referring variables will remain empty. If any
	 * coordinate values are missing or invalid, an exception will be thrown
	 * 
	 * ! THIS CONSTRUCTOR IS NOT MEANT TO BE USED FOR PLUGIN DEVELOPERS !
	 * 
	 * @param an
	 *            unedited line of the coordinate section of a pdb file
	 * @throws Exception
	 *             if no coordinates could be determined properly
	 */
	public Atom_3D(final String line) throws Exception {

		if (line == null)
			return;
		if (line.startsWith("TER"))
			throw new Exception("Atom Entry is a TER Entry");
		try {
			this.recordName = line.substring(0, 7).trim().toUpperCase();

		} catch (final Exception e) {
		}

		try {
			final String sx = line.substring(30, 38);
			final String sy = line.substring(38, 46);
			final String sz = line.substring(46, 54);

			final float x = Float.parseFloat(sx.trim());
			final float y = Float.parseFloat(sy.trim());
			final float z = Float.parseFloat(sz.trim());

			this.position = new Point3f(x, y, z);

		} catch (final Exception e) {
			throw new Exception("Invalid Coordinates");
		}

		try {
			this.name = line.substring(11, 16).trim().toUpperCase();
		} catch (final Exception e) {

		}

		try {
			this.residueName = line.substring(17, 20).trim();
		} catch (final Exception e) {

		}
		// try {
		// this.resSeq = line.substring(23, 27).trim();
		// } catch (final Exception e) {
		//
		// }
		try {
			this.chainID = line.substring(21, 22).trim();
		} catch (final Exception e) {

		}

		try {
			this.occupancy = line.substring(55, 60).trim();
		} catch (final Exception e) {

		}
		try {
			this.tempFactor = line.substring(61, 66).trim();
		} catch (final Exception e) {

		}

		try {
			this.element = line.substring(76, 78).trim().toUpperCase();
			if (this.element.length() < 1) {
				try {
					this.element = ("" + line.charAt(13)).toUpperCase();
					new Integer(this.element);
					this.element = "";
				} catch (Exception e3) {

				}
			}
		} catch (final Exception e) {

			try {
				this.element = ("" + line.charAt(13)).toUpperCase();
				new Integer(this.element);
				this.element = "";
			} catch (Exception e2) {

			}

		}

		try {
			this.charge = line.substring(78, 80).trim();
		} catch (final Exception e) {

		}

	}

	/**
	 * Returns an exact clone of this instance
	 * 
	 * @return
	 */
	public Atom_3D copy() {

		final Atom_3D atom;
		if (this instanceof Particle)
			atom = new Particle(((Particle) this).includedAtoms
					.toArray(new Atom_3D[0]));
		else
			atom = new Atom_3D();

		atom.recordName = "" + this.recordName;
		atom.position = new Point3f(this.position);
		atom.name = "" + this.name;
		atom.residueName = "" + this.residueName;
		atom.occupancy = "" + this.occupancy;
		atom.tempFactor = "" + this.tempFactor;
		atom.element = "" + this.element;
		atom.charge = "" + this.charge;
		atom.color = this.color;
		atom.radiusCov = this.radiusCov;
		atom.radiusVDW = this.radiusVDW;
		atom.comparator = this.comparator;
		atom.chainID=this.chainID;
		atom.massValue=this.massValue;
		if (this instanceof Particle)
			((Particle) atom).particleIdentifier = ((Particle) this).particleIdentifier;
		return atom;
	}

	/**
	 * Returns a String for Debugging
	 */
	public String toString() {
		return this.name + " " + this.position;
	}

	/**
	 * Sets the position. Do not use this method directly, use
	 * applyAtomPosition() in the molecule object instead
	 * 
	 * @param newPosition
	 */
	public void setPosition(final Point3f newPosition) {
		this.position = newPosition;
	}

	/**
	 * The (anstrom) position of this Atom
	 * 
	 * @return
	 */
	public Point3f getPosition() {
		return new Point3f(this.position);
	}

	public int compareTo(Object arg0) {
		if (arg0 instanceof Atom_3D == false) {
			return 0;
		} else {

			int c = ((Atom_3D) arg0).comparator;

			if (this.comparator < c)
				return -1;
			else if (this.comparator > c)
				return 1;
			else {

				// System.out.println("Comparable: Same Comparator");
				// System.out.println(this.name+"  "+((Atom)arg0).name);
				return 0;
			}

		}
	}

}
