/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.dataobjects;

import java.util.Vector;

import javax.vecmath.Point3f;

/**
 * This class is needed to compute the bounds of molecules 
 * by using its atoms.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class MyBounds {

	public Point3f lower = new Point3f();
	public Point3f upper = new Point3f();

	public Point3f center = new Point3f();

	public float widthX = 0;
	public float widthY = 0;
	public float widthZ = 0;

	
	
	public MyBounds() {

	}

	public MyBounds(Point3f[] allPoints) {
		this.init(allPoints);
	}

	public MyBounds(Atom_3D[] allPoints) {
		Point3f[] p = new Point3f[allPoints.length];
		for (int i = 0; i < p.length; i++) {
			p[i] = allPoints[i].position;
		}

		this.init(p);

	}

	public MyBounds(Vector<Atom_3D> allPoints) {
		Point3f[] p = new Point3f[allPoints.size()];
		for (int i = 0; i < p.length; i++) {
			p[i] = allPoints.get(i).position;
		}

		this.init(p);
	}

	private void init(Point3f[] points) {

		float maxx = -1000000, maxy = -1000000, maxz = -1000000, minx = 1000000, miny = 1000000, minz = 1000000;

		final int l = points.length;
		for (int i = 0; i < l; i++) {
			final Point3f p = points[i];
			minx = Math.min(minx, p.x);
			miny = Math.min(miny, p.y);
			minz = Math.min(minz, p.z);

			maxx = Math.max(maxx, p.x);
			maxy = Math.max(maxy, p.y);
			maxz = Math.max(maxz, p.z);
		}

		lower = new Point3f(minx, miny, minz);
		upper = new Point3f(maxx, maxy, maxz);

		final float cx = (minx + ((maxx - minx) / 2f));
		final float cy = (miny + ((maxy - miny) / 2f));
		final float cz = (minz + ((maxz - minz) / 2f));

		center = new Point3f(cx, cy, cz);

		widthX = maxx - minx;
		widthY = maxy - miny;
		widthZ = maxz - minz;

	}

	public void normalize() {
		lower = new Point3f(lower.x - center.x, lower.y - center.y, lower.z
				- center.z);
		upper = new Point3f(upper.x - center.x, upper.y - center.y, upper.z
				- center.z);
		center = new Point3f();

	}

	public void scale(float scale) {
		upper.scale(scale);
		lower.scale(scale);
		center.scale(scale);

		widthX *= scale;
		widthY *= scale;
		widthZ *= scale;
	}

	public MyBounds copy() {
		MyBounds ret = new MyBounds();

		ret.lower = new Point3f(this.lower);
		ret.upper = new Point3f(this.upper);

		ret.center = new Point3f(this.center);

		ret.widthX = this.widthX;
		ret.widthY = this.widthY;
		ret.widthZ = this.widthZ;
		return ret;
	}
}
