/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.engine;

import java.awt.Color;
import java.awt.Point;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.net.Authenticator;
import java.net.PasswordAuthentication;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.vecmath.Vector2f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.Layer;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.MicroDomain_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Particle;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.Engine.J3DOperations;
import org.cellmicrocosmos.cm2.engine.Engine.MembraneOperations;
import org.cellmicrocosmos.cm2.gui.PDBFileBrowser;
import org.cellmicrocosmos.cm2.gui.dialogs.J3DSettingsDialog;
import org.cellmicrocosmos.cm2.gui.dialogs.PdbSettingsWindow;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.io.ppe.toolbox.MIFHandler;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.cellmicrocosmos.cm2.misc.ConfigParser;


/**
 * This class serves as a singleton and will store all shared data.
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Sebastian Rubert, Bjoern Sommer
 *         bjoern@CELLmicrocosmos.org
 */
public class DataPool {

	/*
	 * The (one and only) Instace of the MembraneModel that is currently opened
	 */
	private static MembraneModel currentMembrane;

	// the file currently opened
	private static File currentMembraneFile;

	public static PDBFileBrowser pdbbrowser;

	public static PdbSettingsWindow pdbsettings;

	public static J3DSettingsDialog j3dsettings;

	public static MicroDomain_3D drawingDomain = null;

	public static File workspace;

	public static boolean useSystemLookandFeel = false;

	// public static Vector<JPanel> microPanels = new Vector<JPanel>();

	public static boolean iAmWebstart = false;

	public static Hashtable<String, String[]> rawPdbData = new Hashtable<String, String[]>();

	private static int currentLayerId = 0;

	// TODO MIFHandler here correct?
	private static MIFHandler mifHandler;

	// public static void clearMicroPanels(){
	// microPanels.clear();
	// }

	// public static Vector<JPanel> getMicroPanels() {
	// return microPanels;
	// }
	//
	// public static void setMicroPanels(Vector<JPanel> inputMicroPanels) {
	// microPanels = inputMicroPanels;
	// }
	//
	// public static void addMicroPanel(JPanel inputMicroPanel) {
	// microPanels.add(inputMicroPanel);
	// }

	/**
	 * Initializes the MIFHandler needed for the PDB_TM Settings.
	 */
	public static void initMIFHandler() {
		DataPool.mifHandler = new MIFHandler();
	}

	/**
	 * Gets the MIFHandler needed for the PDB_TM alignment.
	 */
	public static MIFHandler getMifHandler() {
		return DataPool.mifHandler;
	}

	// /**
	// * Sets the MIFHandler needed for the PDB_TM alignment.
	// */
	// public static void setMifHandler(MIFHandler mifh) {
	// mifHandler = mifh;
	// }

	/**
	 * returns the Instance of the MembraneModel currently open
	 * 
	 * @return the Instance of the Membrane
	 */
	public static MembraneModel getMembraneModel() {
		return DataPool.currentMembrane;
	}

	/**
	 * replaces the current Membrane completely
	 * 
	 * @param membrane
	 */
	public static void replaceMembraneModel(final MembraneModel membrane) {
		DataPool.currentLayerId = 0;
		DataPool.currentMembrane = membrane;

	}

	/**
	 * Returns the currently opened Membranefile
	 * 
	 * @return
	 */
	public static File getCurrentMembraneFile() {
		return DataPool.currentMembraneFile;
	}

	/**
	 * sets the currently opened Membranefile
	 * 
	 * @param currentMembraneFile
	 */
	public static void setCurrentMembraneFile(final File currentMembraneFile) {
		DataPool.currentMembraneFile = currentMembraneFile;
	}

	/**
	 * Indicated wether or not the current MembraneModel has been edited again
	 * since last load or save
	 * 
	 * @return
	 */
	public static boolean getMembraneHasBeenEditedAgain() {
		if (DataPool.currentMembrane != null)
			return DataPool.currentMembrane.getMembraneHasBeenEditedAgain();
		else
			return false;
	}

	/**
	 * sets the state of membraneHasBeenEditedAgain
	 * 
	 * @param membraneHasBeenEditedAgain
	 */
	public static void setMembraneHasBeenEditedAgain(
			final boolean membraneHasBeenEditedAgain) {
		if (DataPool.currentMembrane != null)
			DataPool.currentMembrane
					.setMembraneHasBeenEditedAgain(membraneHasBeenEditedAgain);
	}

	/**
	 * Returns the next Protein ID number that should be used for a new
	 * instance. If earlier proteins have been deleted and so their ids can be
	 * reused, then one of this numbers will be returned first
	 * 
	 * @return
	 */
	public static String getNextFreeProteinId() {

		int newId = 0;
		while (true) {
			if (DataPool.currentMembrane.hasProteinWithId(new Integer(newId)
					.toString()) == false)
				return new Integer(newId).toString();
			else
				newId++;
		}
	}

	/**
	 * A subclass that stores our J3d settings. Has functions to restore these
	 * values and to store them into the config parser
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public static class J3DSettings {

		public static float precision_modifier;

		public static int minprecision;

		public static int numberofvertices;

		public static boolean light1;

		public static boolean light2;

		public static float ambientlight;

		public static int shininess_proteins;

		public static int shininess_lipids;

		public static int polygon_mode;

		public static boolean bright_appearance = false;

		public static boolean forbidDraggingInFreeLookMode = false;

		public static boolean enableFastTransitions = true;

		public static boolean enablePPEAutomatic = true;

		public static float mouseFactor;

		public static boolean antialiasing = false;
		
		public static boolean parallelProjection = false;
		
		public static boolean showAllMultiLayers = false;
		
		public static boolean uniformBackground = false;

		public static float minatomdistance = 0f;

		public static boolean shapeCollisionCheck = true;

		public static boolean atomCollisionCheck = true;
		
		public static boolean atomDistanceAbsolute = false;

		public static boolean automaticDetailSettings = true;

		public static int overallRenderMode = Molecule_3D.RENDER_MODE_POLYGON;

		public static boolean lipidPicking = true;

		public static int removeInnerAtomsFactor = 30;

		public static boolean noBackground = !CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.SHOW_UI_BACKGROUND;

		public static boolean isScreenDocked=true;

		public static boolean isInSystemTray=false;

		/**
		 * Restores settings from configParser
		 * 
		 */
		public static void restoreSettings() {

			try {
				J3DSettings.automaticDetailSettings = Boolean
						.parseBoolean(ConfigParser
								.getValue("Automatic_Detail_Settings")[0]);
			} catch (final Exception e) {

			}

			try {
				J3DSettings.forbidDraggingInFreeLookMode = Boolean
						.parseBoolean(ConfigParser
								.getValue("forbid_Dragging_In_FreeLook_Mode")[0]);
			} catch (final Exception e) {

			}

			try {
				enableFastTransitions = Boolean.parseBoolean(ConfigParser
						.getValue("enable_fast_transitions")[0]);
			} catch (Exception e) {

			}

			try {
				enablePPEAutomatic = Boolean.parseBoolean(ConfigParser
						.getValue("enable_ppe_automatic")[0]);
			} catch (Exception e) {

			}

			try {
				J3DSettings.mouseFactor = Float.parseFloat(ConfigParser
						.getValue("Mouse_Sensitivity")[0]);
			} catch (final Exception e) {

				J3DSettings.mouseFactor = CM2_CONSTANTS.J3D.MOUSE_SENSITIVITY;
			}

			try {
				J3DSettings.bright_appearance = Boolean
						.parseBoolean(ConfigParser
								.getValue("Appearance_Bright")[0]);
			} catch (final Exception e) {
				J3DSettings.bright_appearance = false;
			}

			try {
				J3DSettings.precision_modifier = Float.parseFloat(ConfigParser
						.getValue("Appearance_Precision_Modifier")[0]);
			} catch (final Exception e) {

				J3DSettings.precision_modifier = CM2_CONSTANTS.J3D.DETAIL_PRECISION_MODIFIER;
			}

			try {
				J3DSettings.minprecision = Integer.parseInt(ConfigParser
						.getValue("Appearance_Min_Precision")[0]);
			} catch (final Exception e) {
				J3DSettings.minprecision = CM2_CONSTANTS.J3D.DETAIL_MIN_PRECISION;
			}
			try {
				J3DSettings.numberofvertices = Integer.parseInt(ConfigParser
						.getValue("Appearance_Vertices")[0]);

				if (J3DSettings.numberofvertices < 3)
					throw new Exception();

			} catch (final Exception e) {
				J3DSettings.numberofvertices = CM2_CONSTANTS.J3D.DETAIL_DEFAULTVERTICES;
			}

			try {
				J3DSettings.shapeCollisionCheck = Boolean
						.parseBoolean(ConfigParser
								.getValue("Check_Shape_Collisions")[0]);
			} catch (final Exception e) {

			}
			try {
				J3DSettings.atomCollisionCheck = Boolean
						.parseBoolean(ConfigParser
								.getValue("Check_Atom_Collisions")[0]);
			} catch (final Exception e) {

			}
			try {
				J3DSettings.atomDistanceAbsolute = Boolean
						.parseBoolean(ConfigParser
								.getValue("Absolute_Atom_Distance")[0]);
			} catch (final Exception e) {

			}
			try {
				J3DSettings.minatomdistance = Float.parseFloat(ConfigParser
						.getValue("Min_Atom_Distance")[0]);
			} catch (final Exception e) {

			}

			try {
				J3DSettings.light1 = Boolean.parseBoolean(ConfigParser
						.getValue("Appearance_Light1")[0]);
			} catch (final Exception e) {
				J3DSettings.light1 = true;
			}
			try {
				J3DSettings.light2 = Boolean.parseBoolean(ConfigParser
						.getValue("Appearance_Light2")[0]);
			} catch (final Exception e) {
				J3DSettings.light2 = false;
			}
			try {
				J3DSettings.ambientlight = Float.parseFloat(ConfigParser
						.getValue("Appearance_Ambient_Light")[0]);

			} catch (final Exception e) {
				J3DSettings.ambientlight = CM2_CONSTANTS.J3D.DETAIL_AMBIENT_LIGHT;
			}
			try {
				J3DSettings.shininess_proteins = Integer.parseInt(ConfigParser
						.getValue("Appearance_Shininess_Proteins")[0]);
			} catch (final Exception e) {
				J3DSettings.shininess_proteins = CM2_CONSTANTS.J3D.DETAIL_SHININESS_PROTEINS;
			}
			try {
				J3DSettings.shininess_lipids = Integer.parseInt(ConfigParser
						.getValue("Appearance_Shininess_Lipids")[0]);
			} catch (final Exception e) {
				J3DSettings.shininess_lipids = CM2_CONSTANTS.J3D.DETAIL_SHININESS_LIPIDS;
			}
			try {
				J3DSettings.polygon_mode = Integer.parseInt(ConfigParser
						.getValue("Appearance_Polygon_Mode")[0]);
			} catch (final Exception e) {
				J3DSettings.polygon_mode = CM2_CONSTANTS.J3D.DETAIL_POLYGON_MODE;
			}
			try {
				J3DSettings.antialiasing = Boolean.parseBoolean(ConfigParser
						.getValue("Appearance_Antialiasing")[0]);
			} catch (final Exception e) {
				J3DSettings.antialiasing = CM2_CONSTANTS.J3D.VIEW_ANTIALIASING;
			}
			try {
				J3DSettings.parallelProjection = Boolean
				.parseBoolean(ConfigParser
						.getValue("Parallel_Projection")[0]);
			} catch (final Exception e) {
				J3DSettings.parallelProjection =  CM2_CONSTANTS.J3D.VIEW_PARALLEL_PROJECTION;
			}
			try {
				J3DSettings.showAllMultiLayers = Boolean
				.parseBoolean(ConfigParser
						.getValue("Show_All_Multilayers")[0]);
			} catch (final Exception e) {
				J3DSettings.showAllMultiLayers =  CM2_CONSTANTS.J3D.VIEW_ALL_MULTILAYERS;
			}
			try {
				J3DSettings.uniformBackground = Boolean
				.parseBoolean(ConfigParser
						.getValue("Uniform_Background")[0]);
			} catch (final Exception e) {
				J3DSettings.uniformBackground = CM2_CONSTANTS.J3D.VIEW_UNIFORM_BACKGROUND;
			}
			try {
				J3DSettings.removeInnerAtomsFactor = Integer
						.parseInt(ConfigParser.getValue("Remove_Inner_Atoms")[0]);
			} catch (final Exception e) {

			}

		}

		/**
		 * Stores the settings into configParser
		 * 
		 */
		public static void storeSettings() {

			ConfigParser.deleteAllAt("Automatic_Detail_Settings");
			ConfigParser.addValue("Automatic_Detail_Settings", ""
					+ J3DSettings.automaticDetailSettings);

			ConfigParser.deleteAllAt("Appearance_Bright");
			ConfigParser.addValue("Appearance_Bright", ""
					+ J3DSettings.bright_appearance);

			ConfigParser.deleteAllAt("Appearance_Precision_Modifier");
			ConfigParser.addValue("Appearance_Precision_Modifier", ""
					+ J3DSettings.precision_modifier);
			ConfigParser.deleteAllAt("Appearance_Min_Precision");
			ConfigParser.addValue("Appearance_Min_Precision", ""
					+ J3DSettings.minprecision);

			ConfigParser.deleteAllAt("Appearance_Vertices");
			ConfigParser.addValue("Appearance_Vertices", ""
					+ J3DSettings.numberofvertices);

			ConfigParser.deleteAllAt("Check_Shape_Collisions");
			ConfigParser.addValue("Check_Shape_Collisions", ""
					+ J3DSettings.shapeCollisionCheck);

			ConfigParser.deleteAllAt("Check_Atom_Collisions");
			ConfigParser.addValue("Check_Atom_Collisions", ""
					+ J3DSettings.atomCollisionCheck);
			
			ConfigParser.deleteAllAt("Absolute_Atom_Distance");
			ConfigParser.addValue("Absolute_Atom_Distance", ""
					+ J3DSettings.atomDistanceAbsolute);
			
			ConfigParser.deleteAllAt("Min_Atom_Distance");
			ConfigParser.addValue("Min_Atom_Distance", ""
					+ J3DSettings.minatomdistance);

			ConfigParser.deleteAllAt("Appearance_Light1");
			ConfigParser.addValue("Appearance_Light1", "" + J3DSettings.light1);
			ConfigParser.deleteAllAt("Appearance_Light2");
			ConfigParser.addValue("Appearance_Light2", "" + J3DSettings.light2);
			ConfigParser.deleteAllAt("Appearance_Ambient_Light");
			ConfigParser.addValue("Appearance_Ambient_Light", ""
					+ J3DSettings.ambientlight);
			ConfigParser.deleteAllAt("Appearance_Shininess_Proteins");
			ConfigParser.addValue("Appearance_Shininess_Proteins", ""
					+ J3DSettings.shininess_proteins);
			ConfigParser.deleteAllAt("Appearance_Shininess_Lipids");
			ConfigParser.addValue("Appearance_Shininess_Lipids", ""
					+ J3DSettings.shininess_lipids);
			ConfigParser.deleteAllAt("Appearance_Polygon_Mode");
			ConfigParser.addValue("Appearance_Polygon_Mode", ""
					+ J3DSettings.polygon_mode);

			ConfigParser.deleteAllAt("Appearance_Antialiasing");
			ConfigParser.addValue("Appearance_Antialiasing", ""
					+ J3DSettings.antialiasing);
			
			ConfigParser.deleteAllAt("Parallel_Projection");
			ConfigParser.addValue("Parallel_Projection", ""
					+ J3DSettings.parallelProjection);
			
			ConfigParser.deleteAllAt("Uniform_Background");
			ConfigParser.addValue("Uniform_Background", ""
					+ J3DSettings.uniformBackground);

			ConfigParser.deleteAllAt("Show_All_Multilayers");
			ConfigParser.addValue("Show_All_Multilayers", ""
					+ J3DSettings.showAllMultiLayers);
			
			ConfigParser.deleteAllAt("Mouse_Sensitivity");
			ConfigParser.addValue("Mouse_Sensitivity", ""
					+ J3DSettings.mouseFactor);

			ConfigParser.deleteAllAt("Remove_Inner_Atoms");
			ConfigParser.addValue("Remove_Inner_Atoms", ""
					+ J3DSettings.removeInnerAtomsFactor);

			ConfigParser.deleteAllAt("forbid_Dragging_In_FreeLook_Mode");
			ConfigParser.addValue("forbid_Dragging_In_FreeLook_Mode", ""
					+ J3DSettings.forbidDraggingInFreeLookMode);

			ConfigParser.deleteAllAt("enable_fast_transitions");
			ConfigParser.addValue("enable_fast_transitions", ""
					+ enableFastTransitions);

			ConfigParser.deleteAllAt("enable_ppe_automatic");
			ConfigParser.addValue("enable_ppe_automatic", ""
					+ enablePPEAutomatic);

		}

	}

	public static class InternetSettings {

		public static String proxyLogin = "";

		public static String proxyPassword = "";

		public static String proxyHTTPport = "";

		public static String proxyFTPport = "";

		public static String proxyHTTP = "";

		public static String proxyFTP = "";

		public static boolean proxyuse = false;

		public static String rcsbimageLink = CM2_CONSTANTS.IO.DEFAULT_PREVIEW_IMAGE_LINK;

		public static String rcsbLink = CM2_CONSTANTS.IO.DEFAULT_RCSB_SITE_LINK;

		public static String rcsbentriesLink = CM2_CONSTANTS.IO.DEFAULT_SEARCH_ENTRIES_LINK;

		public static String rcsbdownloadLink = CM2_CONSTANTS.IO.DEFAULT_DOWNLOAD_LINK;

		public static String opmdownloadLink = CM2_CONSTANTS.IO.DEFAULT_OPM_DOWNLOAD_LINK;

		public static void restoreSettings() {

			try {
				InternetSettings.proxyLogin = ConfigParser
						.getValue("proxy_Login")[0];
			} catch (final Exception e1) {

			}
			try {
				InternetSettings.proxyPassword = ConfigParser
						.getValue("proxy_Password")[0];
			} catch (final Exception e1) {

			}
			try {
				InternetSettings.proxyHTTPport = ConfigParser
						.getValue("proxy_HTTP_Port")[0];
			} catch (final Exception e1) {

			}
			try {
				InternetSettings.proxyFTPport = ConfigParser
						.getValue("proxy_FTP_Port")[0];
			} catch (final Exception e1) {

			}
			try {
				InternetSettings.proxyHTTP = ConfigParser
						.getValue("proxy_HTTP")[0];
			} catch (final Exception e1) {

			}
			try {
				InternetSettings.proxyFTP = ConfigParser.getValue("proxy_FTP")[0];
			} catch (final Exception e1) {

			}

			try {
				InternetSettings.proxyuse = Boolean.parseBoolean(ConfigParser
						.getValue("proxy_Use")[0]);

			} catch (final Exception e) {

			}

			try {
				InternetSettings.rcsbdownloadLink = ConfigParser
						.getValue("RCSB_download")[0];
			} catch (final Exception e1) {

			}
			try {
				InternetSettings.rcsbLink = ConfigParser.getValue("RCSB_site")[0];
			} catch (final Exception e1) {

			}
			try {
				InternetSettings.rcsbentriesLink = ConfigParser
						.getValue("RCSB_entries")[0];
			} catch (final Exception e1) {

			}
			try {
				InternetSettings.rcsbimageLink = ConfigParser
						.getValue("RCSB_images")[0];
			} catch (final Exception e1) {

			}

		}

		public static void storeSettings() {

			ConfigParser.deleteAllAt("proxy_Use");
			ConfigParser.addValue("proxy_Use", "" + InternetSettings.proxyuse);

			ConfigParser.deleteAllAt("proxy_Login");
			ConfigParser.addValue("proxy_Login", InternetSettings.proxyLogin);

			ConfigParser.deleteAllAt("proxy_Password");
			ConfigParser.addValue("proxy_Password",
					InternetSettings.proxyPassword);

			ConfigParser.deleteAllAt("proxy_HTTP_Port");
			ConfigParser.addValue("proxy_HTTP_Port",
					InternetSettings.proxyHTTPport);

			ConfigParser.deleteAllAt("proxy_FTP_Port");
			ConfigParser.addValue("proxy_FTP_Port",
					InternetSettings.proxyFTPport);

			ConfigParser.deleteAllAt("proxy_HTTP");
			ConfigParser.addValue("proxy_HTTP", InternetSettings.proxyHTTP);

			ConfigParser.deleteAllAt("proxy_FTP");
			ConfigParser.addValue("proxy_FTP", InternetSettings.proxyFTP);

			ConfigParser.deleteAllAt("RCSB_download");
			ConfigParser.addValue("RCSB_download",
					InternetSettings.rcsbdownloadLink);

			ConfigParser.deleteAllAt("RCSB_site");
			ConfigParser.addValue("RCSB_site", InternetSettings.rcsbLink);

			ConfigParser.deleteAllAt("RCSB_entries");
			ConfigParser.addValue("RCSB_entries",
					InternetSettings.rcsbentriesLink);

			ConfigParser.deleteAllAt("RCSB_images");
			ConfigParser
					.addValue("RCSB_images", InternetSettings.rcsbimageLink);
			
		}

		public static void doProxySettings() {

			if (InternetSettings.proxyuse) {

				System.getProperties().put("proxySet", "true");
				System.getProperties().put("ftpProxySet", "true");

				Authenticator.setDefault(new Authenticator() {
					protected PasswordAuthentication getPasswordAuthentication() {
						return new PasswordAuthentication(
								InternetSettings.proxyLogin,
								InternetSettings.proxyPassword.toCharArray());
					}
				});

				System.getProperties().put("proxyPort",
						InternetSettings.proxyHTTPport);

				System.getProperties().put("ftpProxyPort",
						InternetSettings.proxyFTPport);

				System.getProperties().put("proxyHost",
						InternetSettings.proxyHTTP);

				System.getProperties().put("ftpProxyHost",
						InternetSettings.proxyFTP);

			} else {

				System.getProperties().put("proxySet", "false");
				System.getProperties().put("ftpProxySet", "false");

				Authenticator.setDefault(new Authenticator() {
					protected PasswordAuthentication getPasswordAuthentication() {
						return new PasswordAuthentication(null, null);
					}
				});

				System.getProperties().put("proxyPort", "");

				System.getProperties().put("ftpProxyPort", "");

				System.getProperties().put("proxyHost", "");

				System.getProperties().put("ftpProxyHost", "");

			}

		}
	}

	/**
	 * This class is needed to setup the direct path to Jmol, including the
	 * java -jar execution string.
	 * 
	 * @author Bjoern Sommer bjoern@CELLmicrocosmos.org
	 * 
	 */
	public static class JmolSettings {

		public static String jmolExecutionLink = CM2_CONSTANTS.IO.DEFAULT_JMOL_EXECUTION_LINK;

		public static void restoreSettings() {

			try {
				JmolSettings.jmolExecutionLink = ConfigParser
						.getValue("jmol_Execution_Link")[0];
			} catch (final Exception e1) {

			}

		}

		public static void storeSettings() {

			ConfigParser.deleteAllAt("jmol_Execution_Link");

			ConfigParser.addValue("jmol_Execution_Link", ""
					+ JmolSettings.jmolExecutionLink);

		}

	}
	
	/**
	 * This class is needed to setup the direct path to APL@Voro
	 * 
	 * @author Bjoern Sommer bjoern@CELLmicrocosmos.org
	 * 
	 */
	public static class APLVoroSettings {

		public static String aplVoroExecutionLink = CM2_CONSTANTS.IO.DEFAULT_APLVORO_EXECUTION_LINK;

		public static void restoreSettings() {

			try {
				APLVoroSettings.aplVoroExecutionLink = ConfigParser
						.getValue("aplVoro_Execution_Link")[0];
			} catch (final Exception e1) {

			}

		}

		public static void storeSettings() {

			ConfigParser.deleteAllAt("aplVoro_Execution_Link");

			ConfigParser.addValue("aplVoro_Execution_Link", ""
					+ APLVoroSettings.aplVoroExecutionLink);

		}

	}

	/**
	 * A subclass that stores the current limitation settings. Also has
	 * functions to store and restore these settings from config
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public static class LimitationSettings {

		public static boolean caseSensitiveChainIds = true;

		public static int atomserials = 1;

		public static int chains = 1;

		public static boolean assignResSeqNumbers = true;

		public static boolean all99999 = true;
		
		public static boolean restartNumberingWith0 = true;

		public static boolean adoptTitleSections = true;

		public static boolean reverseParsing = true;
		
		public static boolean exportTopology = true;
		
		public static boolean exportAPLVoro = true;
		
		public static int domainID2ResNameAtPosition = -1;
		
		public static int layerID2ResNameAtPosition = -1;
		

		/**
		 * Restores the settings from the config file
		 * 
		 */
		public static void restoreSettings() {

			try {
				final String string = ConfigParser
						.getValue("Limitation_Atom_Serials")[0];
				if (string.equals("Proteins_and_Lipids"))
					LimitationSettings.atomserials = 0;
				else if (string.equals("Proteins_only"))
					LimitationSettings.atomserials = 1;
				else if (string.equals("Unlimited"))
					LimitationSettings.atomserials = 2;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("Limitation_Chains")[0];
				if (string.equals("Proteins_and_Lipids"))
					LimitationSettings.chains = 0;
				else if (string.equals("Proteins_only"))
					LimitationSettings.chains = 1;
				else if (string.equals("Unlimited"))
					LimitationSettings.chains = 2;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("Limitation_Case_Sensitive_Chain_IDs")[0];
				if (string.equals("false"))
					LimitationSettings.caseSensitiveChainIds = false;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("Limitation_All99999")[0];
				if (string.equals("false"))
					LimitationSettings.all99999 = false;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("Limitation_RestartNumberingWith0")[0];
				if (string.equals("false"))
					LimitationSettings.restartNumberingWith0 = false;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("Limitation_AdoptTitleSections")[0];
				if (string.equals("false"))
					LimitationSettings.adoptTitleSections = false;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("Limitation_ReverseParsing")[0];
				if (string.equals("false"))
					LimitationSettings.reverseParsing = false;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("Limitation_ExportTopology")[0];
				if (string.equals("false"))
					LimitationSettings.exportTopology = false;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("Limitation_ExportAPLVoro")[0];
				if (string.equals("false"))
					LimitationSettings.exportAPLVoro = false;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("Limitation_AssingResseqNumbers")[0];
				if (string.equals("false"))
					LimitationSettings.assignResSeqNumbers = false;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("Limitation_DomainID2ResName")[0];
				if (string.equals("1"))
					LimitationSettings.domainID2ResNameAtPosition = 1;
				else if (string.equals("2"))
					LimitationSettings.domainID2ResNameAtPosition = 2;
				else if (string.equals("3"))
					LimitationSettings.domainID2ResNameAtPosition = 3;
				else
					LimitationSettings.domainID2ResNameAtPosition = -1;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("Limitation_LayerID2ResName")[0];
				if (string.equals("1"))
					LimitationSettings.layerID2ResNameAtPosition = 1;
				else if (string.equals("2"))
					LimitationSettings.layerID2ResNameAtPosition = 2;
				else if (string.equals("3"))
					LimitationSettings.layerID2ResNameAtPosition = 3;
				else
					LimitationSettings.layerID2ResNameAtPosition = -1;

			} catch (final Exception e) {

			}
		}

		/**
		 * Stores the settings into the config
		 */
		public static void storeSettings() {
			final String[] strings = { "Proteins_and_Lipids", "Proteins_only",
					"Unlimited" };

			ConfigParser.deleteAllAt("Limitation_Atom_Serials");
			ConfigParser.addValue("Limitation_Atom_Serials",
					strings[LimitationSettings.atomserials]);

			ConfigParser.deleteAllAt("Limitation_Chains");
			ConfigParser.addValue("Limitation_Chains",
					strings[LimitationSettings.chains]);

			ConfigParser.deleteAllAt("Limitation_Case_Sensitive_Chain_IDs");
			ConfigParser.addValue("Limitation_Case_Sensitive_Chain_IDs", ""
					+ LimitationSettings.caseSensitiveChainIds);

			ConfigParser.deleteAllAt("Limitation_All99999");
			ConfigParser.addValue("Limitation_All99999", ""
					+ LimitationSettings.all99999);

			ConfigParser.deleteAllAt("Limitation_RestartNumberingWith0");
			ConfigParser.addValue("Limitation_RestartNumberingWith0", ""
					+ LimitationSettings.restartNumberingWith0);
			
			ConfigParser.deleteAllAt("Limitation_AdoptTitleSections");
			ConfigParser.addValue("Limitation_AdoptTitleSections", ""
					+ LimitationSettings.adoptTitleSections);

			ConfigParser.deleteAllAt("Limitation_ReverseParsing");
			ConfigParser.addValue("Limitation_ReverseParsing", ""
					+ LimitationSettings.reverseParsing);
			
			ConfigParser.deleteAllAt("Limitation_ExportTopology");
			ConfigParser.addValue("Limitation_ExportTopology", ""
					+ LimitationSettings.exportTopology);
			
			ConfigParser.deleteAllAt("Limitation_ExportAPLVoro");
			ConfigParser.addValue("Limitation_ExportAPLVoro", ""
					+ LimitationSettings.exportAPLVoro);

			ConfigParser.deleteAllAt("Limitation_AssingResseqNumbers");
			ConfigParser.addValue("Limitation_AssingResseqNumbers", ""
					+ LimitationSettings.assignResSeqNumbers);
			
			ConfigParser.deleteAllAt("Limitation_DomainID2ResName");
			ConfigParser.addValue("Limitation_DomainID2ResName",
					Integer.toString(LimitationSettings.domainID2ResNameAtPosition));
			
			ConfigParser.deleteAllAt("Limitation_LayerID2ResName");
			ConfigParser.addValue("Limitation_LayerID2ResName",
					Integer.toString(LimitationSettings.layerID2ResNameAtPosition));

		}
	}

	/**
	 * A subclass that stores the current Coordinate Transformation settings.
	 * Also has functions to store and restore these settings from config
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public static class CoordinateTransformationSettings {

		public static boolean swapYZ = false;

		public static int shiftXZ = 0;

		public static int shiftY = 0;
		
		public static boolean sortByType = true;
		
		/**
		 * Restores the settings from the config file
		 * 
		 */
		public static void restoreSettings() {

			try {
				final String string = ConfigParser
						.getValue("CoordinateTransformation_ShiftXZ")[0];
				if (string.equals("None"))
					CoordinateTransformationSettings.shiftXZ = 0;
				else if (string.equals("Physical"))
					CoordinateTransformationSettings.shiftXZ = 1;
				else if (string.equals("Bounds"))
					CoordinateTransformationSettings.shiftXZ = 2;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("CoordinateTransformation_ShiftY")[0];
				if (string.equals("None"))
					CoordinateTransformationSettings.shiftY = 0;
				else if (string.equals("Physical"))
					CoordinateTransformationSettings.shiftY = 1;
				else if (string.equals("Bottom"))
					CoordinateTransformationSettings.shiftY = 2;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("CoordinateTransformation_SwapYZ")[0];
				if (string.equals("true"))
					CoordinateTransformationSettings.swapYZ = true;

			} catch (final Exception e) {

			}
			try {
				final String string = ConfigParser
						.getValue("CoordinateTransformation_SortByType")[0];
				if (string.equals("false"))
					CoordinateTransformationSettings.sortByType = false;

			} catch (final Exception e) {

			}
		}

		/**
		 * Stores the settings into the config
		 */
		public static void storeSettings() {

			ConfigParser.deleteAllAt("CoordinateTransformation_ShiftXZ");
			if (CoordinateTransformationSettings.shiftXZ == 0)
				ConfigParser.addValue("CoordinateTransformation_ShiftXZ",
						"None");
			else if (CoordinateTransformationSettings.shiftXZ == 1)
				ConfigParser.addValue("CoordinateTransformation_ShiftXZ",
						"Physical");
			else
				ConfigParser.addValue("CoordinateTransformation_ShiftXZ",
						"Bounds");

			ConfigParser.deleteAllAt("CoordinateTransformation_ShiftY");
			if (CoordinateTransformationSettings.shiftY == 0)
				ConfigParser
						.addValue("CoordinateTransformation_ShiftY", "None");
			else if (CoordinateTransformationSettings.shiftY == 1)
				ConfigParser.addValue("CoordinateTransformation_ShiftY",
						"Physical");
			else
				ConfigParser.addValue("CoordinateTransformation_ShiftY",
						"Bottom");

			ConfigParser.deleteAllAt("CoordinateTransformation_SwapYZ");
			ConfigParser.addValue("CoordinateTransformation_SwapYZ", ""
					+ CoordinateTransformationSettings.swapYZ);
			
			ConfigParser.deleteAllAt("CoordinateTransformation_SortByType");
			ConfigParser.addValue("CoordinateTransformation_SortByType", ""
					+ CoordinateTransformationSettings.sortByType);
			
		}
	}

	/**
	 * A subclass that stores the current Protein Placement Expansion settings.
	 * Also has functions to store and restore these settings from config
	 * 
	 * @author Bjoern Sommer bjoern@CELLmicrocosmos.org
	 * 
	 */
	public static class PPESettings {

		public static boolean saveOPMDummyLayersInPDB = CM2_CONSTANTS.IO.OPM_DUMMY_LAYERS_IN_PDBEXPORT;

		/**
		 * Restores the settings from the config file
		 * 
		 */
		public static void restoreSettings() {

			try {
				final String string = ConfigParser
						.getValue("PPESettings_SaveOPMDummyLayersInPDB")[0];
				if (string.equals("true"))
					PPESettings.saveOPMDummyLayersInPDB = true;

			} catch (final Exception e) {

			}

		}

		/**
		 * Stores the settings into the config
		 */
		public static void storeSettings() {

			ConfigParser.deleteAllAt("PPESettings_SaveOPMDummyLayersInPDB");
			ConfigParser.addValue("PPESettings_SaveOPMDummyLayersInPDB", ""
					+ PPESettings.saveOPMDummyLayersInPDB);

		}
	}
	
	/**
	 * A subclass that stores the current Algorithm settings.
	 * Also has functions to store and restore these settings from config
	 * 
	 * @author Bjoern Sommer bjoern@CELLmicrocosmos.org
	 * 
	 */
	public static class AlgorithmSettings {

		public static boolean startInBackground = CM2_CONSTANTS.ALGORITHMS.DEFAULT_START_IN_BACKGROUND;
		public static boolean showProtocol = CM2_CONSTANTS.ALGORITHMS.DEFAULT_SHOW_PROTOCOL;
		public static boolean uniqueRandom = CM2_CONSTANTS.ALGORITHMS.DEFAULT_UNIQUE_RANDOM;
		public static long uniqueRandomSeed = CM2_CONSTANTS.ALGORITHMS.DEFAULT_UNIQUE_RANDOM_SEED;

		/**
		 * Restores the settings from the config file
		 * 
		 */
		public static void restoreSettings() {

			try {
				final String string = ConfigParser
						.getValue("Algorithm_StartInBackground")[0];
				if (string.equals("true"))
					AlgorithmSettings.startInBackground = true;
			} catch (final Exception e) {
				AlgorithmSettings.startInBackground = CM2_CONSTANTS.ALGORITHMS.DEFAULT_START_IN_BACKGROUND;
			}
			
			try {
				final String string = ConfigParser
						.getValue("Algorithm_ShowProtocol")[0];
				if (string.equals("true"))
					AlgorithmSettings.showProtocol = true;
			} catch (final Exception e) {
				AlgorithmSettings.showProtocol = CM2_CONSTANTS.ALGORITHMS.DEFAULT_SHOW_PROTOCOL;
			}
			
			try {
				final String string = ConfigParser
						.getValue("Algorithm_UniqueRandom")[0];
				if (string.equals("true"))
					AlgorithmSettings.uniqueRandom = true;
			} catch (final Exception e) {
				AlgorithmSettings.uniqueRandom = CM2_CONSTANTS.ALGORITHMS.DEFAULT_UNIQUE_RANDOM;
			}
			
			try {
				AlgorithmSettings.uniqueRandomSeed = Long.parseLong(ConfigParser
						.getValue("Algorithm_UniqueRandomSeed")[0]);
			} catch (final Exception e) {
				AlgorithmSettings.uniqueRandomSeed = CM2_CONSTANTS.ALGORITHMS.DEFAULT_UNIQUE_RANDOM_SEED;
			}

		}

		/**
		 * Stores the settings into the config
		 */
		public static void storeSettings() {

			ConfigParser.deleteAllAt("Algorithm_StartInBackground");
			ConfigParser.addValue("Algorithm_StartInBackground", ""
					+ AlgorithmSettings.startInBackground);			

			ConfigParser.deleteAllAt("Algorithm_ShowProtocol");
			ConfigParser.addValue("Algorithm_ShowProtocol", ""
					+ AlgorithmSettings.showProtocol);
			
			ConfigParser.deleteAllAt("Algorithm_UniqueRandom");
			ConfigParser.addValue("Algorithm_UniqueRandom", ""
					+ AlgorithmSettings.uniqueRandom);
			
			ConfigParser.deleteAllAt("Algorithm_UniqueRandomSeed");
			ConfigParser.addValue("Algorithm_UniqueRandomSeed", ""
					+ AlgorithmSettings.uniqueRandomSeed);

		}
	}

	/**
	 * Used for Drag and Drop. Can create a Protein that will be used for
	 * dragging around in the 3d view.
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public static class DNDBridge {

		private static Molecule_3D currentMolecule;

		/**
		 * Creates or copies a new Protein Instance to use
		 * 
		 * @param code
		 * @param file
		 * @throws Exception
		 */
		public static void activate(final String code, final String file,
				final int modelnumber) throws Exception {

			final MembraneModel model = DataPool.getMembraneModel();

			Protein_3D newProt;

			Protein_3D other = (Protein_3D) model.getFirstMoleculeThatHasFile(
					new File(file), 0, modelnumber);

			// if (other != null) {
			// final Atom[] altered = DataPool
			// .getAtomsWithAlteredPositions(other);
			// if (altered != null && altered.length > 0)
			// other = null;
			// }
			if (other != null) {
				newProt = (Protein_3D) other.copy(false, true);
			} else {

				final PdbReader_3D reader = new PdbReader_3D();
				newProt = reader.readProtein(new File(file), modelnumber, null);
				newProt.setPDBcode(code);
				newProt.setColor(Cm2_Toolkit.getRandomProteinColor());

			}
			newProt.setID(DataPool.getNextFreeProteinId());

			// ===============================================================
			// TODO: PPE Update 03_2012									 	  |
			// ===============================================================
			// when PPE is enabled the stored molecule transformations will
			// be applied
			if(DataPool.J3DSettings.enablePPEAutomatic) {
				// PPE automatic is enabled
				// -> automatic OPM adjustment
				// -> automatic PDBTM adjustment
				if(newProt.opmAtoms.length > 0) {
					// molecule is a OPM File
					DataPool.getMifHandler().applyOPMAlignmentTo(newProt);
				} else {
					// molecule isn't a OPM file.
					// this also works when molecule isn't a PDBTM file
					// because of its default values stored in
					// Y-translation and rotation matrix.
					newProt.setRotationMatrix(newProt.getPDBTMRotation());
					newProt.setYTranslation(newProt.getPDBTMTranslation());				}
			} else {
				// PPE automatic is disabled
				newProt.setRotationMatrix(Cm2_Toolkit.createAUnitMatrix());
			}
			
			

			DNDBridge.currentMolecule = newProt;
			DNDBridge.currentMolecule.setPositionIsInAValidSpace(false);

		}

		public static void activate(final Lipid_3D mol) {

			if (mol != null) {

				DNDBridge.currentMolecule = mol.copy(true, false);

				if (DNDBridge.currentMolecule instanceof Lipid_3D) {
					final float h = MembranePackingAlgorithm
							.getTotalHeight(DNDBridge.currentMolecule)
							* (1 / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
					if (((Lipid_3D) DNDBridge.currentMolecule)
							.getMembraneSide() == 0)
						DNDBridge.currentMolecule.setYTranslation(h / 2f);
					else
						DNDBridge.currentMolecule.setYTranslation(-(h / 2f));
				}
				DNDBridge.currentMolecule.setPositionIsInAValidSpace(false);

			}
		}

		/**
		 * Discards the current Protein
		 * 
		 */
		public static void deactivate() {

			DNDBridge.currentMolecule = null;
		}

		/**
		 * Returns the current Protein
		 * 
		 * @return
		 */
		public static Molecule_3D getCurrentMolecule() {
			return DNDBridge.currentMolecule;
		}

		/**
		 * Overwrites the current Protein
		 * 
		 * @param currentProtein2
		 */
		public static void overwriteCurrentMolecule(final Molecule_3D current) {
			DNDBridge.currentMolecule = current;

		}

	}

	/**
	 * Return, wether or not a molecule with the passed filepath+model is still
	 * located in the membrane model
	 * 
	 * @param filewithModel
	 * @return
	 */
	@SuppressWarnings("unused")
	private static boolean isStillIn(final String filewithModel) {
		int length = DataPool.currentMembrane.getProteinCount();

		for (int i = 0; i < length; i++) {

			if ((DataPool.currentMembrane.getProteinAt(i).getPDBFile()
					.getAbsolutePath() + DataPool.currentMembrane.getProteinAt(
					i).getModelSet()).equalsIgnoreCase(filewithModel)) {

				return true;
			}

		}
		length = DataPool.currentMembrane.getExampleLipidsCountExtra();
		for (int i = 0; i < length; i++) {

			if ((DataPool.currentMembrane.getExampleLipidAtExtra(i)
					.getPDBFile().getAbsolutePath() + DataPool.currentMembrane
					.getExampleLipidAtExtra(i).getModelSet())
					.equalsIgnoreCase(filewithModel)) {

				return true;
			}

		}
		length = DataPool.currentMembrane.getExampleLipidsCountIntra();
		for (int i = 0; i < length; i++) {

			if ((DataPool.currentMembrane.getExampleLipidAtIntra(i)
					.getPDBFile().getAbsolutePath() + DataPool.currentMembrane
					.getExampleLipidAtIntra(i).getModelSet())
					.equalsIgnoreCase(filewithModel)) {

				return true;
			}

		}
		return false;
	}

	/**
	 * Opens the pdbfile text browser for the specified file. If a previous
	 * instance is still open, this one gets replaced
	 * 
	 * @param file
	 */
	public static void openPdbFileBrowser(final File file) {
		if (DataPool.pdbbrowser != null)
			DataPool.pdbbrowser.dispose();

		DataPool.pdbbrowser = new PDBFileBrowser(file);

	}

	/**
	 * Opens the pdb settings window. If a previous instance is still open, this
	 * one gets replaced
	 * 
	 * @param file
	 */
	public static void openPdbSettingsWindow(
			final MembraneOperations membraneOperations) {
		if (DataPool.pdbsettings != null)
			DataPool.pdbsettings.dispose();

		DataPool.pdbsettings = new PdbSettingsWindow(membraneOperations);

	}

	/**
	 * Opens the j3d settings window. If a previous instance is still open, this
	 * one gets replaced
	 * 
	 * @param file
	 */
	public static void openJ3DSettingsWindow(final J3DOperations j3doperations,
			final Point location) {
		if (DataPool.j3dsettings != null)
			DataPool.j3dsettings.dispose();

		DataPool.j3dsettings = new J3DSettingsDialog(j3doperations, location);
	}

	public static class IO {

		public static File DEFAULTINCOMINGDIRECTORY;
		public static File DEFAULTPDBDIRECTORY;
		public static File DEFAULTMEMBRANEDIRECTORY;
		public static File DEFAULTDBDIRECTORY;
		public static File Default_DB_File;

		public static void initFilePaths() {

			IO.DEFAULTINCOMINGDIRECTORY = new File(DataPool.workspace
					.getAbsolutePath()
					+ File.separator
					+ CM2_CONSTANTS.IO.DEFAULT_INCOMING_DIRECTORY);
			IO.DEFAULTPDBDIRECTORY = new File(DataPool.workspace
					.getAbsolutePath()
					+ File.separator + CM2_CONSTANTS.IO.DEFAULT_PDB_DIRECTORY);
			IO.DEFAULTMEMBRANEDIRECTORY = new File(DataPool.workspace
					.getAbsolutePath()
					+ File.separator
					+ CM2_CONSTANTS.IO.DEFAULT_MEMBRANE_DIRECTORY);
			IO.DEFAULTDBDIRECTORY = new File(DataPool.workspace
					.getAbsolutePath()
					+ File.separator + CM2_CONSTANTS.IO.DEFAULT_DB_DIRECTORY);
			IO.Default_DB_File = new File(DataPool.workspace.getAbsolutePath()
					+ File.separator + CM2_CONSTANTS.IO.DEFAULT_DATABASE);

		}

	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	public static class AtomValues {

		private static Hashtable<String, Float> massValues = new Hashtable<String, Float>();

		public static Hashtable<String, Vector2f> radii = new Hashtable<String, Vector2f>();

		private static Hashtable<String, Color> colors = new Hashtable<String, Color>();
		
		
		public static float biggestCovalentRadius = 0f;

		public static float biggestVanDerWaalsRadius = 0f;

		public static void load() throws Exception {
			final BufferedReader br = new BufferedReader(new FileReader(
					CM2_CONSTANTS.IO.UAMU_FILE));
			br.readLine();
			br.readLine();
			while (true) {
				final String line = br.readLine();
				if (line == null) {
					br.close();

					return;
				}

				try {
					final String[] s = line.split(":");
					if (s.length != 4)
						continue;
					final String key = s[0].trim().toUpperCase();
					AtomValues.massValues.put(key, Float.parseFloat(s[1].trim()
							.replaceAll(",", ".")));

					try {
						final String[] rgb = s[2].trim().split(",");
						final Color color = new Color(Integer.parseInt(rgb[0]),
								Integer.parseInt(rgb[1]), Integer
										.parseInt(rgb[2]));
						AtomValues.colors.put(key, color);
					} catch (final Throwable e) {

					}

					// try {
					// AtomValues.radii.put(key, new
					// Vector2f((Float.parseFloat(s[3]
					// .trim().replaceAll(",", ".")) / 100f),0f));
					// } catch (final Throwable e) {
					//
					// }

					CM2_CONSTANTS.ATOMVALUES.feedWithRadii(AtomValues.radii);

					Enumeration<Vector2f> en = AtomValues.radii.elements();
					while (en.hasMoreElements()) {
						Vector2f info = (Vector2f) en.nextElement();
						biggestCovalentRadius = Math.max(biggestCovalentRadius,
								info.x);
						biggestVanDerWaalsRadius = Math.max(
								biggestVanDerWaalsRadius, info.y);
					}

				} catch (final Throwable e) {
					continue;
				}

			}

		}

		public static float getMassValue(String key) {

			Float f = AtomValues.massValues.get(key);
			if (f != null)
				return f.floatValue();
			
			for (int i = 0; i < 10; i++) {
				key = key.replaceAll("" + i, "");
			}
			f = AtomValues.massValues.get(key);
			if (f != null)
				return f.floatValue();
			
			String properKey = Cm2_Toolkit.convertAtomNameToElementName(key);
			if (properKey == null)
				return 0;
			f = AtomValues.massValues.get(properKey);
			if (f == null)
				return 0;
			return f.floatValue();
		}

		public static Color getColor(final String key) {

			Color c = AtomValues.colors.get(key);
			if (c != null)
				return c;
			String properKey = Cm2_Toolkit.convertAtomNameToElementName(key);

			if (properKey == null)
				return Color.black;
			c = AtomValues.colors.get(properKey);
			if (c == null)
				c = Color.black;
			return c;
		}

		public static float calculateAtomicMass(final Molecule_3D mol) {
			float mass = 0;
			final Atom_3D[] atoms = mol.getAtoms(false, false);
			for (int i = 0; i < atoms.length; i++) {
				mass += DataPool.AtomValues.getMassValue(atoms[i].name);
			}
			return mass;
		}

		private static Vector2f getRadii(final String key) {
//			String properkey = "" + key;
//			for (int i = 0; i < 10; i++) {
//				properkey = properkey.replaceAll("" + i, "");
//			}
//			Vector2f info = null;
//			while (true) {
//
//				info = AtomValues.radii.get(properkey);
//				if (info != null) {
//					break;
//				}
//				if (properkey.length() == 1)
//					break;
//				properkey = properkey.substring(0, properkey.length() - 1);
//			}
			
			
			Vector2f info= AtomValues.radii.get(key);

			return info;

		}

		public static float getCovalentRadius(final String key) {
			Vector2f info = DataPool.AtomValues.getRadii(key);
			if (info == null || info.x == 0f) {
				return CM2_CONSTANTS.ATOMVALUES.DEFAULT_COVALENT_RADIUS;
			} else
				return info.x;

		}

		public static float getVanDerWaalsRadius(final String key) {
			Vector2f info = DataPool.AtomValues.getRadii(key);
			if (info == null || info.y == 0f) {
				return CM2_CONSTANTS.ATOMVALUES.DEFAULT_VANDERWAALS_RADIUS;
			} else
				return info.y;

		}

		public static String[] getAllAtomTypesOfMembraneModelSorted() {

			Hashtable<String, String> hash = new Hashtable<String, String>();

			Enumeration<String> en = AtomValues.radii.keys();
			while (en.hasMoreElements()) {
				String s = en.nextElement();
				// if(s!=null&&s.length()>0)
				hash.put(s, "");
			}

			MembraneModel model = DataPool.getMembraneModel();
			Molecule_3D[] mols = model.getAllMoleculesInMembrane();

			Vector<Molecule_3D> search = new Vector<Molecule_3D>();

			for (int i = 0; i < mols.length; i++) {
				search.addElement(mols[i]);
			}
			for (int i = 0; i < model.getExampleLipidsCountExtra(); i++) {
				search.addElement(model.getExampleLipidAtExtra(i));
			}
			for (int i = 0; i < model.getExampleLipidsCountIntra(); i++) {
				search.addElement(model.getExampleLipidAtIntra(i));
			}

			for (int i = 0; i < search.size(); i++) {
				Atom_3D[] atoms = search.get(i).atoms;
				for (int j = 0; j < atoms.length; j++) {
					hash.put(atoms[j].name, "");
				}
			}

			Vector<String> ret = new Vector<String>();

			Enumeration<String> en2 = hash.keys();
			while (en2.hasMoreElements()) {
				String s = en2.nextElement();
				if (s != null && s.length() > 0)
					ret.addElement(s);
			}

			String[] retval = ret.toArray(new String[0]);

			Arrays.sort(retval);

			return retval;
		}

		public static String[] getAllParticleIDsOfMembraneModelSorted() {

			Hashtable<String, String> hash = new Hashtable<String, String>();

			MembraneModel model = DataPool.getMembraneModel();
			Molecule_3D[] mols = model.getAllMoleculesInMembrane();

			Vector<Molecule_3D> search = new Vector<Molecule_3D>();

			for (int i = 0; i < mols.length; i++) {
				search.addElement(mols[i]);
			}
			for (int i = 0; i < model.getExampleLipidsCountExtra(); i++) {
				search.addElement(model.getExampleLipidAtExtra(i));
			}
			for (int i = 0; i < model.getExampleLipidsCountIntra(); i++) {
				search.addElement(model.getExampleLipidAtIntra(i));
			}

			for (int i = 0; i < search.size(); i++) {
				Particle[] ps = search.get(i).currentCGParticles.toArray(new Particle[0]);
				for (int j = 0; j < ps.length; j++) {

					
						hash.put(ps[j].particleIdentifier, "");
				}
			}

			Vector<String> ret = new Vector<String>();

			Enumeration<String> en2 = hash.keys();
			while (en2.hasMoreElements()) {
				String s = en2.nextElement();
				if (s != null && s.length() > 0)
					ret.addElement(s);
			}

			String[] retval = ret.toArray(new String[0]);

			Arrays.sort(retval);

			return retval;
		}

		public static String[] getAllElementsOfMembraneModelSorted() {
			Hashtable<String, String> hash = new Hashtable<String, String>();

			MembraneModel model = DataPool.getMembraneModel();
			Molecule_3D[] mols = model.getAllMoleculesInMembrane();

			Vector<Molecule_3D> search = new Vector<Molecule_3D>();

			for (int i = 0; i < mols.length; i++) {
				search.addElement(mols[i]);
			}
			for (int i = 0; i < model.getExampleLipidsCountExtra(); i++) {
				search.addElement(model.getExampleLipidAtExtra(i));
			}
			for (int i = 0; i < model.getExampleLipidsCountIntra(); i++) {
				search.addElement(model.getExampleLipidAtIntra(i));
			}

			for (int i = 0; i < search.size(); i++) {
				Atom_3D[] ps = search.get(i).atoms;
				for (int j = 0; j < ps.length; j++) {

					
						hash.put(ps[j].element, "");
				}
			}

			Vector<String> ret = new Vector<String>();

			Enumeration<String> en2 = hash.keys();
			while (en2.hasMoreElements()) {
				String s = en2.nextElement();
				if (s != null && s.length() > 0)
					ret.addElement(s);
			}

			String[] retval = ret.toArray(new String[0]);

			Arrays.sort(retval);

			return retval;
		}

	}

	public static String[] getRawPdbData(final String key) {
		String[] rawPdbDataContent = DataPool.rawPdbData.get(key);
		if (rawPdbDataContent == null) {
			// check, if the key is not found, because the file originally has been safed on
			// another system like Unix/Linux/Mac OS X
			String linuxFormattedKey = key.replace(File.separator, "/").substring(key.indexOf(File.separator));
			rawPdbDataContent = DataPool.rawPdbData.get(linuxFormattedKey);
			// store the data under the system compatible key
			if (rawPdbDataContent != null) {
				DataPool.storeRawPdbData(key, rawPdbDataContent);
			// check if this file was originally created by windows and may look like this:
			//  /home/bsommer/workspace/Cm2_1rev/C:\Dokumente und Einstellungen\bjsommer\Cm2_Workspace\Pdb_Files\Frequently_Used_Lipids\hic-up-lipids\phosphatidylcholine\psc_exp.pdb
			} else if (key.contains("/") && key.contains("\\")){
				String windowsFormattedKey = key.substring(key.lastIndexOf("/")+1);	
				rawPdbDataContent = DataPool.rawPdbData.get(windowsFormattedKey);
				// store the data under the system compatible key
				if (rawPdbDataContent != null) 
					DataPool.storeRawPdbData(key, rawPdbDataContent);
			}
		}
		
		return rawPdbDataContent;
	}

	public static void storeRawPdbData(final String key, final String[] contents) {

		if (contents == null)
			DataPool.rawPdbData.remove(key);
		else
			DataPool.rawPdbData.put(key, contents);
	}

	public static void cleanUpRawData() {

		final MembraneModel model = DataPool.getMembraneModel();

		final Enumeration<String> en = DataPool.rawPdbData.keys();

		while (en.hasMoreElements()) {

			final String key = en.nextElement();
			if (model.containsMoleculeWithAbsoluteFilepath(key) == false)
				DataPool.rawPdbData.remove(key);

		}

	}

	// public static Atom[] getAtomsWithAlteredPositions(final Molecule_3D mol)
	// {
	//
	// final Vector<Atom> ret = new Vector<Atom>();
	//
	// final Atom[] atm = mol.allAtomsSorted;
	//
	// String[] lines;
	// try {
	// lines = new PdbReader_3D().readModel(DataPool.getRawPdbData(mol
	// .getPDBFile().getAbsolutePath()), mol.getModelSet())[0];
	//
	// final Vector<Atom> newAtoms = new Vector<Atom>();
	//
	// for (int i = 0; i < lines.length; i++) {
	//
	// try {
	//
	// final Atom atom = new Atom(lines[i]);
	// newAtoms.addElement(atom);
	//
	// } catch (final Exception e) {
	//
	// continue;
	// }
	// }
	//
	//			
	// float maxx = -1000000, maxy = -1000000, maxz = -1000000, minx = 1000000,
	// miny = 1000000, minz = 1000000;
	// for (int i = 0; i < newAtoms.size(); i++) {
	// final Point3f p = newAtoms.get(i).position;
	// minx = Math.min(minx, p.x);
	// miny = Math.min(miny, p.y);
	// minz = Math.min(minz, p.z);
	//
	// maxx = Math.max(maxx, p.x);
	// maxy = Math.max(maxy, p.y);
	// maxz = Math.max(maxz, p.z);
	// }
	//
	// final float cx = (minx + ((maxx - minx) / 2));
	// final float cy = (miny + ((maxy - miny) / 2));
	// final float cz = (minz + ((maxz - minz) / 2));
	//
	// final float transx = 0 - cx;
	// final float transy = 0 - cy;
	// final float transz = 0 - cz;
	//
	// for (int i = 0; i < newAtoms.size(); i++) {
	//
	// newAtoms.get(i).position.x += transx;
	// newAtoms.get(i).position.y += transy;
	// newAtoms.get(i).position.z += transz;
	//
	// }
	//
	// for (int i = 0; i < atm.length; i++) {
	//
	// // System.out.println(newAtoms.get(i).position+"   "+atm[i].position);
	//
	// final boolean b = Math.abs(newAtoms.get(i).position.x
	// - atm[i].position.x) > 0.001f
	// || Math.abs(newAtoms.get(i).position.y
	// - atm[i].position.y) > 0.001f
	// || Math.abs(newAtoms.get(i).position.z
	// - atm[i].position.z) > 0.001f;
	//
	// if (b)
	// ret.addElement(atm[i]);
	// }
	//
	// return ret.toArray(new Atom[0]);
	// } catch (final Exception e1) {
	// e1.printStackTrace();
	// }
	//
	// return null;
	//
	// }

	public static void setSelectedLayer(final int layerId) {
		DataPool.currentLayerId = layerId;
	}

	public static Layer getCurrentLayer() {
		final MembraneModel model = DataPool.getMembraneModel();
		for (int i = 0; i < model.getLayerCount(); i++) {
			if (model.getLayer(i).getLayerId() == DataPool.currentLayerId)
				return model.getLayer(i);
		}
		return null;
	}

	public static float getCurrentLayerHeight() {
		final Layer layer = DataPool.getCurrentLayer();
		if (layer == null)
			return 0;
		else
			return layer.getYHeight();
	}

	public static int getCurrentLayerID() {
		final Layer layer = DataPool.getCurrentLayer();
		if (layer == null)
			return 0;
		else
			return layer.getLayerId();
	}

}
