/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.functionality3d;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Transparency;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;

import javax.media.j3d.Canvas3D;
import javax.media.j3d.Shape3D;
import javax.swing.JLabel;
import javax.vecmath.Matrix3f;

import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Particle;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


import com.sun.j3d.utils.geometry.Primitive;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.picking.PickCanvas;
import com.sun.j3d.utils.picking.PickResult;

/**
 * The HUD is needed to show different visual options e.g. in the alignment mode.
 *          
 * @author Tim Dingersen dackelnose@gmx.net
 */
public class Hud {

	public static int NORMAL = 0;
	public static int ALIGN = 1;

	public static int ALIGNLIPID = 2;

	public static int MODELCHANGE = 3;
	public int mode = Hud.NORMAL;
	private final PickCanvas pickCanvas;
	private MouseEvent currentMouseEvent;
	private final Canvas3D canvas;
	private boolean cameraWhileAligning = false;
	private final Font origFont = new Font("Arial", Font.PLAIN, 10);
	private Image membraneImage = null;
	private Rectangle[] buttons;
	private final BufferedImage[] arrows = new BufferedImage[4];
	private boolean colliding = false;
	public Molecule_3D currentMoleculeAligning = null;
	
	 Atom_3D atCursor=null;
	  
	  
	  Font font1 = new Font("Arial",Font.BOLD,12);
	  Font font2 = new Font("Arial",Font.PLAIN,12);
	  Font font3 = new Font("Arial",Font.PLAIN,10);

	public Hud(final Canvas3D canvas, final PickCanvas pc) {
		this.canvas = canvas;
		this.pickCanvas = pc;
		final Image arrow = Cm2_IconDepot.getSpecifiedIcon("CV_opened")
				.getImage();

		for (int i = 0; i < 4; i++) {

			if (i > 0)
				this.arrows[i] = Cm2_Toolkit.rotateImage90(this.arrows[i - 1]);
			else {
				this.arrows[i] = new BufferedImage(arrow.getWidth(null), arrow
						.getHeight(null), Transparency.TRANSLUCENT);
				this.arrows[i].getGraphics().drawImage(arrow, 0, 0, null);
			}

		}

	}

	public void drawHud(final Graphics2D g) throws Throwable {

		final Color textBackground = DataPool.J3DSettings.bright_appearance ? Color.white
				: Color.black;
		final Color textForeground = DataPool.J3DSettings.bright_appearance ? Color.black
				: Color.white;

		final int W = this.canvas.getWidth();
		final int H = this.canvas.getHeight();

		if (this.currentMouseEvent != null)
			this.pickCanvas.setShapeLocation(this.currentMouseEvent);

		final PickResult myPickResult = this.pickCanvas.pickClosest();
		
		
		if (this.mode == Hud.NORMAL) {

			if (this.currentMouseEvent != null)
				this.pickCanvas.setShapeLocation(this.currentMouseEvent);

			

			if (myPickResult != null
					&& myPickResult.getObject() instanceof Shape3D
					&& ((Shape3D) myPickResult.getObject()).getUserData() instanceof Molecule_3D) {
				final Molecule_3D mol = (Molecule_3D) ((Shape3D) myPickResult
						.getObject()).getUserData();

				if (mol.getPDBcode() != null) {

					g.setFont(this.origFont.deriveFont(Font.BOLD, 12f));
					final JLabel l = new JLabel();
					l.setFont(g.getFont());
					l.setText(mol.getPDBcode());
					final Rectangle r = new Rectangle(this.currentMouseEvent
							.getX(), this.currentMouseEvent.getY(), l
							.getPreferredSize().width,
							l.getPreferredSize().height);

					g.setColor(textBackground);

					this.fillBackground(g, new Rectangle(r.x + 13, r.y - 5,
							r.width, r.height - 1));

					g.setColor(textForeground);
					g.drawString(mol.getPDBcode(), this.currentMouseEvent
							.getX() + 12, this.currentMouseEvent.getY() + 7);

				}
			} else {

			}

		} else if (this.mode >= Hud.ALIGN) {

			if (this.currentMoleculeAligning == null)
				return;

			final Molecule_3D currentProt = this.currentMoleculeAligning;

			final Color foreground = DataPool.J3DSettings.bright_appearance ? Color.GREEN
					.darker()
					: Color.green;

			if (this.mode != Hud.MODELCHANGE) {
				final Matrix3f matrix = currentProt.getRotationMatrix();

				final String[] strings = { "" + matrix.m00, "" + matrix.m01,
						"" + matrix.m02, "" + matrix.m10, "" + matrix.m11,
						"" + matrix.m12, "" + matrix.m20, "" + matrix.m21,
						"" + matrix.m22 };

				for (int i = 0; i < strings.length; i++) {
					if (strings[i].length() > 5)
						strings[i] = strings[i].substring(0, 5);

				}

				g.setColor(textBackground);
				g.fillRect(4, 3, 127, 55);

				g.fillRect(4, 63, 75, 30);

				g.setFont(this.origFont.deriveFont(Font.PLAIN, 11));
				g.setColor(foreground);

				String string = "" + currentProt.getYTranslation();

				try {
					final int i = string.indexOf(".");
					string = string.substring(0, i + 2);
				} catch (final Exception e) {

				}

				g.drawString(string, 10, 90);

				/*
				 * drawing the matrix values
				 */
				g.drawString(strings[0], 10, 31);
				g.drawString(strings[1], 50, 31);
				g.drawString(strings[2], 90, 31);

				g.drawString(strings[3], 10, 43);
				g.drawString(strings[4], 50, 43);
				g.drawString(strings[5], 90, 43);

				g.drawString(strings[6], 10, 55);
				g.drawString(strings[7], 50, 55);
				g.drawString(strings[8], 90, 55);

				g.setColor(textForeground);

				// g.drawRect(4, 3, 127, 55);

				g.drawString("Rotation 3x3", 10, 16);

				// g.drawLine(4, 18, 131, 18);

				// ytranslation /////////////
				// g.drawRect(4, 63, 50, 30);

				g.drawString("Y-Translation", 10, 75);

				// g.drawLine(4, 78, 54, 78);

				g.setColor(textForeground);
				int hh = H - 9;

				if (this.cameraWhileAligning) {

					g.drawString("Wheel", 6, hh);
					g.drawString(": Zoom", 53, hh);
					hh -= 12;
					if (this.mode != Hud.MODELCHANGE) {
						g.drawString("Right MB", 6, hh);
						g.drawString(": Translate", 53, hh);
						hh -= 12;

						g.drawString("Left MB", 6, hh);
						g.drawString(": Rotate", 53, hh);
					}
				} else {

					if (this.mode != Hud.MODELCHANGE) {
						g.drawString("Right MB", 6, hh);
						g.drawString(": Translate Y", 53, hh);

						hh -= 12;

						g.drawString("Left MB", 6, hh);
						g.drawString(": Rotate X/Y", 53, hh);
					}
				}
			}

			g.setFont(this.origFont.deriveFont(Font.BOLD, 13));

			final JLabel l = new JLabel();
			l.setFont(g.getFont());
			if (this.cameraWhileAligning)
				l.setText("Free-Look mode");
			else if (this.mode != Hud.MODELCHANGE)
				l.setText("Align mode");
			else
				l.setText("Choose a Model");
			g.drawString(l.getText(), W / 2 - l.getPreferredSize().width / 2,
					15);

			if (this.mode != Hud.MODELCHANGE) {

				this.buttons = new Rectangle[5];

				this.buttons[0] = new Rectangle(W - 64, H - 83, 40, 15);
				this.buttons[1] = new Rectangle(W - 21, H - 64, 15, 40);
				this.buttons[2] = new Rectangle(W - 64, H - 20, 40, 15);
				this.buttons[3] = new Rectangle(W - 82, H - 64, 15, 40);

				if (this.mode == Hud.ALIGNLIPID) {

					int w = DataPool.getMembraneModel().getSize().width;
					int h = DataPool.getMembraneModel().getSize().height;
					final float div = 40 / (float) Math.max(w, h);

					w = (int) (div * w);
					h = (int) (div * h);

					this.buttons[4] = new Rectangle(W - 64 + (20 - (w / 2)), H
							- 64 + (20 - (h / 2)), w, h);

				}
				if (this.mode == Hud.ALIGN) {
					this.buttons[4] = new Rectangle(W - 64
							+ (20 - (this.membraneImage.getWidth(null) / 2)), H
							- 64
							+ (20 - (this.membraneImage.getHeight(null) / 2)),
							this.membraneImage.getWidth(null),
							this.membraneImage.getHeight(null));
					g.drawImage(this.membraneImage, this.buttons[4].x,
							this.buttons[4].y, null);
				}

				g.setColor(DataPool.J3DSettings.bright_appearance ? Color.gray
						: Color.LIGHT_GRAY);

				for (int i = 0; i < this.buttons.length; i++) {
					if (this.mode == Hud.ALIGNLIPID && (i != 2) && (i != 4))
						continue;

					if (i < 4) {

						g.drawImage(this.arrows[i], this.buttons[i].x
								+ this.buttons[i].width / 2
								- this.arrows[i].getWidth() / 2,
								this.buttons[i].y + this.buttons[i].height / 2
										- this.arrows[i].getHeight() / 2, null);

						// g.drawRect(buttons[i].x,buttons[i].y,buttons[i].width,buttons[i].height);
					} else {
						g.drawLine(this.buttons[i].x, this.buttons[i].y,
								this.buttons[i].x + this.buttons[i].width,
								this.buttons[i].y);
						g.drawLine(this.buttons[i].x, this.buttons[i].y,
								this.buttons[i].x, this.buttons[i].y
										+ this.buttons[i].height);
						g.drawLine(this.buttons[i].x - 1,
								this.buttons[i].y - 1, this.buttons[i].x
										+ this.buttons[i].width,
								this.buttons[i].y - 1);
						g.drawLine(this.buttons[i].x - 1,
								this.buttons[i].y - 1, this.buttons[i].x - 1,
								this.buttons[i].y + this.buttons[i].height);

						g.drawLine(this.buttons[i].x, this.buttons[i].y,
								this.buttons[i].x - 5, this.buttons[i].y - 5);
						g.drawLine(this.buttons[i].x, this.buttons[i].y + 1,
								this.buttons[i].x - 5, this.buttons[i].y - 4);
					}

				}
				final int bt = this
						.getButton(this.currentMouseEvent.getPoint());
				if (bt != -1) {
					if (this.mode != Hud.ALIGNLIPID || bt == 2 || bt == 4) {
						this.fillBackground(g, this.buttons[bt]);
						g
								.drawRect(this.buttons[bt].x,
										this.buttons[bt].y,
										this.buttons[bt].width,
										this.buttons[bt].height);
					}
				}

				if (this.mode == Hud.ALIGNLIPID) {
					g.drawLine(W / 2 - 5, H / 2, W / 2 + 5, H / 2);
					g.drawLine(W / 2, H / 2 - 5, W / 2, H / 2 + 5);
				}

				if (this.colliding && this.mode == Hud.ALIGN) {

					g.setFont(this.origFont.deriveFont(Font.BOLD, 13));

					final JLabel l2 = new JLabel("Objects intersect currently!");
					l2.setFont(g.getFont());

					g.setColor(textBackground);
					g.fillRect(W / 2 - l2.getPreferredSize().width / 2 - 3, H
							/ 2 - l2.getPreferredSize().height - 6, l2
							.getPreferredSize().width + 6, l2
							.getPreferredSize().height + 5);
					g.setColor(Color.red);
					g.drawString(l2.getText(), W / 2
							- l2.getPreferredSize().width / 2, H / 2
							- l2.getPreferredSize().height / 2);
				}
			}

		}

		
		this.drawAtomRolloverInfo(g);
	}

	
	
	  
	private void drawAtomRolloverInfo(Graphics2D g) {

		PickResult[] pr = this.pickCanvas.pickAllSorted();
		
		if(pr==null||pr.length==0)return;
		Primitive prim=null;
		for(int i=0;i<pr.length;i++) {
			
			Primitive pri = (Primitive) pr[i].getNode(PickResult.PRIMITIVE);
			if (!(pri instanceof Sphere)) {
				continue;
			}
			prim=pri;
			break;
		}
		
	
		
		Sphere pickedSphere = (Sphere) prim;
		if (pickedSphere == null) {
			this.atCursor=null;
			
			return;
		}

		Atom_3D pickedAtom = (Atom_3D) pickedSphere.getUserData();

		this.atCursor=pickedAtom;
		
//		for(int k=0;k<DataPool.getMembraneModel().getProteinAt(0).allAtomsSorted.length;k++) {
//			if(DataPool.getMembraneModel().getProteinAt(0).allAtomsSorted[k]==this.atCursor)
//				System.out.println(k+"   "+this.atCursor);
//		}
		
		
		
//		int W=this.canvas.getWidth();
		int H=this.canvas.getHeight();
		Color fg = DataPool.J3DSettings.bright_appearance?Color.black:Color.white;
	
		
		
		g.setColor(fg);
		
//		JLabel l=new JLabel("A");
//		l.setFont(font1);
		int prefH=H-20;
		g.drawLine(0,prefH,150,prefH);
		g.drawLine(150,prefH,150,H);
		g.drawLine(80,prefH,80,H);
		
		if(this.atCursor!=null) {
			
			g.setFont(font1);
			g.setColor(this.atCursor.color);
			g.fillOval(8,H-15,10,10);
			g.setColor(fg);
			
			g.setFont(font3);
			g.drawString("Name:",85,H-6);
			
			g.setFont(font2);
			g.drawString(""+atCursor.name,118,H-5);
			
		
		
			
			
			
			if(atCursor instanceof Particle) {
				
				JLabel l=new JLabel(""+((Particle)atCursor).particleIdentifier);
				l.setFont(font2);
				int ext= l.getPreferredSize().width+29+150;
				g.setColor(this.atCursor.color);
				g.drawOval(5,H-18,16,16);
				g.setColor(fg);
				
				g.drawLine(150,prefH,ext,prefH);
				g.drawLine(ext,prefH,ext,H);
				g.setFont(font1);
				g.drawString("Particle",28,H-5);
				
				
				g.setFont(font3);
				g.drawString("ID:",155,H-6);
				g.setFont(font2);
				g.drawString(""+((Particle)atCursor).particleIdentifier,174,H-5);
				
				
				
				
				
				
				
			}else {
				g.setFont(font1);
				g.drawString("Atom",31,H-5);
				
				
				
				
				JLabel l=new JLabel(""+atCursor.residueName);
				l.setFont(font2);
				int ext= l.getPreferredSize().width+55+150;
				
				
				g.drawLine(150,prefH,ext,prefH);
				g.drawLine(ext,prefH,ext,H);
				
				
				
				g.setFont(font3);
				g.drawString("Residue:",155,H-6);
				g.setFont(font2);
				g.drawString(""+atCursor.residueName,200,H-5);
				
				
				
				
				
				
				
				
				
				
			}
			
			
		}else {
			int ext= 223;
			g.drawLine(150,prefH,ext,prefH);
			g.drawLine(ext,prefH,ext,H);
		}
		
		
		
		
		
	}

	private void fillBackground(final Graphics g, final Rectangle rectangle) {
		// boolean b=true;
		for (int i = 0; i < rectangle.width; i += 2) {
			// int add=0;
			// if(b)add=1;
			for (int j = 0; j < rectangle.height; j += 3) {
				g.drawLine(rectangle.x + i, rectangle.y + j, rectangle.x + i,
						rectangle.y + j + 1);

			}
		}

	}

	public void forwardMouseEvent(final MouseEvent arg0) {
		this.currentMouseEvent = arg0;

	}

	public void setPickNDragManager(final PickNDragManager pndManager) {

	}

	public void setCameraModeWhileAligning(final boolean b) {
		this.cameraWhileAligning = b;

	}

	public int getButton(final Point point) {
		try {

			for (int i = 0; i < 5; i++) {
				if (this.mode == Hud.ALIGN
						|| (this.mode == Hud.ALIGNLIPID && (i == 2 || i == 4))) {
					if (this.buttons[i].contains(point))
						return i;
				}
			}
			return -1;
		} catch (final Exception e) {
			return -1;
		}
	}

	public void setMembraneImage(final Image mimage) {
		int w = DataPool.getMembraneModel().getSize().width;
		int h = DataPool.getMembraneModel().getSize().height;
		final float div = 40 / (float) Math.max(w, h);

		w = (int) (div * w);
		h = (int) (div * h);

		if (mimage != null)
			this.membraneImage = mimage.getScaledInstance(w, h,
					Image.SCALE_FAST);
		else
			this.membraneImage = null;

	}

	public void setCurrentlyColliding(final boolean b) {
		this.colliding = b;

	}

}
