/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui;

import java.util.Comparator;
import java.util.Vector;

/**
 * This comparator is used by the sortAllRows Method. Sorts the objects that are
 * passed. Gets involved when using the sort-method in the java.util.Collections
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class ColumnSorter implements Comparator {
	protected int colIndex;

	boolean ascending;

	public ColumnSorter(final int colIndex, final boolean ascending) {
		this.colIndex = colIndex;
		this.ascending = ascending;
	}

	/**
	 * compares two passed objects and returns an int value that tells in what
	 * order these objects should be
	 */
	@SuppressWarnings("unchecked")
	public int compare(final Object a, final Object b) {
		final Vector objvectorA = (Vector) a;
		final Vector objvectorB = (Vector) b;
		Object objectA = objvectorA.get(this.colIndex);
		Object objectB = objvectorB.get(this.colIndex);

		// Treat empty strains like nulls
		if (objectA instanceof String && ((String) objectA).length() < 1) {
			objectA = null;
		}
		if (objectB instanceof String && ((String) objectB).length() < 1) {
			objectB = null;
		}

		/*
		 * Sort nulls so they appear last, regardless of sort order
		 */
		if (objectA == null && objectB == null) {
			return 0;
		} else if (objectA == null) {
			if (this.ascending) {
				return 1;
			} else {
				return -1;
			}

		} else if (objectB == null) {
			if (this.ascending) {
				return -1;
			} else {
				return 1;
			}
		} else if (objectA instanceof Comparable) {
			if (this.ascending) {
				return ((Comparable) objectA).compareTo(objectB);
			} else {
				return ((Comparable) objectB).compareTo(objectA);
			}
		} else {
			if (this.ascending) {
				return objectA.toString().compareTo(objectB.toString());
			} else {
				return objectB.toString().compareTo(objectA.toString());
			}
		}
	}

}
