/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
/**
 * 
 */
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.media.j3d.PolygonAttributes;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.engine.DataPool.J3DSettings;
import org.cellmicrocosmos.cm2.engine.Engine.J3DOperations;
import org.cellmicrocosmos.cm2.gui.HelpBtn;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;


/**
 * 
 * The Dialog for the J3d settings. It is divided into 3 different tabs
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
@SuppressWarnings("serial")
public class J3DSettingsDialog extends JDialog implements ActionListener,
		ChangeListener {

	JPanel main = new JPanel();

	public JTabbedPane tab = new JTabbedPane();

	JPanel main2 = new JPanel();

	JPanel main3 = new JPanel();

	JPanel main4 = new JPanel();

	JPanel header = new JPanel() {
		@Override
		public void paintComponent(final Graphics g) {

			g.setFont(new Font("Serif", Font.PLAIN, 18));
			g.setColor(this.getForeground());
			g.drawString("Options for 3D Mode", 2, 22);

			g.setColor(this.getForeground().darker());
			g.drawString("*", 3, 49);
			g.setFont(new Font("Serif", Font.PLAIN, 13));
			g.drawString("may effect performance", 15, 46);

		}
	};

	JPanel btnbar = new JPanel();

	JButton close = new JButton("Close");

	private final J3DOperations j3d;

	JSlider precslider = new JSlider();

	JSlider minprecslider = new JSlider();

	JSlider verticesslider = new JSlider();

	JSlider ambientslider = new JSlider();

	JCheckBox light1check = new JCheckBox("Light1");

	JCheckBox light2check = new JCheckBox("Light2");

	JSlider shinprotslider = new JSlider();

	JSlider shinlipslider = new JSlider();

	JRadioButton filledradio = new JRadioButton("Filled");

	JRadioButton wireframeradio = new JRadioButton("Wireframe");

	JButton defaultBtn = new JButton("Defaults");

	JSlider mousesen = new JSlider();

	JCheckBox shapesCollisions = new JCheckBox("Shape Collision");

//	JSlider atomDistances = new JSlider();
	JSpinner atomDistances = new JSpinner(new SpinnerNumberModel(0.0f, 0.0f, 100.0f, 0.1f));

	JCheckBox atomDistancesAbsolute = new JCheckBox("absolute");
	
	JSlider leaveOutAtomsFactor = new JSlider();

	JCheckBox atomsCollisions = new JCheckBox("Atom Collision");

	JRadioButton rbDetail1 = new JRadioButton("Automatic Detail Settings");

	JRadioButton rbDetail2 = new JRadioButton("Custom Detail Settings");

	JPanel main1 = new JPanel();

	JCheckBox disableDragging = new JCheckBox("Disable");

	JCheckBox enableFastTransitions = new
	JCheckBox("Enable Fast Transitions");
	
	JCheckBox enablePPEAutomatic = new
	JCheckBox("Enable PPE Automatic PDB Adjustment");
	
	JCheckBox antialiasing = new JCheckBox("Antialiasing");
	JCheckBox parallelProjection = new JCheckBox("Parallel Projection");
	JCheckBox showAllMultilayers = new JCheckBox("Show all layers");
	JCheckBox uniformBackground = new JCheckBox("Uniform Background");

	public J3DSettingsDialog(final J3DOperations j3doperations,
			final Point location) {

		this.j3d = j3doperations;
		this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		this.setIconImage(Cm2_IconDepot.getSpecifiedIcon("MV_Detail")
				.getImage());
		this.setTitle("3D Settings");
		this.setAlwaysOnTop(true);

		JScrollPane scrollMain = new JScrollPane(main);
		scrollMain.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollMain.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		
		main.setLayout(new BoxLayout(main, BoxLayout.Y_AXIS));
		main.add(this.header, BorderLayout.NORTH);
		main.add(this.tab, BorderLayout.CENTER);
		main.add(this.btnbar, BorderLayout.SOUTH);
		
		this.getContentPane().add(scrollMain);
		
		this.close.addActionListener(this);
		this.header.add(Box.createVerticalStrut(40));

		this.btnbar.setLayout(new BoxLayout(this.btnbar, BoxLayout.X_AXIS));
		this.btnbar.add(Box.createHorizontalGlue());
		this.btnbar.add(this.defaultBtn);
		this.btnbar.add(Box.createHorizontalStrut(10));
		this.btnbar.add(this.close);
		this.btnbar.add(Box.createHorizontalGlue());
		this.btnbar.add(new HelpBtn(this));
		this.btnbar.add(Box.createHorizontalStrut(5));

		final JPanel p1 = new JPanel();
		p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));

		final JPanel p2 = new JPanel();
		p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));

		p1.add(this.main2);
		p1.add(Box.createHorizontalStrut(300));

		p2.add(this.main4);
		p2.add(Box.createHorizontalStrut(300));

		this.tab.addTab("Geometric Detail", this.main1);
		this.tab.addTab("Appearance", p1);
		this.tab.addTab("Controls", p2);

		this.main2.setLayout(new BoxLayout(this.main2, BoxLayout.Y_AXIS));
		this.main3.setLayout(new BoxLayout(this.main3, BoxLayout.Y_AXIS));
		this.main4.setLayout(new BoxLayout(this.main4, BoxLayout.Y_AXIS));

		this.main1.setLayout(new GridLayout(3, 2));

		final ButtonGroup g2 = new ButtonGroup();
		g2.add(this.filledradio);
		g2.add(this.wireframeradio);

		final ButtonGroup g3 = new ButtonGroup();
		g3.add(this.rbDetail1);
		g3.add(this.rbDetail2);

		final JPanel infopanel1 = new JPanel();
		final JLabel infolabel1 = new JLabel(
				"<html>Set up parameters used when a 3D object are created. NOTE:<br>"
						+ "These settings do not only affect the rendering performance,<br>"
						+ "they also play a major role at the collision detection, meaning<br>"
						+ "that higher settings increase the correctness and minimize<br>"
						+ "overlaps between molecules while lower settings speed up the <br>"
						+ "collision detection significantly. <br>"
						+ "To avoid overlapping of molecules, <font color=\"FFC800\">Shape and Atom Collision</font><br>"
						+ "must be activated. Deactivating Shape Collision will enable<br>"
						+ "Molecules to overlap, increase the performance and increase<br>"
						+ "the molecules/space ratio.<br>"
						+ "In order to apply any changes for already constructed objects<br>"
						+ "and to check the collision detection, use the <font color=\"00AA00\">Refresh Geometry</font><br>"
						+ "Button located in the toolbar.</html>.");

		this.main1.add(infopanel1);

		infopanel1.setLayout(new BoxLayout(infopanel1, BoxLayout.X_AXIS));
		infopanel1.add(Box.createHorizontalStrut(5));
		infopanel1.add(infolabel1);
		infopanel1.add(Box.createHorizontalStrut(5));

		final MySwitcherPanel msp = new MySwitcherPanel(
				"Shape Collision and Construction",
				"<html>Activate the Shape Collision to use the mesh for collision detection.<br>Choose wether the precision of the surfaces creation should be<br>handled automatically or not. "
						+ "Choosing custom detail settings<br>will enable you to set up the parameters manually and will usually"
						+ "<br>increase the performance. Automatic Detail Settings ensure that<br>the shape wraps really all atoms, but may"
						+ "be less convenient<br>on older machines or if completeness is less important."
						+ "</html>", this.rbDetail1, this.rbDetail2, true);

		msp.cp.setLayout(new BoxLayout(msp.cp, BoxLayout.Y_AXIS));

		this.shapesCollisions.setForeground(Color.ORANGE);
		msp.cp.add(this.shapesCollisions, 0);

		this.main1
				.add(new MySliderPanel(
						"Precision Modifier",
						"<html>The dependency between the size of a "
								+ "molecule<br>and the number of levels used while constructing<br>a "
								+ "3D object in vertical order</html>",
						this.precslider, true, null, new JLabel(
								"Levels x (height/100)")));

		this.main1.add(msp);

		this.main1
				.add(new MySliderPanel(
						"Min. Precision",
						"<html>Ensures that all molecules have "
								+ "at least a minimum<br>detail, e.g. when one is very small"
								+ "</html>", this.minprecslider, true, null,
						new JLabel("Levels                        ")));

//		final MySliderPanel mp = new MySliderPanel(
//				"Atom Collision",
//				"<html>Enable this option in order to respect all outer atoms and their<br>radiuses at collision detection. "
//						+ "If activated, use the slider to set<br>an additional minimal distance between the atoms.</html>",
//				this.atomDistances, true, null, new JLabel("angstrom"));
		
		final MySpinnerPanel mp = new MySpinnerPanel(
				"Atom Collision",
				"<html>Enable this option in order to respect all outer atoms and their<br>radiuses at collision detection. " +
				"If activated, use the slider to set<br>an additional minimal distance between the atoms. This distance<br>" +
				"is added to each side of an atom. If absolute is checked, the value<br>" +
				"is used as standard diameter value for each atom.</html>",
				this.atomDistances, true, null, new JLabel("<html>&nbsp;angstrom</html>"));

		mp.cp.add(this.atomDistancesAbsolute, 0);
		
		this.atomsCollisions.setForeground(Color.ORANGE);
		mp.cp.add(this.atomsCollisions, 0);

		this.main1.add(mp);

		this.main1
				.add(new MySliderPanel(
						"Number of Vertices",
						"<html>The number of vertices that are taken into account"
								+ "<br>for an object; the higher the value, the more<br>vertices per level will be used</html>",
						this.verticesslider, true, null, new JLabel(
								"Vertices per Level       ")));

		this.main2.add(new MySliderPanel("Brightness Ambient Light",
				"<html>The overall brightness of all<br>"
						+ "visible objects in the view</html>",
				this.ambientslider, false, new JLabel("-"), new JLabel("+")));

		this.main2.add(new MySwitcherPanel("Directional Lights",
				"<html>Toggle the 2 additional light sources on/off"
						+ "</html>", this.light1check, this.light2check, true));

		this.main2.add(this.main3);

		this.main3.add(new MySliderPanel("Shininess Proteins",
				"<html>The intensity of light reflection<br>"
						+ "on protein surfaces</html>", this.shinprotslider,
				false, new JLabel("-"), new JLabel("+")));
		this.main3.add(new MySliderPanel("Shininess Lipids",
				"<html>The intensity of light reflection<br>"
						+ "on lipid surfaces</html>", this.shinlipslider,
				false, new JLabel("-"), new JLabel("+")));

		this.main3.add(new MySwitcherPanel("Polygon Mode",
				"<html>The 2 possible modes for displaying the objects:<br>"
						+ "- With a closed surface (Filled), or <br>"
						+ "- With a mesh-like surface (Wireframe)" + "</html>",
				this.filledradio, this.wireframeradio, true));

		final MySwitcherPanel msViz = new MySwitcherPanel("Visualization", null,
										this.antialiasing, this.uniformBackground, true);
		msViz.cp.add(this.parallelProjection,0);
		msViz.cp.add(this.showAllMultilayers,0);
		
		this.main3.add(msViz);
		
//		this.main3
//				.add(new MySliderPanel(
//						"Atom Rendering Heuristics",
//						"<html>Leaves out the inner atoms of a protein in 3D mode<br> "
//								+ "in order to speed up rendering. Note that this method<br>"
//								+ "only affect the visuals. Use the slider to set the factor</html>",
//						this.leaveOutAtomsFactor, true, new JLabel("off"),
//						new JLabel("high factor")));

		this.main4.add(new MySliderPanel("Mouse Sensitivity",
				"<html>The mouse sensitivity while looking around<br>"
						+ "in the 3D view</html>", this.mousesen, false,
				new JLabel("-"), new JLabel("+")));

		this.main4.add(new MySwitcherPanel("Dragging of Proteins",
				"<html>Capabiltity to move proteins while using the<br>"
						+ "Free-Look mode. Disabling will avoid any<br>"
						+ "unintended displacements" + "</html>",
				this.disableDragging, null, false));

		main4.add(new MySwitcherPanel("Fast Transitions",
				"<html>If this is activated, the 3D view may be faster in highlighting<br />"
				+
				"updating the view, if you are chaning e.g. the microdomains <br />"
				+ "or the membrane layer. <br />"
				+ "If you have actual hardware, you should switch this off,<br />"
				+ "because the transitions will be more smooth.</html>",
				this.enableFastTransitions, null, false));
		
		main4.add(new MySwitcherPanel("PPE Automatic Protein Placement",
				"<html>The Protein Placement Expansion (PPE) is able to <br />" +
				"place proteins automatically which are found in the PDBTM (Protein Data Bank of<br/>" +
				"Transmembrane Proteins) as well as proteins downloaded from<br/>" +
				"the OPM (Orientation of Proteins in Membranes) database.<br />"+
				"If this is switched off, you still can align the proteins after you<br/>" +
				"placed them in the membrane by using the according buttons<br/>" +
				"in the Align Molecule mode.</html>",
				this.enablePPEAutomatic, null, false));

		this.main4.add(Box.createVerticalGlue());

		this.precslider.setMinimum(10);
		this.precslider.setMaximum(170);

		final Hashtable<Integer, JLabel> prec = new Hashtable<Integer, JLabel>();
		prec.put(new Integer(10), new JLabel("10"));
		prec.put(new Integer(30), new JLabel("30"));
		prec.put(new Integer(50), new JLabel("50"));
		prec.put(new Integer(70), new JLabel("70"));
		prec.put(new Integer(90), new JLabel("90"));
		prec.put(new Integer(110), new JLabel("110"));
		prec.put(new Integer(130), new JLabel("130"));
		prec.put(new Integer(150), new JLabel("150"));
		prec.put(new Integer(170), new JLabel("170"));
		this.precslider.setLabelTable(prec);

		this.precslider.setPaintLabels(true);
		this.precslider.setPaintTicks(true);
		this.precslider.setMajorTickSpacing(1);

		this.precslider.setSnapToTicks(false);

		this.minprecslider.setMinimum(2);
		this.minprecslider.setMaximum(10);

		this.minprecslider.setSnapToTicks(true);

		final Hashtable<Integer, JLabel> mprec = new Hashtable<Integer, JLabel>();
		mprec.put(new Integer(2), new JLabel("2"));
		mprec.put(new Integer(3), new JLabel("3"));
		mprec.put(new Integer(4), new JLabel("4"));
		mprec.put(new Integer(5), new JLabel("5"));
		mprec.put(new Integer(6), new JLabel("6"));
		mprec.put(new Integer(7), new JLabel("7"));
		mprec.put(new Integer(8), new JLabel("8"));
		mprec.put(new Integer(9), new JLabel("9"));
		mprec.put(new Integer(10), new JLabel("10"));
		this.minprecslider.setLabelTable(mprec);

		this.minprecslider.setPaintLabels(true);
		this.minprecslider.setPaintTicks(true);
		this.minprecslider.setMajorTickSpacing(1);

		this.verticesslider.setMinimum(4);
		this.verticesslider.setMaximum(60);

		final Hashtable<Integer, JLabel> vert = new Hashtable<Integer, JLabel>();
		vert.put(new Integer(4), new JLabel("4"));
		vert.put(new Integer(10), new JLabel("10"));
		vert.put(new Integer(20), new JLabel("20"));
		vert.put(new Integer(30), new JLabel("30"));
		vert.put(new Integer(40), new JLabel("40"));
		vert.put(new Integer(50), new JLabel("50"));
		vert.put(new Integer(60), new JLabel("60"));

		this.verticesslider.setLabelTable(vert);

		this.verticesslider.setPaintLabels(true);
		this.verticesslider.setPaintTicks(true);

		this.verticesslider.setSnapToTicks(true);
		this.verticesslider.setMajorTickSpacing(1);

//		this.atomDistances.setMinimum(0);
//		this.atomDistances.setMaximum(50);
//
//		this.atomDistances.setSnapToTicks(true);

		final Hashtable<Integer, JLabel> adtab = new Hashtable<Integer, JLabel>();
		adtab.put(new Integer(0), new JLabel("0"));
		adtab.put(new Integer(10), new JLabel("1"));
		adtab.put(new Integer(20), new JLabel("2"));
		adtab.put(new Integer(30), new JLabel("3"));
		adtab.put(new Integer(40), new JLabel("4"));
		adtab.put(new Integer(50), new JLabel("5"));

//		this.atomDistances.setLabelTable(adtab);
//
//		this.atomDistances.setPaintLabels(true);
//		this.atomDistances.setPaintTicks(true);
//		this.atomDistances.setMajorTickSpacing(1);

		this.leaveOutAtomsFactor.setInverted(false);
		this.leaveOutAtomsFactor.setMinimum(30);
		this.leaveOutAtomsFactor.setMaximum(80);
		this.leaveOutAtomsFactor.setMajorTickSpacing(1);

		final Font f = this.getFont().deriveFont(9f);
		final Enumeration<JLabel> en1 = prec.elements();
		while (en1.hasMoreElements()) {
			en1.nextElement().setFont(f);
		}
		final Enumeration<JLabel> en2 = mprec.elements();
		while (en2.hasMoreElements()) {
			en2.nextElement().setFont(f);
		}
		final Enumeration<JLabel> en3 = vert.elements();
		while (en3.hasMoreElements()) {
			en3.nextElement().setFont(f);
		}

		this.precslider
				.setValue((int) (DataPool.J3DSettings.precision_modifier * 10));
		this.minprecslider.setValue(DataPool.J3DSettings.minprecision);

		this.verticesslider.setValue(DataPool.J3DSettings.numberofvertices);

//		this.atomDistances
//			.setValue((int) (DataPool.J3DSettings.minatomdistance * 10f));
		this.atomDistances
			.setValue(DataPool.J3DSettings.minatomdistance);
		this.atomDistancesAbsolute
			.setSelected(DataPool.J3DSettings.atomDistanceAbsolute);
		this.leaveOutAtomsFactor
				.setValue((int) (DataPool.J3DSettings.removeInnerAtomsFactor));

		this.light1check.setSelected(DataPool.J3DSettings.light1);
		this.light2check.setSelected(DataPool.J3DSettings.light2);

		this.ambientslider.setMinimum(0);
		this.ambientslider.setMaximum(100);

		this.ambientslider
				.setValue((int) (DataPool.J3DSettings.ambientlight * 100));

		this.shinprotslider.setMinimum(0);
		this.shinprotslider.setMaximum(90);
		this.shinlipslider.setMinimum(0);
		this.shinlipslider.setMaximum(90);

		this.shinprotslider.setInverted(true);
		this.shinlipslider.setInverted(true);

		this.shinprotslider.setValue(DataPool.J3DSettings.shininess_proteins);
		this.shinlipslider.setValue(DataPool.J3DSettings.shininess_lipids);

		this.mousesen.setMinimum(10);
		this.mousesen.setMaximum(40);

		this.mousesen.setValue((int) DataPool.J3DSettings.mouseFactor * 10);

		this.disableDragging
				.setSelected(DataPool.J3DSettings.forbidDraggingInFreeLookMode);

		this.antialiasing.setSelected(DataPool.J3DSettings.antialiasing);
		this.parallelProjection.setSelected(DataPool.J3DSettings.parallelProjection);
		this.showAllMultilayers.setSelected(DataPool.J3DSettings.showAllMultiLayers);
		this.uniformBackground.setSelected(DataPool.J3DSettings.uniformBackground);

		this.disableDragging.addActionListener(this);
		this.enableFastTransitions
				.setSelected(DataPool.J3DSettings.enableFastTransitions);
		enableFastTransitions.addActionListener(this);
		this.enablePPEAutomatic
				.setSelected(DataPool.J3DSettings.enablePPEAutomatic);
		enablePPEAutomatic.addActionListener(this);
		this.antialiasing.addActionListener(this);
		this.parallelProjection.addActionListener(this);
		this.showAllMultilayers.addActionListener(this);
		this.uniformBackground.addActionListener(this);
		this.mousesen.addChangeListener(this);

		this.filledradio
				.setSelected(DataPool.J3DSettings.polygon_mode == PolygonAttributes.POLYGON_FILL);
		this.wireframeradio
				.setSelected(DataPool.J3DSettings.polygon_mode == PolygonAttributes.POLYGON_LINE);

		this.precslider.addChangeListener(this);
		this.minprecslider.addChangeListener(this);
		this.verticesslider.addChangeListener(this);

		this.defaultBtn.addActionListener(this);

		this.light1check.addActionListener(this);
		this.light2check.addActionListener(this);

		this.filledradio.addActionListener(this);
		this.wireframeradio.addActionListener(this);
		this.ambientslider.addChangeListener(this);
		this.shinprotslider.addChangeListener(this);
		this.shinlipslider.addChangeListener(this);

		this.rbDetail1.addActionListener(this);
		this.rbDetail2.addActionListener(this);
		this.shapesCollisions.addActionListener(this);
		this.atomDistances.addChangeListener(this);
		this.atomDistancesAbsolute.addActionListener(this);
		this.leaveOutAtomsFactor.addChangeListener(this);
		this.atomsCollisions.addActionListener(this);

		this.pack();

		this.pack();
		this.header.repaint();
		if (location != null)
			this.setLocation(location.x - this.getWidth() / 2, location.y);
		else
			this.setLocationRelativeTo(this.j3d.getRealtiveComponent());

		if (DataPool.J3DSettings.automaticDetailSettings)
			this.rbDetail1.doClick();
		else
			this.rbDetail2.doClick();

		if (DataPool.J3DSettings.shapeCollisionCheck) {
			this.shapesCollisions.setSelected(true);
		} else {
			this.shapesCollisions.setSelected(false);
		}

		if (DataPool.J3DSettings.atomDistanceAbsolute) {
			this.atomDistancesAbsolute.setSelected(true);
		} else {
			this.atomDistancesAbsolute.setSelected(false);
		}
		
		if (DataPool.J3DSettings.atomCollisionCheck) {
			this.atomsCollisions.setSelected(true);
		} else {
			this.atomsCollisions.setSelected(false);
			this.atomDistances.setEnabled(false);
			this.atomDistancesAbsolute.setEnabled(false);
		}

		this.setVisible(true);
	}

	public void actionPerformed(final ActionEvent arg0) {

		if (arg0.getSource() == this.rbDetail1
				|| arg0.getSource() == this.rbDetail2) {

			if (this.rbDetail1.isSelected()) {
				this.precslider
						.setValue((int) (CM2_CONSTANTS.J3D.DETAIL_PRECISION_MODIFIER * 10));
				this.minprecslider
						.setValue(CM2_CONSTANTS.J3D.DETAIL_MIN_PRECISION);
				this.verticesslider
						.setValue(CM2_CONSTANTS.J3D.DETAIL_DEFAULTVERTICES);
			}

			this.precslider.setEnabled(this.rbDetail2.isSelected());
			this.minprecslider.setEnabled(this.rbDetail2.isSelected());
			this.verticesslider.setEnabled(this.rbDetail2.isSelected());

			DataPool.J3DSettings.automaticDetailSettings = this.rbDetail1
					.isSelected();

		}
		if (arg0.getSource() == this.shapesCollisions) {
			DataPool.J3DSettings.shapeCollisionCheck = this.shapesCollisions
					.isSelected();
			if (!this.shapesCollisions.isSelected()
					&& !this.atomsCollisions.isSelected()) {
				if (!(JOptionPane
						.showConfirmDialog(
								this,
								"<html>Neither <font color=\"FFC800\">Atom nor Shape Collision</font> will be activated.<br><br>"
										+ "This means that the algorithms will not work properly and may not<br>"
										+ "stop automatically. You should activate at least one Collision Mode!<br><br>"
										+ "Do you wish to proceed anyway and deactivate all Collision Modes?</html>",
								"Confirm", JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION)) {
					DataPool.J3DSettings.shapeCollisionCheck = true;
					this.shapesCollisions.setSelected(true);
				}
				// automatically switch to AtomCollision
				// DataPool.J3DSettings.atomCollisionCheck = true;
				// this.atomsCollisions.setSelected(true);
				// this.atomDistances.setEnabled(true);
				// JOptionPane.showMessageDialog(this,
				// "One Collision Mode has to be selected!\n"+
				// "Activating Atom Collision Mode.", "Collision Mode Info",
				// JOptionPane.WARNING_MESSAGE);
			}
			this.j3d.refreshViews(false, false, false, false, false, true);
		}
		if (arg0.getSource() == this.atomsCollisions) {
			DataPool.J3DSettings.atomCollisionCheck = this.atomsCollisions
					.isSelected();
			if (!this.shapesCollisions.isSelected()
					&& !this.atomsCollisions.isSelected()) {
				if (!(JOptionPane
						.showConfirmDialog(
								this,
								"<html>Neither <font color=\"FFC800\">Atom nor Shape Collision</font> will be activated.<br><br>"
										+ "This means that the algorithms will not work properly and may not<br>"
										+ "stop automatically. You should activate at least one Collision Mode!<br><br>"
										+ "Do you wish to proceed anyway and deactivate all Collision Modes?</html>",
								"Confirm", JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION)) {
					DataPool.J3DSettings.atomCollisionCheck = true;
					this.atomsCollisions.setSelected(true);
				}
				// automatically switch to ShapeCollision
				// DataPool.J3DSettings.shapeCollisionCheck = true;
				// this.shapesCollisions.setSelected(true);
				// JOptionPane.showMessageDialog(this,
				// "One Collision Mode has to be selected!\n"+
				// "Activating Shape Collision Mode.", "Collision Mode Info",
				// JOptionPane.WARNING_MESSAGE);
			}
			this.atomDistances.setEnabled(this.atomsCollisions.isSelected());
			this.atomDistancesAbsolute.setEnabled(this.atomsCollisions.isSelected());
			this.j3d.refreshViews(false, false, false, false, false, true);
		}

		if (arg0.getSource() == this.atomDistancesAbsolute) {
			DataPool.J3DSettings.atomDistanceAbsolute = this.atomDistancesAbsolute.isSelected();
			this.j3d.refreshViews(false, false, false, false, false, true);
		}
		
		if (arg0.getSource() == this.close) {

			this.dispose();
		}

		if (arg0.getSource() == this.defaultBtn) {

			if (this.tab.getSelectedIndex() == 0) {

				this.precslider
						.setValue((int) (CM2_CONSTANTS.J3D.DETAIL_PRECISION_MODIFIER * 10));
				this.minprecslider
						.setValue(CM2_CONSTANTS.J3D.DETAIL_MIN_PRECISION);

				this.verticesslider
						.setValue(CM2_CONSTANTS.J3D.DETAIL_DEFAULTVERTICES);

				J3DSettings.shapeCollisionCheck = true;
				this.shapesCollisions.setSelected(true);

				this.atomsCollisions.setSelected(true);
				J3DSettings.atomCollisionCheck = true;
				this.atomDistancesAbsolute.setSelected(false);
				this.atomDistancesAbsolute.setEnabled(true);
				
				DataPool.J3DSettings.minatomdistance = 0;
				this.atomDistances.setValue(0);
				
				J3DSettings.atomDistanceAbsolute = false;
				this.atomDistances.setEnabled(true);

				this.precslider.setEnabled(false);

				this.minprecslider.setEnabled(false);

				this.verticesslider.setEnabled(false);

				this.rbDetail1.setSelected(true);
				DataPool.J3DSettings.automaticDetailSettings = true;

			}
			if (this.tab.getSelectedIndex() == 1) {

				if (this.light1check.isSelected() == false)
					this.light1check.doClick();

				if (this.light2check.isSelected() == true)
					this.light2check.doClick();

				this.ambientslider
						.setValue((int) (CM2_CONSTANTS.J3D.DETAIL_AMBIENT_LIGHT * 100));

				this.shinprotslider
						.setValue(CM2_CONSTANTS.J3D.DETAIL_SHININESS_PROTEINS);
				this.shinlipslider
						.setValue(CM2_CONSTANTS.J3D.DETAIL_SHININESS_LIPIDS);
				this.filledradio.doClick();

				if (this.antialiasing.isSelected() == true)
					this.antialiasing.doClick();

				this.leaveOutAtomsFactor.setValue(30);
				DataPool.J3DSettings.removeInnerAtomsFactor = 30;
			}
			if (this.tab.getSelectedIndex() == 2) {

				this.mousesen
						.setValue((int) (CM2_CONSTANTS.J3D.MOUSE_SENSITIVITY * 10));

				this.disableDragging.setSelected(false);

				this.enableFastTransitions.setSelected(CM2_CONSTANTS.J3D.FAST_TRANSITIONS);
				
				this.enablePPEAutomatic.setSelected(CM2_CONSTANTS.J3D.PPE_AUTOMATIC);
			}
		}

		if (arg0.getSource() == this.light1check) {
			DataPool.J3DSettings.light1 = this.light1check.isSelected();
			this.j3d.setLight1Enabled(this.light1check.isSelected());
		}
		if (arg0.getSource() == this.light2check) {
			DataPool.J3DSettings.light2 = this.light2check.isSelected();
			this.j3d.setLight2Enabled(this.light2check.isSelected());
		}
		if (arg0.getSource() == this.disableDragging) {
			DataPool.J3DSettings.forbidDraggingInFreeLookMode = this.disableDragging
					.isSelected();
			this.j3d.setForbidDraggingInFreeLookMode(this.disableDragging
					.isSelected());
		}
		if (arg0.getSource() == this.enableFastTransitions) {
			DataPool.J3DSettings.enableFastTransitions =
				this.enableFastTransitions.isSelected();
		}
		if (arg0.getSource() == this.enablePPEAutomatic) {
			DataPool.J3DSettings.enablePPEAutomatic =
				this.enablePPEAutomatic.isSelected();
		}
		if (arg0.getSource() == this.antialiasing) {

			try {
				this.j3d.setAntialiasingEnabled(this.antialiasing.isSelected());
				DataPool.J3DSettings.antialiasing = this.antialiasing
						.isSelected();
			} catch (final Throwable e) {
				this.antialiasing.setSelected(false);
				DataPool.J3DSettings.antialiasing = false;
			}
		}
		if (arg0.getSource() == this.parallelProjection) {

			try {
				DataPool.J3DSettings.parallelProjection = this.parallelProjection
						.isSelected();
				this.j3d.setParallelProjection(this.parallelProjection.isSelected());
			} catch (final Throwable e) {
				this.parallelProjection.setSelected(false);
				DataPool.J3DSettings.parallelProjection = false;
			}
		}
		if (arg0.getSource() == this.showAllMultilayers) {

			try {
				this.j3d.showAllMultilayers(this.showAllMultilayers.isSelected());
				DataPool.J3DSettings.showAllMultiLayers = this.showAllMultilayers.isSelected();
			} catch (final Throwable e) {
				this.showAllMultilayers.setSelected(false);
				DataPool.J3DSettings.showAllMultiLayers = false;
			}
		}
		if (arg0.getSource() == this.uniformBackground) {

			try {
				this.j3d.setUniformBackground(this.uniformBackground.isSelected());
				DataPool.J3DSettings.uniformBackground = this.uniformBackground.isSelected();
			} catch (final Throwable e) {
				this.uniformBackground.setSelected(false);
				DataPool.J3DSettings.uniformBackground = false;
			}
		}

		if (arg0.getSource() == this.filledradio
				|| arg0.getSource() == this.wireframeradio) {
			final int oldmode = DataPool.J3DSettings.polygon_mode;

			if (this.filledradio.isSelected())
				DataPool.J3DSettings.polygon_mode = PolygonAttributes.POLYGON_FILL;
			else
				DataPool.J3DSettings.polygon_mode = PolygonAttributes.POLYGON_LINE;

			if (DataPool.getMembraneModel() == null
					|| oldmode == DataPool.J3DSettings.polygon_mode)
				return;

			DataPool.getMembraneModel().setPolygonMode(
					DataPool.J3DSettings.polygon_mode);

		}

	}

	public void stateChanged(final ChangeEvent arg0) {

//		if (arg0.getSource() == this.atomDistances) {
//			DataPool.J3DSettings.minatomdistance = (float) this.atomDistances
//					.getValue() / 10f;
//			if (!this.atomDistances.getValueIsAdjusting())
//				this.j3d.refreshViews(false, false, false, false, false, true);
//		}
		if (arg0.getSource().equals(this.atomDistances)) {
			DataPool.J3DSettings.minatomdistance = new Float(this.atomDistances.getValue().toString());
			this.j3d.refreshViews(false, false, false, false, false, true);
		}
		if (arg0.getSource() == this.precslider) {
			DataPool.J3DSettings.precision_modifier = ((float) (this.precslider
					.getValue()) / 10);

		}
		if (arg0.getSource() == this.minprecslider) {
			DataPool.J3DSettings.minprecision = this.minprecslider.getValue();
		}
		if (arg0.getSource() == this.ambientslider) {
			final float f = ((float) (this.ambientslider.getValue()) / 100);
			DataPool.J3DSettings.ambientlight = f;
			this.j3d.setBrightnessForAmbientLight(f);

		}
		if (arg0.getSource() == this.verticesslider) {
			DataPool.J3DSettings.numberofvertices = this.verticesslider
					.getValue();

		}
		if (arg0.getSource() == this.mousesen) {

			DataPool.J3DSettings.mouseFactor = this.mousesen.getValue() / 10;
			this.j3d.setMouseSensitivity(this.mousesen.getValue() / 10);

		}
		if (arg0.getSource() == this.shinprotslider) {

			if (this.shinprotslider.getValue() != DataPool.J3DSettings.shininess_proteins) {
				DataPool.J3DSettings.shininess_proteins = this.shinprotslider
						.getValue();
				if (DataPool.getMembraneModel() != null)
					DataPool.getMembraneModel().setShininess(
							this.shinprotslider.getValue(), 0);
			}
		}
		if (arg0.getSource() == this.shinlipslider) {
			if (this.shinlipslider.getValue() != DataPool.J3DSettings.shininess_lipids) {
				DataPool.J3DSettings.shininess_lipids = this.shinlipslider
						.getValue();
				if (DataPool.getMembraneModel() != null)
					DataPool.getMembraneModel().setShininess(
							this.shinlipslider.getValue(), 1);
			}

		}
		if (arg0.getSource() == this.leaveOutAtomsFactor) {

			DataPool.J3DSettings.removeInnerAtomsFactor = this.leaveOutAtomsFactor
					.getValue();
		}
	}

	/**
	 * A panel that puts a header, a description and a slider with two labels
	 * into a special layout
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class MySliderPanel extends JPanel {

		JPanel dp = new JPanel();

		public JPanel cp = new JPanel();

		JLabel dl = new JLabel();

		public MySliderPanel(String header, final String description,
				final JSlider slider, final boolean performance_relevant,
				final JLabel low, final JLabel high) {
			super();
			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

			this.dp.setLayout(new BoxLayout(this.dp, BoxLayout.X_AXIS));
			this.cp.setLayout(new BoxLayout(this.cp, BoxLayout.X_AXIS));

			this.setBorder(BorderFactory.createEtchedBorder());

			this.add(this.dp);
			this.add(new JLabel(" "));
			this.add(this.cp);

			if (performance_relevant) {

				header += " *";
			}

			this.dp.add(this.dl);
			this.dp.add(new JPanel());
			if (low != null)
				this.cp.add(low);
			this.cp.add(slider);
			if (high != null)
				this.cp.add(high);

			// if(low!=null)
			// low.setFont(low.getFont().deriveFont(17f));
			this.setBorder(BorderFactory.createTitledBorder(""));
			this.setBorder(BorderFactory.createTitledBorder(this.getBorder(),
					header, TitledBorder.LEADING,
					TitledBorder.DEFAULT_POSITION, this.getFont().deriveFont(
							Font.BOLD, 12f)));
			this.dl.setText(description);
		}

	}
	
	/**
	 * A panel that puts a header, a description and a slider with two labels
	 * into a special layout
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class MySpinnerPanel extends JPanel {

		JPanel dp = new JPanel();

		public JPanel cp = new JPanel();

		JLabel dl = new JLabel();

		public MySpinnerPanel(String header, final String description,
				final JSpinner spinner, final boolean performance_relevant,
				final JLabel low, final JLabel high) {
			super();
			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

			this.dp.setLayout(new BoxLayout(this.dp, BoxLayout.X_AXIS));
			this.cp.setLayout(new BoxLayout(this.cp, BoxLayout.X_AXIS));

			this.setBorder(BorderFactory.createEtchedBorder());

			this.add(this.dp);
			this.add(new JLabel(" "));
			this.add(this.cp);

			if (performance_relevant) {

				header += " *";
			}

			this.dp.add(this.dl);
			this.dp.add(new JPanel());
			if (low != null)
				this.cp.add(low);
			spinner.setMaximumSize(new Dimension(60, 50));
			spinner.setMinimumSize(new Dimension(60, 50));
			this.cp.add(spinner);
			if (high != null)
				this.cp.add(high);

			// if(low!=null)
			// low.setFont(low.getFont().deriveFont(17f));
			this.setBorder(BorderFactory.createTitledBorder(""));
			this.setBorder(BorderFactory.createTitledBorder(this.getBorder(),
					header, TitledBorder.LEADING,
					TitledBorder.DEFAULT_POSITION, this.getFont().deriveFont(
							Font.BOLD, 12f)));
			this.dl.setText(description);
		}

	}

	/**
	 * A panel that puts a header, a description and two buttons into a special
	 * layout
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class MySwitcherPanel extends JPanel {

		JPanel dp = new JPanel();

		public JPanel cp = new JPanel();

		JLabel dl = new JLabel();

		public MySwitcherPanel(String header, final String description,
				final AbstractButton b1, final AbstractButton b2,
				final boolean performance_relevant) {
			super();
			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

			this.dp.setLayout(new BoxLayout(this.dp, BoxLayout.X_AXIS));
			this.cp.setLayout(new BoxLayout(this.cp, BoxLayout.X_AXIS));

			this.setBorder(BorderFactory.createEtchedBorder());

			this.add(this.dp);
			this.add(new JLabel(" "));
			this.add(this.cp);

			this.dp.add(this.dl);

			if (performance_relevant) {
				header += " *";
			}

			this.cp.add(b1);
			if (b2 != null)
				this.cp.add(b2);
			this.cp.add(Box.createHorizontalGlue());
			this.setBorder(BorderFactory.createTitledBorder(""));
			this.setBorder(BorderFactory.createTitledBorder(this.getBorder(),
					header, TitledBorder.LEADING,
					TitledBorder.DEFAULT_POSITION, this.getFont().deriveFont(
							Font.BOLD, 12f)));
			this.dl.setText(description);
		}

	}

	public void dispose() {

		super.dispose();
		System.gc();
	}

}
