/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;


/**
 * 
 * The dialog that shows up when a new membrane gets created. Allows to enter
 * the Code, the Classification and the size of the new membrane
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
@SuppressWarnings("serial")
public class NewMembraneDialog extends JDialog implements ActionListener {

	// possible return values when showing up
	public static final int APPROVE = 1;

	public final static int CANCEL = 2;

	private int out = NewMembraneDialog.CANCEL;

	private JButton cancelBtn, okBtn;

	private JPanel buttonbar;

	private JTextField tf1, tf2;

	private JPanel header1, header2;

	private JLabel lx, lz;

	private JSpinner tfx, tfz;

	private JPanel ground1, ground2;

	private JPanel groundxz;

	private JLabel h1, h2;

	private final MembraneModel model;

	public NewMembraneDialog(final MembraneModel model) {
		this.model = model;

	}

	/**
	 * Constructs the Gui
	 * 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public int popUp() {

		this.setTitle("Create a new Membrane...");

		this.tf1 = new JTextField();

		this.tf2 = new JTextField();

		this.tfx = new JSpinner();

		this.tfz = new JSpinner();

		this.lx = new JLabel("X");
		this.lz = new JLabel("  Z");

		this.ground1 = new JPanel();

		this.ground2 = new JPanel();

		this.groundxz = new JPanel();

		this.buttonbar = new JPanel();

		this.header1 = new JPanel();
		this.header2 = new JPanel();

		this.tfx.setModel(new SpinnerNumberModel(
				CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE,
				CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE,
				CM2_CONSTANTS.MEMBRANE_SIZES.MAX_SIZE_OF_MEMBRANE, 1));
		this.tfz.setModel(new SpinnerNumberModel(
				CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE,
				CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE,
				CM2_CONSTANTS.MEMBRANE_SIZES.MAX_SIZE_OF_MEMBRANE, 1));

		this.h1 = new JLabel("Create a new Membrane Model     ",
				SwingConstants.LEADING);
		this.h2 = new JLabel(
				"<html>Enter a Pdb-Code, a Header and a Size for the new<br>"
						+ "Model. These Selections represent only initial<br>"
						+ "Values and can be changed later at any time<br>",
				SwingConstants.LEADING);

		this.header1.add(this.h1);
		this.header2.add(this.h2);

		// header1.setBackground(Color.white);

		this.cancelBtn = new JButton("Cancel");

		this.okBtn = new JButton("Create");

		this.setModal(true);

		this.getContentPane().setLayout(
				new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));

		this.getContentPane().add(this.header1);
		this.getContentPane().add(this.header2);

		this.getContentPane().add(this.ground1);
		this.getContentPane().add(this.ground2);
		this.getContentPane().add(this.groundxz);
		this.getContentPane().add(this.buttonbar);

		this.buttonbar.add(this.okBtn);
		this.buttonbar.add(this.cancelBtn);

		this.ground1.setBorder(BorderFactory
				.createTitledBorder("Header  (max. 40 characters)"));
		this.ground2.setBorder(BorderFactory
				.createTitledBorder("Code   (max. 4 characters)"));

		final String mm = "( "
				+ CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE
				+ " <= X,Z <= "
				+ CM2_CONSTANTS.MEMBRANE_SIZES.MAX_SIZE_OF_MEMBRANE + " )";
		this.groundxz.setBorder(BorderFactory
				.createTitledBorder("Size in angstrom   " + mm));

		this.cancelBtn.addActionListener(this);
		this.okBtn.addActionListener(this);

		this.ground1.add(this.tf1);
		this.ground2.add(this.tf2);

		this.groundxz.add(this.lx);
		this.groundxz.add(this.tfx);
		this.groundxz.add(this.lz);
		this.groundxz.add(this.tfz);

		this.tf1
				.setFont(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.FONT_PDB_TEXTFIELDS);
		this.tf2
				.setFont(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.FONT_PDB_TEXTFIELDS);

		this.tf1.setPreferredSize(new Dimension(333, this.tf1
				.getPreferredSize().height));
		this.tf2.setPreferredSize(new Dimension(45,
				this.tf2.getPreferredSize().height));

		this.tf1.setDocument(new LimitedDocument(this.tf1, 40));
		this.tf2.setDocument(new LimitedDocument(this.tf2, 4));

		this.tfx.setFont(new Font("Arial", Font.PLAIN, 13));
		this.tfz.setFont(new Font("Arial", Font.PLAIN, 13));

		this.h1.setFont(new Font("Serif", Font.PLAIN, 18));

		this.addWindowFocusListener(new WindowFocusListener() {

			public void windowGainedFocus(final WindowEvent e) {

			}

			public void windowLostFocus(final WindowEvent e) {

				NewMembraneDialog.this.requestFocus();

			}

		});

		this.pack();

		final Dimension dim = this.getToolkit().getScreenSize();
		this.setLocation(dim.width / 2 - this.getWidth() / 2, dim.height / 2
				- this.getHeight() / 2);

		this.tfx.setValue(this.model.getSize().width);
		this.tfz.setValue(this.model.getSize().height);

		final JPanel p1 = new JPanel();
		this.ground1.add(p1);
		p1.setPreferredSize(new Dimension(
				340 - this.tf1.getPreferredSize().width - 20, this.tf1
						.getPreferredSize().height));

		final JPanel p2 = new JPanel();
		this.ground2.add(p2);
		p2.setPreferredSize(new Dimension(
				340 - this.tf2.getPreferredSize().width - 20, this.tf2
						.getPreferredSize().height));

		this.tf1.requestFocus();

		this.setResizable(false);
		this.setVisible(true);
		// goes on here when the dialog gets closed
		return this.out;

	}

	/**
	 * Returns the Value the Spinners are set to
	 * 
	 * @return
	 */
	public Dimension getSpinnerValues() {
		Dimension size;
		try {
			size = new Dimension(((Integer) this.tfx.getValue()).intValue(),
					((Integer) this.tfz.getValue()).intValue());
		} catch (final Exception e) {
			size = null;
		}
		return size;
	}

	/**
	 * Actions for the 3 Buttons in the Toolbar
	 */
	@SuppressWarnings("unchecked")
	public void actionPerformed(final ActionEvent arg0) {

		/*
		 * Accepts the Changes
		 */
		if (arg0.getSource() == this.okBtn) {

			this.model.setHeader(this.tf1.getText());
			this.model.setCode(this.tf2.getText());

			if (this.getSpinnerValues() != null)
				this.model.setSize(this.getSpinnerValues());

			this.out = NewMembraneDialog.APPROVE;
			this.dispose();
			return;
		}

		// or cancels the changes
		if (arg0.getSource() == this.cancelBtn) {
			this.dispose();
		}

	}
}
