/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.Authenticator;
import java.net.PasswordAuthentication;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.gui.HelpBtn;
import org.cellmicrocosmos.cm2.gui.MainGui;
import org.cellmicrocosmos.cm2.interfaces.GuiListener;


/**
 * A Dialog that lets the User set up connections preferences
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
@SuppressWarnings("serial")
public class SetUpConnectionsDialog extends JDialog implements ActionListener {

	JRadioButton radio1 = new JRadioButton("Direct connection");

	JRadioButton radio2 = new JRadioButton("Manual proxy configuration");

	JButton okBtn = new JButton("Ok");

	JButton cancelBtn = new JButton("Cancel");

	JButton testBtn = new JButton("Test Configuration");

	JTextField tflogin = new JTextField();

	JPasswordField tfpassword = new JPasswordField();

	JTextField tfhttpport = new JTextField();

	JTextField tfftpport = new JTextField();

	JTextField tfhttp = new JTextField();

	JTextField tfftp = new JTextField();

	JLabel l1 = new JLabel("Username ", SwingConstants.LEADING);

	JLabel l2 = new JLabel("Password ", SwingConstants.LEADING);

	JLabel l3 = new JLabel("HTTP  Proxy ", SwingConstants.LEADING);

	JLabel l4 = new JLabel("FTP    Proxy ", SwingConstants.LEADING);

	JLabel l5 = new JLabel("  Port ");

	JLabel l6 = new JLabel("  Port ");

	JPanel headerpanel = new JPanel();

	JPanel p2 = new JPanel();

	JPanel p3 = new JPanel();

	JPanel btnbar = new JPanel();

	JPanel p21 = new JPanel();

	JPanel p22 = new JPanel();

	JPanel p31 = new JPanel();

	JPanel p32 = new JPanel();

	JPanel p33 = new JPanel();

	JLabel header = new JLabel(" Internet Connection and Jmol Settings",
			SwingConstants.LEADING);

	JPanel p01 = new JPanel();

	JPanel p02 = new JPanel();

	JPanel main1 = new JPanel();

	JPanel main2 = new JPanel();

	JTabbedPane tab = new JTabbedPane();

	JTextArea ta = new JTextArea();

	JPanel linkp1 = new JPanel();

	JPanel linkp2 = new JPanel();

	JPanel linkp3 = new JPanel();

	JPanel linkp4 = new JPanel();

	JPanel linkp5 = new JPanel();
	
	JPanel linkp6 = new JPanel();
	
	JPanel linkp7 = new JPanel();

	JButton defaultBtn = new JButton("Restore Defaults");

	JTextField linktf1 = new JTextField();

	JTextField linktf2 = new JTextField();

	JTextField linktf3 = new JTextField();

	JTextField linktf4 = new JTextField();
	
	JTextField linktf5 = new JTextField();
	
	JTextField linktf6 = new JTextField();

	JLabel linkl1 = new JLabel("Download Link", SwingConstants.LEADING);

	JLabel linkl2 = new JLabel("Entries File", SwingConstants.LEADING);

	JLabel linkl3 = new JLabel("Preview Images", SwingConstants.LEADING);

	JLabel linkl4 = new JLabel("Info Page", SwingConstants.LEADING);
	
	JLabel linkl5 = new JLabel("Jmol Execution", SwingConstants.LEADING);
	
	JLabel linkl6 = new JLabel("APL@Voro Execution", SwingConstants.LEADING);

	private final GuiListener listener;

	/**
	 * Assembles the Gui
	 * 
	 * @param listener
	 */
	public SetUpConnectionsDialog(final GuiListener listener) {

		this.setTitle("Connection Settings");
		this.listener = listener;
		this.setAlwaysOnTop(true);
		this.setModal(true);
		this.setLocationByPlatform(true);
		this.setResizable(false);
		final ButtonGroup bg = new ButtonGroup();
		bg.add(this.radio1);
		bg.add(this.radio2);

		this.getContentPane().add(this.headerpanel, BorderLayout.NORTH);
		this.getContentPane().add(this.tab, BorderLayout.CENTER);
		this.getContentPane().add(this.btnbar, BorderLayout.SOUTH);

		this.tab.addTab("Connection", this.main1);
		this.tab.addTab("RCSB, Jmol & APL@Voro Links", this.main2);

		this.main1.setLayout(new BoxLayout(this.main1, BoxLayout.Y_AXIS));
		this.main2.setLayout(new BoxLayout(this.main2, BoxLayout.Y_AXIS));

		this.main1.add(this.p01);
		this.main1.add(this.p02);
		this.main1.add(new JPanel());
		this.main1.add(this.p2);
		this.main1.add(this.p3);
		this.main1.add(new JPanel());
		this.main1.add(new JPanel());
		this.main1.add(this.p33);

		this.p33.add(this.testBtn);

		this.p01.add(this.radio1);
		this.p02.add(this.radio2);
		this.p01.add(new JPanel());
		this.p02.add(new JPanel());

		this.headerpanel.add(this.header);

		this.p21.setLayout(new BoxLayout(this.p21, BoxLayout.X_AXIS));
		this.p21.add(this.l1);
		this.p21.add(this.tflogin);
		this.p21.add(new JPanel());

		this.p22.setLayout(new BoxLayout(this.p22, BoxLayout.X_AXIS));
		this.p22.add(this.l2);
		this.p22.add(this.tfpassword);
		this.p22.add(new JPanel());

		this.p2.setLayout(new BoxLayout(this.p2, BoxLayout.Y_AXIS));

		this.p2.add(this.p21);
		this.p2.add(new JPanel());
		this.p2.add(this.p22);

		this.p31.setLayout(new BoxLayout(this.p31, BoxLayout.X_AXIS));
		this.p31.add(this.l3);
		this.p31.add(this.tfhttp);
		this.p31.add(this.l5);
		this.p31.add(this.tfhttpport);

		this.p32.setLayout(new BoxLayout(this.p32, BoxLayout.X_AXIS));
		this.p32.add(this.l4);
		this.p32.add(this.tfftp);
		this.p32.add(this.l6);
		this.p32.add(this.tfftpport);

		this.p3.setLayout(new BoxLayout(this.p3, BoxLayout.Y_AXIS));

		this.p3.add(this.p31);
		this.p3.add(new JPanel());
		this.p3.add(this.p32);

		this.btnbar.setLayout(new BoxLayout(this.btnbar, BoxLayout.X_AXIS));
		this.btnbar.add(Box.createHorizontalGlue());
		this.btnbar.add(this.okBtn);
		this.btnbar.add(Box.createHorizontalStrut(10));
		this.btnbar.add(this.cancelBtn);
		this.btnbar.add(Box.createHorizontalGlue());
		this.btnbar.add(new HelpBtn(this));
		this.btnbar.add(Box.createHorizontalStrut(5));

		// headerpanel.setBackground(Color.white);

		this.header.setFont(new Font("Serif", Font.PLAIN, 18));

		this.p2.setBorder(BorderFactory.createTitledBorder("Authentification"));
		this.p3.setBorder(BorderFactory.createTitledBorder("Address"));

		this.p01.setLayout(new BoxLayout(this.p01, BoxLayout.X_AXIS));
		this.p02.setLayout(new BoxLayout(this.p02, BoxLayout.X_AXIS));

		this.headerpanel.setPreferredSize(new Dimension(520, 40));
		this.header.setPreferredSize(new Dimension(520, 30));

		this.tflogin.setPreferredSize(new Dimension(80, 20));
		this.tfpassword.setPreferredSize(new Dimension(80, 20));
		this.tfhttpport.setPreferredSize(new Dimension(50, 20));
		this.tfftpport.setPreferredSize(new Dimension(50, 20));
		this.tfhttp.setPreferredSize(new Dimension(120, 20));
		this.tfftp.setPreferredSize(new Dimension(120, 20));
		this.p01.setPreferredSize(new Dimension(100, 50));
		this.p02.setPreferredSize(new Dimension(100, 30));

		this.l1.setPreferredSize(new Dimension(65, 20));
		this.l2.setPreferredSize(new Dimension(65, 20));
		this.l3.setPreferredSize(new Dimension(65, 20));
		this.l4.setPreferredSize(new Dimension(65, 20));
		this.l5.setPreferredSize(new Dimension(30, 20));
		this.l6.setPreferredSize(new Dimension(30, 20));

		this.radio1.setToolTipText("Use a direct connection to the internet");
		this.radio2.setToolTipText("Use a proxy to connect to the internet");

		this.tflogin
				.setToolTipText("Enter Login name for proxy authentification (only if required)");
		this.tfpassword
				.setToolTipText("Enter Password for proxy authentification (only if required)");

		this.tfhttpport.setToolTipText("Enter Portnumber");
		this.tfftpport.setToolTipText("Enter Portnumber");

		this.tfhttp.setToolTipText("Enter Proxy Domain");
		this.tfftp.setToolTipText("Enter Proxy Domain");

		this.linkp1.setLayout(new BoxLayout(this.linkp1, BoxLayout.X_AXIS));
		this.linkp2.setLayout(new BoxLayout(this.linkp2, BoxLayout.X_AXIS));
		this.linkp3.setLayout(new BoxLayout(this.linkp3, BoxLayout.X_AXIS));
		this.linkp4.setLayout(new BoxLayout(this.linkp4, BoxLayout.X_AXIS));
		this.linkp5.setLayout(new BoxLayout(this.linkp5, BoxLayout.X_AXIS));
		this.linkp6.setLayout(new BoxLayout(this.linkp6, BoxLayout.X_AXIS));
		this.linkp7.setLayout(new BoxLayout(this.linkp7, BoxLayout.X_AXIS));

		this.linkp1.add(this.linkl1);
		this.linkp1.add(this.linktf1);

		this.linkp2.add(this.linkl2);
		this.linkp2.add(this.linktf2);

		this.linkp3.add(this.linkl3);
		this.linkp3.add(this.linktf3);

		this.linkp4.add(this.linkl4);
		this.linkp4.add(this.linktf4);
		
		this.linkp5.add(this.linkl5);
		this.linkp5.add(this.linktf5);
		
		this.linkp6.add(this.linkl6);
		this.linkp6.add(this.linktf6);

		this.linkp7.add(this.defaultBtn);

		this.main2.add(new JPanel());
		this.main2.add(this.ta);

		this.main2.add(this.linkp1);
		this.main2.add(this.linkp2);
		this.main2.add(this.linkp3);
		this.main2.add(this.linkp4);
		this.main2.add(Box.createVerticalStrut(10));
		this.main2.add(this.linkp5);
		this.main2.add(Box.createVerticalStrut(10));
		this.main2.add(this.linkp6);
		this.main2.add(Box.createVerticalStrut(10));
		this.main2.add(this.linkp7);
		this.main2.add(Box.createVerticalStrut(10));

		this.ta.setEnabled(false);
		this.ta.setDisabledTextColor(this.main1.getForeground());
		this.ta.setFont(this.linktf1.getFont());
		this.ta.setOpaque(false);
		this.ta
				.setText("  This section allows to change the links that are used to access the RCSB contents. The default\n"
						+ "  links should work as long as there were no changes made on the RCSB site.\n"
						+ "  If so, you should find out the new proper links and update them here. Otherwise, the\n"
						+ "  certain functions will not be available.\n\n\n"
						+ "  You may insert the keyword #PDBCODE# into the strings. It will be\n"
						+ "  replaced with the pdb code of the selected item when used\n");

		this.linkl1.setPreferredSize(new Dimension(80, 20));
		this.linkl2.setPreferredSize(new Dimension(80, 20));
		this.linkl3.setPreferredSize(new Dimension(80, 20));
		this.linkl4.setPreferredSize(new Dimension(80, 20));
		this.linkl5.setPreferredSize(new Dimension(80, 20));
		this.linkl6.setPreferredSize(new Dimension(80, 20));

		this.linktf1.setPreferredSize(new Dimension(260, 20));
		this.linktf2.setPreferredSize(new Dimension(260, 20));
		this.linktf3.setPreferredSize(new Dimension(260, 20));
		this.linktf4.setPreferredSize(new Dimension(260, 20));
		this.linktf5.setPreferredSize(new Dimension(260, 20));
		this.linktf6.setPreferredSize(new Dimension(260, 20));

		this.linkp1.setPreferredSize(new Dimension(320, 25));
		this.linkp2.setPreferredSize(new Dimension(320, 25));
		this.linkp3.setPreferredSize(new Dimension(320, 25));
		this.linkp4.setPreferredSize(new Dimension(320, 25));
		this.linkp5.setPreferredSize(new Dimension(320, 25));
		this.linkp6.setPreferredSize(new Dimension(320, 25));

		this.pack();

		this.radio1.addActionListener(this);
		this.radio2.addActionListener(this);
		this.okBtn.addActionListener(this);
		this.cancelBtn.addActionListener(this);
		this.testBtn.addActionListener(this);
		this.defaultBtn.addActionListener(this);

		this.p2.setBorder(BorderFactory.createTitledBorder("Authentification"));
		this.p3.setBorder(BorderFactory.createTitledBorder("Address"));

		this.testBtn.setFont(this.testBtn.getFont().deriveFont(Font.BOLD));

		if (DataPool.InternetSettings.proxyuse)
			this.radio2.doClick();
		else
			this.radio1.doClick();

		this.tflogin.setText(DataPool.InternetSettings.proxyLogin);
		this.tfpassword.setText(DataPool.InternetSettings.proxyPassword);
		this.tfhttpport.setText(DataPool.InternetSettings.proxyHTTPport);
		this.tfftpport.setText(DataPool.InternetSettings.proxyFTPport);
		this.tfhttp.setText(DataPool.InternetSettings.proxyHTTP);
		this.tfftp.setText(DataPool.InternetSettings.proxyFTP);

		this.linktf1.setText(DataPool.InternetSettings.rcsbdownloadLink);
		this.linktf2.setText(DataPool.InternetSettings.rcsbentriesLink);
		this.linktf3.setText(DataPool.InternetSettings.rcsbimageLink);
		this.linktf4.setText(DataPool.InternetSettings.rcsbLink);
		
		this.linktf5.setText(DataPool.JmolSettings.jmolExecutionLink);
		
		this.linktf6.setText(DataPool.APLVoroSettings.aplVoroExecutionLink);

		this.linktf1
				.setToolTipText("The link to the pdb file that should be downloaded");
		this.linktf2
				.setToolTipText("The link to the index file of the RCSB database");
		this.linktf3
				.setToolTipText("The link to the preview image used for an entry");
		this.linktf4.setToolTipText("The link to the info site of an entry");
		this.linktf5.setToolTipText("The java -jar Execution Path for Jmol");
		this.linktf6.setToolTipText("The Execution Path for APL@Voro");

		this.linktf1.setCaretPosition(0);
		this.linktf2.setCaretPosition(0);
		this.linktf3.setCaretPosition(0);
		this.linktf4.setCaretPosition(0);
		this.linktf5.setCaretPosition(0);
		this.linktf6.setCaretPosition(0);
	}

	public void actionPerformed(final ActionEvent arg0) {
		if (arg0.getSource() == this.radio1) {
			this.l1.setEnabled(false);
			this.l2.setEnabled(false);
			this.l3.setEnabled(false);
			this.l4.setEnabled(false);
			this.l5.setEnabled(false);
			this.l6.setEnabled(false);

			this.tflogin.setEnabled(false);
			this.tfpassword.setEnabled(false);
			this.tfhttpport.setEnabled(false);
			this.tfftpport.setEnabled(false);
			this.tfhttp.setEnabled(false);
			this.tfftp.setEnabled(false);

			final TitledBorder b1 = (TitledBorder) this.p2.getBorder();

			this.p2.setBorder(BorderFactory.createTitledBorder(b1.getBorder(),
					b1.getTitle(), b1.getTitleJustification(), b1
							.getTitlePosition(), b1.getTitleFont(),
					this.tflogin.getDisabledTextColor()));

			final TitledBorder b2 = (TitledBorder) this.p3.getBorder();

			this.p3.setBorder(BorderFactory.createTitledBorder(b2.getBorder(),
					b2.getTitle(), b2.getTitleJustification(), b2
							.getTitlePosition(), b2.getTitleFont(),
					this.tflogin.getDisabledTextColor()));
		}
		if (arg0.getSource() == this.radio2) {
			this.l1.setEnabled(true);
			this.l2.setEnabled(true);
			this.l3.setEnabled(true);
			this.l4.setEnabled(true);
			this.l5.setEnabled(true);
			this.l6.setEnabled(true);

			this.tflogin.setEnabled(true);
			this.tfpassword.setEnabled(true);
			this.tfhttpport.setEnabled(true);
			this.tfftpport.setEnabled(true);
			this.tfhttp.setEnabled(true);
			this.tfftp.setEnabled(true);

			this.p2.setBorder(BorderFactory
					.createTitledBorder("Authentification"));
			this.p3.setBorder(BorderFactory.createTitledBorder("Address"));

		}
		if (arg0.getSource() == this.cancelBtn) {

			this.dispose();
		}
		if (arg0.getSource() == this.defaultBtn) {
			this.linktf1.setText(CM2_CONSTANTS.IO.DEFAULT_DOWNLOAD_LINK);
			this.linktf2.setText(CM2_CONSTANTS.IO.DEFAULT_SEARCH_ENTRIES_LINK);
			this.linktf3.setText(CM2_CONSTANTS.IO.DEFAULT_PREVIEW_IMAGE_LINK);
			this.linktf4.setText(CM2_CONSTANTS.IO.DEFAULT_RCSB_SITE_LINK);
			this.linktf5.setText(CM2_CONSTANTS.IO.DEFAULT_JMOL_EXECUTION_LINK);
			this.linktf6.setText(CM2_CONSTANTS.IO.DEFAULT_APLVORO_EXECUTION_LINK);
		}
		if (arg0.getSource() == this.okBtn) {

			DataPool.InternetSettings.proxyLogin = this.tflogin.getText();
			final char[] chars = this.tfpassword.getPassword();
			String pw = "";
			for (int i = 0; i < chars.length; i++) {
				pw += chars[i];
			}
			DataPool.InternetSettings.proxyPassword = pw;
			DataPool.InternetSettings.proxyHTTPport = this.tfhttpport.getText();
			DataPool.InternetSettings.proxyFTPport = this.tfftpport.getText();
			DataPool.InternetSettings.proxyHTTP = this.tfhttp.getText();
			DataPool.InternetSettings.proxyFTP = this.tfftp.getText();

			DataPool.InternetSettings.proxyuse = this.radio2.isSelected();

			DataPool.InternetSettings.rcsbdownloadLink = this.linktf1.getText();
			DataPool.InternetSettings.rcsbentriesLink = this.linktf2.getText();
			DataPool.InternetSettings.rcsbimageLink = this.linktf3.getText();
			DataPool.InternetSettings.rcsbLink = this.linktf4.getText();
			
			DataPool.JmolSettings.jmolExecutionLink = this.linktf5.getText();
			
			DataPool.APLVoroSettings.aplVoroExecutionLink = this.linktf6.getText();
			MainGui.setupAnalyzeMembraneButton();

			if (this.listener != null)
				this.listener.doProxySettings();
			this.dispose();
		}
		if (arg0.getSource() == this.testBtn) {
			this.testIt();
		}

	}

	/**
	 * Tests the current settings and shows the results in an optionPane
	 * 
	 */
	private void testIt() {

		final Object[] origs = new Object[6];

		origs[0] = System.getProperties().get("proxySet");
		origs[1] = System.getProperties().get("ftpProxySet");
		origs[2] = System.getProperties().get("proxyPort");
		origs[3] = System.getProperties().get("ftpProxyPort");
		origs[4] = System.getProperties().get("proxyHost");
		origs[5] = System.getProperties().get("ftpProxyHost");

		if (this.radio2.isEnabled()) {
			System.getProperties().put("proxySet", "true");
			System.getProperties().put("ftpProxySet", "true");

			String login = this.tflogin.getText();
			char[] password = this.tfpassword.getPassword();

			if (login == null)
				login = "";
			if (password == null)
				password = new char[0];

			final String s1 = login;
			final char[] s2 = password;

			Authenticator.setDefault(new Authenticator() {
				@Override
				protected PasswordAuthentication getPasswordAuthentication() {
					return new PasswordAuthentication(s1, s2);
				}
			});

			System.getProperties().put("proxyPort", this.tfhttpport.getText());
			System.getProperties()
					.put("ftpProxyPort", this.tfftpport.getText());

			System.getProperties().put("proxyHost", this.tfhttp.getText());
			System.getProperties().put("ftpProxyHost", this.tfftp.getText());

		} else {
			System.getProperties().put("proxySet", "false");
			System.getProperties().put("ftpProxySet", "false");

			Authenticator.setDefault(new Authenticator() {
				protected PasswordAuthentication getPasswordAuthentication() {
					return new PasswordAuthentication(null, null);
				}
			});

			System.getProperties().put("proxyPort", "");
			System.getProperties().put("ftpProxyPort", "");

			System.getProperties().put("proxyHost", "");
			System.getProperties().put("ftpProxyHost", "");
		}

		String http = "OK";
		String ftp = "OK";

		try {
			final URL url = new URL(CM2_CONSTANTS.IO.HTTP_TEST_LINK);
			url.openStream();
		} catch (final Throwable e) {
			http = "FAILED";
		}

		try {
			final URL url = new URL(CM2_CONSTANTS.IO.FTP_TEST_LINK);
			url.openStream();
		} catch (final Throwable e) {
			ftp = "FAILED";
		}

		int messtype;
		if (http.equals("OK") && ftp.equals("OK"))
			messtype = JOptionPane.INFORMATION_MESSAGE;
		else
			messtype = JOptionPane.WARNING_MESSAGE;

		JOptionPane.showMessageDialog(null,
				"Test finished with the following results:\n\n"
						+ "HTTP access: " + http + "\n" + "FTP   access: "
						+ ftp, "Test Results", messtype);

		if (origs[0] != null)
			System.getProperties().put("proxySet", origs[0]);
		if (origs[1] != null)
			System.getProperties().put("ftpProxySet", origs[1]);
		if (origs[2] != null)
			System.getProperties().put("proxyPort", origs[2]);
		if (origs[3] != null)
			System.getProperties().put("ftpProxyPort", origs[3]);
		if (origs[4] != null)
			System.getProperties().put("proxyHost", origs[4]);
		if (origs[5] != null)
			System.getProperties().put("ftpProxyHost", origs[5]);

		this.listener.doProxySettings();

	}

}
