/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.elements;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Comparator;
import java.util.EventObject;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileView;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.gui.ColumnSorter;
import org.cellmicrocosmos.cm2.gui.HelpBtn;
import org.cellmicrocosmos.cm2.gui.MyFCAccessory;
import org.cellmicrocosmos.cm2.gui.dialogs.EditDBEntryDialog;
import org.cellmicrocosmos.cm2.interfaces.DatabaseViewListener;
import org.cellmicrocosmos.cm2.interfaces.EditDBEntryDialogListener;
import org.cellmicrocosmos.cm2.io.DatabaseIO;
import org.cellmicrocosmos.cm2.io.MembraneXmlReader;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.io.PdbReader_3D.PdbAttributes;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.cellmicrocosmos.cm2.misc.ConfigParser;
import org.jdom.Document;
import org.jdom.Element;


/**
 * The local database view. Manages and provides all possible actions for the
 * pdb file contents on the disc. Also provides options to add instances to the
 * membrane
 * 
 * @author Dingi dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
@SuppressWarnings("serial")
public class DatabaseView extends JPanel implements ActionListener,
		EditDBEntryDialogListener, ListSelectionListener, MouseMotionListener {

	// the file this DB currently works with
	private File currentFile;

	// The headers for the table columns in correct order
	private final String[] headers = new String[] { "", "Code", "Header",
			"Description", "Source", "Notes", "Filepath", "Models" };

	// identifiers for columns that are relevant for load/save (names of the xml
	// tags)
	private final String[] relevantContents = { "CODE", "HEADER", "TITLE",
			"SOURCE", "COMMENT", "FILE", "HASMODELS" };

	// the color of the Entries that are already in the current Membrane
	private final Color alreadyinTextColor = new Color(40, 140, 40);

	// the color of the Entries that are invalid
	private final Color invalidTextColor = new Color(200, 70, 70);

	// 4 possible return values for the validity check of a row
	private final int VALIDROW = 0;

	private final int EMPTYFILENAME = 1;

	private final int FILEDOESNOTEXIST = 2;

	private final int FILEISNOTVALID = 3;

	// a flag that determines wheter the a table should be sorted as-or
	// descending
	// toggles each time a table is sorted
	private boolean sortAscending = false;

	// the listener that triggers actions in the parent class
	private DatabaseViewListener listener;

	// the bar for the buttons at the bottom
	private final JToolBar controlpanel = new JToolBar();

	// the two tables
	private final JTable table1;

	private final JTable table2;

	// the 5 buttons in the toolbar
	private final JButton btn1 = new JButton();

	private final JButton btn2 = new JButton();

	// the popupMenu that appears when the user rightclicks on a table
	private final JPopupMenu pop = new JPopupMenu();

	// the items for the popupMenu
	private final JMenuItem item1, item3, item4, item6, item7, item8, item9,item10;
	

	private final JButton b1, b3, b4, b6, b7, b8, b9,b10;

	// the models for the tables that handle the data
	private DefaultTableModel tablemodel1, tablemodel2;

	// det. whether the options to add molecules to the Membrane should be
	// available
	private boolean activateAddOptions;

	private final JLabel namelabel = new JLabel();

	private final JRadioButton radio1 = new JRadioButton("PROTEINS");

	private final JRadioButton radio2 = new JRadioButton("LIPIDS");

	private final ButtonGroup bg = new ButtonGroup();

	private final JScrollPane scr1;

	public JScrollPane scr2;

	private RBRenderer rbrenderer;

	private RBEditor rbeditor;

	private RBRenderer rbrenderer2;

	private RBEditor rbeditor2;

	// private JLabel infolabel = new JLabel("", JLabel.LEADING);

	private final Font defaultTableFont;

	// stores information about the models used for each file
	public static Hashtable<String, Integer> ModelByFile = new Hashtable<String, Integer>();

	/**
	 * The constructor builds up the database Gui without setting any contents
	 * yet
	 */
	public DatabaseView() {

		this.b1 = new JButton();
		this.b3 = new JButton();
		this.b4 = new JButton();
		this.b6 = new JButton();
		this.b7 = new JButton();
		this.b8 = new JButton();
		this.b9 = new JButton();
		this.b10 = new JButton();

		this.radio1.setSelected(true);
		this.radio2.setForeground(Color.gray);

		this.radio1.addActionListener(this);
		this.radio2.addActionListener(this);

		this.controlpanel.add(this.radio1);
		this.controlpanel.add(this.radio2);

		this.namelabel.setBorder(BorderFactory.createRaisedBevelBorder());

		this.controlpanel.addSeparator(new Dimension(10, 0));
		this.controlpanel.addSeparator();
		this.controlpanel.addSeparator(new Dimension(10, 0));
		this.controlpanel.add(this.btn2);
		this.controlpanel.addSeparator(new Dimension(10, 0));

		this.controlpanel.addSeparator();
		this.controlpanel.addSeparator(new Dimension(10, 0));
		this.controlpanel.add(this.b3);
		this.controlpanel.addSeparator(new Dimension(5, 0));
		this.controlpanel.add(this.b4);
		this.controlpanel.addSeparator(new Dimension(5, 0));
		this.controlpanel.addSeparator();
		this.controlpanel.addSeparator(new Dimension(5, 0));
		this.controlpanel.add(this.b1);
		this.controlpanel.addSeparator(new Dimension(5, 0));
		this.controlpanel.add(this.b7);
		this.controlpanel.addSeparator(new Dimension(5, 0));
		this.controlpanel.add(this.b10);
		this.controlpanel.addSeparator(new Dimension(5, 0));
		this.controlpanel.addSeparator();
		this.controlpanel.addSeparator(new Dimension(5, 0));

		this.controlpanel.add(this.b6);
		this.controlpanel.addSeparator(new Dimension(5, 0));
		this.controlpanel.add(this.b8);
		this.controlpanel.addSeparator(new Dimension(5, 0));
		this.controlpanel.add(this.b9);
		this.controlpanel.addSeparator(new Dimension(5, 0));
		this.controlpanel.addSeparator();

		// infolabel.setFont(infolabel.getFont().deriveFont(Font.BOLD,
		// infolabel.getFont().getSize2D() - 1));
		// infolabel
		// .setForeground(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.ALT_TEXT_COLOR);

		// JPanel middle = new JPanel();
		// middle.add(infolabel);
		// controlpanel.add(middle);

		this.controlpanel.add(Box.createHorizontalGlue());

		this.controlpanel.add(this.btn1);
		this.controlpanel.addSeparator(new Dimension(10, 0));
		this.controlpanel.addSeparator();

		this.controlpanel.add(new HelpBtn(this));
		this.controlpanel.addSeparator(new Dimension(10, 0));
		this.controlpanel.add(this.namelabel);

		this.btn1.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_Rcsb"));
		this.btn2.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_Import"));
		this.b3.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_Remove"));

		this.btn1
				.setToolTipText("Download a PDB from the global RCSB Database");
		this.btn2.setToolTipText("Make a new Entry...");
		this.b3.setToolTipText("Remove selections from the active Table");

		this.btn1.addActionListener(this);
		this.btn2.addActionListener(this);

		this.item1 = new JMenuItem("  Edit Entry...");

		this.item3 = new JMenuItem("  Remove");
		this.item4 = new JMenuItem("  Delete");
		this.item6 = new JMenuItem("  View in JMol");
		this.item7 = new JMenuItem("  Change Model");
		this.item8 = new JMenuItem("  Browse File");
		this.item9 = new JMenuItem("  Open Structure Summary");
		this.item10 = new JMenuItem("  Manage Molecule Profiles");
		this.pop.add(this.item1);

		this.pop.add(this.item3);
		this.pop.add(this.item4);
		this.pop.add(this.item7);
		this.pop.add(this.item10);
		this.pop.addSeparator();
		this.pop.add(this.item8);
		this.pop.add(this.item6);
		this.pop.add(this.item9);

		this.item4.setForeground(Color.red.darker().darker());

		this.item1.addActionListener(this.createActionListenerForPopup());
		this.item3.addActionListener(this.createActionListenerForPopup());
		this.item4.addActionListener(this.createActionListenerForPopup());

		this.item6.addActionListener(this.createActionListenerForPopup());
		this.item7.addActionListener(this.createActionListenerForPopup());
		this.item8.addActionListener(this.createActionListenerForPopup());
		this.item9.addActionListener(this.createActionListenerForPopup());
		this.item10.addActionListener(this.createActionListenerForPopup());

		this.b1.addActionListener(this.createActionListenerForPopup());
		this.b3.addActionListener(this.createActionListenerForPopup());
		this.b4.addActionListener(this.createActionListenerForPopup());

		this.b6.addActionListener(this.createActionListenerForPopup());
		this.b7.addActionListener(this.createActionListenerForPopup());
		this.b8.addActionListener(this.createActionListenerForPopup());
		this.b9.addActionListener(this.createActionListenerForPopup());
		this.b10.addActionListener(this.createActionListenerForPopup());

		this.item1.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_EditEntry"));
		this.item3.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_Remove"));
		this.item4.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_Delete"));

		this.item6.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Frame_Molecule_View"));
		this.item7.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_ChangeModel"));
		this.item8.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_Browse"));
		this.item9.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_RCSB_small"));
		this.item10.setIcon(Cm2_IconDepot.getSpecifiedIcon("CG_Manage_Profiles"));
		this.b1.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_EditEntry"));

		this.b4.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_Delete"));

		this.b6.setIcon(Cm2_IconDepot.getSpecifiedIcon("Frame_Molecule_View"));
		this.b7.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_ChangeModel"));
		this.b8.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_Browse"));
		this.b9.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_RCSB_small"));
		this.b10.setIcon(Cm2_IconDepot.getSpecifiedIcon("CG_Manage_Profiles"));

		this.b1.setToolTipText("Open the Properties Dialog");

		this.b4
				.setToolTipText("<html>Delete the Entry's File from HD (only if possible)<br>"
						+ "and remove the Entry from the List</html>");

		this.b6.setToolTipText("Preview the Molecule in JMol");
		this.b7.setToolTipText("Change the active Model of this Entry");

		this.b8.setToolTipText("Browse in the text of the File");
		this.b9.setToolTipText("Open the referring RCSB Info Site");
		this.b10.setToolTipText("Create or edit Conformation/Granularity Profiles for this type");

		this.table1 = this.createNewTable();
		this.table2 = this.createNewTable();

		this.scr1 = new JScrollPane(this.table1);
		this.scr2 = new JScrollPane(this.table2);

		this.controlpanel.setFloatable(false);
		this.add(this.controlpanel, BorderLayout.NORTH);
		this.add(this.scr2, BorderLayout.SOUTH);
		this.add(this.scr1, BorderLayout.SOUTH);

		this.bg.add(this.radio1);
		this.bg.add(this.radio2);

		// creating new tablemodels here
		this.resetModels();

		// should always be inactive at first
		this.activateAddOptions(false);

		this.defaultTableFont = this.table1.getFont();
		this.updateButtonStates();

	}

	/**
	 * Overrides doLayout
	 */
	public void doLayout() {

		this.scr1.setPreferredSize(new Dimension(this.getWidth(), this
				.getHeight() - 40));
		this.scr2.setPreferredSize(new Dimension(this.getWidth(), this
				.getHeight() - 40));

		this.controlpanel.setPreferredSize(new Dimension(this.getWidth(),
				this.radio1.getPreferredSize().height));

		super.doLayout();

	}

	/**
	 * Configurates the Tables
	 */
	public void configureTables() {
		for (int i = 0; i < 2; i++) {

			final JTable table = this.getTable(i);

			// table.setSelectionBackground(new Color(215, 215, 225));
			table.setRowHeight(table.getRowHeight() + 2);
			// table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			// table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
			table.getTableHeader().setReorderingAllowed(false);
			table.getTableHeader().addMouseListener(this.createMouseListener());
			table.addMouseListener(this.createMouseListener());
			table.addKeyListener(this.createKeyListenerForTable());

			table.getColumnModel().getColumn(0).setMaxWidth(35);
			table.getColumnModel().getColumn(0).setMinWidth(35);

			table.getColumnModel().getColumn(1).setMinWidth(45);

			try {
				final String[] strings = ConfigParser.getValue("DB_Columns");

				for (int j = 1; j < table.getColumnCount(); j++) {

					final int size = Integer.parseInt(strings[j - 1]);
					table.getColumnModel().getColumn(j).setPreferredWidth(size);

				}
			} catch (final Exception e) {
				table.getColumnModel().getColumn(1).setPreferredWidth(45);
				table.getColumnModel().getColumn(2).setPreferredWidth(180);
				table.getColumnModel().getColumn(3).setPreferredWidth(300);
				table.getColumnModel().getColumn(4).setPreferredWidth(100);
				table.getColumnModel().getColumn(5).setPreferredWidth(200);
				table.getColumnModel().getColumn(6).setPreferredWidth(300);

			}

			table.getTableHeader().setPreferredSize(new Dimension(0, 20));

			table.setColumnSelectionAllowed(false);

			table.getSelectionModel().addListSelectionListener(this);

			for (int j = 0; j < table.getColumnCount(); j++) {
				table.getColumnModel().getColumn(j).setCellRenderer(
						new DefaultTableCellRenderer());
			}
		}

		this.table1.getTableHeader().getComponent(0).setBackground(
				CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_PROTEIN_COLOR);
		// table1
		// .setGridColor(CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULTPROTEINCOLOR);

		this.table2.getTableHeader().getComponent(0).setBackground(
				CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_LIPID_COLOR);
		// table2.setGridColor(CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULTLIPIDCOLOR);

		this.table1.addMouseMotionListener(this);

		this.rbeditor = new RBEditor(this.table1);
		this.rbrenderer = new RBRenderer();
		this.table1.getColumnModel().getColumn(0).setCellEditor(this.rbeditor);
		this.table1.getColumnModel().getColumn(0).setCellRenderer(
				this.rbrenderer);

		this.rbeditor2 = new RBEditor(this.table2);
		this.rbrenderer2 = new RBRenderer();
		this.table2.getColumnModel().getColumn(0).setCellEditor(this.rbeditor2);
		this.table2.getColumnModel().getColumn(0).setCellRenderer(
				this.rbrenderer2);

	}

	/**
	 * adds a row of data to a table
	 * 
	 * @param rowData
	 *            a vecot containing data for each column
	 * @param group
	 *            which table to add into
	 */
	@SuppressWarnings("unchecked")
	public void addRow(final Vector rowData, final int group) {
		DefaultTableModel model;
		if (group == 0)
			model = this.tablemodel1;
		else if (group == 1)
			model = this.tablemodel2;
		else
			return;

//		rowData.set(6,(String) rowData
//						.get(6));
		model.addRow(rowData);

		final JTable table = this.getTable(group);
		final Point p = new Point(0, table.getHeight());
		if (table.rowAtPoint(p) != -1)
			table.addRowSelectionInterval(table.rowAtPoint(p), table
					.rowAtPoint(p));

	}

	/**
	 * deletes a row from a table
	 * 
	 * @param row
	 *            the rownumber to delete
	 * @param group
	 *            which table
	 */
	private void deleteRow(final int row, final int group) {
		DefaultTableModel model;
		if (group == 0)
			model = this.tablemodel1;
		else if (group == 1)
			model = this.tablemodel2;
		else
			return;

		DatabaseView.ModelByFile.remove(new File((String) this.getValueAt(row,
				6, group)).getAbsolutePath());

		if (this.getRowCount(this.getTableOnFrontIndex()) > 0)
			model.removeRow(row);

	}

	/**
	 * replaces a row by a new one
	 * 
	 * @param rowData
	 *            the new Rowdata
	 * @param row
	 *            the index of the row to replace
	 * @param group
	 *            which table is meant
	 */
	private void setRowData(final Vector<Object> rowData, final int row, final int group) {

		DefaultTableModel model;
		if (group == 0)
			model = this.tablemodel1;
		else if (group == 1)
			model = this.tablemodel2;
		else
			return;

		for (int i = 0; i < rowData.size(); i++) {
			model.setValueAt(rowData.get(i), row, i);
		}

	}

	/**
	 * returns the data of a row
	 * 
	 * @param row
	 *            which row
	 * @param group
	 *            which group
	 * @return a vector containig the values of each column
	 */
	@SuppressWarnings("unchecked")
	private Vector<Object> getRowData(final int row, final int group) {

		DefaultTableModel model;
		if (group == 0)
			model = this.tablemodel1;
		else if (group == 1)
			model = this.tablemodel2;
		else
			return null;

		if (row > this.getRowCount(group))
			return null;

		final Vector vec = new Vector();
		for (int i = 0; i < model.getColumnCount(); i++) {
			vec.add(model.getValueAt(row, i));
		}
		return vec;
	}

	/**
	 * Sets the Value of a single cell
	 * 
	 * @param obj
	 *            the value to set
	 * @param row
	 *            the row
	 * @param column
	 *            the column
	 * @param group
	 *            which table
	 */
	@SuppressWarnings("unused")
	private void setValueAt(final Object obj, final int row, final int column,
			final int group) {
		DefaultTableModel model;
		if (group == 0)
			model = this.tablemodel1;
		else if (group == 1)
			model = this.tablemodel2;
		else
			return;

		model.setValueAt(obj, row, column);

	}

	/**
	 * Sets the Value of a single cell
	 * 
	 * @param row
	 *            the row
	 * @param column
	 *            the column
	 * @param group
	 *            which table
	 * @return the value
	 */
	private Object getValueAt(final int row, final int column, final int group) {

		DefaultTableModel model;

		if (group == 0)
			model = this.tablemodel1;
		else if (group == 1)
			model = this.tablemodel2;
		else
			return null;

		if (row >= model.getRowCount())
			return null;

		return model.getValueAt(row, column);

	}

	/**
	 * See how many rows are in a table recently
	 * 
	 * @param group
	 *            which table
	 * @return
	 */
	private int getRowCount(final int group) {

		if (group == 0)
			return this.tablemodel1.getRowCount();
		else if (group == 1)
			return this.tablemodel2.getRowCount();
		else
			return 0;

	}

	/**
	 * simply returns the tablemodel of the requested table
	 * 
	 * @param group
	 * @return
	 */
	private DefaultTableModel getTableModel(final int group) {

		if (group == 0)
			return this.tablemodel1;
		else if (group == 1)
			return this.tablemodel2;
		else
			return null;

	}

	/**
	 * returns the requested table
	 * 
	 * @param group
	 * @return
	 */
	private JTable getTable(final int group) {

		if (group == 0)
			return this.table1;
		else if (group == 1)
			return this.table2;
		else
			return null;

	}

	/**
	 * Create a DefaultTableModel that specifies the classtypes for the columns
	 * 
	 */
	private DefaultTableModel createNewModel() {
		return new DefaultTableModel() {
			
			public Class<?> getColumnClass(final int index) {

				// if (index == 0)
				// return Boolean.class;
				// else
				return super.getColumnClass(index);

			}

			// Make all Columns, except the first one uneditable
			public boolean isCellEditable(final int row, final int column) {

				if (column != 0)
					return false;
				else
					return true;
			}
			
		};

	}

	/**
	 * Returns a customized JTable that renders invalid Entries or Entries that
	 * are already in the Membrane differently
	 * 
	 * @return
	 */
	private JTable createNewTable() {
		return new JTable() {

			@Override
			public Component prepareRenderer(final TableCellRenderer renderer,
					final int rowIndex, final int vColIndex) {
				final Component c = super.prepareRenderer(renderer, rowIndex,
						vColIndex);

				final Object value = this.getValueAt(rowIndex, 7);

				if (value != null) {
					if (value.equals("Yes"))
						c.setFont(DatabaseView.this.defaultTableFont
								.deriveFont(Font.BOLD));
					else
						c.setFont(DatabaseView.this.defaultTableFont);
				}
				String tooltext;
				final int valid = DatabaseView.this.isValidRow(rowIndex,
						DatabaseView.this.getTableOnFrontIndex());
				if (valid == DatabaseView.this.VALIDROW) {
					if (DatabaseView.this.isRowAlreadyIn(rowIndex,
							DatabaseView.this.getTableOnFrontIndex())) {
						c.setForeground(DatabaseView.this.alreadyinTextColor);

					} else {
						c.setForeground(null);

					}
					if (this.getValueAt(rowIndex, vColIndex) != null
							&& vColIndex > 0) {
						final JComponent jc = (JComponent) c;
						tooltext = (String) this
								.getValueAt(rowIndex, vColIndex);

						if (DatabaseView.this.activateAddOptions
								&& DatabaseView.this.getTableOnFrontIndex() == 0)
							tooltext = tooltext
									+ "\n<font color=\"FFC800\">Hold down left mousebutton to start drag & drop!</font>";

						tooltext = "<html>" + tooltext.replaceAll("\n", "<br>")
								+ "</html>";
						jc.setToolTipText(tooltext);

					} else if (vColIndex > 0) {
						((JComponent) c).setToolTipText("");
					}

					
				} else if (valid == DatabaseView.this.EMPTYFILENAME) {

					((JComponent) c)
							.setForeground(DatabaseView.this.invalidTextColor);
					((JComponent) c)
							.setToolTipText("You have to select a file for this entry");
				} else if (valid == DatabaseView.this.FILEDOESNOTEXIST) {

					((JComponent) c)
							.setForeground(DatabaseView.this.invalidTextColor);
					if (this.getValueAt(rowIndex, 6) != null)
						((JComponent) c).setToolTipText("<html>The file "
								+ this.getValueAt(rowIndex, 6)
								+ "<br>cannot be resolved</html>");
				} else if (valid == DatabaseView.this.FILEISNOTVALID) {

					((JComponent) c)
							.setForeground(DatabaseView.this.invalidTextColor);
					((JComponent) c).setToolTipText("<html>The file "
							+ this.getValueAt(rowIndex, 6)
							+ "<br>is no valid pdb-file</html>");
				}

				return c;
			}

		};
	}

	/**
	 * Checks an Entry for validity
	 * 
	 * @param rowIndex
	 *            the rownumber to check
	 * @param group
	 *            which table
	 * @return an int depending on the results of the check. See the fields
	 *         above
	 */
	private int isValidRow(final int rowIndex, final int group) {

		final Object value = this.getValueAt(rowIndex, 6, group);
		if (value == null)
			return this.EMPTYFILENAME;
		final String filename = value.toString();
		if (filename.length() < 1)
			return this.EMPTYFILENAME;
		if (new File(filename).canRead() == false)
			return this.FILEDOESNOTEXIST;
		if (new File(filename).isFile() == false)
			return this.FILEISNOTVALID;
		if (filename.toLowerCase().endsWith("pdb") == false)
			return this.FILEISNOTVALID;
		else
			return this.VALIDROW;

	}

	/**
	 * Checks if an Entry is already in the currently opened Membrane
	 * 
	 * @param rowIndex
	 *            the row to check
	 * @param group
	 *            which table
	 * @return true if its already in, false otherwise
	 */
	private boolean isRowAlreadyIn(final int rowIndex, final int group) {
		if (DataPool.getMembraneModel() == null)
			return false;

		Object value = this.getValueAt(rowIndex, 1, group);
		if (value == null) {
			String name = null;
			try {
				name = Cm2_Toolkit.getSimpleName(new File((String) this
						.getValueAt(rowIndex, 6, group)));
			} catch (final Exception e) {

			}
			if (name == null)
				return false;
			else
				value = name;
		}

		if (group == 0)
			return DataPool.getMembraneModel().getProteinTypeAlreadyInside(
					value.toString());

		else
			return DataPool.getMembraneModel()
					.getExampleLipidTypeAlreadyInside(value.toString());

	}

	/**
	 * Opens a Dialog where the currently highlighted Entry can be edited
	 * 
	 */
	@SuppressWarnings("unchecked")
	public void editEntry() {
		final int rownumber = this.getTable(this.getTableOnFrontIndex())
				.getSelectedRow();
		final Vector rowdata = this.getRowData(rownumber, this
				.getTableOnFrontIndex());
		final EditDBEntryDialog dialog = new EditDBEntryDialog(this);
		final int val = dialog.askParameters(rowdata);

		// if user applies, the new rowdata is set to the specified row
		if (val == dialog.APPROVE) {

			final JTable table = this.getTable(dialog.getSelectedType());

			// if the type remains, the rowdata gets replaced in the table
			if (dialog.getSelectedType() == this.getTableOnFrontIndex()) {
				final Vector newrowdata = dialog.getResultingRowdata();

				try {
					if (new PdbReader_3D().checkForModels(new File(
							(String) newrowdata.get(6)))) {
						newrowdata.set(7, "Yes");
					} else {
						newrowdata.set(7, "No");
					}

				} catch (final Exception e) {
					newrowdata.set(7, null);
				}

				this.setRowData(newrowdata, rownumber, this
						.getTableOnFrontIndex());
				this.updateModelHashtable();
			}

			else {
				/*
				 * and if the the type of the Entry was changed, it gets moved
				 * into the other table
				 */
				this.deleteRow(rownumber, this.getTableOnFrontIndex());
				this.addRow(dialog.getResultingRowdata(), dialog
						.getSelectedType());

				final Point p = new Point(0, table.getHeight());
				table.addRowSelectionInterval(table.rowAtPoint(p), table
						.rowAtPoint(p));

			}

		}
	}

	/**
	 * Creates a customized MouseListener for the tables
	 * 
	 */
	private MouseListener createMouseListener() {

		return new MouseAdapter() {

			@SuppressWarnings("unchecked")
			public void mouseClicked(final MouseEvent arg0) {

				if (arg0.getSource() instanceof JTableHeader) {
					// sorts the column the user clicked
					if (arg0.getSource() == DatabaseView.this.getTable(
							DatabaseView.this.getTableOnFrontIndex())
							.getTableHeader()) {
						final int colnumber = DatabaseView.this.getTable(
								DatabaseView.this.getTableOnFrontIndex())
								.getTableHeader()
								.columnAtPoint(arg0.getPoint());

						final DefaultTableModel model = DatabaseView.this
								.getTableModel(DatabaseView.this
										.getTableOnFrontIndex());
						final Vector data = model.getDataVector();

						
						Collections.sort(data, colnumber>0?new ColumnSorter(colnumber,
								DatabaseView.this.sortAscending):new FirstRowColumnSorter(DatabaseView.this.sortAscending));
						
						final int[] ints = DatabaseView.this.getColumnSizes();
						model.fireTableStructureChanged();
						DatabaseView.this.setColumnSizes(ints);
						getTable(
								getTableOnFrontIndex()).getColumnModel().getColumn(0).setMaxWidth(35);
						getTable(
								getTableOnFrontIndex()).getColumnModel().getColumn(0).setMinWidth(35);
						DatabaseView.this.sortAscending = !DatabaseView.this.sortAscending;
					}
				}
				if (arg0.getSource() instanceof JTable) {

					final JTable table = (JTable) arg0.getSource();
					if (arg0.getButton() == MouseEvent.BUTTON1
							&& !(arg0.getModifiers() == CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.RIGHT_BUTTON_MODIFIER)) {
						// opens a dialog to change the Entry that the user
						// doubleclicked
						if (arg0.getClickCount() > 1) {

							if (table.getSelectedColumn() == 0)
								return;

							DatabaseView.this.editEntry();
						}
					} else if (arg0.getButton() == MouseEvent.BUTTON3
							|| (arg0.getModifiers() == CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.RIGHT_BUTTON_MODIFIER)) {
						/*
						 * if the user middle or rightclicks an Entry, the row
						 * gets selected and the popupMenu appears
						 */

						table.clearSelection();
						table
								.addRowSelectionInterval(table.rowAtPoint(arg0
										.getPoint()), table.rowAtPoint(arg0
										.getPoint()));

						DatabaseView.this.item4.setEnabled(true);

						DatabaseView.this.item6.setEnabled(true);
						DatabaseView.this.item7.setEnabled(true);
						DatabaseView.this.item8.setEnabled(true);
						DatabaseView.this.item9.setEnabled(true);

						DatabaseView.this.item1
								.setToolTipText("Open the Properties Dialog");

						DatabaseView.this.item3
								.setToolTipText("Remove the Entry from the List");
						DatabaseView.this.item4
								.setToolTipText("<html>Delete the Entry's File from HD (only if possible)<br>"
										+ "and remove the Entry from the List</html>");

						DatabaseView.this.item6
								.setToolTipText("Preview the Molecule in JMol (default model)");
						DatabaseView.this.item7
								.setToolTipText("Change the active Model of this Entry");

						DatabaseView.this.item8
								.setToolTipText("Browse in the text of the File");
						DatabaseView.this.item9
								.setToolTipText("Open the referring RCSB Info Site");
						DatabaseView.this.item10
						.setToolTipText("Create or edit Conformation/Granularity Profiles for this type");
						// dis- or enables some items dependent on the Entry's
						// state
						if (DatabaseView.this.isValidRow(
								table.getSelectedRow(), DatabaseView.this
										.getTableOnFrontIndex()) != DatabaseView.this.VALIDROW) {

							DatabaseView.this.item6.setEnabled(false);

							DatabaseView.this.item6
									.setToolTipText("Required File is not available!");

							DatabaseView.this.item7.setEnabled(false);

							DatabaseView.this.item7
									.setToolTipText("Required File is not available!");

							DatabaseView.this.item8.setEnabled(false);

							DatabaseView.this.item8
									.setToolTipText("Required File is not available!");
							
							DatabaseView.this.item10.setEnabled(false);

							DatabaseView.this.item10
									.setToolTipText("Required File is not available!");
						}

						if (DatabaseView.this.isRowAlreadyIn(table
								.getSelectedRow(), DatabaseView.this
								.getTableOnFrontIndex())) {
							DatabaseView.this.item4.setEnabled(false);
							DatabaseView.this.item4
									.setToolTipText("<html>Cannot delete Entry because the associated File<br>"
											+ "is currently used in the opened Membrane</html>");
						}

						if (DatabaseView.this.getRowData(
								table.getSelectedRow(),
								DatabaseView.this.getTableOnFrontIndex())
								.get(7).equals("Yes") == false) {
							DatabaseView.this.item7.setEnabled(false);

							DatabaseView.this.item7
									.setToolTipText("No Models Found!");
						}

						final int row = table.getSelectedRow();

						final Object code = DatabaseView.this.getRowData(row,
								DatabaseView.this.getTableOnFrontIndex())
								.get(1);

						if (code == null || ((String) code).length() != 4) {
							DatabaseView.this.item9.setEnabled(false);
						}
						DatabaseView.this.pop.show(table, arg0.getX(), arg0
								.getY());

					}
				}

			}

			// public void mouseEntered(MouseEvent arg0) {
			// if (arg0.getSource() == table1) {
			// if (activateAddOptions)
			// infolabel
			// .setText("Hold down left Mouse Button to start Drag n Drop");
			// }
			// }
			//
			// public void mouseExited(MouseEvent arg0) {
			// if (arg0.getSource() == table1) {
			// infolabel.setText("");
			// }
			// }

			public void mousePressed(final MouseEvent arg0) {
				if (!DatabaseView.this.isEnabled())
					return;
				if (arg0.getSource() == DatabaseView.this.getTable(0)) {

					if (arg0.getButton() == MouseEvent.BUTTON1
							&& !(arg0.getModifiers() == CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.RIGHT_BUTTON_MODIFIER)) {
						DatabaseView.this.startDragTimer(arg0);
					}
				}
			}

			public void mouseReleased(final MouseEvent arg0) {
				if (!DatabaseView.this.isEnabled())
					return;
				if (arg0.getSource() == DatabaseView.this.getTable(0)) {

					if (arg0.getButton() == MouseEvent.BUTTON1
							&& !(arg0.getModifiers() == CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.RIGHT_BUTTON_MODIFIER)) {
						DatabaseView.this.quitDnd(false);
					}
				}

			}

		};
	}

	/**
	 * Creates the ActionListener for the popupMenu
	 * 
	 * @return
	 */
	private ActionListener createActionListenerForPopup() {

		return new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {
				final JTable table = DatabaseView.this
						.getTable(DatabaseView.this.getTableOnFrontIndex());

				// opens the dialog to edit the Entry
				if (arg0.getSource() == DatabaseView.this.item1
						|| arg0.getSource() == DatabaseView.this.b1) {

					DatabaseView.this.editEntry();

					/*
					 * adds an Entry to the current Membrane by sending the
					 * rowdata through the listener
					 */
				} else if (arg0.getSource() == DatabaseView.this.item3
						|| arg0.getSource() == DatabaseView.this.b3) {
					final int group = DatabaseView.this.getTableOnFrontIndex();
					int[] todelete = DatabaseView.this
							.getSelections(DatabaseView.this
									.getTableOnFrontIndex());
					if (todelete.length < 1) {

						if (DatabaseView.this.getTable(group).getSelectedRow() != -1) {
							todelete = new int[] { DatabaseView.this.getTable(
									group).getSelectedRow() };
						} else {
							JOptionPane.showMessageDialog(null,
									"No Entry Selected!\n", "Not Possible",
									JOptionPane.ERROR_MESSAGE);
							return;
						}
					}

					for (int i = todelete.length - 1; i > -1; i--) {
						DatabaseView.this.deleteRow(todelete[i],
								DatabaseView.this.getTableOnFrontIndex());
					}

					/*
					 * deletes the file of the Entry, if the user applies in a
					 * confirm dialog that shows up before
					 */
				} else if (arg0.getSource() == DatabaseView.this.item4
						|| arg0.getSource() == DatabaseView.this.b4) {

					DatabaseView.this.getToolkit().beep();
					final int val = JOptionPane.showConfirmDialog(null,
							"Do you really want, if possible, to delete this Files "
									+ "from your Computer?",
							"Confirm Deleting of Selections",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.WARNING_MESSAGE);

					if (val == JOptionPane.YES_OPTION) {

						final int group = DatabaseView.this
								.getTableOnFrontIndex();
						int[] todelete = DatabaseView.this
								.getSelections(DatabaseView.this
										.getTableOnFrontIndex());
						if (todelete.length < 1) {

							if (DatabaseView.this.getTable(group)
									.getSelectedRow() != -1) {
								todelete = new int[] { DatabaseView.this
										.getTable(group).getSelectedRow() };
							} else {
								JOptionPane.showMessageDialog(null,
										"No Entry Selected!\n", "Not Possible",
										JOptionPane.ERROR_MESSAGE);
								return;
							}
						}

						for (int i = todelete.length - 1; i > -1; i--) {
							final Object rdta = DatabaseView.this.getRowData(
									todelete[i],
									DatabaseView.this.getTableOnFrontIndex())
									.get(6);
							if (rdta != null) {
								final File file = new File(rdta.toString());
								if (file.exists()) {
									if (!file.delete())
										file.deleteOnExit();
								}
							}

							DatabaseView.this.deleteRow(todelete[i],
									DatabaseView.this.getTableOnFrontIndex());

						}

					}

				} else if (arg0.getSource() == DatabaseView.this.item6
						|| arg0.getSource() == DatabaseView.this.b6) {

					final int row = table.getSelectedRow();

					final File file = new File(DatabaseView.this.getRowData(
							row, DatabaseView.this.getTableOnFrontIndex()).get(
							6).toString());

					if (DatabaseView.ModelByFile.get(file.getAbsolutePath()) != null)
						DatabaseView.this.listener.userWantToPreviewMolecule(
								file, DatabaseView.ModelByFile.get(
										file.getAbsolutePath()).intValue());
					else
						DatabaseView.this.listener.userWantToPreviewMolecule(
								file, new Integer(0));

				} else if (arg0.getSource() == DatabaseView.this.item7
						|| arg0.getSource() == DatabaseView.this.b7) {
					if (!DatabaseView.this.isEnabled())
						return;
					final int row = table.getSelectedRow();

					final File file = new File(DatabaseView.this.getRowData(
							row, DatabaseView.this.getTableOnFrontIndex()).get(
							6).toString());

					if (DatabaseView.ModelByFile.get(file.getAbsolutePath()) != null)
						DatabaseView.this.listener.userWantToChangeModel(file,
								DatabaseView.ModelByFile.get(
										file.getAbsolutePath()).intValue());
					else
						DatabaseView.this.listener.userWantToChangeModel(file,
								new Integer(0));
				} else if (arg0.getSource() == DatabaseView.this.item8
						|| arg0.getSource() == DatabaseView.this.b8) {

					final int row = table.getSelectedRow();

					final File file = new File(DatabaseView.this.getRowData(
							row, DatabaseView.this.getTableOnFrontIndex()).get(
							6).toString());
					DataPool.openPdbFileBrowser(file);
				} else if (arg0.getSource() == DatabaseView.this.item9) {

					final int row = table.getSelectedRow();

					final Object code = DatabaseView.this.getRowData(row,
							DatabaseView.this.getTableOnFrontIndex()).get(1);
					if (code == null)
						return;
					Cm2_Toolkit
							.openWebPageOnSystemBrowser(DataPool.InternetSettings.rcsbLink
									.replaceAll("#PDBCODE#", (String) code));
				}else if (arg0.getSource() == DatabaseView.this.item10
						|| arg0.getSource() == DatabaseView.this.b10) {
					
					final int row = table.getSelectedRow();

					final File file = new File(DatabaseView.this.getRowData(
							row, DatabaseView.this.getTableOnFrontIndex()).get(
							6).toString());
					listener.openMoleculeEditorFromDB(file,DatabaseView.this.getTableOnFrontIndex());
				}

			}

		};
	}

	/**
	 * Creates a KeyListener for the Table
	 * 
	 * @return
	 */
	private KeyListener createKeyListenerForTable() {

		return new KeyAdapter() {

			public void keyPressed(final KeyEvent arg0) {

				// do nothing if no row is in the table
				if (DatabaseView.this.getRowCount(DatabaseView.this
						.getTableOnFrontIndex()) < 1)
					return;

				// deletes the highlighted Entry from the list when DEL is
				// pressed
				if (arg0.getKeyCode() == KeyEvent.VK_DELETE) {
					DatabaseView.this.deleteRow(DatabaseView.this.getTable(
							DatabaseView.this.getTableOnFrontIndex())
							.getSelectedRow(), DatabaseView.this
							.getTableOnFrontIndex());
				}
				// if ENTER is pressed, the dialog for editing the Entry shows
				// up
				if (arg0.getKeyCode() == KeyEvent.VK_ENTER) {

					DatabaseView.this.editEntry();

				}
				if (arg0.getSource() == DatabaseView.this.table1
						&& arg0.getKeyCode() == KeyEvent.VK_ESCAPE) {

					DatabaseView.this.quitDnd(true);

				}

			}

		};
	}

	/**
	 * Tests if an Entry with the passed File already exists in both tables
	 * 
	 * @param file
	 * @return true if so, false otherwise
	 */
	public boolean dbContainsPdbFile(final File file) {

		for (int group = 0; group < 2; group++) {
			for (int i = 0; i < this.getRowCount(group); i++) {

				if (this.getValueAt(i, 6, group) == null)
					continue;

				if (new File((String) this.getValueAt(i, 6, group))
						.getAbsolutePath().equals(file.getAbsolutePath())) {

					return true;

				}
			}

		}
		return false;
	}

	/**
	 * Returns an array that contains the rownumbers of all Entries in the
	 * showing list which have their checkbox set on selected
	 * 
	 * @param group
	 *            which table
	 * @return
	 */
	private int[] getSelections(final int group) {

		final int[] ints = this.getTable(group).getSelectedRows();

		return ints;
	}

	/**
	 * sets new Models on the tables which means that they get completely
	 * cleared
	 * 
	 */
	public void resetModels() {

		this.tablemodel1 = this.createNewModel();
		this.tablemodel2 = this.createNewModel();

		this.table1.setModel(this.tablemodel1);
		this.table2.setModel(this.tablemodel2);

		this.tablemodel1.setColumnIdentifiers(this.headers);
		this.tablemodel2.setColumnIdentifiers(this.headers);

	}

	/**
	 * Returns the Sizes of each Column in an Array
	 * 
	 * @return
	 */
	private int[] getColumnSizes() {

		final TableColumnModel model = this.getTable(
				this.getTableOnFrontIndex()).getColumnModel();
		final int[] ints = new int[model.getColumnCount()];
		for (int i = 0; i < model.getColumnCount(); i++) {

			ints[i] = model.getColumn(i).getWidth();
		}
		return ints;
	}

	/**
	 * Sets the size of each Column according to the passed ints
	 * 
	 * @param ints
	 */
	private void setColumnSizes(final int[] ints) {

		final TableColumnModel model1 = this.table1.getColumnModel();

		model1.getColumn(0).setMaxWidth(18);
		model1.getColumn(0).setMinWidth(18);

		model1.getColumn(7).setMaxWidth(50);
		model1.getColumn(7).setMinWidth(50);

		model1.getColumn(1).setMinWidth(45);
		for (int i = 0; i < ints.length; i++) {
			if (model1.getColumn(i) != null && i > 0 && i != 7)
				model1.getColumn(i).setPreferredWidth(ints[i]);
		}
		model1.getColumn(0).setCellEditor(this.rbeditor);
		model1.getColumn(0).setCellRenderer(this.rbrenderer);

		final TableColumnModel model2 = this.table2.getColumnModel();

		model2.getColumn(0).setMaxWidth(18);
		model2.getColumn(0).setMinWidth(18);

		model2.getColumn(7).setMaxWidth(50);
		model2.getColumn(7).setMinWidth(50);

		model2.getColumn(1).setMinWidth(45);
		for (int i = 0; i < ints.length; i++) {
			if (model2.getColumn(i) != null && i > 0 && i != 7)
				model2.getColumn(i).setPreferredWidth(ints[i]);
		}
		model2.getColumn(0).setCellEditor(this.rbeditor2);
		model2.getColumn(0).setCellRenderer(this.rbrenderer2);

	}

	/**
	 * Just performs an update on the two tables. Should be called when another
	 * Membrane gets loaded in the application
	 */
	public void fireUpdateTablesOnMembraneModel() {
		this.tablemodel1.fireTableDataChanged();
		this.tablemodel2.fireTableDataChanged();
	}

	/**
	 * Set up the tables values according to a DOM-Tree in an Document that was
	 * produced in the Parser. Used when loading a Database.
	 * 
	 * @param doc
	 *            the DOM Document created from the parser
	 */
	@SuppressWarnings("unchecked")
	public void setContents(final Document doc) {

		this.resetModels();

		final List mainelements = doc.getRootElement().getChildren();

		// Reading through each Entry
		for (int i = 0; i < mainelements.size(); i++) {

			final List subelements = ((Element) mainelements.get(i))
					.getChildren();

			final Hashtable<String, String> hash = new Hashtable<String, String>();

			// Values are put into a hashtable first
			for (int j = 0; j < subelements.size(); j++) {

				final Element subelement = ((Element) subelements.get(j));

				if (subelement.getName() != null
						&& subelement.getValue() != null)
					hash.put(subelement.getName(), subelement.getValue());

			}

			// ignores Entries without the tag "GROUP"
			if (hash.get("GROUP") == null)
				continue;

			int group;

			if (hash.get("GROUP")
					.equals(MembraneXmlReader.TAGNAMES_MEMBRANE[0]))
				group = 0;
			else if (hash.get("GROUP").equals(
					MembraneXmlReader.TAGNAMES_MEMBRANE[1].substring(0, 6)))
				group = 1;
			else
				continue;

			final Vector rowdata = new Vector();

			// adding the boolean for the checkboxes
			rowdata.add(null);

			/*
			 * adding the Values in a rowdata Vector referring to the order of
			 * the relevantContents Array
			 */
			for (int k = 0; k < this.relevantContents.length; k++) {

				if (hash.get(this.relevantContents[k]) != null)
					rowdata.add(hash.get(this.relevantContents[k]));
				else
					rowdata.add(null);

			}

			this.addRow(rowdata, group);

			try {
				final Integer model = new Integer(hash.get("MODEL_USED"));
				DatabaseView.ModelByFile.put(new File(hash.get("FILE"))
						.getAbsolutePath(), model);
			} catch (final Exception e) {

			}
		}

		final int[] ints = this.getColumnSizes();

		this.setColumnSizes(ints);

		this.updateAllFileAttributes();

		this.tablemodel1.fireTableDataChanged();
		this.tablemodel2.fireTableDataChanged();
	}

	/**
	 * Prepares a Document for the Dom Parser to save and return it
	 * 
	 * @return a Document
	 */
	public Document getContents() {

		// creates an RootElement with our default name
		final Document doc = new Document();
		final Element rootelement = new Element(DatabaseIO.ROOTELEMENTNAME_DB);

		// Works through each of the two tables
		for (int group = 0; group < 2; group++) {

			// works through each row of the current table
			for (int i = 0; i < this.getRowCount(group); i++) {

				final Element mainelement = new Element("ENTRY");

				final Element groupelement = new Element("GROUP");
				if (group == 0)
					groupelement
							.setText(MembraneXmlReader.TAGNAMES_MEMBRANE[group]);
				else
					groupelement
							.setText(MembraneXmlReader.TAGNAMES_MEMBRANE[group]
									.substring(0, 6));

				mainelement.addContent(groupelement);

				final Vector<Object> rowData = this.getRowData(i, group);
				// go through each column of the current row
				for (int j = 0; j < rowData.size(); j++) {

					// leaving out all non-String or empty cells
					if (rowData.get(j) instanceof String == false)
						continue;
					else if (((String) rowData.get(j)).length() < 1)
						continue;

					final Element subelement = new Element(
							this.relevantContents[j - 1]);
					subelement.setText(rowData.get(j).toString());

					mainelement.addContent(subelement);

				}

				Integer model = -1;
				try {
					model = DatabaseView.ModelByFile.get(new File(
							(String) rowData.get(6)).getAbsolutePath());
				} catch (final Exception e) {

				}

				if (model != null) {
					final Element sub2 = new Element("MODEL_USED");
					sub2.setText("" + model.intValue());
					mainelement.addContent(sub2);
				} else if (rowData.get(7) != null
						&& rowData.get(7).equals("Yes")) {
					final Element sub2 = new Element("MODEL_USED");
					sub2.setText("" + 0);
					mainelement.addContent(sub2);
				}
				rootelement.addContent(mainelement);
			}

		}

		doc.setRootElement(rootelement);

		return doc;
	}

	/**
	 * Sets the listener to invoke Actions for Parent Classes
	 * 
	 * @param listener
	 */
	public void addDatabaseViewListener(final DatabaseViewListener listener) {
		this.listener = listener;

	}

	/**
	 * Returns the current File the DB is set to
	 * 
	 * @return
	 */
	public File getCurrentFile() {
		return this.currentFile;
	}

	/**
	 * Sets the current File. The Border around the DB Gui gets the File's
	 * simple name as Title
	 * 
	 * @param currentFile
	 */
	public void setCurrentFile(final File currentFile) {
		this.currentFile = currentFile;
		if (currentFile != null)
			this.namelabel.setText(Cm2_Toolkit.getSimpleName(currentFile));
		else
			this.namelabel.setText("");

	}

	/**
	 * Turn the Options to add an Entry to the Membrane on or off
	 * 
	 * @param b
	 */
	public void activateAddOptions(final boolean b) {
		this.activateAddOptions = b;

	}

	/**
	 * Pops up a Filechooser where the User may select a PDB File.
	 * 
	 * @return the file the user selects, null if cancelled
	 * @param dir
	 *            the file initially selected
	 */
	public File askPDBFile(final File dir) {

		final JFileChooser fc = new JFileChooser();

		// make sure user can only apply having ONE NON-DIRECTORY PDB file
		// selected
		fc.setMultiSelectionEnabled(false);
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setAcceptAllFileFilterUsed(false);

		// setting the selected file, if possible
		if (dir != null)

			fc.setSelectedFile(dir);

		// customizing a filefilter that accepts only .xml's
		final FileFilter filter = new FileFilter() {

			@Override
			public boolean accept(final File arg0) {
				if (arg0.getName().toLowerCase().endsWith(".pdb")
						|| arg0.isDirectory())
					return true;
				else
					return false;

			}

			@Override
			public String getDescription() {
				return ".pdb";
			}

		};

		/*
		 * customizing a fileview that returns icons according to the file's
		 * presence in the db
		 */
		final FileView fv = new FileView() {

			public Icon getIcon(final File arg0) {

				if (arg0.isDirectory())
					return null;

				if (DatabaseView.this.dbContainsPdbFile(arg0))
					return Cm2_IconDepot.getSpecifiedIcon("FC_Pdb_In");
				else
					return Cm2_IconDepot.getSpecifiedIcon("FC_Pdb_Ok");
			}
		};

		fc.setFileView(fv);

		fc.setAccessory(new MyFCAccessory(fc, 2, "Pdb Info"));

		// setting the filefilter
		fc.setFileFilter(filter);

		if (dir == null) {
			try {
				File filepath;

				/*
				 * trying to get the last used directory for the filechooser
				 * (only relevant if no init. selected file is passed
				 */

				filepath = new File(ConfigParser
						.getValue("last_PdbFiles_directory")[0]);

				// checks if the retrieved file is a valid directory
				if (!filepath.exists() || !filepath.isDirectory())
					throw new Exception();

				// ...and if so sets the filechoosers path
				fc.setCurrentDirectory(filepath);

			} catch (final Exception e) {

				// setting the default paths
				if (DataPool.IO.DEFAULTPDBDIRECTORY.exists())
					fc.setCurrentDirectory(DataPool.IO.DEFAULTPDBDIRECTORY);

			}
		}

		if (this.getTableOnFrontIndex() == 0)
			fc.setDialogTitle("Import Protein");
		else
			fc.setDialogTitle("Import Lipid");

		final int val = fc.showOpenDialog(null);
		if (val == JFileChooser.APPROVE_OPTION) {
			final File file = fc.getSelectedFile();

			// storing the new Path
			ConfigParser.deleteAllAt("last_PdbFiles_directory");
			ConfigParser.addValue("last_PdbFiles_directory", new File(file
					.getParent()).getAbsolutePath());
			return file;

		} else {
			// return null if user cancels
			return null;
		}

	}

	private File[] askPDBFiles() {

		final JFileChooser fc = new JFileChooser();

		fc.setMultiSelectionEnabled(true);
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setAcceptAllFileFilterUsed(false);

		// customizing a filefilter that accepts only .xml's
		final FileFilter filter = new FileFilter() {

			@Override
			public boolean accept(final File arg0) {
				if (arg0.getName().toLowerCase().endsWith(".pdb")
						|| arg0.isDirectory())
					return true;
				else
					return false;

			}

			@Override
			public String getDescription() {
				return ".pdb";
			}

		};

		/*
		 * customizing a fileview that returns icons according to the file's
		 * presence in the db
		 */
		final FileView fv = new FileView() {

			public Icon getIcon(final File arg0) {

				if (arg0.isDirectory())
					return null;

				if (DatabaseView.this.dbContainsPdbFile(arg0))
					return Cm2_IconDepot.getSpecifiedIcon("FC_Pdb_In");
				else
					return Cm2_IconDepot.getSpecifiedIcon("FC_Pdb_Ok");
			}
		};

		fc.setFileView(fv);

		fc.setAccessory(new MyFCAccessory(fc, 2, "Pdb Info"));

		// setting the filefilter
		fc.setFileFilter(filter);

		try {
			File filepath;

			/*
			 * trying to get the last used directory for the filechooser (only
			 * relevant if no init. selected file is passed
			 */

			filepath = new File(ConfigParser
					.getValue("last_PdbFiles_directory")[0]);

			// checks if the retrieved file is a valid directory
			if (!filepath.exists() || !filepath.isDirectory())
				throw new Exception();

			// ...and if so sets the filechoosers path
			fc.setCurrentDirectory(filepath);

		} catch (final Exception e) {

			// setting the default paths
			if (DataPool.IO.DEFAULTPDBDIRECTORY.exists())
				fc.setCurrentDirectory(DataPool.IO.DEFAULTPDBDIRECTORY);

		}

		if (this.getTableOnFrontIndex() == 0)
			fc.setDialogTitle("Import Proteins");
		else
			fc.setDialogTitle("Import Lipids");

		final int val = fc.showOpenDialog(null);
		if (val == JFileChooser.APPROVE_OPTION) {
			final File[] files = fc.getSelectedFiles();

			ConfigParser.deleteAllAt("last_PdbFiles_directory");
			ConfigParser.addValue("last_PdbFiles_directory", fc
					.getCurrentDirectory().getAbsolutePath());
			return files;

		} else {
			// return null if user cancels
			return null;
		}

	}

	/**
	 * Possible clicks on one of the 4 DB-Toolbar Buttons are handled here
	 */
	@SuppressWarnings("unchecked")
	public void actionPerformed(final ActionEvent arg0) {

		if (arg0.getSource() == this.radio1) {

			this.scr1.setVisible(true);
			this.scr2.setVisible(false);
			this.radio1.setForeground(Color.black);
			this.radio2.setForeground(Color.gray);
			this.doLayout();
			this.updateButtonStates();
		}
		if (arg0.getSource() == this.radio2) {

			this.scr2.setVisible(true);
			this.scr1.setVisible(false);

			this.radio2.setForeground(Color.black);
			this.radio1.setForeground(Color.gray);
			this.doLayout();
			this.updateButtonStates();
		}

		/*
		 * opens the search window for downloading PDB's
		 */
		if (arg0.getSource() == this.btn1) {
			if (this.listener != null)
				this.listener.userOpensGlobalDBView();
		}
		/*
		 * Makes a new Entry in the DB
		 */
		else if (arg0.getSource() == this.btn2) {

			// retrieve a file...
			final File[] files = this.askPDBFiles();

			if (files != null) {

				final Vector<File> outtakes = new Vector<File>();

				for (int i = 0; i < files.length; i++) {

					if (this.dbContainsPdbFile(files[i])
							&& CM2_CONSTANTS.MISC.ALLOW_MULTIPLE_ENTRIES_IN_DB == false) {
						outtakes.addElement(files[i]);
						continue;
					}

					Vector rowdata = this.getEmptyRowData(files[i]);

					try {
						final PdbReader_3D reader = new PdbReader_3D();
						final PdbAttributes attrs = reader
								.readAttributes(files[i]);

						String code = attrs.getCode();
						if (code == null || code.length() < 1) {
							code = Cm2_Toolkit.getSimpleName(files[i]);
						}
						
						rowdata.set(0, null);
						rowdata.set(1, code);
						rowdata.set(2, attrs.getHeader());
						rowdata.set(3, attrs.getTitles());
						rowdata.set(4, attrs.getSpecies());
						rowdata.set(5, null);

					} catch (final IOException e) {
						rowdata = this.getEmptyRowData(files[i]);
					}
					try {
						if (new PdbReader_3D().checkForModels(new File(
								(String) rowdata.get(6)))) {
							rowdata.set(7, "Yes");
						} else {
							rowdata.set(7, "No");
						}
					} catch (final Exception e) {
						rowdata.set(7, null);
					}

					this.addRow(rowdata, this.getTableOnFrontIndex());

				}

				this.updateModelHashtable();

				if (this.getTableOnFrontIndex() == 0)
					this.radio1.doClick();
				else
					this.radio2.doClick();

				if (!outtakes.isEmpty()) {

					String s = "\n";
					for (int i = 0; i < outtakes.size(); i++) {
						s += outtakes.get(i).getName() + "\n";
					}
					String pre = "The file";
					if (outtakes.size() > 1)
						pre = "The files";
					String post = "has already been added to the current Database!";
					if (outtakes.size() > 1)
						post = "have already been added to the current Database!";
					JOptionPane.showMessageDialog(this, pre + s + post,
							"Information", JOptionPane.INFORMATION_MESSAGE);
				}

			}

		}

	}

	private void updateModelHashtable() {

		for (int i = 0; i < this.table1.getRowCount(); i++) {
			if (this.getValueAt(i, 7, 0) != null
					&& this.getValueAt(i, 7, 0).equals("Yes")) {
				final Integer integer = DatabaseView.ModelByFile.get(new File(
						(String) this.getValueAt(i, 6, 0)).getAbsolutePath());
				if (integer == null || integer.intValue() < 1)
					DatabaseView.ModelByFile.put(new File((String) this
							.getValueAt(i, 6, 0)).getAbsolutePath(),
							new Integer(1));
			}
		}

		for (int i = 0; i < this.table2.getRowCount(); i++) {
			if (this.getValueAt(i, 7, 1) != null
					&& this.getValueAt(i, 7, 1).equals("Yes")) {
				final Integer integer = DatabaseView.ModelByFile.get(new File(
						(String) this.getValueAt(i, 6, 1)).getAbsolutePath());
				if (integer == null || integer.intValue() < 1)
					DatabaseView.ModelByFile.put(new File((String) this
							.getValueAt(i, 6, 1)).getAbsolutePath(),
							new Integer(1));
			}
		}

	}

	/**
	 * Returns which table is currently on Front
	 */
	public int getTableOnFrontIndex() {

		if (this.radio1.isSelected())
			return 0;
		else
			return 1;
	}

	/**
	 * Creates and returns a new rowdata that is empty except for the path of
	 * the passed file
	 * 
	 * @param file
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Vector getEmptyRowData(final File file) {
		final Vector rowdata = new Vector();

		for (int i = 0; i < this.getTable(0).getColumnCount(); i++) {
			if (i != 6)
				rowdata.add(null);
			else
				rowdata.add(file.getAbsolutePath());
		}

		return rowdata;
	}

	/**
	 * Stores the sizes of the columns in the Config
	 * 
	 */
	public void storeSizes() {
		final JTable table = this.getTable(this.getTableOnFrontIndex());
		ConfigParser.deleteAllAt("DB_Columns");
		for (int k = 1; k < table.getColumnCount(); k++) {

			ConfigParser.addValue("DB_Columns", ""
					+ table.getColumnModel().getColumn(k).getWidth());
		}

	}

	/**
	 * Returns all rowdatas for Proteins
	 * 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Vector<String>[] getAllProteins() {
		final int rowcount = this.getRowCount(0);
		final Vector<Vector> v = new Vector<Vector>();
		for (int i = 0; i < rowcount; i++) {
			final Vector vec = this.getRowData(i, 0);
			vec.remove(0);
			v.add(vec);
		}
		return v.toArray(new Vector[0]);
	}

	/**
	 * Returns all rowdatas for Lipids
	 * 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Vector<String>[] getAllLipids() {
		final int rowcount = this.getRowCount(1);
		final Vector<Vector> v = new Vector<Vector>();
		for (int i = 0; i < rowcount; i++) {
			final Vector vec = this.getRowData(i, 0);
			vec.remove(0);
			v.add(vec);
		}
		return v.toArray(new Vector[0]);
	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class RBEditor extends AbstractCellEditor implements
			TableCellEditor, ActionListener {
		final JButton btn = new JButton();

		private final JTable table;

		private int row = -1;

		public RBEditor(final JTable table) {
			this.table = table;
			this.btn.addActionListener(this);
			this.btn.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_IntoMembrane"));
		}

		public Component getTableCellEditorComponent(final JTable table,
				final Object value, final boolean isSelected, final int row,
				final int column) {
			this.row = row;
			if (row == -1)
				return this.btn;
			this.btn
					.setEnabled(DatabaseView.this.isValidRow(row,
							DatabaseView.this.getTableOnFrontIndex()) == DatabaseView.this.VALIDROW
							&& DatabaseView.this.activateAddOptions);
			return this.btn;
		}

		public boolean isCellEditable(final EventObject evt) {

			return true;

		}

		// Returns the spinners current value.
		public Object getCellEditorValue() {
			if (this.row == -1)
				return this.btn;
			this.btn
					.setEnabled(DatabaseView.this.isValidRow(this.row,
							DatabaseView.this.getTableOnFrontIndex()) == DatabaseView.this.VALIDROW
							&& DatabaseView.this.activateAddOptions);
			return this.btn;
		}

		public void actionPerformed(final ActionEvent arg0) {
			if (!DatabaseView.this.isEnabled())
				return;
			if (arg0.getSource() instanceof JButton) {

				String code = null;
				String file = null;
				if (this.row != -1) {
					code = (String) ((DefaultTableModel) this.table.getModel())
							.getValueAt(this.row, 1);

					file = (String) ((DefaultTableModel) this.table.getModel())
							.getValueAt(this.row, 6);

				}
				if (code == null || code.length() < 1) {
					code = Cm2_Toolkit.getSimpleName(new File(file));
				}

				int modelnumber = -1;
				try {
					modelnumber = DatabaseView.ModelByFile.get(
							new File(file).getAbsolutePath()).intValue();
				} catch (final Exception e) {

				}
				if (this.table == DatabaseView.this.table1) {

					this.popUpDomainSelection((JButton) arg0.getSource(), 0,
							code, file, modelnumber);

				} else if (this.table == DatabaseView.this.table2) {

					this.popUpDomainSelection((JButton) arg0.getSource(), 1,
							code, file, modelnumber);

				} else
					return;

			}

		}

		private void popUpDomainSelection(final JButton button, final int mode,
				final String code, final String file, final int modelnumber) {

			final MembraneModel model = DataPool.getMembraneModel();

			if (model.countMicroDomains() == 0) {
				if (mode == 0)
					DatabaseView.this.listener.userAddsProtein(code, file,
							modelnumber, 0);
				else {

					final JPopupMenu popup = new JPopupMenu();

					final JMenuItem item11 = new JMenuItem("EC / IC");
					final JMenuItem item22 = new JMenuItem("EC only");
					final JMenuItem item33 = new JMenuItem("IC only");

					popup.add(item11);
					popup.add(item22);
					popup.add(item33);

					final ActionListener a = new ActionListener() {

						public void actionPerformed(final ActionEvent arg0) {
							final JMenuItem item = (JMenuItem) arg0.getSource();
							if (item.getText().equals("EC / IC"))
								DatabaseView.this.listener.userAddsLipid(code,
										file, modelnumber, 0, -1);
							else if (item.getText().equals("EC only"))
								DatabaseView.this.listener.userAddsLipid(code,
										file, modelnumber, 0, 0);
							else
								DatabaseView.this.listener.userAddsLipid(code,
										file, modelnumber, 0, 1);
						}

					};

					item11.addActionListener(a);
					item22.addActionListener(a);
					item33.addActionListener(a);

					final boolean ECin = DataPool.getMembraneModel()
							.containsExampleLipidExtra(code, 0,
									DataPool.getCurrentLayerID())
							|| !DataPool.getMembraneModel()
									.canAddMoreExampleLipidsExtra(0,
											DataPool.getCurrentLayerID());
					final boolean ICin = DataPool.getMembraneModel()
							.containsExampleLipidIntra(code, 0,
									DataPool.getCurrentLayerID())
							|| !DataPool.getMembraneModel()
									.canAddMoreExampleLipidsIntra(0,
											DataPool.getCurrentLayerID());
					if (ECin || ICin)
						item11.setEnabled(false);

					if (ECin)
						item22.setEnabled(false);

					if (ICin)
						item33.setEnabled(false);

					popup.show(button, 0, 0);

				}

			} else {

				final JPopupMenu popup = new JPopupMenu();
				final JPanel p = new JPanel();
				final JLabel label = new JLabel("Add to...");
				p.add(label);

				label.setFont(label.getFont().deriveFont(
						label.getFont().getSize2D() + 3));

				p.setBackground(label.getBackground().darker());
				popup.add(p);

				final MouseAdapter ma1 = new MouseAdapter() {

					public void mouseEntered(final MouseEvent arg0) {
						for (int i = 0; i < popup.getComponentCount(); i++) {
							if (popup.getComponent(i) == arg0.getSource()) {
								DatabaseView.this.listener
										.highlightMicroDomain(i - 1);

								break;
							}
						}

					}

				};

				if (mode == 0) {
					final JMenuItem[] items = new JMenuItem[model
							.countMicroDomains() + 1];

					final ActionListener al = new ActionListener() {

						public void actionPerformed(final ActionEvent arg0) {
							for (int i = 0; i < popup.getComponentCount(); i++) {
								if (popup.getComponent(i) == arg0.getSource()) {
									DatabaseView.this.listener.userAddsProtein(
											code, file, modelnumber, i - 1);

									break;
								}
							}

						}

					};

					items[0] = new JMenuItem("Default Area");
					items[0]
							.setFont(items[0].getFont().deriveFont(Font.ITALIC));
					items[0].addActionListener(al);
					items[0].addMouseListener(ma1);

					popup.add(items[0]);

					final int layerID = DataPool.getCurrentLayerID();
					int c = 0;
					for (int i = 0; i < model.countMicroDomains(); i++) {

						if (model.getMicroDomain(i).layerId != layerID)
							continue;
						c++;
						items[c] = new JMenuItem("Domain "
								+ model.getMicroDomain(i).id);
						items[c].addActionListener(al);
						items[c].addMouseListener(ma1);

						popup.add(items[c]);
					}
				} else {

					final JMenu[] menus = new JMenu[model.countMicroDomains() + 1];

					final ActionListener a2 = new ActionListener() {

						public void actionPerformed(final ActionEvent arg0) {

							int membraneside;
							final JMenuItem source = (JMenuItem) arg0
									.getSource();
							if (source.getText().equals("EC / IC"))
								membraneside = -1;
							else if (source.getText().equals("EC only"))
								membraneside = 0;
							else
								membraneside = 1;

							for (int i = 0; i < popup.getComponentCount(); i++) {
								if (popup.getComponent(i) instanceof JMenu) {
									final JMenu menu = (JMenu) popup
											.getComponent(i);
									for (int j = 0; j < menu.getItemCount(); j++) {
										if (menu.getItem(j) == arg0.getSource()) {
											DatabaseView.this.listener
													.userAddsLipid(
															code,
															file,
															modelnumber,
															menu
																	.getText()
																	.startsWith(
																			"Default") ? 0
																	: Integer
																			.parseInt(menu
																					.getText()
																					.replaceAll(
																							"Domain",
																							"")
																					.trim()),
															membraneside);
											break;
										}
									}

								}
							}

						}

					};

					menus[0] = new JMenu("Default Area");
					menus[0]
							.setFont(menus[0].getFont().deriveFont(Font.ITALIC));

					final JMenuItem item1 = new JMenuItem("EC / IC");
					final JMenuItem item2 = new JMenuItem("EC only");
					final JMenuItem item3 = new JMenuItem("IC only");

					menus[0].add(item1);
					menus[0].add(item2);
					menus[0].add(item3);

					menus[0].addMouseListener(ma1);

					item1.addActionListener(a2);
					item2.addActionListener(a2);
					item3.addActionListener(a2);

					final boolean ECin = DataPool.getMembraneModel()
							.containsExampleLipidExtra(code, 0,
									DataPool.getCurrentLayerID())
							|| !DataPool.getMembraneModel()
									.canAddMoreExampleLipidsExtra(0,
											DataPool.getCurrentLayerID());
					final boolean ICin = DataPool.getMembraneModel()
							.containsExampleLipidIntra(code, 0,
									DataPool.getCurrentLayerID())
							|| !DataPool.getMembraneModel()
									.canAddMoreExampleLipidsIntra(0,
											DataPool.getCurrentLayerID());
					if (ECin || ICin)
						item1.setEnabled(false);

					if (ECin)
						item2.setEnabled(false);

					if (ICin)
						item3.setEnabled(false);

					popup.add(menus[0]);

					final int layerID = DataPool.getCurrentLayerID();
					int c = 0;
					for (int i = 0; i < model.countMicroDomains(); i++) {

						if (model.getMicroDomain(i).layerId != layerID)
							continue;
						c++;

						menus[c] = new JMenu("Domain "
								+ model.getMicroDomain(i).id);
						final JMenuItem item11 = new JMenuItem("EC / IC");
						final JMenuItem item22 = new JMenuItem("EC only");
						final JMenuItem item33 = new JMenuItem("IC only");

						menus[c].add(item11);
						menus[c].add(item22);
						menus[c].add(item33);

						menus[c].addMouseListener(ma1);

						item11.addActionListener(a2);
						item22.addActionListener(a2);
						item33.addActionListener(a2);

						final boolean ECin2 = DataPool.getMembraneModel()
								.containsExampleLipidExtra(code, i + 1,
										DataPool.getCurrentLayerID())
								|| !DataPool.getMembraneModel()
										.canAddMoreExampleLipidsExtra(i + 1,
												DataPool.getCurrentLayerID());
						final boolean ICin2 = DataPool.getMembraneModel()
								.containsExampleLipidIntra(code, i + 1,
										DataPool.getCurrentLayerID())
								|| !DataPool.getMembraneModel()
										.canAddMoreExampleLipidsIntra(i + 1,
												DataPool.getCurrentLayerID());
						if (ECin2 || ICin2)
							item11.setEnabled(false);

						if (ECin2)
							item22.setEnabled(false);

						if (ICin2)
							item33.setEnabled(false);

						if (ECin2 && ICin2)
							menus[i + 1].setEnabled(false);
						// items[i+1].setBorder(BorderFactory.createLineBorder(model.getMicroDomain(i).getColor(),2));
						popup.add(menus[c]);
					}
				}

				label.setPreferredSize(new Dimension(
						popup.getPreferredSize().width, label
								.getPreferredSize().height + 4));
				popup.show(button, 0, 0);
			}

		}

	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
	 * 
	 */
	private class RBRenderer implements TableCellRenderer {

		public Component getTableCellRendererComponent(final JTable table,
				final Object color, final boolean isSelected,
				final boolean hasFocus, final int row, final int column) {

			final JButton btn = new JButton();

			String text = "Add this Type to the Model";
				
			btn.setEnabled(DatabaseView.this.isValidRow(row, DatabaseView.this
					.getTableOnFrontIndex()) == DatabaseView.this.VALIDROW
					&& DatabaseView.this.activateAddOptions);

			// choose the button image for the actual row
			if ( (table.getValueAt(row, 1)!=null) && (table == DatabaseView.this.table1) ) {
				String actualProtein = table.getValueAt(row, 1).toString();
				
				// OPM button
				if (actualProtein.contains(CM2_CONSTANTS.IO.OPM_FILENAME_ADDON) || // this line is added support custom OPM files
						(actualProtein.contains(CM2_CONSTANTS.IO.OPM_FILENAME_ADDON) &&
						DataPool.getMifHandler().isElementOfOPM(actualProtein.replace(CM2_CONSTANTS.IO.OPM_FILENAME_ADDON, "")))) {
					btn.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_OPM"));
					text = "Add a new Instance to current Model. This is an OPM file with alignment info.";
				// PDBTM button
				} else if (DataPool.getMifHandler().isElementOfPDBTM(actualProtein.toLowerCase())) {
					btn.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_PDBTM"));
					text = "Add a new Instance to current Model. For this file PDBTM alignment info exist.";
				// Standard button
				}  else {
					btn.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_IntoMembrane"));
					text = "Add a new Instance to current Model.";
				}
			// Standard button
			} else {
				btn.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_IntoMembrane"));
			}
			
			btn.setToolTipText(text);

			return btn;
		}

	}

	/**
	 * Add a protein or lipid to the database. 
	 * The value is not added if the PDB file is already contained in the database.
	 * @param file PDB file path
	 * @param group protein: 0, lipid: 1
	 */
	public void addFromRSCB(final File file, int group) {
		
		// check if the PDB file is already located in the database
		if (!dbContainsPdbFile(file)) {
			
			final JTable table = this.getTable(0);
			table.clearSelection();
			this.addRow(this.getEmptyRowData(file), group);

			this.updateAllFileAttributes();
		
		} else {
			
			JOptionPane
			.showMessageDialog(
					null,
					"The file "+file.toString()+" is already contained in the databases. It is updated now.", "File already in DB",
					JOptionPane.WARNING_MESSAGE);
			
		}
		

	}

	/**
	 * Add a protein to the database.
	 * @param file PDB file path
	 */
	public void addFromRSCB(final File file) {
		
		this.addFromRSCB(file, MembranePackingAlgorithm.PROTEINTYPE);
		
	}

	public void valueChanged(final ListSelectionEvent arg0) {

		SwingUtilities.invokeLater(new Thread() {
			public void run() {
				DatabaseView.this.updateButtonStates();
			}
		});

		if (arg0.getValueIsAdjusting())
			return;

		final JTable table = this.getTable(this.getTableOnFrontIndex());

		File file;
		if (table.getSelectedRowCount() != 1) {
			file = null;
		} else {

			file = new File(this.getRowData(table.getSelectedRow(),
					this.getTableOnFrontIndex()).get(6).toString());
		}

		if (this.listener != null)
			this.listener.userWantToSeeProperties(file, this
					.getTableOnFrontIndex());

	}

	// /DND Section starts here///////////////

	DNDThread dndThread;

	/**
	 * Starts -or restarts the Timer for Drag N Drop. Called each time the left
	 * Mouse Button is pressed down.
	 * 
	 * @param arg0
	 */
	@SuppressWarnings("deprecation")
	private void startDragTimer(final MouseEvent arg0) {

		if (this.dndThread != null) {
			if (this.dndThread.isAlive())
				this.dndThread.stop();
		}
		this.dndThread = new DNDThread();
		this.dndThread.start();
	}

	/**
	 * Ends the Dnd process. Restores the function of the Table and delivers the
	 * event to the engine
	 * 
	 * @param cancelled
	 *            - if true, the Dragged Protein will be discarded
	 */
	@SuppressWarnings("deprecation")
	private void quitDnd(final boolean cancelled) {
		this.setCursor(Cursor.getDefaultCursor());
		this.getTable(0).setRowSelectionAllowed(true);
		if (this.dndThread == null)
			return;
		this.dndThread.stop();

		if (this.activateAddOptions == false)
			return;
		if (this.dndThread.getRow() == -1)
			return;
		this.listener.DNDFinished(cancelled);

		DataPool.DNDBridge.deactivate();

		this.table1.setAutoscrolls(true);
		this.getTable(0).setRowSelectionInterval(this.dndThread.getRow(),
				this.dndThread.getRow());

	}

	/**
	 * The Thread that initializes Dnd Operation, unless it doesnt get stopped
	 * before countdown exceeds
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class DNDThread extends Thread {

		// Protein_3D newProt = null;

		int currentRow = -1;

		public void run() {
			try {

				// newProt = null;
				this.currentRow = -1;

				Thread
						.sleep(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.DND_TIMER_COUNTDOWN);

				if (DatabaseView.this.activateAddOptions == false)
					return;
				if (DatabaseView.this.getTable(0).getSelectedRowCount() != 1)
					return;
				if (DatabaseView.this.isValidRow(DatabaseView.this.getTable(0)
						.getSelectedRow(), DatabaseView.this
						.getTableOnFrontIndex()) != DatabaseView.this.VALIDROW)
					return;

				DatabaseView.this.setCursor(Cm2_IconDepot.DNDCursor);
				this.currentRow = DatabaseView.this.getTable(0)
						.getSelectedRow();
				DatabaseView.this.getTable(0).setRowSelectionAllowed(false);
				DatabaseView.this.table1.setAutoscrolls(false);
				String code = null;
				String file = null;
				if (this.currentRow != -1) {
					code = (String) ((DefaultTableModel) DatabaseView.this
							.getTable(0).getModel()).getValueAt(
							this.currentRow, 1);

					file = (String) ((DefaultTableModel) DatabaseView.this
							.getTable(0).getModel()).getValueAt(
							this.currentRow, 6);

				}

				if (code == null || code.length() < 1) {
					code = Cm2_Toolkit.getSimpleName(new File(file));
				}

				int modelnumber = -1;
				try {

					modelnumber = DatabaseView.ModelByFile.get(
							new File(file).getAbsolutePath()).intValue();
				} catch (final Exception e) {

				}
				try {
					DataPool.DNDBridge.activate(code, file, modelnumber);
				} catch (final Exception e) {
					DataPool.DNDBridge.deactivate();

					JOptionPane
							.showMessageDialog(
									null,
									"Cannot initialize Drag N Drop!\nAn Error occured while reading the File!\nReason:\n\n"
											+ e.getMessage(), "ERROR",
									JOptionPane.ERROR_MESSAGE);
					e.printStackTrace();
					DatabaseView.this.quitDnd(true);
				}
				// newProt = new Protein_3D();
			} catch (final InterruptedException e) {

			}
		}

		// public Protein_3D getNewProtein() {
		// return newProt;
		// }

		public int getRow() {
			return this.currentRow;
		}
	}

	/*
	 * ###Drag and drop movements###
	 * 
	 * Handled in the databaseview, because if dragging starts here, then no
	 * events are produced in the membrane view
	 */
	private Point lastPoint = new Point();

	public void mouseDragged(final MouseEvent arg0) {
		if (DataPool.DNDBridge.getCurrentMolecule() == null)
			return;
		if (arg0.getPoint().equals(this.lastPoint))
			return;
		this.lastPoint = arg0.getPoint();

		final Point loc = arg0.getPoint();

		SwingUtilities.convertPointToScreen(loc, this.table1);

		this.listener.DnDEventOccured(loc);

	}

	public void mouseMoved(final MouseEvent arg0) {

	}

	@SuppressWarnings("unchecked")
	public void updateAllFileAttributes() {

		for (int a = 0; a < 2; a++) {
			final JTable table = this.getTable(a);
			for (int i = 0; i < table.getRowCount(); i++) {

				final Vector data = this.getRowData(i, a);

				final File file = new File((String) data.get(6));
				if (file.canRead() == false)
					continue;

				try {
					final PdbReader_3D reader = new PdbReader_3D();
					final PdbAttributes attrs = reader.readAttributes(file);
					final Vector newdata = new Vector();
					newdata.addElement(null);
					if ( attrs.getCode()==null && !file.getName().isEmpty() && 
						file.getName().contains(CM2_CONSTANTS.IO.OPM_FILENAME_ADDON))
							newdata.addElement(file.getName().replace(".pdb", ""));
					else
						newdata.addElement(attrs.getCode());
//					System.out.println("PDBTM for "+attrs.getCode()+": "+DataPool.getMifHandler().isElementOfPDBTM(attrs.getCode()));
					newdata.addElement(attrs.getHeader());
					newdata.addElement(attrs.getTitles());
					newdata.addElement(attrs.getSpecies());
					newdata.addElement(this.getValueAt(i, 5, a));
					newdata.addElement(file.getAbsolutePath());

					try {
						if (new PdbReader_3D().checkForModels(file)) {
							newdata.addElement("Yes");

						} else {
							newdata.addElement("No");
						}

					} catch (final Exception e) {
						newdata.addElement("No");
					}

					this.setRowData(newdata, i, a);

				} catch (final IOException e) {
					e.printStackTrace();
					continue;
				}

			}
		}
		this.updateModelHashtable();
	}

	public void changeModelForFile(final File file, final int model) {
		DatabaseView.ModelByFile
				.put(file.getAbsolutePath(), new Integer(model));

	}

	public String retrieveUserCommentFromDB(final File file, final int mode) {
		final JTable table = this.getTable(mode);
		for (int i = 0; i < table.getRowCount(); i++) {
			try {
				final String s = (String) table.getValueAt(i, 6);
				if (new File(s).getAbsolutePath().equalsIgnoreCase(
						file.getAbsolutePath())) {
					final String usercomment = (String) table.getValueAt(i, 5);
					if (usercomment.length() > 0)
						return usercomment;
					else
						return null;
				}
			} catch (final Exception e) {
				continue;
			}

		}
		return null;
	}

	

	public boolean isCurrentlyEmpty() {

		return this.table1.getRowCount() < 1 && this.table2.getRowCount() < 1;

	}

	public void autofill() {

		try {
			for (int a = 0; a < 2; a++) {
				final File[] files = a == 0 ? DataPool.IO.DEFAULTPDBDIRECTORY
						.listFiles() : new File(DataPool.workspace
						+ File.separator + "Pdb_Files" + File.separator
						+ "Frequently_Used_Lipids").listFiles();

				for (int i = 0; i < files.length; i++) {
					if (files[i].isDirectory()
							|| files[i].getName().toLowerCase().endsWith("pdb") == false)
						continue;

					this.addRow(this.getEmptyRowData(files[i]), a);

				}

			}
			this.updateAllFileAttributes();
		} catch (final Throwable e) {
			return;
		}
	}

	public boolean dbContainsCode(final String code) {
		if (code == null)
			return false;
		if (code.length() < 1)
			return false;

		for (int a = 0; a < 2; a++) {
			final JTable table = this.getTable(a);
			for (int i = 0; i < table.getRowCount(); i++) {

				final Vector<Object> data = this.getRowData(i, a);
				try {
					final String s = (String) data.get(1);
					if (s.equalsIgnoreCase(code))
						return true;
				} catch (final Exception e) {

				}
			}
		}
		return false;
	}

	private void updateButtonStates() {

		final JTable table = this.getTable(this.getTableOnFrontIndex());

		final int row = table.getSelectedRow();

		if (row < 0) {
			this.b1.setEnabled(false);
			this.b3.setEnabled(false);
			this.b4.setEnabled(false);

			this.b6.setEnabled(false);
			this.b7.setEnabled(false);
			this.b8.setEnabled(false);
			this.b9.setEnabled(false);
			this.b10.setEnabled(false);

			return;
		}

		boolean bool1 = true;
		final boolean bool3 = true;
		boolean bool4 = true;
		boolean bool6 = true;
		boolean bool7 = true;
		boolean bool8 = true;
		boolean bool9 = true;
		boolean bool10 = true;

		if (this.getSelections(this.getTableOnFrontIndex()).length > 1) {
			bool1 = false;
			bool6 = false;
			bool7 = false;
			bool8 = false;
			bool9 = false;
			bool10 = false;
		}

		if (this.isRowAlreadyIn(row, this.getTableOnFrontIndex()))
			bool4 = false;

		if (this.isValidRow(row, this.getTableOnFrontIndex()) != this.VALIDROW) {

			bool6 = false;
			bool7 = false;
			bool8 = false;
			bool10 = false;

		}

		if ("Yes".equals(this.getRowData(row, this.getTableOnFrontIndex()).get(
				7)) == false) {
			bool7 = false;

		}

		final Object code = this.getRowData(row, this.getTableOnFrontIndex())
				.get(1);

		if (code == null || ((String) code).length() != 4) {
			bool9 = false;
		}

		this.b1.setEnabled(bool1);
		this.b3.setEnabled(bool3);
		this.b4.setEnabled(bool4);

		this.b6.setEnabled(bool6);
		this.b7.setEnabled(bool7);
		this.b8.setEnabled(bool8);
		this.b9.setEnabled(bool9);
		this.b10.setEnabled(bool10);
	}
	
	
	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	@SuppressWarnings("unchecked")
	private class FirstRowColumnSorter implements Comparator {
	

		boolean ascending;

		public FirstRowColumnSorter(final boolean ascending) {
			
			this.ascending = ascending;
		}

		/**
		 * compares two passed objects and returns an int value that tells in what
		 * order these objects should be
		 */
		public int compare(final Object a, final Object b) {
			final Vector objvectorA = (Vector) a;
			final Vector objvectorB = (Vector) b;
			Object objectA = objvectorA.get(1);
			Object objectB = objvectorB.get(1);

			// Treat empty strains like nulls
			if (objectA instanceof String && ((String) objectA).length() < 1) {
				objectA = null;
			}
			if (objectB instanceof String && ((String) objectB).length() < 1) {
				objectB = null;
			}

			
			if (objectA == null && objectB == null) {
				return 0;
			} else if (objectA == null) {
				if (this.ascending) {
					return 1;
				} else {
					return -1;
				}

			} else if (objectB == null) {
				if (this.ascending) {
					return -1;
				} else {
					return 1;
				}
			} else{
				
				String C1;
				String C2;
					
					
					if (this.ascending) {
						 C1=(String)objectA;
						 C2=(String)objectB;
						
					}else {
						 C1=(String)objectB;
						 C2=(String)objectA;
					}
						
						
					if(DataPool.getMifHandler().isElementOfPDBTM(C1))
						return 1;
					else if(DataPool.getMifHandler().isElementOfOPM(C1.replaceAll(CM2_CONSTANTS.IO.OPM_FILENAME_ADDON,""))&&!DataPool.getMifHandler().isElementOfPDBTM(C2))
						return 1;
					else 
						return -1;
					
					
					
					
				}
			} 
		

	}
}
