/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.elements;

import java.awt.AWTException;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.MenuItem;
import java.awt.Point;
import java.awt.PopupMenu;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.CapabilityNotSetException;
import javax.media.j3d.ColoringAttributes;
import javax.media.j3d.LineArray;
import javax.media.j3d.LineAttributes;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Transform3D;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JToolBar;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.vecmath.Color3f;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.Layer;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.MicroDomain_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.functionality3d.CameraMode;
import org.cellmicrocosmos.cm2.functionality3d.Hud;
import org.cellmicrocosmos.cm2.functionality3d.IntersectManager;
import org.cellmicrocosmos.cm2.functionality3d.IntersectionsDisplayer;
import org.cellmicrocosmos.cm2.functionality3d.Membrane3D;
import org.cellmicrocosmos.cm2.gui.HelpBtn;
import org.cellmicrocosmos.cm2.gui.MyJRadioButton;
import org.cellmicrocosmos.cm2.gui.PDBFileBrowser;
import org.cellmicrocosmos.cm2.gui.dialogs.AlgorithmWizzard;
import org.cellmicrocosmos.cm2.gui.dialogs.EnterMatrixDialog;
import org.cellmicrocosmos.cm2.gui.dialogs.StatusDialog;
import org.cellmicrocosmos.cm2.interfaces.MembraneViewListener;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.cellmicrocosmos.cm2.misc.ConfigParser;
import org.cellmicrocosmos.cm2.pluginmanager.CalculationManager;


import com.sun.j3d.utils.behaviors.mouse.MouseRotate;


/**
 * The MembraneView Class. Builds up the MembranView-GUI.
 * 
 * @author Sebastian Schneider, Tim Dingerson, Bjoern Sommer bjoern@CELLmicrocosmos.org
 */

/*
 */
@SuppressWarnings("serial")
public class MembraneView extends JPanel implements ActionListener,
		MouseMotionListener, MouseListener {

	private MembraneViewListener listener;

	private final Membrane3D m3d;

	private final JToolBar toolbar2;

	private final JToolBar toolbar;

	private final MyAlignButtonBar toolbar3;
	
	private JPanel calcuPanel; 

	private JButton calcu, clearLip, clearAll, align, remove, detailBtn;

	private MyJRadioButton intraBtn, extraBtn, freelookBtn;

	private final JCheckBox showAxis = new JCheckBox();

	private final JCheckBox showRaster = new JCheckBox();
	
	public JCheckBox toggleOPMLayersButton;

	private JLabel opmLabel = new JLabel();
	

	private final JComboBox comboRender = new JComboBox();

	private final JToolBar bilayerinfo = new JToolBar();

	public IntersectionPanel intersectPanel = new IntersectionPanel();

	// private JLabel perspectivelabel = new JLabel("Camera: ",
	// JLabel.TRAILING);

	

	private final AtomCountPanel atomcountpanel = new AtomCountPanel();

	private final int EXTRA = Membrane3D.EXTRA;

	private final int INTRA = Membrane3D.INTRA;

	private final int FREE = Membrane3D.FREE;

	public final static int GUI_MODE_ALIGN_PROTEIN = 0;

	public final static int GUI_MODE_ALIGN_SAMPPLELIPID = 1;

	public final static int GUI_MODE_MODEL_CHANGE = 2;

	JPopupMenu popcanvas = new JPopupMenu();

	final JMenuItem alPopItem = new JMenuItem("Align Molecule");

	final JMenuItem rmPopItem = new JMenuItem("Remove Selected");

	final JMenuItem cmPopItem = new JMenuItem("Change Model");

	final JMenuItem bfPopItem = new JMenuItem("Browse File");

	JPanel popPanel = new JPanel();

	JLabel popLabel = new JLabel();

	IntersectionsDisplayer intersectionsDisplayer = new IntersectionsDisplayer(
			this);

	AlgorithmInfo ainfo = new AlgorithmInfo();

	Vector<Protein_3D> otherProteins;
	Vector<Matrix3f> oldMatrices;
	Vector<Float> oldYTranslations;

	JPanel placeholder = new JPanel();

	MultiLayerPanel multilayerpanel = new MultiLayerPanel();

	private Vector<Molecule_3D> invalidones;

	private PaintDomainDialog paintDomainDialog = null;
	


	/**
	 * constructor MembraneView
	 * 
	 */
	public MembraneView() {

		final BoxLayout layout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(layout);
		this.m3d = new Membrane3D();
		this.m3d.getCanvas().addKeyListener(new KeyAdapter() {

			public void keyPressed(final KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_DELETE)
					MembraneView.this.remove.doClick();
			}

		});

		this.toolbar2 = this.createToolbar2();
		this.toolbar = this.createToolBar();
		this.toolbar3 = new MyAlignButtonBar();
		this.add(this.toolbar);

		this.add(this.m3d);
		this.add(this.placeholder);
		this.add(this.toolbar2);
		this.add(this.toolbar3);
		this.placeholder.add(this.multilayerpanel);
		this.toolbar3.setVisible(false);
		this.placeholder.setVisible(false);
		
		placeholder.setLayout(new BoxLayout(placeholder,BoxLayout.Y_AXIS));

		this.setEnabled(false);

		final ButtonGroup group = new ButtonGroup();
		group.add(this.extraBtn);
		group.add(this.intraBtn);
		group.add(this.freelookBtn);

		this.initPopupMenu();
	}

	/**
	 * creates the upper toolbar
	 * 
	 * @return
	 */
	private JToolBar createToolBar() {
		final JToolBar tb = new JToolBar();
		tb.setFloatable(false);

		this.clearLip = new JButton("Clear Lipids");
		this.clearLip.addActionListener(this);

		this.clearAll = new JButton("Clear All");
		this.clearAll.addActionListener(this);

		this.align = new JButton();
		this.align.addActionListener(this);

		this.remove = new JButton();

		this.remove.addActionListener(this);

		this.detailBtn = new JButton();
		this.detailBtn.addActionListener(this);
		
		
		this.toggleOPMLayersButton = new JCheckBox() {
			public void setVisible(boolean b) {
				super.setVisible(b);
				opmLabel.setVisible(b);
			}
		};
//		this.toggleOPMLayersButton.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_OPM"));
		this.toggleOPMLayersButton.setSelected(true);
		opmLabel.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_OPM"));
		
		this.toggleOPMLayersButton.setToolTipText("Toggle the OPM Membrane Dummy Markers on and off");
		

		
		this.toggleOPMLayersButton.addActionListener(this);

		try {
			this.showAxis.setSelected(Boolean.parseBoolean(ConfigParser
					.getValue("ShowAxis3D")[0]));
		} catch (final Exception e) {
			this.showAxis.setSelected(true);
		}

		this.showAxis.addActionListener(this);

		try {
			this.showRaster.setSelected(Boolean.parseBoolean(ConfigParser
					.getValue("ShowRaster3D")[0]));
		} catch (final Exception e) {
			this.showRaster.setSelected(true);
		}

		this.showRaster.addActionListener(this);

		this.comboRender.addItem("Shape");
		this.comboRender.addItem("Covalent");
		this.comboRender.addItem("Van Der Waals");
		this.comboRender.addItem("Sticks");
		this.comboRender.addItem("Balls & Sticks");

		this.comboRender.setRenderer(new DefaultListCellRenderer() {
			public Component getListCellRendererComponent(final JList list,
					final Object value, int index, final boolean isSelected,
					final boolean cellHasFocus) {
				final Component comp = super.getListCellRendererComponent(list,
						value, index, isSelected, cellHasFocus);
				final JLabel label = new JLabel();
				if (index == -1)
					index = DataPool.J3DSettings.overallRenderMode;
				label.setOpaque(true);
				label.setForeground(comp.getForeground());
				label.setBackground(comp.getBackground());
				label.setIcon(Cm2_IconDepot.getSpecifiedIcon("renderMode_"
						+ (index>1?index-1:index)));
				label.setText((String) value);

				return label;
			}
		});

		this.comboRender.addActionListener(this);
		this.comboRender.setMaximumSize(new Dimension(25, 150));
		this.comboRender.setMinimumSize(new Dimension(25, 150));

		this.align.setIcon(Cm2_IconDepot.getSpecifiedIcon("MV_AlignSingle"));

		this.remove.setIcon(Cm2_IconDepot.getSpecifiedIcon("MV_Remove"));

		this.detailBtn.setIcon(Cm2_IconDepot.getSpecifiedIcon("MV_Detail"));

		this.clearLip
				.setToolTipText("<html>Clear all Lipids from the Model<br>"
						+ "(Does not effect Lipid templates in the Component View)</html>");
		this.clearAll
				.setToolTipText("<html>Clear all Proteins and Lipids from the Model<br>"
						+ "(Does not effect Lipid templates in the Component View)</html>");
		this.align
				.setToolTipText("<html>Align the selected Molecule. Needs a single Instance<br>selected for Reference</html>");

		this.remove
				.setToolTipText("Remove the selected Instance from the Model");

		this.detailBtn.setToolTipText("Detail & Appearance");

		this.showAxis.setToolTipText("Toggle Axis On/Off");
		this.showRaster.setToolTipText("Toggle Raster On/Off");
		this.comboRender
				.setToolTipText("Change the current render-mode for all molecules");

		tb.addSeparator();
		tb.add(this.clearAll);

		tb.addSeparator();

		tb.add(this.clearLip);

		tb.addSeparator();

		tb.addSeparator();

		tb.addSeparator(new Dimension(5, 0));
		tb.add(this.remove);
		tb.addSeparator(new Dimension(5, 0));

		tb.addSeparator(new Dimension(5, 0));
		tb.add(this.align);
		tb.addSeparator(new Dimension(5, 0));

		tb.add(new JPanel());

		tb.add(this.showAxis);
		tb.addSeparator(new Dimension(5, 0));
		tb.add(this.showRaster);
		tb.addSeparator(new Dimension(25, 0));
		if (CM2_CONSTANTS.MISC.ALLOW_ATOM_MODE) {
			tb.add(this.toggleOPMLayersButton);
			
			tb.add(opmLabel);
			tb.addSeparator(new Dimension(20, 0));
			tb.add(new JLabel("Render Mode: "));
			tb.add(this.comboRender);
			tb.addSeparator(new Dimension(30, 0));
		}

		this.showRaster
				.setIcon(Cm2_IconDepot.getSpecifiedIcon("MV_ShowRaster"));
		this.showAxis.setIcon(Cm2_IconDepot.getSpecifiedIcon("MV_ShowAxis"));

		this.showRaster.setSelectedIcon(Cm2_IconDepot
				.getSpecifiedIcon("MV_ShowRaster_Selected"));
		this.showAxis.setSelectedIcon(Cm2_IconDepot
				.getSpecifiedIcon("MV_ShowAxis_Selected"));

		tb.add(this.detailBtn);
		tb.addSeparator();

		tb.add(new HelpBtn(this));

		return tb;

	}

	/**
	 * sets up the BiLayerPanel
	 * 
	 * @return
	 */
	private JToolBar createToolbar2() {

		final JToolBar bar = new JToolBar();

		this.calcu = new JButton("<html>Calculate<br>Bilayer</html>");
		this.calcu.addActionListener(this);

		this.calcu.setMinimumSize(this.calcu.getPreferredSize());
		this.calcu.setMaximumSize(this.calcu.getPreferredSize());

		this.intraBtn = new MyJRadioButton(1);
		this.intraBtn.addActionListener(this);
		this.extraBtn = new MyJRadioButton(0);
		this.extraBtn.addActionListener(this);

		this.freelookBtn = new MyJRadioButton(-1);
		this.freelookBtn.addActionListener(this);

		bar.setLayout(new BoxLayout(bar, BoxLayout.X_AXIS));
		this.bilayerinfo.setLayout(new BoxLayout(this.bilayerinfo,
				BoxLayout.X_AXIS));
		this.bilayerinfo.setFloatable(false);

		calcuPanel = new JPanel();
		calcuPanel.setBackground(Color.RED);
		calcuPanel.add(this.calcu);
		bar.add(calcuPanel);

		bar.add(this.ainfo);

		bar.add(this.bilayerinfo);
		bar.add(Box.createHorizontalGlue());

		final JPanel p = new JPanel();

		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));

		p.add(this.extraBtn);
		p.add(this.intraBtn);

		bar.add(this.freelookBtn);
		bar.add(p);

		this.calcu
				.setToolTipText("Opens the Calculation Wizard for the Bilayer");
		
		this.bilayerinfo.addSeparator();
		this.bilayerinfo.add(this.atomcountpanel);

		this.bilayerinfo.add(Box.createHorizontalGlue());

		this.bilayerinfo.addSeparator();

		this.bilayerinfo.add(this.intersectPanel);
		this.bilayerinfo.add(Box.createHorizontalStrut(8));
		this.bilayerinfo.add(this.intersectPanel.lipidsBtn);
		// bilayerinfo.addSeparator();

		this.bilayerinfo.addSeparator(new Dimension(10, 0));
		// bilayerinfo.add(new JPanel());
		// bilayerinfo.add(perspectivelabel);
		this.bilayerinfo.addSeparator();
		bar.setFloatable(false);
		return bar;
	}

	/**
	 * returns the MembraneDisplay instance
	 * 
	 * @return
	 */
	public Membrane3D getMembrane3D() {
		return this.m3d;
	}

	/**
	 * sets calculation button enabled if ExampleLipids are added to the
	 * membranemodel
	 * 
	 */
	public void setCalculationEnabled(final boolean b) {
		final MembraneModel model = DataPool.getMembraneModel();
		if (model == null) {
			this.calcu.setEnabled(false);
			return;
		}
		if (model.getExampleLipidsCountExtra() > 0
				|| model.getExampleLipidsCountIntra() > 0) {
			this.calcu.setEnabled(b);
			this.calcuPanel.setBackground(new Color(255, 80, 80));
		} else {
			this.calcu.setEnabled(false);
			this.calcuPanel.setBackground(this.toolbar.getBackground());
		}
	}

	/**
	 * ActionPerformed method for the ComboBox and the Buttons
	 */
	public void actionPerformed(final ActionEvent arg0) {
		final MembraneModel model = DataPool.getMembraneModel();
		// Button "calculate"
		if (arg0.getSource() == this.calcu) {

			if (model.getAllExampleLipidsExtraInDomain(-1, DataPool
					.getCurrentLayerID()).length == 0
					&& model.getAllExampleLipidsIntraInDomain(-1, DataPool
							.getCurrentLayerID()).length == 0) {

				JOptionPane
						.showMessageDialog(
								this,
								"Make sure you have at least one Type of Lipids added!",
								"Missing Lipids",
								JOptionPane.INFORMATION_MESSAGE);
				return;

			}
			
			// this method is skipped if absolute values are used for the lipid computation
			if (!model.getComputeAbsoluteValues()) {
				
				if (model.areLipidPercentagesOk() == false) {
	
					JOptionPane
							.showMessageDialog(
									this,
									"In order to proceed, you'll have to set your Lipid Percentages"
											+ "\nto an overall of 100% on each side of the Model!",
									"Invalid Lipid Percentages",
									JOptionPane.INFORMATION_MESSAGE);
					return;
	
				}
				
			} else {
				if (model.areLipidAbsoluteValuesOk() == false) {
					
					JOptionPane
							.showMessageDialog(
									this,
									"In order to proceed, you'll have to set the absoulte values" +
									"to the value of at least 1.",
									"Invalid Lipid Percentages",
									JOptionPane.INFORMATION_MESSAGE);
					return;
	
				}
			}

			new AlgorithmWizzard(this.listener.getPluginManager(),
					this.listener);

		}
		if (arg0.getSource() == this.clearAll) {

			final JLabel l = new JLabel("Select categories");
			l.setFont(l.getFont().deriveFont(Font.BOLD));
			final JCheckBox c1 = new JCheckBox("Proteins");
			final JCheckBox c2 = new JCheckBox("Lipids in Membrane");
			final JCheckBox c3 = new JCheckBox("Sample Lipids");
			final JCheckBox c4 = new JCheckBox("MicroDomains");
			final JButton ok = new JButton("Ok");
			final JButton cancel = new JButton("Cancel");

			final JDialog dia = new JDialog();
			final JLabel ret = new JLabel("");
			ok.addActionListener(new ActionListener() {

				public void actionPerformed(final ActionEvent arg0) {
					
					ret.setText("true");
					dia.dispose();

				}

			});
			cancel.addActionListener(new ActionListener() {

				public void actionPerformed(final ActionEvent arg0) {

					dia.dispose();

				}

			});

			final JPanel p1 = new JPanel();
			final JPanel p2 = new JPanel();
			p1.setLayout(new BoxLayout(p1, BoxLayout.Y_AXIS));
			dia.getContentPane().add(p1, BorderLayout.NORTH);
			dia.getContentPane().add(p2, BorderLayout.SOUTH);

			p1.add(Box.createVerticalStrut(5));
			p1.add(l);
			p1.add(Box.createVerticalStrut(5));
			p1.add(c1);
			p1.add(c2);
			p1.add(c3);
			p1.add(c4);

			p2.add(ok);
			p2.add(cancel);

			if (model.getProteinCount() < 1) {
				c1.setSelected(false);
				c1.setEnabled(false);
			} else
				c1.setSelected(false);
			if (model.getLipidCountExtra() < 1
					&& model.getLipidCountIntra() < 1) {
				c2.setSelected(false);
				c2.setEnabled(false);
			} else
				c2.setSelected(false);
			if (model.getExampleLipidsCountExtra() < 1
					&& model.getExampleLipidsCountIntra() < 1) {
				c3.setSelected(false);
				c3.setEnabled(false);
			} else
				c3.setSelected(false);

			if (model.countMicroDomains() < 1) {
				c4.setSelected(false);
				c4.setEnabled(false);
			} else
				c4.setSelected(false);

			dia.setTitle("Clear All...");
			dia.setModal(true);
			dia.setAlwaysOnTop(true);
			dia.pack();
			dia.setLocation(this.clearAll.getLocationOnScreen());
			dia.setVisible(true);

			if (!Boolean.parseBoolean(ret.getText()))
				return;

			this.m3d.clearAll(c1.isSelected(), c2.isSelected(),
					c3.isSelected(), DataPool.getCurrentLayerID());
			this.m3d.clearSelection();

			if (c4.isSelected()) {
				this.listener.highlightMicroDomain(0);
				for (int i = model.countMicroDomains() - 1; i > -1; i--) {

					if (model.getMicroDomain(i).layerId == DataPool
							.getCurrentLayerID())
						model.removeMicroDomain(model.getMicroDomain(i));
				}

				DataPool.getMembraneModel().organizeDomains();
				this.listener.updateMicroDomainsIn3D();
			}

		}
		if (arg0.getSource() == this.clearLip) {

			if (model.countMicroDomains() < 1) {
				this.m3d.clearLipids(-1, DataPool.getCurrentLayerID());
				return;
			}

			final JDialog menu = new JDialog();
			menu.getContentPane().setLayout(
					new GridLayout(3 + model.countMicroDomains(), 1));

			final ActionListener ac = new ActionListener() {

				public void actionPerformed(final ActionEvent arg0) {
					final JButton item = (JButton) arg0.getSource();

					if (item.getText().equals("All Domains")) {

						MembraneView.this.m3d.clearLipids(-1, DataPool
								.getCurrentLayerID());
						menu.dispose();
					} else if (item.getText().equals("Default Area")) {
						MembraneView.this.m3d.clearLipids(0, DataPool
								.getCurrentLayerID());
						item.setVisible(false);
						menu.dispose();
					} else {
						final int id = Integer.parseInt(item.getText()
								.substring(item.getText().lastIndexOf(" "))
								.trim());
						MembraneView.this.m3d.clearLipids(id, DataPool
								.getCurrentLayerID());
						item.setVisible(false);
						menu.dispose();
					}

				}

			};
			final MouseAdapter ma = new MouseAdapter() {

				public void mouseEntered(final MouseEvent arg0) {
					final JButton item = (JButton) arg0.getSource();

					if (item.getText().equals("All Domains")) {
						MembraneView.this.listener.highlightMicroDomain(0);
					} else if (item.getText().equals("Default Area")) {
						MembraneView.this.listener.highlightMicroDomain(0);
					} else {
						final int id = Integer.parseInt(item.getText()
								.substring(item.getText().lastIndexOf(" "))
								.trim());
						MembraneView.this.listener.highlightMicroDomain(id);
					}
				}

			};

			final JLabel label = new JLabel("Select Domain to clear");
			label.setFont(label.getFont().deriveFont(Font.BOLD));
			label.setBackground(label.getBackground().darker());
			final JButton item1 = new JButton("All Domains");
			item1.setFont(item1.getFont().deriveFont(Font.ITALIC));
			final JButton item2 = new JButton("Default Area");

			item1.addActionListener(ac);
			item2.addActionListener(ac);
			item1.addMouseListener(ma);
			item2.addMouseListener(ma);
			menu.getContentPane().add(label);
			menu.getContentPane().add(item1);
			menu.getContentPane().add(item2);

			final int layerID = DataPool.getCurrentLayerID();
			final Vector<JButton> v = new Vector<JButton>();
			for (int i = 0; i < model.countMicroDomains(); i++) {

				if (model.getMicroDomain(i).layerId != layerID)
					continue;
				final JButton item = new JButton("Domain "
						+ model.getMicroDomain(i).id);
				item.addActionListener(ac);
				item.addMouseListener(ma);
				menu.getContentPane().add(item);
				v.addElement(item);
			}

			menu.setUndecorated(true);
			menu.setAlwaysOnTop(true);
			menu.pack();

			menu.setResizable(false);
			menu.setLocationRelativeTo(this.clearLip);
			menu.setVisible(true);
			menu.requestFocus();
			menu.addFocusListener(new FocusListener() {

				public void focusGained(final FocusEvent arg0) {

				}

				public void focusLost(final FocusEvent arg0) {
					if (arg0.getOppositeComponent() != item1
							&& arg0.getOppositeComponent() != item2
							&& !v.contains(arg0.getOppositeComponent()))
						menu.dispose();

				}

			});

		}
		
		if(arg0.getSource()==this.toggleOPMLayersButton) {
			listener.refreshViews(false,false,false,false,false,false);
		}

		if (arg0.getSource() == this.intraBtn) {

			this.m3d.setViewByMode(this.INTRA, null);

		}
		if (arg0.getSource() == this.extraBtn) {

			this.m3d.setViewByMode(this.EXTRA, null);

		}
		if (arg0.getSource() == this.freelookBtn) {

			this.m3d.setViewByMode(this.FREE, null);

		}
		if (arg0.getSource() == this.showAxis) {
			((Membrane3D) this.m3d).setShowAxis(this.showAxis.isSelected());

			ConfigParser.deleteAllAt("ShowAxis3D");
			ConfigParser
					.addValue("ShowAxis3D", "" + this.showAxis.isSelected());

		}
		if (arg0.getSource() == this.showRaster) {
			((Membrane3D) this.m3d).setShowRaster(this.showRaster.isSelected());

			ConfigParser.deleteAllAt("ShowRaster3D");
			ConfigParser.addValue("ShowRaster3D", ""
					+ this.showRaster.isSelected());

		}
		if (arg0.getSource() == this.detailBtn) {

			this.listener.popup3DSettingsDialog(this.detailBtn
					.getLocationOnScreen());

		}

		if (arg0.getSource() == this.align) {

			if (this.m3d.getSelectedMolecules().size() > 0) {

				this.listener.alignSingleProtein(this.m3d
						.getSelectedMolecules().get(0));
			}
		}

		if (arg0.getSource() == this.remove) {
			((Membrane3D) this.m3d).removeMoleculesFrom3D(this.m3d
					.getSelectedMolecules().toArray(new Molecule_3D[0]));
			for (int i = 0; i < this.m3d.getSelectedMolecules().size(); i++) {

				DataPool.getMembraneModel().deleteMolecule(
						this.m3d.getSelectedMolecules().get(i));
			}
			this.m3d.clearSelection();
			model.organizeDomains();
			this.listener.refreshViews(true, true, true, true, false, true);
			this.listener.setShowAxis(this.listener.areAxisVisible());
			this.listener.pickMolecules(null);

		}
		if (arg0.getSource() == this.comboRender) {

			
		if(this.calcu.isVisible()) {
			this.switchRenderMode_Normal(this.comboRender.getSelectedIndex());
		}else {
			listener.requestRenderModeChange();
		}

		}

	}

	private void switchRenderMode_Normal(int selectedIndex) {
		if (selectedIndex != Molecule_3D.RENDER_MODE_POLYGON
				&& DataPool.getMembraneModel().countAtoms(-1, -1, DataPool.getCurrentLayerID(),false) > CM2_CONSTANTS.J3D.CRITICAL_ATOM_COUNT) {
			final int val = JOptionPane
					.showConfirmDialog(
							this,
							"Displaying that many elements will lead to a significant\n"
									+ "downgrade of the performance. This might also stall\n"
									+ "your machine for a while!\n\n"
									+ "Do you want to continue?",
							"Warning", JOptionPane.YES_NO_OPTION,
							JOptionPane.WARNING_MESSAGE);
			if (val != JOptionPane.YES_OPTION) {
				this.comboRender.removeActionListener(this);
				this.comboRender
						.setSelectedIndex(Molecule_3D.RENDER_MODE_POLYGON);
				this.comboRender.addActionListener(this);
				return;
			}

		}

		DataPool.J3DSettings.overallRenderMode = selectedIndex;

		if (DataPool.getMembraneModel().isEmpty())
			return;

		final JButton b1 = new JButton("Break");

		final StatusDialog dia;

		if (DataPool.J3DSettings.overallRenderMode != Molecule_3D.RENDER_MODE_POLYGON) {
			dia = new StatusDialog(b1, true);
			dia.setProgress("Inserting Elements", 0, false);
		} else {
			dia = new StatusDialog(false);
			dia.setProgress("Restoring, please wait...", 0, true);
		}

		final Vector<String> goon = new Vector<String>();

		b1.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent e) {
				goon.addElement("");
				dia.dispose();
			}

		});

		this.listener.setAllEnabled(false);
		this.comboRender.setEnabled(false);

		final Thread T = new Thread() {
			public void run() {

				boolean broke = false;
				final float step = 100f / DataPool.getMembraneModel()
						.countAtoms(-1, -1, DataPool.getCurrentLayerID(),false);
				float progress = 0f;

				MembraneView.this.m3d.dockScreen(false);

				final Molecule_3D[] allMols = DataPool.getMembraneModel()
						.getAllMoleculesInMembrane();
				boolean toggle = true;

				for (int i = 0; i < allMols.length; i++) {
					toggle = !toggle;

					if (DataPool.getCurrentLayerID() == allMols[i]
							.getLayerID())
						allMols[i]
								.setRenderMode(DataPool.J3DSettings.overallRenderMode);
					else
						allMols[i]
								.setRenderMode(Molecule_3D.RENDER_MODE_POLYGON);

					progress += (step * allMols[i].getAtomCount());

					if (goon.isEmpty() == false) {

						broke = true;
						break;
					}
					if (toggle
							&& DataPool.J3DSettings.overallRenderMode != Molecule_3D.RENDER_MODE_POLYGON)
						dia.setProgress("Inserting Atoms", (int) progress,
								true);
				}

				dia.dispose();

				MembraneView.this.m3d.dockScreen(true);
				MembraneView.this.listener.setAllEnabled(true);
				MembraneView.this.comboRender.setEnabled(true);

				if (broke) {
					SwingUtilities.invokeLater(new Thread() {
						public void run() {
							MembraneView.this.comboRender
									.setSelectedIndex(Molecule_3D.RENDER_MODE_POLYGON);

						}
					});
				}

			}
		};

		T.start();
		
	}
	
	public void switchRenderMode_While_Algorithm_Runs() {
		if (this.comboRender.getSelectedIndex() != Molecule_3D.RENDER_MODE_POLYGON
				&& DataPool.getMembraneModel().countAtoms(-1, -1, DataPool.getCurrentLayerID(),false) > CM2_CONSTANTS.J3D.CRITICAL_ATOM_COUNT) {
			final int val = JOptionPane
					.showConfirmDialog(
							this,
							"Displaying that many elements will lead to a significant\n"
									+ "downgrade of the performance. This might also stall\n"
									+ "your machine for a while!\n\n"
									+ "Do you want to continue?",
							"Warning", JOptionPane.YES_NO_OPTION,
							JOptionPane.WARNING_MESSAGE);
			if (val != JOptionPane.YES_OPTION) {
				this.comboRender.removeActionListener(this);
				this.comboRender
						.setSelectedIndex(Molecule_3D.RENDER_MODE_POLYGON);
				this.comboRender.addActionListener(this);
				return;
			}

		}

		DataPool.J3DSettings.overallRenderMode = this.comboRender.getSelectedIndex();

		if (DataPool.getMembraneModel().isEmpty())
			return;

		final StatusDialog dia;
		 
		dia = new StatusDialog(false);
		dia.setProgress("Please wait...", 0, true);

		this.comboRender.setEnabled(false);

		MembraneView.this.m3d.dockScreen(false);

		final Molecule_3D[] allMols = DataPool.getMembraneModel()
				.getAllMoleculesInMembrane();
		

		for (int i = 0; i < allMols.length; i++) {

			if (DataPool.getCurrentLayerID() == allMols[i]
					.getLayerID())
				allMols[i]
						.setRenderMode(DataPool.J3DSettings.overallRenderMode);
			else
				allMols[i]
						.setRenderMode(Molecule_3D.RENDER_MODE_POLYGON);

		}

		dia.dispose();

		MembraneView.this.m3d.dockScreen(true);
		MembraneView.this.comboRender.setEnabled(true);

	}
	

	/**
	 * MembranViewListener
	 * 
	 * @param listener
	 */
	public void addMembraneViewListener(final MembraneViewListener listener) {
		this.listener = listener;
		((Membrane3D) this.m3d).addMembraneViewListener(listener);

	}

	/**
	 * sets the prefferedSize of the DrawableComponent ScrollPane
	 */
	public void doLayout() {

		this.multilayerpanel
				.setPreferredSize(new Dimension(this.getWidth(), 25));
		this.toolbar2.setPreferredSize(new Dimension(this.getWidth(), 35));
		this.toolbar3.setPreferredSize(new Dimension(this.getWidth(), 35));
		this.toolbar.setPreferredSize(new Dimension(this.getWidth(), 20));

		if (this.placeholder.isVisible())
			this.m3d.setPreferredSize(new Dimension(this.getWidth(), this
					.getHeight()
					- 80 - this.multilayerpanel.getPreferredSize().height));
		else
			this.m3d.setPreferredSize(new Dimension(this.getWidth(), this
					.getHeight() - 70));

		super.doLayout();

	}

	/**
	 * enables buttons in the toolbar if a membrane model exists
	 * 
	 */
	public void setEnabled(final boolean b) {
		super.setEnabled(b);
		this.calcu.setEnabled(b);
		this.m3d.setEnabled(b);
		// perspectivelabel.setVisible(b);

		if (b == false)
			this.clearLip.setEnabled(b);
		if (b == false)
			this.clearAll.setEnabled(b);

		this.intraBtn.setVisible(b);
		this.extraBtn.setVisible(b);
		this.freelookBtn.setVisible(b);
		this.extraBtn.setSelected(b);
		this.showAxis.setEnabled(b);
		this.showRaster.setEnabled(b);
//		this.comboRender.setEnabled(b);

		if (b)
			this.setCalculationEnabled(true);

	
		this.atomcountpanel.setVisible(b);

		if (b == false)
			this.align.setEnabled(b);

		if (b == false)
			this.remove.setEnabled(b);

	}

	/**
	 * Updates the information shown in the toolbars
	 * 
	 */
	public void updatePanels() {

		final MembraneModel model = DataPool.getMembraneModel();

		if (model == null || this.isEnabled() == false)
			return;

		if (model.getLipidCountExtra() < 1 && model.getLipidCountIntra() < 1) {
			this.clearLip.setEnabled(false);

		} else
			this.clearLip.setEnabled(true);

		if (model.getLipidCountExtra() < 1 && model.getLipidCountIntra() < 1
				&& model.getProteinCount() < 1
				&& model.getExampleLipidsCountExtra() < 1
				&& model.getExampleLipidsCountIntra() < 1
				&& model.countMicroDomains() < 1) {
			this.clearAll.setEnabled(false);

		} else
			this.clearAll.setEnabled(true);

		this.comboRender.removeActionListener(this);
		this.comboRender
				.setSelectedIndex(DataPool.J3DSettings.overallRenderMode);
		this.comboRender.addActionListener(this);

		
		this.atomcountpanel.updateContents();

		if (this.m3d.getSelectedMolecules() != null
				&& this.m3d.getSelectedMolecules().size() < 1) {

			this.align.setEnabled(false);

			this.remove.setEnabled(false);

		} else {

			if (this.m3d.getSelectedMolecules() != null
					&& this.m3d.getSelectedMolecules().size() > 1) {
				this.align.setEnabled(false);

			} else {
				this.align.setEnabled(true);

			}

			this.remove.setEnabled(true);

			if (this.m3d.getSelectedMolecules() != null
					&& this.m3d.getSelectedMolecules().size() > 1)
				this.remove.setToolTipText("Remove selected Instances ( "
						+ (this.m3d.getSelectedMolecules().size() - 1)
						+ " ) from the Model");
			else
				this.remove
						.setToolTipText("Remove the selected Instance from the Model");
		}

		final Molecule_3D[] mols = model.getAllMoleculesInMembrane();
		for (int i = 0; i < mols.length; i++) {
			mols[i].setPicked(false);
		}

		for (int i = 0; i < this.m3d.getSelectedMolecules().size(); i++) {
			this.m3d.getSelectedMolecules().get(i).setPicked(true);

		}

	}

//	/**
//	 * The label on the lower toolbar that shows the state of the membrane
//	 * 
//	 * @author Tim Dingersen dackelnose@gmx.net
//	 * 
//	 */
//	private class MembraneStateLabel extends JLabel {
//
//		public MembraneStateLabel() {
//
//			this.setFont(new Font("Arial", Font.BOLD, 14));
//			this.setText("");
//
//		}
//
//		/**
//		 * Updates the shown information regarding to the state of the current
//		 * model
//		 * 
//		 */
//		public void updateText() {
//
//			if (this.isEnabled() == false)
//				return;
//
//			final MembraneModel model = DataPool.getMembraneModel();
//
//			if (model == null) {
//				this.setIcon(Cm2_IconDepot.getSpecifiedIcon("MS_Closed"));
//				return;
//			}
//
//			final int state = model.getMembraneState();
//
//			if (state == MembraneModel.STATE_OK) {
//				this.setIcon(Cm2_IconDepot.getSpecifiedIcon("MS_Ready"));
//				this
//						.setToolTipText("<html>The Lipid Layers have been correctly calculated.<br>"
//								+ "Membrane is ready to be Generated!</html>");
//			} else if (state == MembraneModel.STATE_INCONSISTENT) {
//				this.setIcon(Cm2_IconDepot.getSpecifiedIcon("MS_Inconsistent"));
//				this
//						.setToolTipText("<html>The Lipid Layers are inconsistent because some values have<br>"
//								+ "been changed since the last Calculation.<br>"
//								+ "Membrane needs to be recalculated!</html>");
//			} else if (state == MembraneModel.STATE_NOLIPIDS) {
//				this.setIcon(Cm2_IconDepot.getSpecifiedIcon("MS_Empty"));
//				this
//						.setToolTipText("<html>The Lipid Layers have not been calculated yet.</html>");
//
//			}
//
//		}
//
//	}

	/**
	 * The panel that shows the number of atoms and chains in the current
	 * membrane and their amount towards the limit
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class AtomCountPanel extends JPanel {

		private int ac = 0;

		private int rac = 0;

		private final int w = 40;

		private int maxatoms;

		private float percent;

		private String pstring = "";

		private int drawingwidth;

		private int cc;

		private int rcc;

		private int maxchains;

		private float percent2;

		private String pstring2 = "";

		private int drawingwidth2;

		public boolean maygenerate = false;

		public AtomCountPanel() {
			this.setPreferredSize(new Dimension(300, 30));

		}

		public void updateContents() {

			if (DataPool.getMembraneModel() == null) {

				return;
			}

			this.ac = 0;
			this.rac = 0;
			final int domainID = MembraneView.this.listener
					.getHighlightedMicroDomain();
			int layerID = DataPool.getCurrentLayerID();
			if (domainID < 0)
				layerID = -1;

			this.rac = (DataPool.getMembraneModel().countAtoms(0, domainID,
					layerID,DataPool.PPESettings.saveOPMDummyLayersInPDB)+DataPool.getMembraneModel().countTERs(0, domainID,
							layerID));
			if (DataPool.LimitationSettings.atomserials == 1)
				this.ac = this.rac;

			this.rac += (DataPool.getMembraneModel().countAtoms(1, domainID,
					layerID,DataPool.PPESettings.saveOPMDummyLayersInPDB)+DataPool.getMembraneModel().countTERs(0, domainID,
							layerID));
			this.rac += (DataPool.getMembraneModel().countAtoms(2, domainID,
					layerID,DataPool.PPESettings.saveOPMDummyLayersInPDB)+DataPool.getMembraneModel().countTERs(0, domainID,
							layerID));

			if (DataPool.LimitationSettings.atomserials == 0)
				this.ac = this.rac;
			if (DataPool.LimitationSettings.atomserials == 2)
				this.ac = 0;

			this.maxatoms = CM2_CONSTANTS.MEMBRANE_SIZES.MAX_ATOMS_FOR_PROPER_SECONDARY_STRUCTURE;
			this.percent = (100 / (float) this.maxatoms) * this.ac;
			this.pstring = ((int) this.percent) + " %";
			this.drawingwidth = (int) (((float) (this.w - 1) / 100) * this.percent);
			if (this.drawingwidth > this.w)
				this.drawingwidth = this.w;

			this.cc = 0;
			this.rcc = 0;

			this.rcc = DataPool.getMembraneModel().countChains(0, domainID,
					layerID);
			if (DataPool.LimitationSettings.chains == 1)
				this.cc = this.rcc;

			this.rcc += DataPool.getMembraneModel().countChains(1, domainID,
					layerID);
			this.rcc += DataPool.getMembraneModel().countChains(2, domainID,
					layerID);

			if (DataPool.LimitationSettings.chains == 0)
				this.cc = this.rcc;
			if (DataPool.LimitationSettings.chains == 2)
				this.cc = 0;

			if (DataPool.LimitationSettings.caseSensitiveChainIds)
				this.maxchains = 52;
			else
				this.maxchains = 26;

			this.percent2 = (100 / (float) this.maxchains) * this.cc;
			this.pstring2 = ((int) this.percent2) + " %";
			this.drawingwidth2 = (int) (((float) (this.w - 1) / 100) * this.percent2);
			if (this.drawingwidth2 > this.w)
				this.drawingwidth2 = this.w;

			boolean b1 = true;
			if (DataPool.LimitationSettings.atomserials == 0
					&& this.rac > CM2_CONSTANTS.MEMBRANE_SIZES.MAX_ATOMS_FOR_PROPER_SECONDARY_STRUCTURE)
				b1 = false;
			else if (DataPool.LimitationSettings.atomserials == 1
					&& this.ac > CM2_CONSTANTS.MEMBRANE_SIZES.MAX_ATOMS_FOR_PROPER_SECONDARY_STRUCTURE)
				b1 = false;

			boolean b2 = true;
			if (DataPool.LimitationSettings.chains == 0
					&& this.rcc > this.maxchains)
				b2 = false;
			else if (DataPool.LimitationSettings.chains == 1
					&& this.cc > this.maxchains)
				b2 = false;

			// TODO Why a limitation to 9999 lipids +proteins? 
			//      This limitation must have been here for a long time!
//			final boolean b3 = 9999 >= (DataPool.getMembraneModel()
//					.getProteinCount()
//					+ DataPool.getMembraneModel().getLipidCountExtra() + DataPool
//					.getMembraneModel().getLipidCountIntra());
//
//			this.maygenerate = b1 && b2 && b3;
			
			this.maygenerate = b1 && b2;
			this.repaint();

		}

		public void paintComponent(final Graphics g) {
			super.paintComponent(g);
			if (DataPool.getMembraneModel() == null) {

				return;
			}

			g.setFont(g.getFont().deriveFont(Font.PLAIN, 11f));
			g.drawString("Counters", 1, 8);
			g.drawString("Limits", 100, 8);

			g.setFont(g.getFont().deriveFont(Font.BOLD, 9f));
			g.drawString("Atoms", 1, 18);
			g.drawString("Chains", 1, 29);

			g.drawString("" + this.rac, 40, 18);
			g.drawString("" + this.rcc, 40, 29);

			g.setFont(g.getFont().deriveFont(Font.PLAIN, 11f));

			if (DataPool.LimitationSettings.atomserials != 2) {
				Color c = new Color(130, 190, 70);

				if (this.ac > this.maxatoms) {
					c = new Color(230, 90, 90);

				} else if (this.percent >= 80)
					c = Color.orange;

				g.setColor(Color.white);

				g.fillRect(100, 9, this.w, 10);
				g.setColor(c);

				g.fillRect(101, 9, this.drawingwidth, 10);

				g.setColor(Color.DARK_GRAY);
				g.drawRect(100, 9, this.w, 10);
				g.setColor(this.getForeground());
				g.drawString(this.pstring, 145, 19);

			} else
				g.drawString("Unlimited", 100, 19);

			if (DataPool.LimitationSettings.chains != 2) {
				Color c2 = new Color(130, 190, 70);

				if (this.cc > this.maxchains) {
					c2 = new Color(230, 90, 90);

				} else if (this.percent2 >= 80)
					c2 = Color.orange;

				g.setColor(Color.white);

				g.fillRect(100, 20, this.w, 10);
				g.setColor(c2);

				g.fillRect(101, 20, this.drawingwidth2, 10);

				g.setColor(Color.DARK_GRAY);
				g.drawRect(100, 20, this.w, 10);
				g.setColor(this.getForeground());
				g.drawString(this.pstring2, 145, 30);

			} else
				g.drawString("Unlimited", 100, 30);

		}

	}

	public boolean mayGenerate() {
		return this.atomcountpanel.maygenerate;
	}

	private void initPopupMenu() {

		this.alPopItem.setIcon(this.align.getIcon());

		this.rmPopItem.setIcon(this.remove.getIcon());

		this.cmPopItem
				.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_ChangeModel"));

		this.bfPopItem.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_Browse"));

		final ActionListener ls = new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {
				if (arg0.getSource() == MembraneView.this.alPopItem)
					MembraneView.this.align.doClick();

				if (arg0.getSource() == MembraneView.this.rmPopItem)
					MembraneView.this.remove.doClick();
				if (arg0.getSource() == MembraneView.this.cmPopItem) {
					
					MembraneView.this.listener
							.startModelChange(MembraneView.this.m3d
									.getSelectedMolecules().firstElement());
				}
				if (arg0.getSource() == MembraneView.this.bfPopItem) {
					final String[] s = DataPool
							.getRawPdbData(MembraneView.this.m3d
									.getSelectedMolecules().firstElement()
									.getPDBFile().getAbsolutePath());
					if (s != null)
						new PDBFileBrowser(s, Cm2_Toolkit
								.getSimpleName(MembraneView.this.m3d
										.getSelectedMolecules().firstElement()
										.getPDBFile()));
				}

			}

		};

		this.alPopItem.addActionListener(ls);

		this.rmPopItem.addActionListener(ls);

		this.cmPopItem.addActionListener(ls);

		this.bfPopItem.addActionListener(ls);

		this.popcanvas.add(this.popPanel);
		this.popcanvas.addSeparator();
		this.popcanvas.add(this.alPopItem);

		this.popcanvas.add(this.rmPopItem);

		this.popcanvas.add(this.cmPopItem);

		this.popcanvas.addSeparator();
		this.popcanvas.add(this.bfPopItem);

		this.popPanel.add(this.popLabel);
		this.popPanel.setBackground(this.popcanvas.getBackground().darker());
		this.popcanvas.setBorder(BorderFactory.createEtchedBorder());

	}

	public JPopupMenu createCanvasPopUpMenu() {

		final Molecule_3D mol = this.m3d.getSelectedMolecules().get(0);
		final String type = mol instanceof Protein_3D ? "Protein" : "Lipid";
		if (this.m3d.getSelectedMolecules().size() < 2) {
			this.popLabel.setText("<html>Options for " + type + "<br>Type:"
					+ mol.getPDBcode());
		} else {
			this.popLabel.setText("<html>Options for all selected<br>" + type
					+ "s of<br>Type: " + mol.getPDBcode());
		}

		this.alPopItem.setToolTipText(this.align.getToolTipText());

		this.rmPopItem.setToolTipText(this.remove.getToolTipText());

		this.cmPopItem
				.setToolTipText("Change the current model for this instance");

		this.cmPopItem.setEnabled(this.m3d.getSelectedMolecules()
				.firstElement().getModelcount() > 0);

		return this.popcanvas;
	}

	public void setToolbarsEnabled(final boolean b) {

		for (int i = 0; i < this.toolbar.getComponentCount(); i++) {
			if (this.toolbar.getComponent(i) != this.showAxis
					&& this.toolbar.getComponent(i) != this.showRaster
					&& this.toolbar.getComponent(i) != this.intersectPanel
					&& this.toolbar.getComponent(i) != this.toggleOPMLayersButton
					&& this.toolbar.getComponent(i) != this.opmLabel
					&& this.toolbar.getComponent(i) != this.comboRender)
				this.toolbar.getComponent(i).setEnabled(b);
		}
		this.multilayerpanel.combo.setEnabled(b);
		this.multilayerpanel.changeHeight.setEnabled(b);
		this.multilayerpanel.remove.setEnabled(b);

		if (b)
			this.updatePanels();

	}

	

	private Molecule_3D currentMoleculeAligning;

	private Point oldCursor;

	private double translating_Distance = 0;

	private int oldModel = 0;

	private boolean haveToReInsert = false;

	Vector<JComponent> comps = new Vector<JComponent>();
	public void setInAlgorithmMode(final JButton stop,
			final JCheckBox dockScreen, final JCheckBox showStats,
			final JButton runtimeControls, final Component controlcompo) {
		this.calcu.setVisible(false);
		this.calcuPanel.setVisible(false);

		this.toolbar2.add(stop, 0);
		comps.addElement(stop);
		if (runtimeControls != null) {
			
			this.toolbar2.add(runtimeControls, 2);
			comps.addElement(runtimeControls);
			this.toolbar2.add(dockScreen, 4);
			comps.addElement(dockScreen);
			this.toolbar2.add(showStats, 5);
			comps.addElement(showStats);
//			this.toolbar2.add(new JToolBar.Separator(), 6);

			this.placeholder.remove(this.multilayerpanel);
			this.placeholder.add(controlcompo);
			this.placeholder.setVisible(true);

		} else {
			
			this.toolbar2.add(dockScreen, 3);
			comps.addElement(dockScreen);
			this.toolbar2.add(showStats, 4);
			comps.addElement(showStats);
//			this.toolbar2.add(new JToolBar.Separator(), 5);
		}

		this.ainfo.setVisible(true);
		this.bilayerinfo.setVisible(false);
		this.atomcountpanel.setVisible(false);

		this.freelookBtn.doClick();

	}

	public void setNormalMode() {

		this.toolbar2.getComponent(0).setVisible(false);
			
			for(int i=0;i<comps.size();i++) {
				this.toolbar2.remove(comps.get(i));
			}

			// this.runtimeDialog.dispose();
		

		this.placeholder.removeAll();
		this.placeholder.add(this.multilayerpanel);
		this.placeholder.setVisible(DataPool.getMembraneModel()
				.hasMultipleLayers());

		this.calcu.setVisible(true);
		this.calcuPanel.setVisible(true);
		this.ainfo.setVisible(false);
		this.bilayerinfo.setVisible(true);
		this.atomcountpanel.setVisible(true);

		this.extraBtn.doClick();
	}

	/**
	 * The component shown while performing a lipid filling algorithm Provides a
	 * progressbar for displaying information from the algorithm
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class AlgorithmInfo extends JPanel {

		public JProgressBar bar = new JProgressBar();

		public AlgorithmInfo() {
			this.setVisible(false);
			this.bar.setVisible(false);
			// bar.setForeground(new Color(130, 190, 70));

			this.bar.setMinimum(0);
			this.bar.setMaximum(100);

			this.add(this.bar);

			this.bar.setStringPainted(true);
		}

		public void setVisible(final boolean b) {

			if (MembraneView.this.toolbar2 != null)

				if (b == false) {
					this.bar.setVisible(false);
					this.bar.setValue(0);
					this.bar.setString("");
				}
			super.setVisible(b);

		}

		public void doLayout() {
			if (MembraneView.this.toolbar2 != null)
				this.bar.setPreferredSize(new Dimension(this.getWidth() - 20,
						30));
			super.doLayout();
		}

	}

	public void setAlgoritmProgress(final int progress) {
		if (this.ainfo.bar.isVisible() == false)
			this.ainfo.bar.setVisible(true);

		this.ainfo.bar.setValue(progress);
		// try {
		// SystemTray.getSystemTray().getTrayIcons()[0].setToolTip("<html>Running "+
		// CalculationManager.getAlgo().getName()+"<br>Progress: "+(ainfo.bar.getValue()>0?ainfo.bar.getValue()+" %":"NaN")+"<br>"+"Message: "+ainfo.bar.getString()+"</html>");
		// }catch(Exception e) {
		// // e.printStackTrace();
		// }

	}

	public void displayAlgorithmInfo(final String info) {
		if (this.ainfo.bar.isVisible() == false)
			this.ainfo.bar.setVisible(true);

		 ainfo.bar.setString(info);
		 
		 
		// try {
		// SystemTray.getSystemTray().getTrayIcons()[0].setToolTip("<html>Running "+
		// CalculationManager.getAlgo().getName()+"<br>Progress: "+(ainfo.bar.getValue()>0?ainfo.bar.getValue()+" %":"NaN")+"<br>"+"Message: "+ainfo.bar.getString()+"</html>");
		// }catch(Exception e) {
		// // e.printStackTrace();
		// }
	}

	public void startDomainMode(final MicroDomain_3D domainToChange) {
		this.listener.setAllEnabled(false);

		this.m3d.getCamera().setViewByMode(this.EXTRA, null);

		this.m3d.getCamera().setEnabled(false);

		this.m3d.getPickNDragManager().allowDragging(false);

		this.calcu.setEnabled(false);
		this.freelookBtn.setEnabled(false);

		if (domainToChange == null)
			this.paintDomainDialog = new PaintDomainDialog("New Microdomain");
		else
			this.paintDomainDialog = new PaintDomainDialog("Change Boundaries");

		this.m3d.getMicroDomainManager().setAcceptButton(
				this.paintDomainDialog.ok);
		this.m3d.getMicroDomainManager().startDomainMode(domainToChange);
		this.listener.highlightMicroDomain(0);

	}

	private void finishDomainMode(final boolean apply) {

		this.paintDomainDialog.dispose();

		this.m3d.getMicroDomainManager().finishDomainMode(apply);
		this.m3d.getPickNDragManager().allowDragging(true);
		this.m3d.getCamera().setEnabled(true);
		this.calcu.setEnabled(true);
		this.freelookBtn.setEnabled(true);

		this.listener.setAllEnabled(true);

		this.listener.updateMicroDomainsIn3D();

		if (apply) {
			this.listener.refreshViews(true, true, false, false, false, false);
			this.listener.highlightMicroDomain(DataPool.drawingDomain.id);
		} else {
			if (DataPool.drawingDomain.id < 0)
				this.listener.highlightMicroDomain(0);
			else
				this.listener.highlightMicroDomain(DataPool.drawingDomain.id);
		}

	}

	private class PaintDomainDialog extends JDialog implements ActionListener {

		JRadioButton b1;

		JRadioButton b2;

		JRadioButton b3;

		JButton ok;

		JButton discard;

		public PaintDomainDialog(final String title) {

			this.b1 = new JRadioButton("Rectangular", Cm2_IconDepot
					.getSpecifiedIcon("Paint_Rect"));
			this.b2 = new JRadioButton("Ellipse", Cm2_IconDepot
					.getSpecifiedIcon("Paint_Ellipse"));
			this.b3 = new JRadioButton("Polygon", Cm2_IconDepot
					.getSpecifiedIcon("Paint_Polygon"));
			this.ok = new JButton("Ok", Cm2_IconDepot
					.getSpecifiedIcon("JMol_Apply"));
			this.discard = new JButton("Discard", Cm2_IconDepot
					.getSpecifiedIcon("JMol_Close"));

			final ButtonGroup bg = new ButtonGroup();
			bg.add(this.b1);
			bg.add(this.b2);
			bg.add(this.b3);

			this.b3.setSelected(true);

			this.b1.addActionListener(this);
			this.b2.addActionListener(this);
			this.b3.addActionListener(this);
			this.ok.addActionListener(this);
			this.discard.addActionListener(this);
			this.ok.setEnabled(false);

			final JPanel main = new JPanel();
			final JPanel sec = new JPanel();

			main.setLayout(new BoxLayout(main, BoxLayout.Y_AXIS));

			this.getContentPane().add(main, BorderLayout.NORTH);

			final JLabel info1 = new JLabel(" Choose Paint Mode");
			info1.setFont(info1.getFont().deriveFont(
					info1.getFont().getSize2D() + 2));

			final JPanel p = new JPanel();
			p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
			p.add(info1);
			p.add(Box.createHorizontalGlue());

			// p.setBackground(p.getBackground().darker());
			main.add(p);

			final JPanel p1 = new JPanel();
			final JPanel p2 = new JPanel();
			final JPanel p3 = new JPanel();

			p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));
			p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
			p3.setLayout(new BoxLayout(p3, BoxLayout.X_AXIS));

			p1.add(this.b1);
			p2.add(this.b2);
			p3.add(this.b3);

			p1.add(Box.createHorizontalGlue());
			p2.add(Box.createHorizontalGlue());
			p3.add(Box.createHorizontalGlue());

			main.add(p3);
			main.add(p1);
			main.add(p2);

			main.add(sec);
			sec.add(this.ok);
			sec.add(this.discard);

			p.setBorder(BorderFactory.createEtchedBorder());

			this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

			this.addWindowListener(new WindowAdapter() {

				public void windowClosing(final WindowEvent e) {
					MembraneView.this.finishDomainMode(false);
				}
			});

			this.setTitle(title);
			this.setAlwaysOnTop(true);
			this.setResizable(false);
			this.pack();

			// b1.setIconTextGap(40);
			// b2.setIconTextGap(66);
			// b3.setIconTextGap(58);

			this.setLocation(MembraneView.this.m3d.getLocationOnScreen());
			this.setVisible(true);

		}

		public void actionPerformed(final ActionEvent arg0) {

			if (arg0.getSource() == this.b1) {
				// b1.setSelected(true);
				// b2.setSelected(false);
				// b3.setSelected(false);
				MembraneView.this.m3d.getMicroDomainManager()
						.setDomainPaintMode(MicroDomain_3D.RECTANGULAR);
			}
			if (arg0.getSource() == this.b2) {
				// b1.setSelected(false);
				// b2.setSelected(true);
				// b3.setSelected(false);
				MembraneView.this.m3d.getMicroDomainManager()
						.setDomainPaintMode(MicroDomain_3D.ELLIPSE);
			}
			if (arg0.getSource() == this.b3) {
				// b1.setSelected(false);
				// b2.setSelected(false);
				// b3.setSelected(true);
				MembraneView.this.m3d.getMicroDomainManager()
						.setDomainPaintMode(MicroDomain_3D.POLYGON);
			}

			if (arg0.getSource() == this.ok) {
				MembraneView.this.finishDomainMode(true);
			}
			if (arg0.getSource() == this.discard) {
				MembraneView.this.finishDomainMode(false);
			}

		}

	}

	public void showRuntimeControls() {

		this.placeholder.setVisible(!this.placeholder.isVisible());

		SwingUtilities.invokeLater(new Thread() {
			public void run() {
				MembraneView.this.doLayout();
			}
		});

	}

	public void startSpecialMode(final Molecule_3D mol, final int mode) {

		// listener.highlightMicroDomain(-1);

		this.listener.setAllEnabled(false);
		this.currentMoleculeAligning = mol;
		this.m3d.getHud().currentMoleculeAligning = this.currentMoleculeAligning;
		this.toolbar3.currentMode = mode;

		this.invalidones = new Vector<Molecule_3D>();

		final Molecule_3D[] mols = DataPool.getMembraneModel()
				.getAllMoleculesInMembrane();

		for (int i = 0; i < mols.length; i++) {

			if (!mols[i].isPositionInAValidSpace())
				this.invalidones.addElement(mols[i]);

			mols[i].setPositionIsInAValidSpace(true);

		}

		this.haveToReInsert = mol.getBranchGroup().getParent() != null;

		if (mode > 0) {

			if (mode == MembraneView.GUI_MODE_MODEL_CHANGE)
				mol.setPicked(false);

			this.m3d.getBasementBranchGroup().removeChild(
					this.m3d.getMainBranchGroup());

			if (mol.getBranchGroup().getParent() == this.m3d
					.getMainBranchGroup())
				this.m3d.getMainBranchGroup().removeChild(mol.getBranchGroup());

			this.m3d.getBasementBranchGroup().addChild(mol.getBranchGroup());

			if (mode == MembraneView.GUI_MODE_ALIGN_SAMPPLELIPID) {

				final Shape3D shape = new Shape3D();
				final BranchGroup br = new BranchGroup();
				Cm2_Toolkit.setAllCapabilitysForShape3D(shape);
				Cm2_Toolkit.setAllCapabilitysForBranchGroup(br);

				shape.setPickable(false);
				br.setUserData(this);
				final Appearance appearance = new Appearance();
				final ColoringAttributes coloringAttrs = new ColoringAttributes(
						new Color3f(
								DataPool.J3DSettings.bright_appearance ? Color.black
										: Color.white),
						ColoringAttributes.SHADE_GOURAUD);
				appearance.setColoringAttributes(coloringAttrs);
				final LineAttributes la = new LineAttributes();
				la.setPatternMask(LineAttributes.PATTERN_DOT);
				la.setLineWidth(1f);
				// PolygonAttributes polyattrs = new PolygonAttributes();
				// polyattrs.setPolygonMode(PolygonAttributes.POLYGON_LINE);
				// appearance.setPolygonAttributes(polyattrs);
				// appearance.setLineAttributes(la);
				shape.setAppearance(appearance);

				final Point3f[] plaPts = new Point3f[2];
				final float h = mol.getUnrotatedBounds().getYdimension();
				plaPts[0] = new Point3f(0, h * 3, 0);
				plaPts[1] = new Point3f(0, -(h * 3), 0);
				final LineArray pla = new LineArray(2, LineArray.COORDINATES);
				pla.setCoordinates(0, plaPts);
				shape.addGeometry(pla);
				pla.setCapability(LineArray.ALLOW_INTERSECT);

				br.addChild(shape);
				this.m3d.getBasementBranchGroup().addChild(br);

			}
		}

		this.toolbar3.buildUp();

		this.getMembrane3D().getHud().setCameraModeWhileAligning(false);

		this.getMembrane3D().getCamera().setEnabled(false);
		this.getMembrane3D().getPickNDragManager().setActive(false);
		this.getMembrane3D().getPickNDragManager().allowDragging(false);

		this.setToolbarsEnabled(false);
		this.toolbar2.setVisible(false);
		this.toolbar3.setVisible(true);
		this.toolbar3.nocamBtn.setSelected(true);
		if (mode < 1) {
			this
					.getMembrane3D()
					.getHud()
					.setMembraneImage(
							Cm2_Toolkit.Membrane_Image2D_Generator
									.create2DImageOfCurrentMembrane(
											DataPool.getMembraneModel(),
											Cm2_Toolkit.Membrane_Image2D_Generator.MODE_POLYGON,
											true, false, false,DataPool.getCurrentLayerID()));
			this.getMembrane3D().getHud().mode = Hud.ALIGN;
		} else if (mode == MembraneView.GUI_MODE_ALIGN_SAMPPLELIPID)
			this.getMembrane3D().getHud().mode = Hud.ALIGNLIPID;
		else if (mode == MembraneView.GUI_MODE_MODEL_CHANGE)
			this.getMembrane3D().getHud().mode = Hud.MODELCHANGE;
		this.m3d.getCanvas().addMouseMotionListener(this);
		this.m3d.getCanvas().addMouseListener(this);

		this.getMembrane3D().getCamera().forwardButton(2,
				this.currentMoleculeAligning);

		final MouseRotate behavior = new MouseRotate() {
			public void transformChanged(final Transform3D arg) {
				final Matrix3f matrix = new Matrix3f();
				arg.get(matrix);
				MembraneView.this.currentMoleculeAligning
						.setRotationMatrix(matrix);

				if (mode < 1)
					MembraneView.this
							.checkAndSetCollisionState(MembraneView.this.currentMoleculeAligning);
				MembraneView.this.handleOkButton();
			}

		};
		behavior.setTransformGroup(this.currentMoleculeAligning
				.getTransformGroup());
		final BranchGroup bg = new BranchGroup();
		bg.addChild(behavior);
		this.currentMoleculeAligning.getTransformGroup().addChild(bg);

		this.currentMoleculeAligning.setMouseBehavior(behavior);

		if (mode == MembraneView.GUI_MODE_ALIGN_PROTEIN) {
			
			if (DataPool.J3DSettings.enableFastTransitions)
				this.getMembrane3D().dockScreen(false);

			if (DataPool.J3DSettings.overallRenderMode == Molecule_3D.RENDER_MODE_POLYGON)
				mol.setRenderMode(Molecule_3D.RENDER_MODE_COVALENT);

			if (mol instanceof Lipid_3D) {
				this.toolbar3.applyOthers.setVisible(false);
				this.toolbar3.resetOthers.setVisible(false);
			}

			final Molecule_3D[] allMols = DataPool.getMembraneModel()
					.getAllMoleculesInMembrane();

			final Atom_3D[] as = mol.getAtoms(false, false);

			float max = 0;
			for (int i = 0; i < as.length; i++) {

				max = Math.max(Math.abs(new Vector3f(as[i].position).length())
						+ (as[i].radiusCov), max);
			}
			max += (40f * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
			if (mol instanceof Lipid_3D)
				max += (20f * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
			final Vector<Molecule_3D> temp = new Vector<Molecule_3D>();
			for (int i = 0; i < allMols.length; i++) {

				if (allMols[i] == this.currentMoleculeAligning)
					continue;

				final Atom_3D[] atoms = allMols[i].getAtoms(true, false);
				boolean in = false;
				for (int k = 0; k < atoms.length; k++) {
					final Vector3f v = new Vector3f(atoms[k].position);
					v.sub(mol.getPosition());

					if (allMols[i] instanceof Protein_3D
							|| ((Math.abs(v.length()) + (atoms[k].radiusCov)) <= max)) {
						temp.addElement(allMols[i]);
						in = true;
						break;
					}

				}

				if (!in)
					this.m3d.getMainBranchGroup().removeChild(
							allMols[i].getBranchGroup());

			}

			int count = 0;
			for (int a = 0; a < temp.size(); a++) {
				if (temp.get(a) instanceof Lipid_3D)
					count += temp.get(a).getAtomCount();
			}

			final boolean tooMany = CM2_CONSTANTS.J3D.CRITICAL_ATOM_COUNT < (count + mol
					.getAtomCount());

			for (int a = 0; a < temp.size(); a++) {
				final Molecule_3D other = temp.get(a);
				if (other instanceof Protein_3D)
					other.setTransparent(true);
				else if (tooMany)
					other.setTransparent(true);
				else if (!tooMany) {
					int m = DataPool.J3DSettings.polygon_mode;
					DataPool.J3DSettings.polygon_mode=PolygonAttributes.POLYGON_LINE;
					other.setRenderMode(Molecule_3D.RENDER_MODE_BALLS_N_STICKS);
					DataPool.J3DSettings.polygon_mode=m;
				}
				other.setPositionIsInAValidSpace(true);
			}
			
//			mol.displayOPMAtoms(true);
			
			
			for(int i=0;i<allMols.length;i++){
				if(allMols[i]!=mol)
			allMols[i].displayOPMAtoms(false);
			}
			if (DataPool.J3DSettings.enableFastTransitions)
				this.getMembrane3D().dockScreen(true);

		} else if (mode == MembraneView.GUI_MODE_ALIGN_SAMPPLELIPID) {
//			if (DataPool.J3DSettings.overallRenderMode != Molecule_3D.RENDER_MODE_BALLS_N_STICKS)
			mol.setRenderMode(Molecule_3D.RENDER_MODE_BALLS_N_STICKS);

		} else if (mode == MembraneView.GUI_MODE_MODEL_CHANGE) {
			

			mol.setPositionIsInAValidSpace(true);
//			if (DataPool.J3DSettings.overallRenderMode == Molecule_3D.RENDER_MODE_POLYGON)
			mol.setRenderMode(Molecule_3D.RENDER_MODE_BALLS_N_STICKS);
		}

		
		
		
		this.currentMoleculeAligning.behavior.setEnable(true);
		this.currentMoleculeAligning.behavior
				.setSchedulingBounds(this.currentMoleculeAligning.getShape()
						.getBounds());

		this.currentMoleculeAligning.behavior.setFactor(0.01d);

		final Protein_3D[] allProts = DataPool.getMembraneModel()
				.getAllProteinsOfType(
						this.currentMoleculeAligning.getPDBcode(),
						this.currentMoleculeAligning.getDomainID(), -1);

		this.otherProteins = new Vector<Protein_3D>();
		this.oldMatrices = new Vector<Matrix3f>();
		this.oldYTranslations = new Vector<Float>();

		this.oldMatrices.addElement(Cm2_Toolkit
				.copyMatrix(this.currentMoleculeAligning.getRotationMatrix()));
		this.oldYTranslations.addElement(this.currentMoleculeAligning
				.getYTranslation());

		this.oldModel = this.currentMoleculeAligning.getModelSet();

		for (int i = 0; i < allProts.length; i++) {
			if (allProts[i] != this.currentMoleculeAligning) {
				this.otherProteins.addElement(allProts[i]);
				this.oldMatrices.addElement(Cm2_Toolkit.copyMatrix(allProts[i]
						.getRotationMatrix()));
				this.oldYTranslations.addElement(allProts[i].getYTranslation());
			}

			this.checkAndSetCollisionState(allProts[i]);

		}
		
		if (mode < 1)
			this.handleOkButton();

		// toolbar3.camBtn.setText("Adjust Camera");

		if (this.otherProteins.isEmpty()) {

			this.toolbar3.applyOthers.setVisible(false);
			this.toolbar3.resetOthers.setVisible(false);
		} else {

			this.toolbar3.applyOthers.setVisible(true);
			this.toolbar3.resetOthers.setVisible(true);
		}

		if (mode == MembraneView.GUI_MODE_MODEL_CHANGE)
			this.toolbar3.camBtn.setSelected(true);
		
		// check the actual molecule for intersections
		this.checkAndSetCollisionState(mol);

	}

	/**
	 * 
	 * @param apply
	 */
	public void finishSpecialMode(final boolean apply) {

		if (DataPool.J3DSettings.enableFastTransitions)
			this.getMembrane3D().dockScreen(false);
		
		if (this.toolbar3.currentMode < 0)
			return;

		if (this.toolbar3.currentMode == MembraneView.GUI_MODE_ALIGN_SAMPPLELIPID
				&& apply) {

			final MembraneModel model = DataPool.getMembraneModel();
			boolean others = false;

			for (int i = 0; i < model.getExampleLipidsCountExtra(); i++) {
				final Molecule_3D mol = model.getExampleLipidAtExtra(i);
				if (mol == this.currentMoleculeAligning)
					continue;
				if (this.currentMoleculeAligning.getPDBFile().getAbsolutePath()
						.equals(mol.getPDBFile().getAbsolutePath()))
					others = true;
			}
			for (int i = 0; i < model.getExampleLipidsCountIntra(); i++) {
				final Molecule_3D mol = model.getExampleLipidAtIntra(i);
				if (mol == this.currentMoleculeAligning)
					continue;
				if (this.currentMoleculeAligning.getPDBFile().getAbsolutePath()
						.equals(mol.getPDBFile().getAbsolutePath()))
					others = true;
			}

			if (others) {

				final int val = JOptionPane
						.showOptionDialog(
								this,
								"Do you want to apply your alignment to any other\n"
										+ "lipid templates of the same type in this membrane?",
								"Finish Alignment",
								JOptionPane.YES_NO_CANCEL_OPTION,
								JOptionPane.QUESTION_MESSAGE, null,
								new String[] { "Yes, to All", "No" },
								"Yes, to All");

				if (val == 0) {

					Matrix3f mExtra;
					Matrix3f mIntra;

					if (model.getAllExampleLipidsExtra().contains(
							this.currentMoleculeAligning)) {
						mExtra = Cm2_Toolkit
								.copyMatrix(this.currentMoleculeAligning
										.getRotationMatrix());
						mIntra = Cm2_Toolkit
								.copyMatrix(this.currentMoleculeAligning
										.getRotationMatrix());
						Cm2_Toolkit.rotate_Z_Axis_Of_CoordinateSystem(mIntra,
								180);
					} else {
						mExtra = Cm2_Toolkit
								.copyMatrix(this.currentMoleculeAligning
										.getRotationMatrix());
						mIntra = Cm2_Toolkit
								.copyMatrix(this.currentMoleculeAligning
										.getRotationMatrix());
						Cm2_Toolkit.rotate_Z_Axis_Of_CoordinateSystem(mExtra,
								180);
					}

					for (int i = 0; i < model.getExampleLipidsCountExtra(); i++) {
						final Molecule_3D mol = model.getExampleLipidAtExtra(i);
						if (mol == this.currentMoleculeAligning)
							continue;
						if (this.currentMoleculeAligning.getPDBFile()
								.getAbsolutePath().equals(
										mol.getPDBFile().getAbsolutePath())) {
							mol.setRotationMatrix(mExtra);
							mol.recalculatePoints();
							mol.move();
						}

					}
					for (int i = 0; i < model.getExampleLipidsCountIntra(); i++) {
						final Molecule_3D mol = model.getExampleLipidAtIntra(i);
						if (mol == this.currentMoleculeAligning)
							continue;
						if (this.currentMoleculeAligning.getPDBFile()
								.getAbsolutePath().equals(
										mol.getPDBFile().getAbsolutePath())) {
							mol.setRotationMatrix(mIntra);
							mol.recalculatePoints();
							mol.move();
						}

					}

				}
			}

		}

		if (this.toolbar3.currentMode > 0) {

			if (this.toolbar3.currentMode == MembraneView.GUI_MODE_MODEL_CHANGE)
				this.currentMoleculeAligning.setPicked(true);

			final Enumeration<?> en = this.m3d.getBasementBranchGroup()
					.getAllChildren();

			while (en.hasMoreElements()) {
				Object o = en.nextElement();

				if (o instanceof BranchGroup) {
					try {

						this.m3d.getBasementBranchGroup().removeChild(
								(BranchGroup) o);
						o = null;
					} catch (final CapabilityNotSetException e) {

					}

				}
			}

			if (this.haveToReInsert) {
				this.m3d.getMainBranchGroup().addChild(
						this.currentMoleculeAligning.getBranchGroup());
			}

			this.m3d.getBasementBranchGroup().addChild(
					this.m3d.getMainBranchGroup());

		}

		// if (DataPool.J3DSettings.enableFastTransitions)
		// this.getMembrane3D().dockScreen(false);

		this.listener.setAllEnabled(true);

		if (!apply) {
			this.currentMoleculeAligning.setRotationMatrix(this.oldMatrices
					.firstElement());
			this.currentMoleculeAligning.setYTranslation(this.oldYTranslations
					.firstElement());

			if (this.currentMoleculeAligning.getModelSet() != this.oldModel) {

				this.currentMoleculeAligning.changeAndApplyModel(this.oldModel);
				if (DataPool.J3DSettings.overallRenderMode == this.currentMoleculeAligning
						.getRenderMode())
					this.currentMoleculeAligning.setRenderMode(null);

			}
			this.currentMoleculeAligning.setRotationMatrix(this.oldMatrices
					.firstElement());
			this.currentMoleculeAligning.move();

			for (int i = 0; i < this.otherProteins.size(); i++) {
				this.otherProteins.get(i).setRotationMatrix(
						this.oldMatrices.get(i + 1));
				this.otherProteins.get(i).setYTranslation(
						this.oldYTranslations.get(i + 1));
				this.otherProteins.get(i).move();

			}
		} else {

			if (this.toolbar3.currentMode == MembraneView.GUI_MODE_MODEL_CHANGE) {
				this.currentMoleculeAligning.setRotationMatrix(this.oldMatrices
						.firstElement());
				this.currentMoleculeAligning.move();
				this.listener.userChangedModelForFile(
						this.currentMoleculeAligning.getPDBFile(),
						this.currentMoleculeAligning.getModelSet());
			}
			DataPool.getMembraneModel().setMembraneHasBeenEditedAgain(true);
			

		}

		final Molecule_3D[] allMols = DataPool.getMembraneModel()
				.getAllMoleculesInMembrane();
		for (int i = 0; i < allMols.length; i++) {

			
			allMols[i].setRenderMode(DataPool.J3DSettings.overallRenderMode);
			
			if (allMols[i].getBranchGroup().getParent() == null) {
				this.m3d.getMainBranchGroup().addChild(
						allMols[i].getBranchGroup());
			}
			if (allMols[i].getLayerID() == this.currentMoleculeAligning.getLayerID()
					&& allMols[i].isTransparent())
				allMols[i].setTransparent(false);
			if (allMols[i].getLayerID() != this.currentMoleculeAligning.getLayerID())
				if (DataPool.J3DSettings.showAllMultiLayers)
					allMols[i].setTransparent(false, false);
				else
					allMols[i].setTransparent(true);
			if (allMols[i].getLayerID() == this.currentMoleculeAligning.getLayerID()) {
				allMols[i].setPositionIsInAValidSpace(true);
			}
		}

		for (int i = 0; i < this.invalidones.size(); i++) {
			this.invalidones.get(i).setPositionIsInAValidSpace(false);

		}

		this.invalidones.clear();

		final int olddomain = this.currentMoleculeAligning.getDomainID();

		this.currentMoleculeAligning.behavior.setEnable(false);

		this.otherProteins = null;
		this.oldMatrices = null;
		this.oldYTranslations = null;

		this.getMembrane3D().getCamera().setEnabled(true);
		this.getMembrane3D().getPickNDragManager().setActive(true);

		if (this.getMembrane3D().getCamera().getMode() == CameraMode.FREE)
			this.getMembrane3D().getPickNDragManager().allowDragging(
					!DataPool.J3DSettings.forbidDraggingInFreeLookMode);

		else
			this.getMembrane3D().getPickNDragManager().allowDragging(true);

		if (this.toolbar3.currentMode == MembraneView.GUI_MODE_MODEL_CHANGE
				&& apply) {
			
			this.currentMoleculeAligning.recalculatePoints();

			// if (DataPool.J3DSettings.enableFastTransitions)
			// this.getMembrane3D().dockScreen(true);
		}


		for (int i = 0; i < allMols.length; i++) {
			allMols[i].displayOPMAtoms(this.toggleOPMLayersButton.isSelected());
		}
		
		this.toolbar3.currentMode = -1;
		this.currentMoleculeAligning = null;
		this.m3d.getHud().currentMoleculeAligning = null;
		this.setToolbarsEnabled(true);
		this.toolbar3.setVisible(false);
		this.toolbar2.setVisible(true);
		this.getMembrane3D().getHud().setMembraneImage(null);
		this.getMembrane3D().getHud().mode = Hud.NORMAL;

		this.m3d.getCanvas().removeMouseMotionListener(this);
		this.m3d.getCanvas().removeMouseListener(this);

		this.listener.highlightMicroDomain(olddomain);

		this.extraBtn.doClick();

		this.toolbar3.currentMode = -1;
		this.listener.refreshViews(true, true, false, false, true, true);
		
		if (DataPool.J3DSettings.enableFastTransitions)
			this.getMembrane3D().dockScreen(true);

	}

	class MyAlignButtonBar extends JPanel implements ActionListener {

		
		JButton ok = new JButton("OK");
		JButton applyOthers = new JButton("Apply to Others");
		JButton resetOthers = new JButton("Reset Others");
		
		// ===============================================================
		// TODO: PPE Update 03_2012										  |
		// ===============================================================
		JButton setPDBAlignmentButton = new JButton("PDB Alignment");
		JButton setPDBTMAlignmentButton = new JButton("PDBTM Alignment");
		JButton setOPMAlignmentButton = new JButton("OPM Alignment");
		JButton setFlipButton = new JButton("Flip Protein");


		JButton cancel = new JButton("Cancel");
		JButton enter = new JButton("Enter...");
		JButton reset = new JButton("Reset");
		JButton left = new JButton();
		JButton right = new JButton();
		JComboBox combo = new JComboBox();
		JButton rotateUpwards = new JButton("Auto Rotation");
		MyJRadioButton camBtn = new MyJRadioButton(3);
		MyJRadioButton nocamBtn = new MyJRadioButton(4);
		
		int directionPDBTMAlignment = 0;

		public int currentMode = -1;

		public MyAlignButtonBar() {
			super();

			this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

	

			this.ok.addActionListener(this);
			this.applyOthers.addActionListener(this);
			this.resetOthers.addActionListener(this);
			this.cancel.addActionListener(this);
			this.enter.addActionListener(this);
			this.reset.addActionListener(this);

			this.left.addActionListener(this);
			this.right.addActionListener(this);
			this.rotateUpwards.addActionListener(this);
			
			
			// ===============================================================
			// TODO: PPE Update 03_2012										  |
			// ===============================================================
			this.setPDBAlignmentButton.addActionListener(this);
			this.setPDBAlignmentButton.setToolTipText
				("Apply this molecule's original PDB alignment and center it about the membrane.");
			this.setPDBTMAlignmentButton.addActionListener(this);
			this.setPDBTMAlignmentButton.setToolTipText
				("Apply this molecule's PDBTM alignment.");
			this.setOPMAlignmentButton.addActionListener(this);
			this.setOPMAlignmentButton.setToolTipText
				("Apply this molecule's OPM alignment.");
			this.setFlipButton.addActionListener(this);
			this.setFlipButton.setToolTipText
				("Change membrane side of this protein immediately (180).");
			
			
			this.camBtn.addActionListener(this);
			this.nocamBtn.addActionListener(this);

			final ButtonGroup bg = new ButtonGroup();

			bg.add(this.nocamBtn);
			bg.add(this.camBtn);

			this.ok.setIcon(Cm2_IconDepot.getSpecifiedIcon("JMol_Apply"));

			this.cancel.setIcon(Cm2_IconDepot.getSpecifiedIcon("JMol_Close"));

			this.left.setIcon(Cm2_IconDepot.getSpecifiedIcon("CV_left"));
			this.right.setIcon(Cm2_IconDepot.getSpecifiedIcon("CV_closed"));

		}

		public void buildUp() {

			MembraneView.this.toolbar3.removeAll();

			if (this.currentMode == MembraneView.GUI_MODE_ALIGN_PROTEIN) {
				
				
				// ===============================================================
				// TODO: PPE Update 03_2012										  |
				// ===============================================================
				// following buttons should be visible in special cases
				this.setPDBTMAlignmentButton.setVisible(false);
				this.setOPMAlignmentButton.setVisible(false);
				// following buttons should always be visible
				this.setPDBAlignmentButton.setVisible(true);
				this.setFlipButton.setVisible(true);
				// switching PDBTM or OPM alignment button visible when possible
				String pdbcode = null;
				if(MembraneView.this.currentMoleculeAligning.opmAtoms.length > 0) {
					// molecule is a OPM file
					this.setOPMAlignmentButton.setVisible(true);
				} else {
					// following also becomes null when molecule is a OPM File
					pdbcode = MembraneView.this.currentMoleculeAligning.getPDBcode().toLowerCase();
				}
				if((pdbcode != null) && (DataPool.getMifHandler().isElementOfPDBTM(pdbcode))) {
					// molecule is a PDBTM file
					this.setPDBTMAlignmentButton.setVisible(true);
				}
				
				
				this.add(this.ok);
				this.add(Box.createHorizontalStrut(6));
				this.add(this.cancel);

				this.add(Box.createHorizontalGlue());
				
				
				// ===============================================================
				// TODO: PPE Update 03_2012										  |
				// ===============================================================
				this.add(Box.createHorizontalStrut(12));
				this.add(this.setPDBAlignmentButton);
				this.add(Box.createHorizontalStrut(6));
				this.add(this.setFlipButton);
				this.add(Box.createHorizontalStrut(12));
				// only one of the following buttons is visible at the same time
				this.add(this.setPDBTMAlignmentButton);
				this.add(this.setOPMAlignmentButton);
				if(this.setPDBTMAlignmentButton.isVisible() || this.setOPMAlignmentButton.isVisible()) {
					this.add(Box.createHorizontalStrut(12));
				}
				
				
				this.add(Box.createHorizontalGlue());
				
				this.add(this.enter);
				this.add(Box.createHorizontalStrut(6));
				this.add(this.reset);
				
				this.add(Box.createHorizontalGlue());
				
				this.add(Box.createHorizontalStrut(12));
				this.add(this.applyOthers);
				this.add(Box.createHorizontalStrut(6));
				this.add(this.resetOthers);
				this.add(Box.createHorizontalStrut(12));
				
				this.add(Box.createHorizontalGlue());
				
				this.add(this.nocamBtn);
				this.add(this.camBtn);
				
			} else if (this.currentMode == MembraneView.GUI_MODE_ALIGN_SAMPPLELIPID) {
				this.add(this.ok);
				this.add(Box.createHorizontalStrut(6));
				this.add(this.cancel);
				this.add(Box.createHorizontalStrut(10));

				this.add(this.rotateUpwards);
				this.add(Box.createHorizontalStrut(6));
				this.add(this.reset);
				this.add(Box.createHorizontalGlue());
				this.add(this.nocamBtn);

			} else if (this.currentMode == MembraneView.GUI_MODE_MODEL_CHANGE) {
				
				this.combo.removeActionListener(this);
				this.combo.removeAllItems();
				for (int i = 0; i < MembraneView.this.currentMoleculeAligning
						.getModelcount(); i++) {
					this.combo.addItem("Model " + (i + 1));
				}
				this.combo
						.setSelectedIndex(MembraneView.this.currentMoleculeAligning
								.getModelSet() - 1);
				this.combo.addActionListener(this);

				this.add(this.ok);
				this.add(Box.createHorizontalStrut(6));
				this.add(this.cancel);
				this.add(Box.createHorizontalGlue());
				// this.add(left);
				this.add(this.combo);
				// this.add(right);
				this.add(Box.createHorizontalGlue());
				this.add(this.camBtn);

			}

		}

		public void actionPerformed(final ActionEvent e) {
			Molecule_3D mol = MembraneView.this.currentMoleculeAligning;
			if (e.getSource() == this.ok) {
				// show all dummy atoms again if they are hidden away
//				if (opmSavedDummyPositions!=null && !opmSavedDummyPositions.isEmpty()) {
//					Atom[] allAtomsOfCurrentMolecule = MembraneView.this.currentMoleculeAligning.atoms;
//					int j=0;
//					for (int i=0;i<allAtomsOfCurrentMolecule.length;i++) {
//						if(allAtomsOfCurrentMolecule[i].residueName.equals("DUM")) {
//							allAtomsOfCurrentMolecule[i].setPosition(opmSavedDummyPositions.get(j++));
//						}
//					}
//					MembraneView.this.currentMoleculeAligning.updateGeometry();
//					opmSavedDummyPositions.removeAllElements();
//					opmSavedDummyPositions = null;
//				}
				MembraneView.this.finishSpecialMode(true);

			} else if (e.getSource() == this.cancel) {
				// show all dummy atoms again if they are hidden away
//				if (opmSavedDummyPositions!=null && !opmSavedDummyPositions.isEmpty()) {
//					Atom[] allAtomsOfCurrentMolecule = MembraneView.this.currentMoleculeAligning.atoms;
//					int j=0;
//					for (int i=0;i<allAtomsOfCurrentMolecule.length;i++) {
//						if(allAtomsOfCurrentMolecule[i].residueName.equals("DUM")) {
//							allAtomsOfCurrentMolecule[i].setPosition(opmSavedDummyPositions.get(j++));
//						}
//					}
//					MembraneView.this.currentMoleculeAligning.updateGeometry();
//					opmSavedDummyPositions.removeAllElements();
//					opmSavedDummyPositions = null;
//				}
				MembraneView.this.finishSpecialMode(false);
			}

			
			// filling the alignment buttons with functionality!
			// ===============================================================
			// TODO: PPE Update 03_2012										  |
			// ===============================================================
			else if(e.getSource() == this.setPDBTMAlignmentButton)
				{
				// reset of PDBTM translation and rotation
				mol.setRotationMatrix(mol.getPDBTMRotation());
				mol.setYTranslation(mol.getPDBTMTranslation());
				// update transform group
				mol.move();
				MembraneView.this.checkAndSetCollisionState(mol);
				MembraneView.this.listener
					.setShowAxis(MembraneView.this.listener.areAxisVisible());
				MembraneView.this.handleOkButton();
				} 
			else if(e.getSource() == this.setOPMAlignmentButton)
				{
				// reset of OPM alignment
				DataPool.getMifHandler().applyOPMAlignmentTo(mol);
				MembraneView.this.checkAndSetCollisionState(mol);
				MembraneView.this.listener
					.setShowAxis(MembraneView.this.listener.areAxisVisible());
				MembraneView.this.handleOkButton();
				}
			else if(e.getSource() == this.setPDBAlignmentButton)
				{
				// set translation and rotation neutral
				mol.setRotationMatrix(Cm2_Toolkit.createAUnitMatrix());
				mol.setYTranslation(0f);
				// update transform group
				mol.move();
				MembraneView.this.checkAndSetCollisionState(mol);
				MembraneView.this.listener
					.setShowAxis(MembraneView.this.listener.areAxisVisible());
				MembraneView.this.handleOkButton();
				} 
			else if(e.getSource() == this.setFlipButton)
				{
				// rotation about z-axis (local coordinates)
				// when protein is in origin PDB alignment
				Matrix3f zRotation = new Matrix3f(-1, 0, 0,
												   0,-1, 0,
												   0, 0, 1);
				Matrix3f rotation = mol.getRotationMatrix();
				// transformation
				zRotation.mul(rotation);
				mol.setRotationMatrix(zRotation);
				mol.setYTranslation(-mol.getYTranslation());
				// update transform group
				mol.move();
				MembraneView.this.checkAndSetCollisionState(mol);
				MembraneView.this.listener
					.setShowAxis(MembraneView.this.listener.areAxisVisible());
				MembraneView.this.handleOkButton();
				} 
			
			
			else if (e.getSource() == this.camBtn
					|| e.getSource() == this.nocamBtn) {

				if (this.camBtn.isSelected()) {
					MembraneView.this.getMembrane3D().getHud()
							.setCameraModeWhileAligning(true);
					MembraneView.this.getMembrane3D().getCamera()
							.setEnabled(true);
					final Point3f center = new Point3f(
							MembraneView.this.currentMoleculeAligning
									.getPosition());
					center.scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
					MembraneView.this.getMembrane3D().getCamera()
							.setViewByMode(CameraMode.FREE, center);
					MembraneView.this.currentMoleculeAligning.behavior
							.setEnable(false);
					
				} else {
					MembraneView.this.getMembrane3D().getHud()
							.setCameraModeWhileAligning(false);
					MembraneView.this.getMembrane3D().getCamera()
							.setEnabled(false);

					MembraneView.this.currentMoleculeAligning.behavior
							.setEnable(true);

				}

				MembraneView.this.getMembrane3D().getCanvas().repaint();
			} else if (e.getSource() == this.enter) {

				final EnterMatrixDialog matrixpopup = new EnterMatrixDialog(
						MembraneView.this.currentMoleculeAligning, this
								.getLocationOnScreen());

				final Matrix3f newmatrix = matrixpopup.getNewMatrix();
				if (newmatrix == null)
					return;

				MembraneView.this.currentMoleculeAligning
						.setRotationMatrix(newmatrix);
				MembraneView.this.currentMoleculeAligning
						.setYTranslation(matrixpopup.getNewYTranslation());
				MembraneView.this.currentMoleculeAligning.move();
				MembraneView.this
						.checkAndSetCollisionState(MembraneView.this.currentMoleculeAligning);
				MembraneView.this.listener
						.setShowAxis(MembraneView.this.listener
								.areAxisVisible());
				MembraneView.this.handleOkButton();

			} else if (e.getSource() == this.applyOthers) {

				for (int i = 0; i < MembraneView.this.otherProteins.size(); i++) {
					MembraneView.this.otherProteins
							.get(i)
							.setRotationMatrix(
									Cm2_Toolkit
											.copyMatrix(MembraneView.this.currentMoleculeAligning
													.getRotationMatrix()));
					MembraneView.this.otherProteins.get(i).setYTranslation(
							MembraneView.this.currentMoleculeAligning
									.getYTranslation());
					MembraneView.this.otherProteins.get(i).move();
					MembraneView.this
							.checkAndSetCollisionState(MembraneView.this.otherProteins
									.get(i));
					MembraneView.this.listener
							.setShowAxis(MembraneView.this.listener
									.areAxisVisible());
				}

				MembraneView.this.handleOkButton();

			} else if (e.getSource() == this.resetOthers) {
				for (int i = 0; i < MembraneView.this.otherProteins.size(); i++) {
					MembraneView.this.otherProteins.get(i).setRotationMatrix(
							MembraneView.this.oldMatrices.get(i + 1));
					MembraneView.this.otherProteins.get(i).setYTranslation(
							MembraneView.this.oldYTranslations.get(i + 1));
					MembraneView.this.otherProteins.get(i).move();
					MembraneView.this
							.checkAndSetCollisionState(MembraneView.this.otherProteins
									.get(i));
					MembraneView.this.listener
							.setShowAxis(MembraneView.this.listener
									.areAxisVisible());
				}
				MembraneView.this.handleOkButton();

			} else if (e.getSource() == this.reset) {
				MembraneView.this.currentMoleculeAligning
						.setRotationMatrix(MembraneView.this.oldMatrices
								.firstElement());
				MembraneView.this.currentMoleculeAligning
						.setYTranslation(MembraneView.this.oldYTranslations
								.firstElement());
				MembraneView.this.currentMoleculeAligning.move();
				if (MembraneView.this.toolbar3.currentMode == MembraneView.GUI_MODE_ALIGN_PROTEIN) {
					MembraneView.this
							.checkAndSetCollisionState(MembraneView.this.currentMoleculeAligning);
					MembraneView.this.listener
							.setShowAxis(MembraneView.this.listener
									.areAxisVisible());
					MembraneView.this.handleOkButton();
				}
			} else if (e.getSource() == this.combo) {

				final int model = this.combo.getSelectedIndex();
				
				if ((model+1) == MembraneView.this.currentMoleculeAligning
						.getModelSet())
					return;
				
				// do not update the position during changing the model (-> var: false)
				MembraneView.this.currentMoleculeAligning
						.changeAndApplyModel(this.combo.getSelectedIndex() + 1, false);

			}
			// else if(e.getSource()==left) {
			// int model = combo.getSelectedIndex()+1;
			//				
			// if(model==1)combo.setSelectedIndex(combo.getItemCount()-1);
			// else combo.setSelectedIndex(model-1);
			//				
			//			
			// }
			// else if(e.getSource()==right) {
			// int model = combo.getSelectedIndex()+1;
			//				
			// if(model==combo.getItemCount())combo.setSelectedIndex(0);
			// else combo.setSelectedIndex(model);
			//				
			//			
			// }
			else if (e.getSource() == this.rotateUpwards) {

				final Matrix3f matrix = PdbReader_3D
						.autorotateUpwards((Lipid_3D) MembraneView.this.currentMoleculeAligning);

				if (!DataPool.getMembraneModel().getAllExampleLipidsExtra()
						.contains(MembraneView.this.currentMoleculeAligning))
					Cm2_Toolkit.rotate_Z_Axis_Of_CoordinateSystem(matrix, 180);

				MembraneView.this.currentMoleculeAligning
						.setRotationMatrix(matrix);
				MembraneView.this.currentMoleculeAligning.move();

			}

		}

	}

	public int getCurrentGuiMode() {

		return this.toolbar3.currentMode;
	}

	public void mouseDragged(final MouseEvent e) {

		if (e.getModifiers() != MouseEvent.BUTTON3_MASK
				|| this.toolbar3.currentMode != MembraneView.GUI_MODE_ALIGN_PROTEIN)
			return;
		if (this.oldCursor == null
				|| this.getMembrane3D().getCamera().isEnabled())
			return;

		final Point newPoint = e.getPoint();

		final int factor = this.oldCursor.y - newPoint.y;

		final float oldtrans = this.currentMoleculeAligning.getYTranslation();

		this.currentMoleculeAligning
				.setYTranslation(this.currentMoleculeAligning.getYTranslation()
						+ (float) (factor * (this.translating_Distance / 50)));
		this.currentMoleculeAligning.move();

		this.checkAndSetCollisionState(this.currentMoleculeAligning);
		this.handleOkButton();

		final float diff = this.currentMoleculeAligning.getYTranslation()
				- oldtrans;
		this.getMembrane3D().getCamera().moveY(
				diff * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);

		this.oldCursor = newPoint;
	}

	public void mouseMoved(final MouseEvent e) {

	}

	public void mouseClicked(final MouseEvent e) {

	}

	public void mouseEntered(final MouseEvent e) {

	}

	public void mouseExited(final MouseEvent e) {

	}

	public void mousePressed(final MouseEvent e) {

		if (e.getButton() != MouseEvent.BUTTON3)
			return;

		this.oldCursor = e.getPoint();
		final Point3d eye_pos = new Point3d();
		this.getMembrane3D().getCanvas().getCenterEyeInImagePlate(eye_pos);
		final Transform3D motion = new Transform3D();
		this.getMembrane3D().getCanvas().getImagePlateToVworld(motion);
		motion.transform(eye_pos);

		final Point3d pos = new Point3d(this.currentMoleculeAligning
				.getPosition());

		pos.scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);

		this.translating_Distance = eye_pos.distance(pos);
	}

	public void mouseReleased(final MouseEvent e) {

		this.oldCursor = null;

		if (this.toolbar3.currentMode == MembraneView.GUI_MODE_ALIGN_PROTEIN)
			this.listener.setShowAxis(this.listener.areAxisVisible());

	}

	private void checkAndSetCollisionState(final Molecule_3D currentMoleculeAligning2) {
		if (this.getCurrentGuiMode() != MembraneView.GUI_MODE_ALIGN_PROTEIN)
			return;

		if ((this.getMembrane3D().getIntersectManager()
				.intercectsBorderOrAnyMolecule(currentMoleculeAligning2, true))) {
			currentMoleculeAligning2.setPositionIsInAValidSpace(false);
			this.m3d.getHud().setCurrentlyColliding(true);

		} else {
			currentMoleculeAligning2.setPositionIsInAValidSpace(true);
			this.m3d.getHud().setCurrentlyColliding(false);

		}

	}

	private void handleOkButton() {

		// if (this.currentMoleculeAligning.isPositionisinavalidspace() ==
		// false) {
		// toolbar3.ok.setEnabled(false);
		// return;
		// }

		// for (int i = 0; i < this.otherProteins.size(); i++) {
		// if (this.otherProteins.get(i).isPositionisinavalidspace() == false) {
		// toolbar3.ok.setEnabled(false);
		// return;
		// }
		// }
		// toolbar3.ok.setEnabled(true);
	}

	public void updateLayers() {

		this.multilayerpanel.update();

		if (DataPool.getMembraneModel().hasMultipleLayers() == false)
			this.placeholder.setVisible(false);
		else
			this.placeholder.setVisible(true);

	}

	class MultiLayerPanel extends JPanel implements ActionListener {

		JComboBox combo = new JComboBox(new DefaultComboBoxModel());
		JLabel hlabel = new JLabel();
		// JSpinner spinner = new JSpinner(new
		// SpinnerNumberModel(0,(CM2_CONSTANTS.MEMBRANE_SIZES.MAX_Y_TRANSLATION*-1)/2,CM2_CONSTANTS.MEMBRANE_SIZES.MAX_Y_TRANSLATION/2,));
		JButton remove = new JButton("Remove Layer");
		JButton changeHeight = new JButton("Change Y-height");

		public MultiLayerPanel() {
			this.setBorder(BorderFactory.createEtchedBorder());

			this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			this.changeHeight.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("MV_ChangeHeight"));
			this.remove.setIcon(Cm2_IconDepot.getSpecifiedIcon("MV_Remove"));
			this.add(new JLabel("  Active Bilayer:  "));
			this.add(this.combo);
			this.add(Box.createHorizontalGlue());

			this.add(this.hlabel);
			this.add(Box.createHorizontalStrut(10));
			this.add(this.changeHeight);
			this.add(Box.createHorizontalStrut(5));
			this.add(this.remove);
			this.combo.setMinimumSize(new Dimension(140, 30));
			this.combo.setMaximumSize(new Dimension(140, 30));
			
			this.combo.addActionListener(this);
			this.remove.addActionListener(this);
			this.changeHeight.addActionListener(this);
			
			
			
//			this.combo.setRenderer(new DefaultListCellRenderer() {
//				public Component getListCellRendererComponent(final JList list,
//						final Object value, int index, final boolean isSelected,
//						final boolean cellHasFocus) {
//					final Component comp = super.getListCellRendererComponent(list,
//							value, index, isSelected, cellHasFocus);
//					final JLabel label = new JLabel();
//					
//					label.setOpaque(true);
//					label.setForeground(comp.getForeground());
//					label.setBackground(comp.getBackground());
//					
////					int layerID=0;
////					try {
////						layerID=Integer.parseInt(((String)value).replaceFirst("Layer","").trim());
////					}catch(Exception e) {
////						
////					}
//					
////				BufferedImage bi =	Cm2_Toolkit.Membrane_Image2D_Generator.create2DImageOfCurrentMembrane(DataPool.getMembraneModel(),Cm2_Toolkit.Membrane_Image2D_Generator.MODE_POLYGON,true,false,false,layerID);
////					label.setIcon(new ImageIcon(bi.getScaledInstance(22,22,BufferedImage.SCALE_FAST)));
//					
//					
//					label.setText("  "+(String) value);
//					label.setFont(label.getFont().deriveFont(Font.BOLD));
//
//					return label;
//				}
//			});
		}

		public void update() {
			final MembraneModel model = DataPool.getMembraneModel();

			final Layer current = DataPool.getCurrentLayer();
			this.combo.removeActionListener(this);
			this.combo.removeAllItems();

			this.combo.addItem("Default Layer");

			for (int i = 0; i < model.getLayerCount(); i++) {
				this.combo.addItem("Layer " + model.getLayer(i).getLayerId());

			}

			if (current != null) {
				int a = 0;
				for (int i = 0; i < this.combo.getItemCount(); i++) {

					try {
						String s = (String) this.combo.getItemAt(i);
						s = s.replaceAll("Layer", "").trim();

						final int id = Integer.parseInt(s);
						if (id == current.getLayerId()) {
							a = i;
							break;
						}

					} catch (final Exception e) {

					}

				}
				this.combo.setSelectedIndex(a);
				this.hlabel.setText("Y-Translation: "
						+ (current.getYHeight() <= 0 ? current.getYHeight()
								: "+" + current.getYHeight()) + "  ");
				this.hlabel.setVisible(true);
				this.changeHeight.setVisible(true);
				this.remove.setVisible(true);

			} else {

				this.combo.setSelectedIndex(0);

				this.hlabel.setVisible(false);
				this.changeHeight.setVisible(false);
				this.remove.setVisible(false);

			}

			this.combo.addActionListener(this);
		}

		public void actionPerformed(final ActionEvent e) {

			if (e.getSource() == this.combo) {

				String s = (String) this.combo.getSelectedItem();

				// int oldLayer=0;
				final float oldY = DataPool.getCurrentLayerHeight();
				// if(DataPool.getCurrentLayer()!=null)
				// oldLayer=DataPool.getCurrentLayer().getLayerId();

				if (s.startsWith("Default")) {

					DataPool.setSelectedLayer(0);
					// if(oldLayer!=0) {
					MembraneView.this.m3d.getMainBranchGroup().detach();
					DataPool.getMembraneModel().translateAllMolecules(
							-oldY + DataPool.getCurrentLayerHeight(), true,
							true);
					MembraneView.this.m3d.getBasementBranchGroup().addChild(
							MembraneView.this.m3d.getMainBranchGroup());
					MembraneView.this.m3d.updateAfterLayerChange();
					// }

				} else {
					s = s.replaceAll("Layer", "").trim();

					DataPool.setSelectedLayer(Integer.parseInt(s));
					// if(oldLayer!=Integer.parseInt(s)) {

					MembraneView.this.m3d.getMainBranchGroup().detach();
					DataPool.getMembraneModel().translateAllMolecules(
							-oldY + DataPool.getCurrentLayerHeight(), true,
							true);
					MembraneView.this.m3d.getBasementBranchGroup().addChild(
							MembraneView.this.m3d.getMainBranchGroup());

					// =======
					// if (DataPool.J3DSettings.enableFastTransitions)
					// m3d.dockScreen(false);
					// DataPool.setSelectedLayer(0);
					// // if(oldLayer!=0) {
					// DataPool.getMembraneModel().translateAllMolecules(-oldY+DataPool.getCurrentLayerHeight(),true,true);
					// >>>>>>> 1.111
					MembraneView.this.m3d.updateAfterLayerChange();

					// }
					// =======
					// if (DataPool.J3DSettings.enableFastTransitions)
					// m3d.dockScreen(true);
					// // }
					//				
					// }else {
					// if (DataPool.J3DSettings.enableFastTransitions)
					// m3d.dockScreen(false);
					// s=s.replaceAll("Layer","").trim();
					//					
					// DataPool.setSelectedLayer(Integer.parseInt(s));
					// // if(oldLayer!=Integer.parseInt(s)) {
					// DataPool.getMembraneModel().translateAllMolecules(-oldY+DataPool.getCurrentLayerHeight(),true,true);
					// m3d.updateAfterLayerChange();
					// if (DataPool.J3DSettings.enableFastTransitions)
					// m3d.dockScreen(true);
					// }
					// >>>>>>> 1.111
				}

				MembraneView.this.listener.refreshViews(true, true, true, true,
						true, false);

			} else if (e.getSource() == this.remove) {

				final int val = JOptionPane.showConfirmDialog(this.remove,
						"Are you sure?", "Confirm removing",
						JOptionPane.YES_NO_OPTION);
				if (val == JOptionPane.NO_OPTION)
					return;

				// =======
				//				
				//				
				// listener.refreshViews(true,true,true,true,true);
				//				
				//				
				// }else if(e.getSource()==remove) {
				//				
				// int val =
				// JOptionPane.showConfirmDialog(remove,"Are you sure?","Confirm removing",JOptionPane.YES_NO_OPTION);
				// if(val==JOptionPane.NO_OPTION)return;
				//				
				// if (DataPool.J3DSettings.enableFastTransitions)
				// m3d.dockScreen(false);
				//				
				// >>>>>>> 1.111
				final MembraneModel model = DataPool.getMembraneModel();
				final Layer layer = DataPool.getCurrentLayer();
				final float oldY = layer.getYHeight();

				MembraneView.this.m3d.clearAll(true, true, true, layer
						.getLayerId());
				MembraneView.this.m3d.clearSelection();

				for (int i = model.countMicroDomains() - 1; i > -1; i--) {
					if (model.getMicroDomain(i).layerId == layer.getLayerId())
						MembraneView.this.listener.removeMicroDomain(model
								.getMicroDomain(i));
				}

				DataPool.getMembraneModel().removeLayer(layer);
				DataPool.setSelectedLayer(0);
				MembraneView.this.m3d.getMainBranchGroup().detach();
				DataPool.getMembraneModel().translateAllMolecules(
						-oldY + DataPool.getCurrentLayerHeight(), true, true);
				MembraneView.this.m3d.getBasementBranchGroup().addChild(
						MembraneView.this.m3d.getMainBranchGroup());
				DataPool.getMembraneModel().organizeDomains();
				MembraneView.this.m3d.updateAfterLayerChange();

				MembraneView.this.listener.refreshViews(true, true, true, true,
						true, true);

			} else if (e.getSource() == this.changeHeight) {

			
				final Double height = new Double(MembraneView.this.askForYHeight(new Double(DataPool.getCurrentLayer().getYHeight())));

				if (height == null)
					return;

				final float old = DataPool.getCurrentLayer().getYHeight();
				
				
				Molecule_3D[] mols = DataPool.getMembraneModel().getAllMoleculesInMembrane(-1,DataPool.getCurrentLayerID());
				DataPool.getCurrentLayer().setYHeight(new Float(height));
				double diff=height-old;
				 m3d.dockScreen(false);
				
				
				DataPool.getMembraneModel().translateAllMolecules(
						new Float(-old + height), true, false);
			
				for(int i=0;i<mols.length;i++) {
					mols[i].setYTranslation(new Float(mols[i].getYTranslation()+diff));
					mols[i].move();
					}
				
				

				DataPool.getMembraneModel().setMembraneHasBeenEditedAgain(true);
			

				MembraneView.this.listener.refreshViews(true, true, true, true,
						true, true);
				 m3d.dockScreen(true);
			}

			
		}

	}

	class AskForYHeightDialog extends JDialog implements ActionListener {
		JPanel main = new JPanel();

		public JButton cancelBtn, okBtn;

		JPanel buttonbar;

		JPanel header1, header2;

		JLabel lx;

		public JSpinner tfx;
		public Double returnValue = null;
		public Double originalValue = null;
		JPanel groundxz;

		JLabel h1;

		MembraneModel model;

		public AskForYHeightDialog(Double currentHeight) {

			this.originalValue = currentHeight;
			
			this.setTitle("Enter Y-Translation");

			this.tfx = new JSpinner();

			this.lx = new JLabel("Y");

			this.groundxz = new JPanel();

			this.buttonbar = new JPanel();

			this.header1 = new JPanel();
			this.header2 = new JPanel();

			this.tfx.setModel(new SpinnerNumberModel(0,
					CM2_CONSTANTS.MEMBRANE_SIZES.MIN_Y_TRANSLATION+
							CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.STANDARD_MULTILAYERY_MOLECULE_HEIGHT,
					CM2_CONSTANTS.MEMBRANE_SIZES.MAX_Y_TRANSLATION-
							CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.STANDARD_MULTILAYERY_MOLECULE_HEIGHT, 
					1));

			this.h1 = new JLabel("Enter a height for the layer (0=Default Layer)",
					JLabel.LEADING);

			this.header1.add(this.h1);

			this.cancelBtn = new JButton("Cancel");

			this.okBtn = new JButton("Ok");

			this.setModal(true);

			this.getContentPane().setLayout(
					new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));

			this.getContentPane().add(this.header1);
			this.getContentPane().add(this.header2);

			this.getContentPane().add(this.groundxz);
			this.getContentPane().add(this.buttonbar);

			this.buttonbar.add(this.okBtn);
			this.buttonbar.add(this.cancelBtn);

			final String mm = (CM2_CONSTANTS.MEMBRANE_SIZES.MIN_Y_TRANSLATION+
										CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.STANDARD_MULTILAYERY_MOLECULE_HEIGHT)
								+ " <= Y <= "
								+ (CM2_CONSTANTS.MEMBRANE_SIZES.MAX_Y_TRANSLATION-
										CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.STANDARD_MULTILAYERY_MOLECULE_HEIGHT) 
								+ " Angstrom";
			this.groundxz.setBorder(BorderFactory
					.createTitledBorder(mm));

			this.groundxz.add(this.lx);
			this.groundxz.add(this.tfx);

			this.tfx.setFont(new Font("Arial", Font.PLAIN, 13));

			this.h1.setFont(new Font("Serif", Font.PLAIN, 18));

			this.addWindowFocusListener(new WindowFocusListener() {

				public void windowGainedFocus(final WindowEvent e) {

				}

				public void windowLostFocus(final WindowEvent e) {

					AskForYHeightDialog.this.requestFocus();

				}

			});

			this.pack();

			final Dimension dim = this.getToolkit().getScreenSize();
			this.setLocation(dim.width / 2 - this.getWidth() / 2, dim.height
					/ 2 - this.getHeight() / 2);

			this.tfx.setValue(this.originalValue);

			this.tfx.requestFocus();

			this.setResizable(false);

			this.okBtn.addActionListener(this);
			this.cancelBtn.addActionListener(this);

		}

		public void actionPerformed(final ActionEvent e) {

			if (e.getSource() == this.okBtn) {
				try {
					this.returnValue = Double.parseDouble(""
							+ this.tfx.getValue());

				} catch (final Exception e2) {
					this.returnValue = this.originalValue;
				}
			}
			if (e.getSource() == this.cancelBtn) {
				try {
					this.returnValue = this.originalValue;

				} catch (final Exception e2) {
					this.returnValue = 0d;
				}
			}
			this.dispose();
		}
	}

	public Float askForYHeight(Double currentHeight) {

		final AskForYHeightDialog dia = new AskForYHeightDialog(currentHeight);

		dia.setModal(true);
		dia.setVisible(true);

		return new Float(dia.returnValue);

	}

	public void switchToLayer(final int i) {
		if (DataPool.getMembraneModel().hasMultipleLayers())
			this.multilayerpanel.combo.setSelectedIndex(i);
		else
			this.m3d.updateAfterLayerChange();
	}

	public boolean isInSpecialMode() {

		return this.toolbar3.currentMode > -1;
	}

	public class IntersectionPanel extends JPanel implements ActionListener {

		public JLabel label = new JLabel();
		public JLabel nonelabel = new JLabel("None");
		public JCheckBox showBtn = new JCheckBox();
		public JButton lipidsBtn = new JButton();

		public IntersectionPanel() {

			boolean b = false;

			try {
				b = Boolean.parseBoolean(ConfigParser
						.getValue("Display_Intersections")[0]);
			} catch (final Exception e) {

			}

			this.showBtn.setSelected(b);

			this.lipidsBtn.setVisible(false);

			this.label.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("CV_No_Collisions"));

			this.lipidsBtn.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("CV_Remove_Colliding_Lipids"));

			this.label.setToolTipText("No current intersections!");
			this.showBtn.setToolTipText("Display current intersections");
			this.lipidsBtn
					.setToolTipText("Remove all lipids that intersect with proteins");

			this.showBtn.setText("Display");

			this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			this.add(Box.createHorizontalStrut(2));
			final JLabel l = new JLabel("<html>Current<br>Intersections</html>");
			this.add(l);

			this.add(Box.createHorizontalStrut(10));
			this.add(this.nonelabel);

			this.add(this.label);

			this.add(this.showBtn);

			l.setMinimumSize(new Dimension(80, 30));
			l.setMaximumSize(new Dimension(80, 30));
			// l.setBorder(BorderFactory.createEtchedBorder());
			this.nonelabel.setFont(this.nonelabel.getFont().deriveFont(12f));
			this.showBtn.addActionListener(this);
			this.lipidsBtn.addActionListener(this);

			// selectionlabel
			// .setForeground(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.ALT_TEXT_COLOR);
			// selectionlabel.setFont(this.getFont().deriveFont(Font.BOLD, 16));

			// selectionlabel.setText(selection);
		}

		public void actionPerformed(final ActionEvent e) {
			if (e.getSource() == this.lipidsBtn) {
				final MembraneModel model = DataPool.getMembraneModel();
				final Molecule_3D[] mols = model.getAllMoleculesInMembrane();
				final Vector<Molecule_3D> prots = new Vector<Molecule_3D>();
				final Vector<Molecule_3D> lips = new Vector<Molecule_3D>();
				final Vector<Molecule_3D> toRemove = new Vector<Molecule_3D>();
				final IntersectManager inter = new IntersectManager();

				for (int i = 0; i < mols.length; i++) {
					if (mols[i] instanceof Protein_3D)
						prots.addElement(mols[i]);
					else
						lips.addElement(mols[i]);
				}

				for (int i = 0; i < prots.size(); i++) {
					if (inter.intersectsMoleculeMoleculesVector(prots.get(i),
							lips, false)) {
						final Molecule_3D[] collided = inter
								.getCollisionObjects();

						for (int j = 0; j < collided.length; j++) {
							toRemove.addElement(collided[j]);
						}

					}

				}
				if (!toRemove.isEmpty())
					MembraneView.this.listener.removeLipidsFrom3D(toRemove
							.toArray(new Lipid_3D[0]));

				for (int i = 0; i < toRemove.size(); i++) {
					model.deleteLipidExtra((Lipid_3D) toRemove.get(i));
					model.deleteLipidIntra((Lipid_3D) toRemove.get(i));
				}

				MembraneView.this.listener.refreshViews(false, true, false,
						true, true, true);

			} else if (e.getSource() == this.showBtn) {
				MembraneView.this.intersectionsDisplayer
						.updateIntersectionDispay();
			}

		}

	}

	public IntersectionsDisplayer getIntersectionsDisplayer() {

		return this.intersectionsDisplayer;
	}

	public void storeSettings() {

		ConfigParser.deleteAllAt("Display_Intersections");
		ConfigParser.addValue("Display_Intersections", ""
				+ this.intersectPanel.showBtn.isSelected());

	}

	public void toSystemTray() {
		if (!SystemTray.isSupported())
			return;

		final SystemTray tray = SystemTray.getSystemTray();
		final Image image = Cm2_IconDepot.getSpecifiedIcon("Cm2_Icon")
				.getImage();
		final PopupMenu popup = new PopupMenu();
		final TrayIcon trayIcon = new TrayIcon(image, "Running "
				+ CalculationManager.getAlgo().getName(), popup);
		this.listener.getMainFrame().setVisible(false);

		final MouseListener mouseListener = new MouseListener() {

			public void mouseClicked(final MouseEvent e) {
				if (e.getClickCount() > 1) {
					MembraneView.this.returnFromSystemTray();
				}
			}

			public void mouseEntered(final MouseEvent e) {

			}

			public void mouseExited(final MouseEvent e) {

			}

			public void mousePressed(final MouseEvent e) {

			}

			public void mouseReleased(final MouseEvent e) {

			}
		};

		final MenuItem defaultItem = new MenuItem("Restore");
		defaultItem.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent e) {
				MembraneView.this.returnFromSystemTray();

			}

		});
		popup.add(defaultItem);

		trayIcon.setImageAutoSize(true);

		trayIcon.addMouseListener(mouseListener);

		try {
			tray.add(trayIcon);

		} catch (final AWTException e) {

		}

	}

	public void returnFromSystemTray() {
		if (!SystemTray.isSupported())
			return;

		try {
			
			final TrayIcon icon = SystemTray.getSystemTray().getTrayIcons()[SystemTray.getSystemTray().getTrayIcons().length-1];
			SystemTray.getSystemTray().remove(icon);
//			this.m3d.dockScreen(true);
			final ActionListener ac = ((JCheckBox) this.toolbar2
					.getComponentAtIndex(4)).getActionListeners()[0];
			((JCheckBox) this.toolbar2.getComponentAtIndex(4))
					.removeActionListener(ac);
			((JCheckBox) this.toolbar2.getComponentAtIndex(4))
					.setSelected(true);
			((JCheckBox) this.toolbar2.getComponentAtIndex(4))
					.addActionListener(ac);

			

		} catch (final Exception e) {
			// e.printStackTrace();
		}
		DataPool.J3DSettings.isInSystemTray=false;
		this.listener.getMainFrame().setVisible(true);
	}
	
}
