/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.interfaces;

import java.awt.Point;
import java.io.File;
import java.util.Vector;

import org.cellmicrocosmos.cm2.dataobjects.MicroDomain_3D;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.CoordinateTransformationHints;
import org.cellmicrocosmos.cm2.pluginmanager.PluginManager;


/**
 * This interface provides all relevant events occured in the Gui
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public interface GuiListener {

	/**
	 * Called when User chooses to create a new Membrane
	 */
	public void userChoseNewMembrane();

	/**
	 * Called when User chooses to load a membrane
	 */
	public void userChoseLoadMembrane();

	/**
	 * Called when User chooses to save a membrane
	 */
	public void userChoseSaveMembrane();

	/**
	 * Called when User chooses to quicksave the current Membrane
	 */
	public void userChoseQuickSaveMembrane();
	
	/**
	 * Called when User chooses to save a Screenshot
	 */
	public void userChoseSaveImage();
	
	/**
	 * Called when User chooses to save the algorithm time protocol
	 */
	public void userChoseSaveTimeProtocol(Vector<String> timeProtocol);

	/**
	 * Called when User chooses to show the algorithm time protocol
	 */
	public void userChoseShowTimeProtocol();
	
	/**
	 * Called when User chooses to create a new DB
	 */
	public void userChoseNewDB();

	/**
	 * Called when User chooses to load a DB
	 */
	public void userchoseLoadDB();

	/**
	 * Called when User chooses to generate a PDB file
	 * (Uses the current membrane if the last opened file was a restored PDB file
	 */
	public void userChoseGeneratePDBFile(File pdbFile);

	/**
	 * Called when User chooses to quit the application
	 */
	public void exitApplication();

	public void userClickedOpenRCSB();

	public void userChoseQuickLoadMembrane(File file);

	public void doProxySettings();

	public void userChosePreviewMembrane(int mode);
	
	public void userChoseAnalyzeMembrane();

	public void popUpAttrsDialog();

	public void userClickedOpenExternal();

	public void userClicked3DSettings(Point location);

	public void userClickedEditorMode(boolean bright_App);

	public void popUpChangeSize(Point locationOnScreen);

	public void userClickedOpenRestrictionDialog();

	public void reloadGeometry();

	public void startDomainMode(MicroDomain_3D domainToChange);

	public void userChangedStereoEnable(boolean stereoEnabled);

	public void userChangedStereoSlider();

	public PluginManager getPluginManager();

	public void addMultilayerPressed();

	public boolean restoreMembraneFromPDBPressed();
	
	public boolean restoreMembraneFromPDB(String absolutePathOfPDB, String absolutePathToCm2, CoordinateTransformationHints overwriteHints, boolean checkMembraneState);

	public void setPercentages(int mode, int newval, String code, int domainID);
	
	public void setComputeAbsoluteValues(boolean computeAbsoluteValues);
	
}
