/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.io.ppe.network.http;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import org.cellmicrocosmos.cm2.io.ppe.toolbox.ConfParser;
import org.cellmicrocosmos.cm2.io.ppe.toolbox.PPE_CONSTANTS.CONF_PARSER;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * 
 * Initializes the OPM Connection.
 * 
 * @author Bjoern Sommer bjoern@CELLmicrocosmos.org
 */
public class OPMConnection extends Connection {

	/**
	 * 
	 */
	private String countOfOpm = "0";

	/**
	 * 
	 */
	public OPMConnection() {	
		// initialize connection
		try {
			final ConfParser confparser = new ConfParser();
			this.setList(confparser.parseForPortConfig(CONF_PARSER.OPM_KEY));
			confparser.freeSpace();
			this.setUrl(new URL(this.getList().get(0),this.getList().get(1),Integer.parseInt(this.getList().get(2)),this.getList().get(3)));
		} catch (final MalformedURLException e) {
			System.out
					.println(">> MalformedURLException at OPMConnection(): ");
			e.printStackTrace();
			Cm2_Toolkit.complainInternet();
		} catch (final Exception e) {
			System.err.println(">> Exception at OPMConnection(): ");
			System.err.println(e);
			Cm2_Toolkit.complainInternet();
		}
	}

	/**
	 * 
	 * @return
	 */
	public String getCountOfOPM() {
		return this.countOfOpm;
	}

	/**
	 * 
	 */
	public void setCountOfOPM(String count) {
		this.countOfOpm = count;
	}

	/**
	 * 
	 * @param tempurl
	 * @return
	 */
	@SuppressWarnings("unused")
	private String getBody(final URL tempurl) {
		try {
			final HttpURLConnection con = (HttpURLConnection) tempurl
					.openConnection();
			con.connect();
			final Reader instream = new InputStreamReader(con.getInputStream());
			final BufferedReader input = new BufferedReader(instream);
			final String body = input.readLine();
			
			StringBuilder allLines = new StringBuilder();
			String thisLine;

			while ((thisLine = input.readLine()) != null) { // while loop begins here
				allLines.append(thisLine);
		       }			
			instream.close();
			input.close();
			con.disconnect();
			return body;
		} catch (final IOException e) {
			System.out.println(">> IOException at getBody(URL): ");
			e.printStackTrace();
			Cm2_Toolkit.complainInternet();
		} catch (final Exception e) {
			System.err.println(">> Exception at getBody(URL): ");
			System.err.println(e);
			Cm2_Toolkit.complainInternet();
		}
		return null;
	}
	
	public String getWholeDocument() {
		String wholeDocument = "";
		try {
			final HttpURLConnection con = (HttpURLConnection) this.getUrl().openConnection();
			final BufferedReader input = new BufferedReader(new InputStreamReader(con.getInputStream()));
			String thisLine;
			StringBuilder wholeDocumentBuilder = new StringBuilder();
			while ((thisLine = input.readLine()) != null)
				wholeDocumentBuilder.append(thisLine);
			wholeDocument = wholeDocumentBuilder.toString();
		} catch (IOException e) {
			System.err.println(">> IOException at getWholeDocument(): ");
			e.printStackTrace();
			Cm2_Toolkit.complainInternet();
		} catch (final Exception e) {
			System.err.println(">> Exception at getWholeDocument(): ");
			System.err.println(e);
			Cm2_Toolkit.complainInternet();
		}
		return wholeDocument;
	}

	/**
	 * 
	 * @return
	 */
	public InputStream getInputStreamFromXML() {
		try {
			
			final HttpURLConnection con = (HttpURLConnection) getUrl()
					.openConnection();
			con.connect();
			return con.getInputStream();
		} catch (final MalformedURLException e) {
			System.out
					.println(">> MalformedURLException at getInputStreamFromXML(): ");
			System.out.println(e);
			Cm2_Toolkit.complainInternet();
		} catch (final IOException e) {
			System.out.println(">> IOException at getInputStreamFromXML(): ");
			System.out.println(e);
			Cm2_Toolkit.complainInternet();
		} catch (final Exception e) {
			System.err.println(">> Exception at getInputStreamFromXML(): ");
			System.err.println(e);
			Cm2_Toolkit.complainInternet();
		}
		return null;
	}

}
