/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.io.ppe.transform.xml;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.cellmicrocosmos.cm2.io.ppe.network.http.Connection;
import org.cellmicrocosmos.cm2.io.ppe.network.http.OPMConnection;
import org.cellmicrocosmos.cm2.io.ppe.toolbox.ConfParser;
import org.cellmicrocosmos.cm2.io.ppe.toolbox.PPE_CONSTANTS;
import org.cellmicrocosmos.cm2.io.ppe.toolbox.PPE_CONSTANTS.IO;
import org.cellmicrocosmos.cm2.io.ppe.toolbox.PPE_CONSTANTS.MIFILE;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;


/**
 * The SAXCustomizer reduces the pdbtmall.xml in a program-internal
 * file format. He is an implementation of the {@link XMLCustomizer}. The
 * redundant method <code>customizeThisFile</code> accepts a file or an
 * InputStream.
 * 
 * Options for the Config-File ppe.conf
 * 
 * &_PORT_CONFIG:			->	Key for the connection configuration
 * http						-> 	protocol
 * w2.enzim.hu				-> 	domain
 * 80						-> 	port
 * /pdbtm/pdbtm/			->	sub-directories
 * Basic Z3Vlc3Q6Z3Vlc3Q=	->	connection type for authorization with key
 * 
 * &_XML_CONFIG: 			->	Key for XML configuration
 * pdbtm					->	tag type: <pdbtm> 
 * < ID > [ 0 ]				->	tag value: <pdbtm ID="XXX">; what should be parsed: [0 -> ID value]
 * TEXT						-> 	tag type: <text>
 * [ 1 ]					->	ignore tag value; what should be parsed: 
 * 								[1 -> text between starting and ending tag, ?<> escapes					
 * HTMLTEXT					->	tag type: <HTMLTEXT>
 * [ 2 ]					->	ignore tag value; what should be parsed:
 * 								[2 -> HTML text between starting and ending tag, 
 * 								special html code parsing has to be defined in the JAVA code]
 * 
 * @author Christian Gamroth, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class SAXCustomizer extends DefaultHandler implements XMLCustomizer {

	private Connection con;
	private SAXParser parser;
	private File mifile;
	private File mifileBackup;
	private XMLOutputFactory factory;
	private XMLStreamWriter streamwriter;
	private FileOutputStream outputStream;
	private String confkey;
	private ArrayList<String> list;
//	boolean initialTagWritten = false;
	private boolean needsOneSpecialAttribute = false;
	private boolean hasOneSpecialAttribute = false;
	private String actualTag = "";
	private String actualTagValue = "";
	private boolean isCharSet = false;
	private boolean isHTMLSet = false;
	private String wholeDocument;
	private int database = 0;
	private InputStream instream = null;

	/**
	 * Constructs a new SAXCustomizer.
	 */
	public SAXCustomizer(int database, Connection con) {

		this.database = database;
		this.con = con;
		
//		buf = new StringBuilder();
//		buf.setLength(100000);
		// TODO TempFile handeln. renameTo(File) sehr unzuverlaessig!
		try {
			if (database == PPE_CONSTANTS.DB.PDBTM) {
				final SAXParserFactory factory = SAXParserFactory.newInstance();
				this.parser = factory.newSAXParser();
//				this.mifileBackup  = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_PDBTM);
//				if (mifileBackup.exists())
//					this.mifileBackup.renameTo(new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_PDBTM+IO.BACKUP_FILENAME_ADDON));
				
				// create the pdbtm.mif backup file
				this.mifile = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_PDBTM);
				this.mifileBackup  = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_PDBTM+IO.BACKUP_FILENAME_ADDON);
				Cm2_Toolkit.copyFile(this.mifile, this.mifileBackup);
				mifileBackup.setLastModified(mifile.lastModified());
				
//				this.mifile.createNewFile();
				
				this.factory = XMLOutputFactory.newInstance();
				outputStream = new FileOutputStream(this.mifile);
				this.streamwriter = this.factory
						.createXMLStreamWriter(outputStream);
			} else if (database == PPE_CONSTANTS.DB.OPM) {
				final SAXParserFactory factory = SAXParserFactory.newInstance();
				// ignore external DTDs, otherwise there will be problems with the DTD from the OPM page
				factory.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
				this.parser = factory.newSAXParser();

				// create the opm.mif backup file
				this.mifile = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_OPM);
				this.mifileBackup = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_OPM+IO.BACKUP_FILENAME_ADDON);
				Cm2_Toolkit.copyFile(this.mifile, this.mifileBackup);
				mifileBackup.setLastModified(mifile.lastModified());
				
//				this.mifileBackup = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_OPM);
//				if (mifileBackup.exists())
//					this.mifileBackup.renameTo(new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_OPM+IO.BACKUP_FILENAME_ADDON));
				
//				this.mifile = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_OPM);
//				this.mifile.createNewFile();
				
				this.factory = XMLOutputFactory.newInstance();
				outputStream = new FileOutputStream(this.mifile);
				this.streamwriter = this.factory
						.createXMLStreamWriter(outputStream);
				
//				this.parser.getXMLReader().setProperty("http://apache.org/xml/properties/input-buffer-size", 
//					      new Integer(200000000));
				
//				instream.
//				final HttpURLConnection con = (HttpURLConnection) tempurl
//				.openConnection();
//				con.connect();
//				final Reader instream = new InputStreamReader(con.getInputStream());
				
				con = new OPMConnection();
				wholeDocument = ((OPMConnection)con).getWholeDocument();
			}
		} catch (final ParserConfigurationException e) {
			System.err
					.println(">> ParserConfigurationException at SAXCustomizer(): ");
			e.printStackTrace();
		} catch (final SAXException e) {
			System.err.println(">> SAXException at SAXCustomizer(): ");
			e.printStackTrace();
		} catch (final IOException e) {
			System.err.println(">> IOException at PdbTMCustomizer(): ");
			e.printStackTrace();
		} catch (final XMLStreamException e) {
			System.err.println(">> XMLStreamException at PdbTMCustomizer(): ");
			e.printStackTrace();
		}
	}

	/**
	 * Transforms the pdbtmall.xml from the given file into the pdbtm.mif.
	 * 
	 * @param xmlFile The pdbtmall.xml file.
	 * @param key The key for identifying the source by the {@link ConfParser}.
	 */
	public void customizeThisFile(final File xmlFile, final String key) {
		try {
			this.confkey = key;
			this.parser.parse(xmlFile, this);
		} catch (final SAXException e) {
			System.err.println(">> SAXException at customizeThisFile(File): ");
			e.printStackTrace();
		} catch (final IOException e) {
			System.err.println(">> IOException at customizeThisFile(File): ");
			e.printStackTrace();
		}
	}

	/**
	 * Transforms the pdbtmall.xml from the given InputStream into the pdbtm.mif.
	 * 
	 * @param instream The InputStream to the pdbtmall.xml.
	 * @param key The key for identifying the source by the {@link ConfParser}.
	 */
	public void customizeThisFile(final InputStream instream, final String key) {
		try {
			this.instream = instream;
			this.confkey = key;
			this.parser.parse(instream, this, instream.toString());
		} catch (final SAXException e) {
			System.err
					.println(">> SAXException at customizeThisFile(InputStream): ");
			e.printStackTrace();
		} catch (final IOException e) {
			System.err
					.println(">> IOException at customizeThisFile(InputStream): ");
			e.printStackTrace();
		}
	}

	/**
	 * 
	 */
	@Override
	public void startDocument() {
		try {
			final ConfParser confparser = new ConfParser();
			this.list = confparser.parseForElements(this.confkey);
			confparser.freeSpace();
			this.streamwriter.writeStartDocument();
			this.streamwriter.writeStartElement(MIFILE.START_ELEMENT);
//			System.out.println(">> SAXParser startet das Dokument ...");
		} catch (final XMLStreamException e) {
			System.err.println(">> XMLStreamException at startDocument(): ");
			e.printStackTrace();
		}
	}

	/**
	 * 
	 */
	@Override
	public void startElement(final String uri, final String localName,
			final String qName, final Attributes attributes) {

		try {
			if (this.list.contains(qName)) {
				
				
				String token = null;
				actualTag = qName;
			
				final String listentry1 = this.list
						.get(this.list.indexOf(qName) + 1);
//				System.out.println("listentry: "+listentry.toString());
				final StringTokenizer tokenizer1 = new StringTokenizer(listentry1);
				while (!(token = tokenizer1.nextToken()).equals("[")) {
					if (token.equals("<")) {
						token = tokenizer1.nextToken();
//						System.out.println("tokenPRE "+token.toString());
						actualTag = qName;
						actualTagValue = token;
						token = token.replace("\"", "");
						String value = "";
						if (token.contains("=")) {
							value = token.substring(token.indexOf("=")+1);
							token = token.substring(0, token.indexOf("="));
							needsOneSpecialAttribute = true;
						}
						if (needsOneSpecialAttribute) {
							for (int countUp = 0; countUp < attributes.getLength(); countUp++) {
								if (attributes.getQName(countUp).equals(token) && 
									attributes.getValue(countUp).equals(value)) {
									hasOneSpecialAttribute = true;
								}
							}
						}
					}
				}
				

				if (!needsOneSpecialAttribute || (needsOneSpecialAttribute && hasOneSpecialAttribute)) {		
					
					if (this.list.indexOf(qName) == 0) {
						this.streamwriter.writeStartElement(MIFILE.NEW_ENTRY);
					}
					this.streamwriter.writeStartElement(qName);
					final String listentry2 = this.list
							.get(this.list.indexOf(qName) + 1);
	//				System.out.println("listentry: "+listentry.toString());
					final StringTokenizer tokenizer2 = new StringTokenizer(listentry2);
					while (!(token = tokenizer2.nextToken()).equals("[")) {
						if (token.equals("<")) {
							token = tokenizer2.nextToken();
							token = token.replace("\"", "");
							String value = "";
							if (token.contains("=")) {
								value = token.substring(token.indexOf("=")+1);
								token = token.substring(0, token.indexOf("="));
							}
							for (int countUp = 0; countUp < attributes.getLength(); countUp++) {
								if ( 	(value.isEmpty() && 
											attributes.getQName(countUp).equals(token))
									||  (!value.isEmpty() &&
											attributes.getQName(countUp).equals(token) && 
											attributes.getValue(countUp).equals(value))){
									
	//								System.out.println("attri.Q: "+attributes.getQName(countUp));
	//								System.out.println("attri.L: "+attributes.getLocalName(countUp));
	//								System.out.println("attri.V: "+attributes.getValue(countUp));
	//								System.out.println("");
									this.streamwriter.writeAttribute(token, attributes.getValue(countUp));
								}
							}
						}
					}
					token = tokenizer2.nextToken();
					// if [ 0 ] the text between the tags is not parsed
					// if [ 1 ] the text between the tags is parsed as regular 
					//          XMLStreamWriter.writeCharacters text, this means
					//          & , < and > are escape sequences
					// if [ 2 ] all text between the tags is parsed
					if (token.equals("1")) {
						this.isCharSet = true;
					} else if (token.equals("2")) {
						this.isHTMLSet = true;
					}
				}
			}
		} catch (final XMLStreamException e) {
			System.err.println(">> XMLStreamException at startElement(): ");
			System.err.println(e);
		}
	}

	/**
	 *  Note: Character maybe run different times for one document pass!
	 */
	@Override
	public void characters(final char[] ch, final int start, final int length) {
		try {
			if (this.isCharSet) {
				this.streamwriter.writeCharacters(ch, start, length);				
			}
		} catch (final XMLStreamException e) {
			System.err.println(">> XMLStreamException at characters(): ");
			System.err.println(e);
		}
	}

	/**
	 * 
	 */
	@Override
	public void endElement(final String uri, final String localName,
			final String qName) {
		try {
			if ( this.list.contains(qName) && 
				(!needsOneSpecialAttribute || 
						(needsOneSpecialAttribute && hasOneSpecialAttribute)) ) {
				if (this.list.indexOf(qName) == 0) {
					this.streamwriter.writeEndElement();
				}
				if (database != PPE_CONSTANTS.DB.OPM)
					this.streamwriter.writeEndElement();
				needsOneSpecialAttribute = false;
				hasOneSpecialAttribute = false;
			}
		} catch (final XMLStreamException e) {
			System.err.println(">> XMLStreamException at endElement(): ");
			e.printStackTrace();
		}
		if (this.isCharSet) 
			this.isCharSet = false;
		if (this.isHTMLSet) {
			wholeDocument = wholeDocument.substring(
					wholeDocument.indexOf(
							actualTag+" "+actualTagValue)+actualTag.length()+actualTagValue.length()+2, 
							wholeDocument.length());
//			System.out.println("wholeDocument PRE "+wholeDocument.toString());
			int posOfActualClosingTag = wholeDocument.indexOf("</"+actualTag);
//			System.out.println("posOfActualClosingTag"+posOfActualClosingTag+" "+"</"+actualTag);
			wholeDocument = wholeDocument.substring(0, posOfActualClosingTag);
//			System.out.println("wholeDocument POST "+wholeDocument.toString());
			wholeDocument = wholeDocument.toLowerCase();
			wholeDocument = wholeDocument.replaceAll("<br />", "\n").replaceAll("<br/>", "\n").replaceAll("<br>", "\n");
			wholeDocument = wholeDocument.replaceAll("<h\\d>.*</h\\d>", "");
			wholeDocument = wholeDocument.replaceAll("<p\\d>.*</p\\d>", "");
			wholeDocument = wholeDocument.replaceAll("\"", "");
			wholeDocument = wholeDocument.replaceAll(" ", "");
			wholeDocument = wholeDocument.trim();
			
			String opmEntries = "";
			
			// special handling for OPM
			if (database == PPE_CONSTANTS.DB.OPM) {
				StringTokenizer wholeDocumentTokenizer = new StringTokenizer(wholeDocument);
				String actualLine = ""; 
				try {
					// DIV
					this.streamwriter.writeEndElement();
					// ENTRY
					this.streamwriter.writeEndElement();
				} catch (XMLStreamException e1) {
					e1.printStackTrace();
				}
				while (wholeDocumentTokenizer.hasMoreTokens() && (actualLine = wholeDocumentTokenizer.nextToken()) != "\n" ) {
					try {
						if (actualLine.contains("pdbentries")) {
							opmEntries = actualLine.replace("pdbentries", "");
						} else {
							this.streamwriter.writeStartElement(MIFILE.NEW_ENTRY);
							this.streamwriter.writeStartElement(MIFILE.OPM_ENTRY);
							this.streamwriter.writeAttribute("ID", actualLine);
							this.streamwriter.writeEndElement();
							this.streamwriter.writeEndElement();
						}
					} catch (XMLStreamException e) {
						e.printStackTrace();
					}
				}
				
				// write counter number to update
				((OPMConnection)con).setCountOfOPM(opmEntries);
				
			// otherwise the regular HTML content is written, tags are transformed to special chars
			} else {
				try {
					this.streamwriter.writeCharacters(wholeDocument);
				} catch (XMLStreamException e) {
					e.printStackTrace();
				}
			}
			this.isHTMLSet = false;
		}
	}

	/**
	 * 
	 */
	@Override
	public void endDocument() {
		try {
			this.streamwriter.writeEndDocument();
			this.streamwriter.flush();
			this.streamwriter.close();
			File mifileBackupNew = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_PDBTM+IO.BACKUP_FILENAME_ADDON);
			if (mifileBackupNew.exists())
				mifileBackupNew.delete();
			if (this.mifileBackup!=null && this.mifileBackup.exists()) {
				this.mifileBackup.delete();
			}
//			System.out.println(">> SAXParser beendet das Dokument ...");
		} catch (final XMLStreamException e) {
			System.err.println(">> XMLStreamException at endDocument(): ");
			e.printStackTrace();
		}
	}
	
	/**
	 * Tries to close all streams connected with the opened files.
	 *
	 */
	public void closeStreams() {

		try {
			if (this.streamwriter!=null) {
				this.streamwriter.writeEndDocument();
//				this.streamwriter.flush();
				this.streamwriter.close();
//				this.streamwriter.flush();
			}
		} catch (final Exception e) {
			System.err.println("Error closing writer.");
		}
		try {
			if (this.outputStream != null) {
				this.outputStream.close();
				this.outputStream.flush();
			}
		} catch (final Exception e) {
			System.err.println("Error closing outstream.");
		}
		try {
			if (this.instream!=null)
				this.instream.close();
		} catch (final Exception e) {
			System.err.println("Error closing instream.");
		}
		
//		if (this.mifileBackup!=null && this.mifileBackup.exists()) {
//			if (this.database == PPE_CONSTANTS.DB.PDBTM) {
//				File mifileBackupNew = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_PDBTM+IO.BACKUP_FILENAME_ADDON);
//				if (mifileBackupNew.exists()) {
//					File mifileNew = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_PDBTM);
//					// this is a windows workaround if the deletion does not work
//					if (!mifileNew.delete()) {
//						try {
//							Cm2_Toolkit.unlockFile(mifileNew);
//						} catch (IOException e) {
//							e.printStackTrace();
//						}
//						mifileNew.delete();
//					}
//					System.out.println("rename PDBTM BCUP: "+mifileBackupNew.renameTo(new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_PDBTM)));
//				}
//			} else if (this.database == PPE_CONSTANTS.DB.OPM) {
//				File mifileBackupNew = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_OPM+IO.BACKUP_FILENAME_ADDON);
//				if (mifileBackupNew.exists()) {
//					File mifileNew = new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_OPM);
//					if (mifileNew.exists())
//						// this is a windows workaround if the deletion does not work
//						if (!mifileNew.delete()) {
//							try {
//								Cm2_Toolkit.unlockFile(mifileNew);
//							} catch (IOException e) {
//								e.printStackTrace();
//							}
//							mifileNew.delete();
//						}
//					System.out.println("rename OPM BCUP: "+mifileBackupNew.renameTo(new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_OPM)));
//				}
//			}
//		}
	}

}
