/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.IllegalOperationException;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithmGUI;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Molecule;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * A first linear algorithm created for the MembraneEditor.
 * 
 * @author Sebastian Schneider
 * 
 */
class LinearAlgorithm extends MembranePackingAlgorithm implements ItemListener,
		ChangeListener {

	private boolean hardcheck = false;

	private int modus = 0;

	boolean breake = false;
	private JCheckBox chk, chk1, chk2, chk3, chk4;

	private JSlider slider1;

	private JLabel det;

	private float cor = 0.01f;

	Vector<Molecule> testLipsNew = new Vector<Molecule>();

	Vector<Molecule> testLipsOld = new Vector<Molecule>();

	Vector<Molecule> testLipsOldOld = new Vector<Molecule>();

	/**
	 * builds up GUI for user settings
	 * 
	 */
	public boolean buildAndShowGui() {
		final JPanel mainpanel = new JPanel();
		mainpanel.setLayout(new BoxLayout(mainpanel, BoxLayout.Y_AXIS));

		final JPanel panel2 = new JPanel();
		panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));
		panel2.setBorder(BorderFactory.createEtchedBorder());

		final JPanel panel3 = new JPanel();
		panel3.setLayout(new BoxLayout(panel3, BoxLayout.X_AXIS));
		this.chk1 = new JCheckBox();
		this.chk1.addItemListener(this);
		this.chk1.setSelected(true);
		panel3.add(this.chk1);
		panel3
				.add(new JLabel(
						"<html>Activate for rotating lipids along the Y-Axis in 1 degree steps<br> (1,2,3....360)</html>"));

		final JPanel panel4 = new JPanel();
		panel4.setLayout(new BoxLayout(panel4, BoxLayout.X_AXIS));
		this.chk2 = new JCheckBox();
		this.chk2.addItemListener(this);
		this.chk2.setSelected(false);
		panel4.add(this.chk2);
		panel4
				.add(new JLabel(
						"<html>Activate for rotating lipids along the Y-Axis in 45 degree steps<br>(0,45,90...315,360</html>"));

		final JPanel panel5 = new JPanel();
		panel5.setLayout(new BoxLayout(panel5, BoxLayout.X_AXIS));
		this.chk3 = new JCheckBox();
		this.chk3.addItemListener(this);
		this.chk3.setSelected(false);
		panel5.add(this.chk3);
		panel5
				.add(new JLabel(
						"<html>Activate for rotating lipids along the Y-Axis in 90 degree steps<br>(90,180,270,360)</html>"));

		final JPanel panel6 = new JPanel();
		panel6.setLayout(new BoxLayout(panel6, BoxLayout.X_AXIS));
		this.chk4 = new JCheckBox();
		this.chk4.addItemListener(this);
		this.chk4.setSelected(false);
		panel6.add(this.chk4);
		panel6.add(new JLabel("Activate for no rotation"));

		panel2.add(panel3);
		panel2.add(panel4);
		panel2.add(panel5);
		panel2.add(panel6);

		mainpanel.add(panel2);

		return MembranePackingAlgorithmGUI.popUpGUI(mainpanel, "LinearAlgorithm",
				"Choose settings for calculation");
	}

	/**
	 * fill with lipid method. Calls simple mode method.
	 * 
	 */
	public void fillWithLipids() {

		if (this.hardcheck == false) {
			this.simpleMode();

		} else {
			this.hardMode();
		}
	}

	/**
	 * creates rows of lipids.
	 * 
	 */
	public void simpleMode() {
		final MembraneModel model = DataPool.getMembraneModel();
		final Dimension dim = model.getSize();

		int height = 0;

		if (this.hasMembraneAnyExtracellularLipids()) {
			while (height <= dim.getHeight() + 4) {
				if (this.breake == true)
					return;
				Lipid oldLip = null;

				Lipid newLip = this.createNextExtracellularLipid();

				newLip.rotY((this.getRandomNumber(this.modus)));
				newLip
						.setPosition(new Point3f(newLip.getMaxRadius_XZ(),
								newLip.getPosition().y, height
										+ newLip.getMaxRadius_XZ() + 2));

				if (this.intersectsBorders(newLip)) {
					if (this.intersectsBorderNorth(newLip)) {
						while (this.intersectsBorderNorth(newLip)) {
							final Point3f p = new Point3f(newLip.getPosition());
							p.z += 0.01f;
							newLip.setPosition(p);

						}
					}
					if (this.intersectsBorderWest(newLip)) {
						while (this.intersectsBorderWest(newLip)) {
							final Point3f p = new Point3f(newLip.getPosition());
							p.x += 0.01f;
							newLip.setPosition(p);

						}
					}

				}

				while (this.intersectsBorderEast(newLip) == false) {

					if (this.intersectsWithProteins(newLip) == false
							&& this.intersectsBorders(newLip) == false) {
						try {
							this.addExtracellularLipid(newLip);

						} catch (final IllegalOperationException e) {

							e.printStackTrace();
						}
					}

					oldLip = newLip;

					newLip = this.createNextExtracellularLipid();

					newLip.rotY(this.getRandomNumber(this.modus));
					final Point3f p1 = new Point3f(oldLip.getPosition());
					p1.x = p1.x + oldLip.getMaxRadius_XZ()
							+ newLip.getMaxRadius_XZ() + 2;

					newLip.setPosition(p1);

				}

				height += (int) newLip.getMaxRadius_XZ() + 2;

			}
		}

		height = 0;
		if (this.hasMembraneAnyIntracellularLipids()) {
			while (height <= dim.getHeight() + 4) {
				if (this.breake == true)
					return;
				Lipid oldLip = null;

				Lipid newLip = this.createNextIntracellularLipid();
				newLip
						.setPosition(new Point3f(newLip.getMaxRadius_XZ(),
								newLip.getPosition().y, height
										+ newLip.getMaxRadius_XZ() + 2));

				newLip.rotY(this.getRandomNumber(this.modus));

				if (this.intersectsBorders(newLip)) {
					if (this.intersectsBorderNorth(newLip)) {
						while (this.intersectsBorderNorth(newLip)) {
							final Point3f p = new Point3f(newLip.getPosition());
							p.z += 0.01f;
							newLip.setPosition(p);

						}
					}
					if (this.intersectsBorderWest(newLip)) {
						while (this.intersectsBorderWest(newLip)) {
							final Point3f p = new Point3f(newLip.getPosition());
							p.x += 0.01f;
							newLip.setPosition(p);
						}
					}

				}

				while (this.intersectsBorderEast(newLip) == false) {

					if (this.intersectsWithProteins(newLip) == false
							&& this.intersectsBorders(newLip) == false) {
						try {
							this.addIntracellularLipid(newLip);

						} catch (final IllegalOperationException e) {

							e.printStackTrace();
						}
					}

					oldLip = newLip;

					newLip = this.createNextIntracellularLipid();

					final Point3f p1 = new Point3f(oldLip.getPosition());
					p1.x = p1.x + oldLip.getMaxRadius_XZ()
							+ newLip.getMaxRadius_XZ() + 2;

					newLip.setPosition(p1);

					newLip.rotY(this.getRandomNumber(this.modus));
				}

				height += (int) newLip.getMaxRadius_XZ() + 2;
			}
		}
	}

	/**
	 * creates rows of lipids in a strange way.
	 * 
	 * @deprecated
	 */
	@Deprecated
	public void hardMode() {
		final MembraneModel model = DataPool.getMembraneModel();
		final Dimension dim = model.getSize();

		this.setFirstRow(0);
		int height = 5;
		while (height <= dim.height) {
			this.createRow(height, 0);
			// slowRow(height,0);
			height += this.testLipsOld.firstElement().getMaxRadius_XZ();

		}

	}

	/**
	 * Places first element of a row.
	 * 
	 * @param newLip
	 * @param mode
	 * @return
	 */
	public Lipid slowFirstElement(Lipid newLip, final int mode) {

		// move away from border
		if (this.intersectsBorderWest(newLip)) {
			int d = 0;
			while (this.intersectsBorderWest(newLip)) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.x += this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
			// move to border
		} else {
			int d = 0;
			while (true) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.x -= this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersectsBorderWest(newLip)) {
					final Point3f pp = new Point3f(newLip.getPosition());
					pp.x += this.cor;
					newLip.setPosition(new Point3f(pp));
					break;
				}
				d++;
				if (d > 1000)
					break;
			}
		}

		if (this.intersects(newLip) == false) {
			int d = 0;
			while (true) {

				final Point3f p = new Point3f(newLip.getPosition());
				p.z -= this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersects(newLip)) {
					final Point3f pp = new Point3f(newLip.getPosition());
					pp.z += this.cor;
					newLip.setPosition(new Point3f(pp));
					break;
				}
				d++;
				if (d > 1000)
					break;
			}
			// System.out.println(this.intersectsMoleculeWithMolecules(newLip,
			// testLipsOld));

		} else {
			// move away from lipid
			if (this.intersects(newLip)) {
				// System.out.println("move away");
				int d = 0;
				while (this.intersects(newLip)) {
					final Point3f p = new Point3f(newLip.getPosition());
					p.z += this.cor;
					newLip.setPosition(new Point3f(p));
					d++;
					if (d > 1000)
						break;
				}
			}

		}

		if (this.intersects(newLip) == false
				&& this.intersectsBorders(newLip) == false) {
			if (mode == 0) {
				try {
					this.addExtracellularLipid(newLip);
					this.testLipsNew.add(newLip);
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			} else {
				try {
					this.addIntracellularLipid(newLip);
					this.testLipsNew.add(newLip);
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			}
		} else {
			System.out.println("INTERSECTS FIRST LIPID IN ROW");
			System.out.println("border" + this.intersectsBorderWest(newLip));
			System.out.println("lip oben"
					+ this.intersectsMoleculeWithMolecules(newLip,
							this.testLipsOld));
			System.out.println("lip ganz oben"
					+ this.intersectsMoleculeWithMolecules(newLip,
							this.testLipsOldOld));
		}

		final Point3f ppp = new Point3f(newLip.getPosition());
		ppp.x += newLip.getMaxRadius_XZ();
		if (mode == 0) {
			newLip = this.createNextExtracellularLipid();
		} else {
			newLip = this.createNextIntracellularLipid();
		}

		// ppp.x += newLip.getMaxRadius_XZ();
		// oPP.y = newLip.getYTranslation();
		// oPP.z = newLip.getMaxRadius_XZ();
		newLip.setPosition(new Point3f(ppp));
		System.out.println(newLip.getPosition());
		return newLip;

	}

	/**
	 * creates a single row
	 * 
	 * @param height
	 * @param mode
	 */
	public void slowRow(final int height, final int mode) {
		System.out.println("begin");
		Lipid newLip;
		if (mode == 0) {
			newLip = this.createNextExtracellularLipid();
		} else {
			newLip = this.createNextIntracellularLipid();
		}
		newLip.setPosition(new Point3f(newLip.getMaxRadius_XZ(), newLip
				.getYTranslation(), newLip.getMaxRadius_XZ() + height));

		Lipid l = this.slowFirstElement(newLip, mode);
		l.rotY(this.getRandomNumber(this.modus));

		int testCOunt = 2;
		while (this.intersectsBorderEast(l) == false) {
			System.out.println("...............Lip Number : " + testCOunt
					+ " ............");
			testCOunt++;
			// System.out.println(l.getPosition().z);
			System.out.println("leftrigth");
			this.slowLeftRight(l);
			if (l.getPosition().x < 0 || l.getPosition().y < 0
					|| l.getPosition().z < 0) {
				System.out.println(l.getPosition());
				// break;
			}
			System.out.println("updown");
			this.slowUpDown(l);
			if (l.getPosition().x < 0 || l.getPosition().y < 0
					|| l.getPosition().z < 0) {
				System.out.println(l.getPosition());
				// break;
			}
			System.out.println("prot");
			this.slowProt(l);
			if (l.getPosition().x < 0 || l.getPosition().y < 0
					|| l.getPosition().z < 0) {
				System.out.println(l.getPosition());
				// break;
			}
			// if(setUpProtein(l)){
			// setUpNeighbor(l);
			// }

			if (this.intersects(l) == false
					&& this.intersectsBorders(l) == false) {
				try {
					if (mode == 0) {
						this.addExtracellularLipid(l);
						this.testLipsNew.add(l);
					} else {
						this.addIntracellularLipid(l);
						this.testLipsNew.add(l);
					}
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			} else {
				System.out.println("INTERSECTS");
				System.out.println("prot " + this.intersectsWithProteins(l));
				System.out.println("oben "
						+ this.intersectsMoleculeWithMolecules(l,
								this.testLipsOld));
				System.out.println("links "
						+ this.intersectsMoleculeWithMolecules(l,
								this.testLipsNew));
				System.out.println("borders " + this.intersectsBorders(l));
				System.out.println("old old "
						+ this.intersectsMoleculeWithMolecules(l,
								this.testLipsOldOld));
				// break;
			}
			final Point3f pp = new Point3f(l.getPosition());
			pp.x += l.getMaxRadius_XZ();

			if (mode == 0) {
				l = this.createNextExtracellularLipid();
			} else {
				l = this.createNextIntracellularLipid();
			}

			// pp.x += l.getMaxRadius_XZ()/3;

			l.setPosition(new Point3f(pp));
			l.rotY(this.getRandomNumber(this.modus));
			if (l.getPosition().x < 0 || l.getPosition().y < 0
					|| l.getPosition().z < 0) {
				System.out.println(l.getPosition());
				// break;
			}
		}
	}

	/**
	 * moves lipids to left or right depending on distance to the neighbors
	 * 
	 * @param newLip
	 */
	public void slowLeftRight(final Lipid newLip) {
		// checks intersection left side
		if (this.intersects(newLip)) {
			int d = 0;
			while (this.intersects(newLip)) {
				// System.out.println("while1");
				final Point3f p = new Point3f(newLip.getPosition());
				p.x += this.cor;
				p.z += this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
			// puts the lipid as near as possible to the neighbor
		} else {
			int d = 0;
			while (true) {
				// System.out.println("while2");
				if (this.intersects(newLip)
						|| this.intersectsBorderWest(newLip)) {
					final Point3f p = new Point3f(newLip.getPosition());
					p.x += this.cor;
					newLip.setPosition(new Point3f(p));
					break;
				}
				final Point3f p = new Point3f(newLip.getPosition());
				p.x -= this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
		}
	}

	/**
	 * moves lipid up or down depending on distance to the neighbors
	 * 
	 * @param newLip
	 */
	public void slowUpDown(final Lipid newLip) {
		// checks intersection with upper neighbors
		if (this.intersects(newLip)) {
			int d = 0;
			while (true) {
				if (this.intersects(newLip) == false) {
					break;
				}

				final Point3f p = new Point3f(newLip.getPosition());
				p.x += this.cor;
				p.z += this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
			// puts the lipid as near as possible to the upper
			// neighbor
		} else if (this.intersects(newLip) == false) {
			int d = 0;
			while (true) {
				if (this.intersects(newLip)) {
					final Point3f p = new Point3f(newLip.getPosition());
					p.z += this.cor;
					newLip.setPosition(new Point3f(p));
					break;
				}

				final Point3f p = new Point3f(newLip.getPosition());
				p.z -= this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
		}
	}

	/**
	 * moves lipid near to a protein
	 * 
	 * @param newLip
	 */
	public void slowProt(final Lipid newLip) {
		if (this.intersectsWithProteins(newLip)) {
			int d = 0;
			while (true) {
				System.out.println("prot1");
				if (this.intersectsWithProteins(newLip) == false) {
					break;
				}
				final Point3f p = new Point3f(newLip.getPosition());
				p.x += 1f;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
			d = 0;
			while (true) {
				System.out.println("prot2");
				if (this.intersectsWithProteins(newLip)) {
					final Point3f p = new Point3f(newLip.getPosition());
					p.x += this.cor;
					newLip.setPosition(new Point3f(p));
					break;
				}
				final Point3f p = new Point3f(newLip.getPosition());
				p.x -= this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}

		}
		if (this.intersects(newLip)) {
			int d = 0;
			while (true) {
				System.out.println("prot 3");
				if (this.intersects(newLip) == false) {
					break;
				}
				final Point3f p = new Point3f(newLip.getPosition());
				p.z += this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
		}
	}

	/**
	 * same as slowLeftRight but faster
	 * 
	 * @param newLip
	 */
	public void setLeftRight(final Lipid newLip) {
		// checks intersection left side
		if (this.intersectsMoleculeWithMolecules(newLip, this.testLipsNew)) {
			int d = 0;
			while (this.intersectsMoleculeWithMolecules(newLip,
					this.testLipsNew)) {
				// System.out.println("while1");
				final Point3f p = new Point3f(newLip.getPosition());
				p.x += this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
				// if(this.intersectsMoleculeWithMolecules(newLip,testLipsOld)||
				// this.intersectsMoleculeWithMolecules(newLip,
				// testLipsOldOld)){
				// this.setUpNeighbor(newLip);
				// }
			}
			// puts the lipid as near as possible to the neighbor
		} else {
			int d = 0;
			while (true) {
				// System.out.println("while2");
				if (this.intersectsMoleculeWithMolecules(newLip,
						this.testLipsNew)
						|| this.intersectsBorderWest(newLip)) {
					final Point3f p = new Point3f(newLip.getPosition());
					p.x += this.cor;
					newLip.setPosition(new Point3f(p));
					break;
				}
				// if(this.intersectsMoleculeWithMolecules(newLip,testLipsOld)||
				// this.intersectsMoleculeWithMolecules(newLip,
				// testLipsOldOld)){
				// this.setUpNeighbor(newLip);
				// }
				final Point3f p = new Point3f(newLip.getPosition());
				p.x -= this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
		}
	}

	/**
	 * same as slowUpDown but faster
	 * 
	 * @param newLip
	 */
	public void setUpNeighbor(final Lipid newLip) {
		// checks intersection with upper neighbors
		if (this.intersectsMoleculeWithMolecules(newLip, this.testLipsOld)
				|| this.intersectsMoleculeWithMolecules(newLip,
						this.testLipsOldOld)) {
			// System.out.println("move away");
			int d = 0;
			while (true) {
				if (this.intersectsMoleculeWithMolecules(newLip,
						this.testLipsOld) == false
						&& this.intersectsMoleculeWithMolecules(newLip,
								this.testLipsOldOld) == false) {
					break;
				}
				// if(this.intersectsMoleculeWithMolecules(newLip,
				// testLipsNew)){
				// setLeftRight(newLip);
				// }
				final Point3f p = new Point3f(newLip.getPosition());
				p.z += this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
			// puts the lipid as near as possible to the upper
			// neighbor
		} else if (this.intersectsMoleculeWithMolecules(newLip,
				this.testLipsOld) == false
				&& this.intersectsMoleculeWithMolecules(newLip,
						this.testLipsOldOld) == false
				&& this.intersectsWithProteins(newLip) == false) {
			// System.out.println("move toward");
			int d = 0;
			while (true) {
				// if(this.intersectsMoleculeWithMolecules(newLip,
				// testLipsNew)){
				// setLeftRight(newLip);
				// }
				// System.out.println(this.intersectsMoleculeWithMolecules(newLip,
				// testLipsOld));
				// System.out.println(this.intersectsMoleculeWithMolecules(newLip,
				// testLipsOldOld));
				// System.out.println(this.intersectsWithProteins(newLip));
				// if(this.intersectsMoleculeWithMolecules(newLip,
				// testLipsOld)||
				// this.intersectsMoleculeWithMolecules(newLip,
				// testLipsOldOld)||this.intersectsWithProteins(newLip)){
				if (this.intersects(newLip)) {
					final Point3f p = new Point3f(newLip.getPosition());
					p.z += this.cor;
					newLip.setPosition(new Point3f(p));
					break;
				}

				// System.out.println("....move....");
				// System.out.println(this.intersectsMoleculeWithMolecules(newLip,
				// testLipsOld));
				// System.out.println(this.intersectsMoleculeWithMolecules(newLip,
				// testLipsOldOld));
				// System.out.println(this.intersectsWithProteins(newLip));

				final Point3f p = new Point3f(newLip.getPosition());
				p.z -= this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
		}
	}

	/**
	 * same as slowProt but faster
	 * 
	 * @param newLip
	 * @return
	 */
	public boolean setUpProtein(final Lipid newLip) {
		boolean test = false;
		if (this.intersectsWithProteins(newLip)) {
			int d = 0;
			while (true) {
				// System.out.println("prot1");
				if (this.intersectsWithProteins(newLip) == false) {
					break;
				}
				final Point3f p = new Point3f(newLip.getPosition());
				p.x += 1f;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
			d = 0;
			while (true) {
				// System.out.println("prot2");
				if (this.intersectsWithProteins(newLip)) {
					final Point3f p = new Point3f(newLip.getPosition());
					p.x += this.cor;
					newLip.setPosition(new Point3f(p));
					break;
				}
				final Point3f p = new Point3f(newLip.getPosition());
				p.x -= this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
			test = true;
		}
		if (this.intersectsMoleculeWithMolecules(newLip, this.testLipsOld)
				|| this.intersectsMoleculeWithMolecules(newLip,
						this.testLipsOldOld)) {
			int d = 0;
			while (true) {
				// System.out.println("prot 3");
				if (this.intersectsMoleculeWithMolecules(newLip,
						this.testLipsOld) == false
						&& this.intersectsMoleculeWithMolecules(newLip,
								this.testLipsOldOld) == false) {
					break;
				}
				final Point3f p = new Point3f(newLip.getPosition());
				p.z += this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
		}
		return test;
	}

	/**
	 * same as slowFirstElement but faster
	 * 
	 * @param newLip
	 * @param mode
	 * @return
	 */
	public Lipid setFirstElementInRow(Lipid newLip, final int mode) {

		// move away from border
		if (this.intersectsBorderWest(newLip)) {
			int d = 0;
			while (this.intersectsBorderWest(newLip)) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.x += this.cor;
				newLip.setPosition(new Point3f(p));
				d++;
				if (d > 1000)
					break;
			}
			// move to border
		} else {
			int d = 0;
			while (true) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.x -= this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersectsBorderWest(newLip)) {
					final Point3f pp = new Point3f(newLip.getPosition());
					pp.x += this.cor;
					newLip.setPosition(new Point3f(pp));
					break;
				}
				d++;
				if (d > 1000)
					break;
			}
		}

		if (this.intersectsMoleculeWithMolecules(newLip, this.testLipsOld) == false
				&& this.intersectsMoleculeWithMolecules(newLip,
						this.testLipsOldOld) == false) {
			int d = 0;
			while (true) {

				final Point3f p = new Point3f(newLip.getPosition());
				p.z -= this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersectsMoleculeWithMolecules(newLip,
						this.testLipsOld)
						|| this.intersectsMoleculeWithMolecules(newLip,
								this.testLipsOldOld)) {
					final Point3f pp = new Point3f(newLip.getPosition());
					pp.z += this.cor;
					newLip.setPosition(new Point3f(pp));
					break;
				}
				d++;
				if (d > 1000)
					break;
			}
			// System.out.println(this.intersectsMoleculeWithMolecules(newLip,
			// testLipsOld));

		} else {
			// move away from lipid
			if (this.intersectsMoleculeWithMolecules(newLip, this.testLipsOld)
					|| this.intersectsMoleculeWithMolecules(newLip,
							this.testLipsOldOld)) {
				// System.out.println("move away");
				int d = 0;
				while (this.intersectsMoleculeWithMolecules(newLip,
						this.testLipsOld)
						|| this.intersectsMoleculeWithMolecules(newLip,
								this.testLipsOldOld)) {
					final Point3f p = new Point3f(newLip.getPosition());
					p.z += this.cor;
					newLip.setPosition(new Point3f(p));
					d++;
					if (d > 1000)
						break;
				}
			}

		}

		if (this.intersects(newLip) == false || this.intersectsBorders(newLip)) {
			if (mode == 0) {
				try {
					this.addExtracellularLipid(newLip);
					this.testLipsNew.add(newLip);
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			} else {
				try {
					this.addIntracellularLipid(newLip);
					this.testLipsNew.add(newLip);
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			}
		} else {
			System.out.println("INTERSECTS FIRST LIPID IN ROW");
			System.out.println("border" + this.intersectsBorderWest(newLip));
			System.out.println("lip oben"
					+ this.intersectsMoleculeWithMolecules(newLip,
							this.testLipsOld));
			System.out.println("lip ganz oben"
					+ this.intersectsMoleculeWithMolecules(newLip,
							this.testLipsOldOld));
		}

		final Point3f ppp = new Point3f(newLip.getPosition());
		ppp.x += newLip.getMaxRadius_XZ();
		if (mode == 0) {
			newLip = this.createNextExtracellularLipid();
		} else {
			newLip = this.createNextIntracellularLipid();
		}

		// ppp.x += newLip.getMaxRadius_XZ();
		// oPP.y = newLip.getYTranslation();
		// oPP.z = newLip.getMaxRadius_XZ();
		newLip.setPosition(new Point3f(ppp));
		// System.out.println(newLip.getPosition());
		return newLip;
	}

	/**
	 * sets the first row in a membrane
	 * 
	 * @param mode
	 */
	@SuppressWarnings("unchecked")
	public void setFirstRow(final int mode) {
		Lipid newLip;
		Lipid oldLip = null;
		if (mode == 0) {
			newLip = this.createNextExtracellularLipid();
		} else {
			newLip = this.createNextIntracellularLipid();
		}
		newLip.setPosition(new Point3f(newLip.getMaxRadius_XZ(), newLip
				.getYTranslation(), newLip.getMaxRadius_XZ()));
		newLip.rotY(this.getRandomNumber(this.modus));
		// checks if intersects with left border and moves it away
		if (this.intersectsBorderWest(newLip)) {
			int d = 0;
			while (true) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.x += this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersectsBorderWest(newLip) == false)
					break;

				d++;
				if (d > 1000)
					break;
			}
			// checks with left border and moves lipid toward
		} else {
			int d = 0;
			while (true) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.x -= this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersectsBorderWest(newLip)) {
					final Point3f pp = new Point3f(newLip.getPosition());
					pp.x += this.cor;
					newLip.setPosition(new Point3f(pp));
					break;
				}
				d++;
				if (d > 1000)
					break;
			}
		}
		// checks upper border and moves it away
		if (this.intersectsBorderNorth(newLip)) {
			int d = 0;
			while (true) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.z += this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersectsBorderNorth(newLip) == false)
					break;

				d++;
				if (d > 1000)
					break;
			}
		} else {
			int d = 0;
			while (true) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.z -= this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersectsBorderNorth(newLip)) {
					final Point3f pp = new Point3f(newLip.getPosition());
					pp.z += this.cor;
					newLip.setPosition(new Point3f(pp));
					break;
				}
				d++;
				if (d > 1000)
					break;
			}
		}
		if (this.intersects(newLip) == false) {
			if (mode == 0) {
				try {
					this.addExtracellularLipid(newLip);
					this.testLipsNew.add(newLip);
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			} else {
				try {
					this.addIntracellularLipid(newLip);
					this.testLipsNew.add(newLip);
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			}
		}
		if (mode == 0) {
			oldLip = this.getExtracellularLipidByPDBCode(newLip
					.getOriginalMolecule().getPDBcode());
		} else {
			oldLip = this.getIntracellularLipidByPDBCode(newLip
					.getOriginalMolecule().getPDBcode());
		}
		oldLip.setPosition(new Point3f(newLip.getPosition()));
		oldLip.setRotationMatrix(Cm2_Toolkit.copyMatrix(newLip
				.getRotationMatrix()));

		if (mode == 0) {
			newLip = this.createNextExtracellularLipid();
		} else {
			newLip = this.createNextIntracellularLipid();
		}

		final Point3f oP = new Point3f(oldLip.getPosition());
		oP.x = newLip.getMaxRadius_XZ() + oldLip.getMaxRadius_XZ();
		oP.y = newLip.getYTranslation();
		oP.z = newLip.getMaxRadius_XZ();
		newLip.setPosition(new Point3f(oP));
		newLip.rotY(this.getRandomNumber(this.modus));
		while (this.intersectsBorderEast(newLip) == false) {
			// rightshift
			if (this.intersectsMoleculeWithMolecules(newLip, this.testLipsNew)) {
				int d = 0;
				while (true) {
					d++;
					if (d > 1000)
						break;
					final Point3f p = new Point3f(newLip.getPosition());
					p.x += this.cor;
					newLip.setPosition(new Point3f(p));
					if (this.intersectsMoleculeWithMolecules(newLip,
							this.testLipsNew) == false)
						break;
				}
				// leftshift
			} else {
				int d = 0;
				while (true) {
					d++;
					if (d > 1000)
						break;
					if (this.intersectsMoleculeWithMolecules(newLip,
							this.testLipsNew)) {
						final Point3f pp = new Point3f(newLip.getPosition());
						pp.x += this.cor;
						newLip.setPosition(new Point3f(pp));
						break;
					}
					final Point3f p = new Point3f(newLip.getPosition());
					p.x -= this.cor;
					newLip.setPosition(new Point3f(p));

				}
			}

			// checks upper border and moves it away
			if (this.intersectsBorderNorth(newLip)) {
				int d = 0;
				while (true) {
					d++;
					if (d > 1000)
						break;
					final Point3f p = new Point3f(newLip.getPosition());
					p.z += this.cor;
					newLip.setPosition(new Point3f(p));
					if (this.intersectsBorderNorth(newLip) == false)
						break;
				}
			} else {
				int d = 0;
				while (true) {
					d++;
					if (d > 1000)
						break;
					final Point3f p = new Point3f(newLip.getPosition());
					p.z -= this.cor;
					newLip.setPosition(new Point3f(p));
					if (this.intersectsBorderNorth(newLip)) {
						final Point3f pp = new Point3f(newLip.getPosition());
						pp.z += this.cor;
						newLip.setPosition(new Point3f(pp));
						break;
					}
				}
			}

			this.setUpProtein(newLip);
			if (true) {// this.intersects(newLip) == false) {
				if (mode == 0) {
					try {
						this.addExtracellularLipid(newLip);
						this.testLipsNew.add(newLip);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				} else {
					try {
						this.addIntracellularLipid(newLip);
						this.testLipsNew.add(newLip);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				}
			}
			if (mode == 0) {
				oldLip = this.getExtracellularLipidByPDBCode(newLip
						.getOriginalMolecule().getPDBcode());
			} else {
				oldLip = this.getIntracellularLipidByPDBCode(newLip
						.getOriginalMolecule().getPDBcode());
			}
			oldLip.setPosition(new Point3f(newLip.getPosition()));
			oldLip.setRotationMatrix(Cm2_Toolkit.copyMatrix(newLip
					.getRotationMatrix()));

			if (mode == 0) {
				newLip = this.createNextExtracellularLipid();
			} else {
				newLip = this.createNextIntracellularLipid();
			}

			final Point3f oPP = new Point3f(oldLip.getPosition());
			oPP.x = newLip.getMaxRadius_XZ() + oldLip.getPosition().x;
			// oPP.y = newLip.getYTranslation();
			// oPP.z = newLip.getMaxRadius_XZ();
			newLip.setPosition(new Point3f(oPP));
			newLip.rotY(this.getRandomNumber(this.modus));

		}
		this.testLipsOld = (Vector<Molecule>) this.testLipsNew.clone();
		this.testLipsNew.clear();
	}

	/**
	 * same as slowROw but faster
	 * 
	 * @param height
	 * @param mode
	 */
	@SuppressWarnings("unchecked")
	public void createRow(final int height, final int mode) {
		// System.out.println("begin");
		Lipid newLip;
		if (mode == 0) {
			newLip = this.createNextExtracellularLipid();
		} else {
			newLip = this.createNextIntracellularLipid();
		}
		newLip.setPosition(new Point3f(newLip.getMaxRadius_XZ(), newLip
				.getYTranslation(), newLip.getMaxRadius_XZ() + height));

		// Lipid l = setFirstElementInRow(newLip, mode);
		Lipid l = this.slowFirstElement(newLip, mode);
		l.rotY(this.getRandomNumber(this.modus));

		int testCOunt = 2;
		while (this.intersectsBorderEast(l) == false) {
			// System.out.println("...............Lip Number : "+testCOunt+"
			// ............");
			testCOunt++;
			// System.out.println(l.getPosition().z);
			System.out.println("prot");
			this.setUpProtein(l);
			if (l.getPosition().x < 0 || l.getPosition().y < 0
					|| l.getPosition().z < 0) {
				System.out.println(l.getPosition());
				// break;
			}
			System.out.println("updown");
			// setUpNeighbor(l);
			if (l.getPosition().x < 0 || l.getPosition().y < 0
					|| l.getPosition().z < 0) {
				System.out.println(l.getPosition());
				// break;
			}
			System.out.println("leftrigth");
			this.setLeftRight(l);
			if (l.getPosition().x < 0 || l.getPosition().y < 0
					|| l.getPosition().z < 0) {
				// System.out.println(l.getPosition());
				// break;
			}

			// if(setUpProtein(l)){
			// setUpNeighbor(l);
			// }

			if (this.intersects(l) == false
					&& this.intersectsBorders(l) == false) {
				try {
					if (mode == 0) {
						this.addExtracellularLipid(l);
						this.testLipsNew.add(l);
					} else {
						this.addIntracellularLipid(l);
						this.testLipsNew.add(l);
					}
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			} else {
				System.out.println("INTERSECTS");
				System.out.println("prot " + this.intersectsWithProteins(l));
				System.out.println("oben "
						+ this.intersectsMoleculeWithMolecules(l,
								this.testLipsOld));
				System.out.println("links "
						+ this.intersectsMoleculeWithMolecules(l,
								this.testLipsNew));
				System.out.println("borders " + this.intersectsBorders(l));
				System.out.println("old old "
						+ this.intersectsMoleculeWithMolecules(l,
								this.testLipsOldOld));
				// break;
			}
			System.out.println(l.getPosition().z);
			final Point3f pp = new Point3f(l.getPosition());
			pp.x += l.getMaxRadius_XZ();

			if (mode == 0) {
				l = this.createNextExtracellularLipid();
			} else {
				l = this.createNextIntracellularLipid();
			}

			// pp.x += l.getMaxRadius_XZ()/3;

			l.setPosition(new Point3f(pp));
			l.rotY(this.getRandomNumber(this.modus));
			// if(l.getPosition().x<0||l.getPosition().y<0||l.getPosition().z<0){
			// System.out.println(l.getPosition());
			// //break;
			// }
		}
		this.testLipsOldOld = (Vector<Molecule>) this.testLipsOld.clone();
		this.testLipsOld = (Vector<Molecule>) this.testLipsNew.clone();
		this.testLipsNew.clear();
	}

	/**
	 * 
	 * returns a random int
	 * 
	 * @param mode
	 * @return
	 */
	public int getRandomNumber(final int mode) {
		int r = 0;
		if (mode == 0) {
			r = this.random.nextInt(360);
		} else if (mode == 1) {
			final int r2 = this.random.nextInt(7);
			if (r2 == 1) {
				r = 45;
			} else if (r2 == 2) {
				r = 90;
			} else if (r2 == 3) {
				r = 135;
			} else if (r2 == 4) {
				r = 180;
			} else if (r2 == 5) {
				r = 225;
			} else if (r2 == 6) {
				r = 270;
			} else if (r2 == 7) {
				r = 315;
			}
		} else if (mode == 2) {
			final int r3 = this.random.nextInt(3);
			if (r3 == 1) {
				r = 90;
			} else if (r3 == 2) {
				r = 180;
			} else if (r3 == 3) {
				r = 270;
			}
		} else if (mode == 3) {
			r = 0;
		}

		return r;
	}

	/**
	 * 
	 * returns full information about this algorithm
	 */
	public String getFullInformation() {
		final String s = "<html>" + "This algorithm orders the lipids in <br>"
				+ "a linear way. Which means that one lipid ist<br>"
				+ "placed beneath the next one on a ~line~</html>";
		return s;
	}

	/**
	 * 
	 * returns the name of this algorithm
	 */
	public String getName() {
		return "Linear Algorithm";
	}

	/**
	 * returns short information about this algorithm
	 */
	public String getShortInformation() {
		return "Sorts Lipids linear in rows (Version 1.01)";
	}

	/**
	 * handles users GUI settings
	 */
	public void itemStateChanged(final ItemEvent arg0) {

		if (arg0.getSource().equals(this.chk)) {
			this.hardcheck = this.chk.isSelected();
			if (this.chk.isSelected()) {
				this.slider1.setEnabled(true);
				this.det.setForeground(Color.BLACK);
			} else {
				this.slider1.setEnabled(false);
				this.det.setForeground(Color.GRAY);
			}
		} else if (arg0.getSource().equals(this.chk1)) {
			if (this.chk1.isSelected()) {
				if (this.chk2 != null && this.chk3 != null) {
					this.chk2.setSelected(false);
					this.chk3.setSelected(false);
					this.chk4.setSelected(false);
				}
				this.modus = 0;
			}

		} else if (arg0.getSource().equals(this.chk2)) {
			if (this.chk2.isSelected()) {
				this.chk1.setSelected(false);
				this.chk3.setSelected(false);
				this.chk4.setSelected(false);
				this.modus = 1;
			}

		} else if (arg0.getSource().equals(this.chk3)) {
			if (this.chk3.isSelected()) {
				this.chk1.setSelected(false);
				this.chk2.setSelected(false);
				this.chk4.setSelected(false);
				this.modus = 2;
			}
		} else if (arg0.getSource().equals(this.chk4)) {
			if (this.chk4.isSelected()) {
				this.chk1.setSelected(false);
				this.chk2.setSelected(false);
				this.chk3.setSelected(false);
				this.modus = 3;
			}
		}

		if (this.chk1.isSelected() == false && this.chk2.isSelected() == false
				&& this.chk3.isSelected() == false
				&& this.chk4.isSelected() == false) {
			this.chk1.setSelected(true);
			this.modus = 0;
		}

	}

	/**
	 * handles users GUI settings
	 */
	public void stateChanged(final ChangeEvent arg0) {
		if (arg0.getSource().equals(this.slider1)) {
			if (this.slider1.getValue() == 0) {
				this.cor = 0.001f;
			} else

				this.cor = this.slider1.getValue() / 1000f;

		}

	}

	/**
	 * 
	 * returns a string representation of users GUI settings
	 */
	public String getStringRepresentationOfSetupParameters() {
		return "rotation: " + this.modus;
	}

	public void stopAlgorithm() {
		this.breake = true;
	}

	public boolean supportsAbsoluteValues() {
		return false;
	}
	
	public boolean supportsMicrodomains() {
		return false;
	}

	public boolean overrideExistingLipids() {

		return true;
	}

	public Component getRuntimeControlPanel() {
		return null;
	}

}
