/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms;

import java.awt.Component;
import java.awt.Dimension;
import java.util.Random;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.IllegalOperationException;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithmGUI;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Molecule;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * A Simple Algorithm. Places lipids at random position.
 * 
 * @author Sebastian Schneider
 * 
 */
class SimpleAlgorithm extends MembranePackingAlgorithm implements ChangeListener {

	public int tries = 1;
	private boolean better = true;
	public JSlider slider, slider2;
	int firstStateTries = 50;
	private MyRandom random;
	public boolean breake = false;
	private final int width = 370;
	private JCheckBox chk;

	/**
	 * Builds up GUI for user settings
	 * 
	 */
	public boolean buildAndShowGui() {
		final JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

		final JPanel panel1 = new JPanel();
		panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
		this.slider = new JSlider(SwingConstants.HORIZONTAL, 1, 10, 1);
		this.slider.setSnapToTicks(true);
		this.slider.setMajorTickSpacing(1);
		this.slider.setSize(new Dimension(this.width, 50));
		this.slider.addChangeListener(this);
		this.slider.setPaintLabels(true);
		this.slider.setPaintTicks(true);

		panel1.add(new JLabel("<html>Choose random<br>placing factor:</html>"));
		panel1.add(this.slider);

		final JPanel panel2 = new JPanel();
		panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));
		this.chk = new JCheckBox();
		this.chk.addChangeListener(this);
		this.chk.setSelected(true);
		panel2.add(new JLabel("Activate for better placing"));
		panel2.add(this.chk);

		final JPanel panel3 = new JPanel();
		panel3.setLayout(new BoxLayout(panel3, BoxLayout.X_AXIS));

		panel3.add(new JLabel(
				"<html>Number of<br>better placing<br>attempts:</html>"));

		this.slider2 = new JSlider(SwingConstants.HORIZONTAL, 0, 10000, 2000);
		this.slider2.setSize(new Dimension(this.width * 2, 50));
		this.slider2.setSnapToTicks(true);
		this.slider2.setMajorTickSpacing(2000);
		this.slider2.setMinorTickSpacing(1000);
		this.slider2.setPaintLabels(true);
		this.slider2.setPaintTicks(true);

		panel3.add(this.slider2);

		mainPanel.add(panel1);
		mainPanel.add(new JLabel(" "));
		mainPanel.add(panel2);
		mainPanel.add(new JLabel(" "));
		mainPanel.add(panel3);
		mainPanel.add(new JLabel(" "));
		return MembranePackingAlgorithmGUI.popUpGUI(mainPanel,
				"Simple Algorithms choices", "Make your choices");
	}

	/**
	 * fillWithLipids method. Places lipids at random position and orientation.
	 * If activated, better positioning via testForBetterPosition is tried to
	 * find.
	 * 
	 */
	public void fillWithLipids() {
		this.random = new MyRandom(0);
		final int count = this.getMembraneSize().height
				* this.getMembraneSize().width;
		final MembraneModel model = DataPool.getMembraneModel();

		if (this.hasMembraneAnyExtracellularLipids()) {
			for (int i = 0; i < count * this.tries; i++) {
				if (this.breake == true)
					return;
				final Lipid lip = this.createNextExtracellularLipid();

				final Random rand = new Random();
				final int x = rand.nextInt(model.getSize().width);
				final int z = rand.nextInt(model.getSize().height);

				final Point3f p = lip.getPosition();
				p.x = x;
				p.z = z;

				lip.setPosition(p);

				lip.rotY(rand.nextInt(360));

				if (this.better) {
					this.testForBetterPosition(lip, MembranePackingAlgorithm.EXTRA);
				}
				if (this.intersects(lip) == false
						&& this.intersectsBorders(lip) == false) {
					try {
						this.addExtracellularLipid(lip);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				}

			}
		}

		if (this.hasMembraneAnyIntracellularLipids()) {
			for (int i = 0; i < count * this.tries; i++) {
				if (this.breake == true)
					return;
				final Lipid lip = this.createNextIntracellularLipid();

				final Random rand = new Random();
				final int x = rand.nextInt(model.getSize().width);
				final int z = rand.nextInt(model.getSize().height);

				final Point3f p = lip.getPosition();
				p.x = x;
				p.z = z;

				lip.setPosition(p);

				lip.rotY(rand.nextInt(360));

				if (this.better) {
					this.testForBetterPosition(lip, MembranePackingAlgorithm.INTRA);
				}
				if (this.intersects(lip) == false
						&& this.intersectsBorders(lip) == false) {
					try {
						this.addIntracellularLipid(lip);

					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				}
			}
		}

		this.displayInformation("TEST");

	}

	/**
	 * Tests if a new Random Position is better then the old Position.
	 * 
	 * 
	 * @param lip
	 * @param membraneside
	 * @return
	 */
	public Lipid testForBetterPosition(final Lipid lip, final int membraneside) {

		for (int tries = 0; tries < this.firstStateTries; tries++) {

			this.getMoleculesInRadiusOnMySide(lip, 30, membraneside);
			final Vector<Molecule> exclude = new Vector<Molecule>();

			final Molecule mol1 = this.getNearestMoleculeOnMySide(lip,
					membraneside);

			if (mol1 != null) {
				final float d1 = this.distance(lip, mol1, true);

				exclude.add(mol1);

				final Molecule mol2 = this.getNearestMoleculeExceptOnMySide(
						lip, exclude, membraneside);

				if (mol2 != null) {
					final float d2 = this.distance(lip, mol2, true);

					final Point3f p = (new Point3f(lip.getPosition()));
					final Matrix3f m = (Cm2_Toolkit.copyMatrix(lip
							.getRotationMatrix()));

					this.shakeMolecule(lip);

					final Molecule test1 = this.getNearestMoleculeOnMySide(lip,
							membraneside);
					final float testD = this.distance(lip, test1, true);

					final Vector<Molecule> testExclude = new Vector<Molecule>();
					testExclude.add(test1);

					final Molecule test2 = this
							.getNearestMoleculeExceptOnMySide(lip, testExclude,
									membraneside);

					final float testD2 = this.distance(lip, test2, true);

					if (testD + testD2 < d1 + d2
							&& this.intersects(lip) == false) {
						return lip;
					} else {
						lip.setPosition(new Point3f(p));
						lip.setRotationMatrix(Cm2_Toolkit.copyMatrix(m));
						return lip;
					}

				} else {

					final Point3f p = (new Point3f(lip.getPosition()));
					final Matrix3f m = (Cm2_Toolkit.copyMatrix(lip
							.getRotationMatrix()));

					this.shakeMolecule(lip);

					final Molecule test1 = this.getNearestMoleculeOnMySide(lip,
							membraneside);
					final float testD = this.distance(lip, test1, true);

					final Vector<Molecule> testExclude = new Vector<Molecule>();
					testExclude.add(test1);

					if (testD < d1 && this.intersects(lip) == false) {
						return lip;
					} else {
						lip.setPosition(new Point3f(p));
						lip.setRotationMatrix(Cm2_Toolkit.copyMatrix(m));
						return lip;
					}
				}
			} else
				return lip;

		}
		return lip;
	}

	/**
	 * Simple method to change position and rotation of a molecule at random.
	 * 
	 * @param mol
	 */
	public void shakeMolecule(final Molecule mol) {
		final Point3f p = new Point3f(mol.getPosition());
		p.x += (this.random.getNextInt(100, false)) / 100f;
		p.z += (this.random.getNextInt(100, false)) / 100f;
		mol.setPosition(new Point3f(p));
		mol.rotY(this.random.getNextInt(360, true));
	}

	/**
	 * returns full information of this algorithm
	 */
	public String getFullInformation() {
		return "<html> The Lipids will be placed at random. <br> "
				+ "You can activate better positioning to minimize holse</html>";
	}

	/**
	 * returns short information about this algorithm
	 * 
	 */
	public String getShortInformation() {
		return "Random positioning";
	}

	/**
	 * returns algorithms name
	 */
	public String getName() {
		return "Random Positioning";
	}

	/**
	 * 
	 * handles users settings
	 */
	public void stateChanged(final ChangeEvent arg) {
		if (arg.getSource().equals(this.slider)) {
			this.tries = this.slider.getValue();
		}
		if (arg.getSource().equals(this.slider2)) {
			this.firstStateTries = this.slider2.getValue();
		}
		if (arg.getSource().equals(this.chk)) {
			if (this.chk.isSelected()) {
				this.better = true;
			} else
				this.better = false;
		}
	}

	/**
	 * 
	 * returns string representation of users GUI settings
	 */
	public String getStringRepresentationOfSetupParameters() {
		final String s = "Random placing factor: " + this.tries + "\n"
				+ "Better placing: " + this.better + "\n"
				+ "better placing amount: " + this.firstStateTries;
		return s;
	}

	/**
	 * when called, this algorithm is stoped at the next breakpoint.
	 * 
	 */
	public void stopAlgorithm() {
		this.breake = true;

	}

	/**
	 * 
	 * Private Random Class generates new Random numbers. This makes sure that
	 * you always get the same random numbers if same starting value is used.
	 * 
	 * @author Maloche
	 * 
	 */
	private class MyRandom {

		long seed;
		Random rand;

		/**
		 * Creates new MyRandom instance.
		 * 
		 * @param seed
		 */
		public MyRandom(final long seed) {
			this.seed = seed;
			this.rand = new Random();
			this.rand.setSeed(seed);
		}

		/**
		 * Returns the next int value in the given range. The absval boolean
		 * indicates if the int is in the range from -scale to +scale or the
		 * absolut value.
		 * 
		 * 
		 * @param scale
		 * @param absval
		 * @return
		 */
		public int getNextInt(final int scale, final boolean absval) {
			this.rand.setSeed(this.seed);
			int i = this.rand.nextInt(scale);
			this.seed = this.rand.nextLong();
			if (absval == false) {

				if (this.seed % 2 == 0)
					i = -i;
			}

			return i;
		}

		/**
		 * Returns the next float The absval boolean indicates if the float is
		 * in the range from - to + or the absolut value.
		 * 
		 * 
		 * @return
		 */
		public float getNextFloat(final boolean absval) {
			this.rand.setSeed(this.seed);
			float f = this.rand.nextFloat();

			this.seed = this.rand.nextLong();

			if (absval == false) {
				if (this.seed % 2 == 0)
					f = -f;
			}

			return f;
		}

		/**
		 * Returns the next double The absval boolean indicates if the double is
		 * in the range from - to + or the absolut value.
		 * 
		 * @return
		 */
		public double getNextDouble(final boolean absval) {
			this.rand.setSeed(this.seed);
			double d = this.rand.nextDouble();

			this.seed = this.rand.nextLong();

			if (absval == false) {
				if (this.seed % 2 == 0)
					d = -d;
			}

			return d;
		}

	}

	public boolean supportsAbsoluteValues() {
		return false;
	}

	public boolean supportsMicrodomains() {
		return false;
	}

	public boolean overrideExistingLipids() {

		return false;
	}

	public Component getRuntimeControlPanel() {
		return null;
	}

}
