/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms;

import java.awt.Component;
import java.awt.Dimension;
import java.util.Random;
import java.util.Vector;

import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.membranepackingalgorithm.IllegalOperationException;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Molecule;


/**
 * See algorithm description...
 * 
 * @author Sebastian Schneider (Bjoern Sommer: adjustment to new version)
 * 
 */

public class Tetris extends MembranePackingAlgorithm {

	int modus = 3;

	float cor = 0.9f;

	Vector<Molecule> testLipsNew = new Vector<Molecule>();

	Vector<Molecule> testLipsOld = new Vector<Molecule>();

	Vector<Molecule> testLipsOld2 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld3 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld4 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld5 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld6 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld7 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld8 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld9 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld10 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld11 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld12 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld13 = new Vector<Molecule>();

	Vector<Molecule> testLipsOld14 = new Vector<Molecule>();

	public boolean buildAndShowGui() {
		return true;
	}

	public void fillWithLipids() {
		final Dimension dim = this.getMembraneSize();
		this.setFirstRow(0);
		int height = 20;
		while (height < dim.height) {
			this.createRow(height, 0);

			height = (int) this.testLipsNew.firstElement().getPosition().z + 15;
		}

		this.setFirstRow(1);
		height = 20;
		while (height < dim.height) {
			this.createRow(height, 1);

			height = (int) this.testLipsNew.firstElement().getPosition().z + 10;
		}
	}

	public void setFirstRow(final int mode) {
		Lipid newLip;
		Lipid oldLip = null;
		if (mode == 0) {
			newLip = this.createNextExtracellularLipid();
		} else {
			newLip = this.createNextIntracellularLipid();
		}
		newLip.setPosition(new Point3f(newLip.getMaxRadius_XZ(), newLip
				.getYTranslation(), newLip.getMaxRadius_XZ()));
		newLip.rotY(this.getRandomNumber(this.modus));
		// checks if intersects with left border and moves it away
		if (this.intersectsBorderWest(newLip)) {
			int d = 0;
			while (true) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.x += this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersectsBorderWest(newLip) == false)
					break;

				d++;
				if (d > 1000)
					break;
			}
			// checks with left border and moves lipid toward
		} else {
			int d = 0;
			while (true) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.x -= this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersectsBorderWest(newLip)) {
					final Point3f pp = new Point3f(newLip.getPosition());
					pp.x += this.cor;
					newLip.setPosition(new Point3f(pp));
					break;
				}
				d++;
				if (d > 1000)
					break;
			}
		}
		// checks upper border and moves it away
		if (this.intersectsBorderNorth(newLip)) {
			int d = 0;
			while (true) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.z += this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersectsBorderNorth(newLip) == false)
					break;

				d++;
				if (d > 1000)
					break;
			}
		} else {
			int d = 0;
			while (true) {
				final Point3f p = new Point3f(newLip.getPosition());
				p.z -= this.cor;
				newLip.setPosition(new Point3f(p));
				if (this.intersectsBorderNorth(newLip)) {
					final Point3f pp = new Point3f(newLip.getPosition());
					pp.z += this.cor;
					newLip.setPosition(new Point3f(pp));
					break;
				}
				d++;
				if (d > 1000)
					break;
			}
		}
		if (this.intersects(newLip) == false) {
			if (mode == 0) {
				try {
					this.addExtracellularLipid(newLip);
					this.testLipsNew.add(newLip);
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			} else {
				try {
					this.addIntracellularLipid(newLip);
					this.testLipsNew.add(newLip);
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}
			}
		}
		if (mode == 0) {
			oldLip = this.getExtracellularLipidByPDBCode(newLip.getType());
		} else {
			oldLip = this.getIntracellularLipidByPDBCode(newLip.getType());
		}
		oldLip.setPosition(new Point3f(newLip.getPosition()));
		oldLip.setRotationMatrix(this.copyMatrix(newLip.getRotationMatrix()));

		if (mode == 0) {
			newLip = this.createNextExtracellularLipid();
		} else {
			newLip = this.createNextIntracellularLipid();
		}

		final Point3f oP = new Point3f(oldLip.getPosition());
		oP.x = newLip.getMaxRadius_XZ() + oldLip.getMaxRadius_XZ();
		oP.y = newLip.getYTranslation();
		oP.z = newLip.getMaxRadius_XZ();
		newLip.setPosition(new Point3f(oP));
		newLip.rotY(this.getRandomNumber(this.modus));
		while (this.intersectsBorderEast(newLip) == false) {
			// rightshift
			if (this.intersectsMoleculeWithMolecules(newLip, this.testLipsNew)) {
				int d = 0;
				while (true) {
					d++;
					if (d > 1000)
						break;
					final Point3f p = new Point3f(newLip.getPosition());
					p.x += this.cor;
					newLip.setPosition(new Point3f(p));
					if (this.intersectsMoleculeWithMolecules(newLip,
							this.testLipsNew) == false)
						break;
				}
				// leftshift
			} else {
				int d = 0;
				while (true) {
					d++;
					if (d > 1000)
						break;
					if (this.intersectsMoleculeWithMolecules(newLip,
							this.testLipsNew)) {
						final Point3f pp = new Point3f(newLip.getPosition());
						pp.x += this.cor;
						newLip.setPosition(new Point3f(pp));
						break;
					}
					final Point3f p = new Point3f(newLip.getPosition());
					p.x -= this.cor;
					newLip.setPosition(new Point3f(p));

				}
			}

			// checks upper border and moves it away
			if (this.intersectsBorderNorth(newLip)) {
				int d = 0;
				while (true) {
					d++;
					if (d > 1000)
						break;
					final Point3f p = new Point3f(newLip.getPosition());
					p.z += this.cor;
					newLip.setPosition(new Point3f(p));
					if (this.intersectsBorderNorth(newLip) == false)
						break;
				}
			} else {
				int d = 0;
				while (true) {
					d++;
					if (d > 1000)
						break;
					final Point3f p = new Point3f(newLip.getPosition());
					p.z -= this.cor;
					newLip.setPosition(new Point3f(p));
					if (this.intersectsBorderNorth(newLip)) {
						final Point3f pp = new Point3f(newLip.getPosition());
						pp.z += this.cor;
						newLip.setPosition(new Point3f(pp));
						break;
					}
				}
			}

			if (true) {// this.intersects(newLip) == false) {
				if (mode == 0) {
					try {
						this.addExtracellularLipid(newLip);
						this.testLipsNew.add(newLip);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				} else {
					try {
						this.addIntracellularLipid(newLip);
						this.testLipsNew.add(newLip);
					} catch (final IllegalOperationException e) {
						e.printStackTrace();
					}
				}
			}
			if (mode == 0) {
				oldLip = this.getExtracellularLipidByPDBCode(newLip.getType());
			} else {
				oldLip = this.getIntracellularLipidByPDBCode(newLip.getType());
			}
			oldLip.setPosition(new Point3f(newLip.getPosition()));
			oldLip.setRotationMatrix(this
					.copyMatrix(newLip.getRotationMatrix()));

			if (mode == 0) {
				newLip = this.createNextExtracellularLipid();
			} else {
				newLip = this.createNextIntracellularLipid();
			}

			final Point3f oPP = new Point3f(oldLip.getPosition());
			oPP.x = newLip.getMaxRadius_XZ() + oldLip.getPosition().x;
			// oPP.y = newLip.getYTranslation();
			// oPP.z = newLip.getMaxRadius_XZ();
			newLip.setPosition(new Point3f(oPP));
			newLip.rotY(this.getRandomNumber(this.modus));

		}
		// testLipsNew.clear();
	}

	public void createRow(final int height, final int mode) {
		this.testLipsNew.clear();
		final Vector<Molecule> testMols = new Vector<Molecule>();
		Lipid l;
		if (mode == 0) {
			l = this.createNextExtracellularLipid();
		} else
			l = this.createNextIntracellularLipid();
		l.rotY(this.getRandomNumber(this.modus));
		l.setPosition(new Point3f(l.getMaxRadius_XZ(), l.getYTranslation(),
				height));
		if (this.intersectsBorderWest(l)) {
			while (this.intersectsBorderWest(l)) {
				final Point3f p = new Point3f(l.getPosition());
				p.x += this.cor;
				l.setPosition(new Point3f(p));
			}

		} else {
			while (this.intersectsBorderWest(l) == false) {
				final Point3f p = new Point3f(l.getPosition());
				p.x -= this.cor;
				l.setPosition(new Point3f(p));
			}
			final Point3f p = new Point3f(l.getPosition());
			p.x += this.cor;
			l.setPosition(new Point3f(p));
		}
		try {
			if (mode == 0)
				this.addExtracellularLipid(l);
			else
				this.addIntracellularLipid(l);

			testMols.add(l);
		} catch (final IllegalOperationException e) {
			e.printStackTrace();
		}

		final Point3f p = new Point3f(l.getPosition());
		p.x += l.getMaxRadius_XZ();

		if (mode == 0)
			l = this.createNextExtracellularLipid();
		else
			l = this.createNextIntracellularLipid();
		p.x += l.getMaxRadius_XZ();
		l.setPosition(p);
		l.rotY(this.getRandomNumber(this.modus));

		while (this.intersectsBorderEast(l) == false) {

			if (this.intersectsMoleculeWithMolecules(l, testMols)) {
				int d = 0;
				while (this.intersectsMoleculeWithMolecules(l, testMols)) {
					// System.out.println("while1");
					final Point3f pp = new Point3f(l.getPosition());
					pp.x += this.cor;
					l.setPosition(new Point3f(pp));
					d++;
					if (d > 1000)
						break;
				}
				// puts the lipid as near as possible to the neighbor
			} else {
				int d = 0;
				while (true) {
					// System.out.println("while2");
					if (this.intersectsMoleculeWithMolecules(l, testMols)
							|| this.intersectsBorderWest(l)) {
						final Point3f pp = new Point3f(l.getPosition());
						pp.x += this.cor;
						l.setPosition(new Point3f(pp));
						break;
					}
					final Point3f pp = new Point3f(l.getPosition());
					pp.x -= this.cor;
					l.setPosition(new Point3f(pp));
					d++;
					if (d > 1000)
						break;
				}
			}
			if (this.intersectsMoleculeWithMolecules(l, testMols) == false) {
				try {
					if (mode == 0)
						this.addExtracellularLipid(l);
					else
						this.addIntracellularLipid(l);
					testMols.add(l);
				} catch (final IllegalOperationException e) {
					e.printStackTrace();
				}

			}
			final Point3f o = new Point3f(l.getPosition());
			o.x += l.getMaxRadius_XZ();

			if (mode == 0)
				l = this.createNextExtracellularLipid();
			else
				l = this.createNextIntracellularLipid();

			o.x += l.getMaxRadius_XZ();

			l.setPosition(new Point3f(o));
			l.rotY(this.getRandomNumber(this.modus));
			// l.rotY((float)Math.toRadians(90));
		}

		// System.out.println("-> "+testMols.size());
		for (int i = 0; i < testMols.size(); i++) {
			final Lipid v = (Lipid) testMols.get(i);

			final Vector<Molecule> collLipids = this.getLipidsInRadiusExcept(v,
					500, testMols);
			while (this.intersectsMoleculeWithMolecules(v, collLipids) == false
					&& this.intersectsWithProteins(v) == false) {
				final Point3f pp = new Point3f(v.getPosition());
				pp.z -= 0.1f;
				v.setPosition(pp);
			}
			final Point3f pp = new Point3f(v.getPosition());
			pp.z += 0.1f;
			v.setPosition(pp);

			if (this.intersectsWithProteins(v)) {
				if (mode == MembranePackingAlgorithm.EXTRA) {
//					try {
						this.removeLipidsFromMembrane(new Lipid[]{v});
//						this.removeExtracellularLipid(v);
//					} catch (final IllegalOperationException e) {
//						e.printStackTrace();
//					}
				} else if (mode == MembranePackingAlgorithm.INTRA) {
//					try {
						this.removeLipidsFromMembrane(new Lipid[]{v});
//						this.removeIntracellularLipid(v);
//					} catch (final IllegalOperationException e) {
//						e.printStackTrace();
//					}
				}
			} else {
				this.testLipsNew.add(v);
			}
			// if(this.intersectsMoleculeWithMolecules(add, testLipsNew)){
			//				
			// }

		}
		// System.out.println(tmp.size());
	}

	public int getRandomNumber(final int mode) {
		final Random rand = new Random();
		int r = 0;
		if (mode == 0) {
			r = rand.nextInt(360);
		} else if (mode == 1) {
			final int r2 = rand.nextInt(7);
			if (r2 == 1) {
				r = 45;
			} else if (r2 == 2) {
				r = 90;
			} else if (r2 == 3) {
				r = 135;
			} else if (r2 == 4) {
				r = 180;
			} else if (r2 == 5) {
				r = 225;
			} else if (r2 == 6) {
				r = 270;
			} else if (r2 == 7) {
				r = 315;
			}
		} else if (mode == 2) {
			final int r3 = rand.nextInt(3);
			if (r3 == 1) {
				r = 90;
			} else if (r3 == 2) {
				r = 180;
			} else if (r3 == 3) {
				r = 270;
			}
		} else if (mode == 3) {
			r = 0;
		}

		return r;
	}

	public String getFullInformation() {
		return "Adds line by line\nAuthor: Sebastian Schneider\nVersion: 0.91 (Bjoern Sommer: adjustment to new version)\n\n"
				+ "note: this algorithm was only a test,\nso it may not run very fine";
	}

	public String getName() {
		return "Flow Line Algorithm";
	}

	public String getShortInformation() {
		return "Adds line by line (Version 0.91)";
	}

	public String getStringRepresentationOfSetupParameters() {
		return null;
	}

	public void stopAlgorithm() {

	}
	
	public boolean supportsAbsoluteValues() {
		return false;
	}	

	public boolean supportsMicrodomains() {
		return false;
	}

	public boolean overrideExistingLipids() {

		return true;
	}

	public Component getRuntimeControlPanel() {
		return null;
	}

}
