/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.pluginmanager;

import java.awt.Font;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.InetAddress;
import java.util.Calendar;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.engine.Engine.BasicOperations;
import org.cellmicrocosmos.cm2.engine.Engine.DatabaseOperations;
import org.cellmicrocosmos.cm2.engine.Engine.MembraneOperations;
import org.cellmicrocosmos.cm2.gui.elements.MembraneView;
import org.cellmicrocosmos.cm2.gui.elements.PropertiesView;
import org.cellmicrocosmos.cm2.gui.elements.StatisticsView;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;


import com.sun.j3d.utils.universe.SimpleUniverse;

/**
 * 
 * The CalculationManager performs the computation of the different algorithms.
 * The MembranePackingAlgorithms are run and administrated here. Also the options
 * available after the algorithm has finished are located in this file.
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class CalculationManager {

	private static MembranePackingAlgorithm algo;
	private final MembraneOperations membraneOperations;
	private final MembraneView membraneview;
	private final BasicOperations basicoperations;
	private final DatabaseOperations databaseoperations;
	private final PropertiesView propertiesview;
	private final StatisticsView statisticsview;
	private boolean stopRequested = false;
	private long startingTime = 0;
	private Vector<String> timeStatistic = new Vector<String>();
	final JCheckBox showStats = new JCheckBox();
	public CalculationThread  T;

	public CalculationManager(final MembraneOperations membraneOperations,
			final MembranePackingAlgorithm memb, final MembraneView membraneview,
			final BasicOperations basicOperations, final DatabaseOperations databaseOperations,
			final StatisticsView stats, final PropertiesView props) {
		CalculationManager.algo = memb;
		this.membraneOperations = membraneOperations;
		this.membraneview = membraneview;
		this.basicoperations = basicOperations;
		this.databaseoperations = databaseOperations;
		this.statisticsview = stats;
		this.propertiesview = props;

	}

	public boolean performLipidCheck() {
		if (CalculationManager.algo.supportsMicrodomains() == false) {
			if (DataPool.getMembraneModel().getAllExampleLipidsExtraInDomain(0,
					0).length < 1
					&& DataPool.getMembraneModel()
							.getAllExampleLipidsIntraInDomain(0, 0).length < 1) {
				JOptionPane
						.showMessageDialog(
								null,
								"This Algorithm requires some lipids added to the default area!",
								"Missing Lipids", JOptionPane.ERROR_MESSAGE);
				return false;
			}
		}

		if (CalculationManager.algo.overrideExistingLipids()
				&& (DataPool.getMembraneModel().getAllLipidsExtraInDomain(-1,
						DataPool.getCurrentLayerID()).length > 0 || DataPool
						.getMembraneModel().getAllLipidsIntraInDomain(-1,
								DataPool.getCurrentLayerID()).length > 0)) {

			final int val = JOptionPane
					.showOptionDialog(
							this.membraneview,
							"This Algorithm will have to remove all existing Lipids first!\n\n",

							"Remove Lipids?", JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE, null, new String[] {
									"Clear Lipids & Continue",
									"Abort Algorithm" }, null);

			if (val != JOptionPane.YES_NO_OPTION)
				return false;

		} else if (DataPool.getMembraneModel().getAllLipidsExtraInDomain(-1,
				DataPool.getCurrentLayerID()).length > 0
				|| DataPool.getMembraneModel().getAllLipidsIntraInDomain(-1,
						DataPool.getCurrentLayerID()).length > 0) {
			// final int val = JOptionPane.showOptionDialog(this.membraneview,
			// "Do you want to maintain your existing lipids?\n\n",
			//
			// "Maintain Lipids?", JOptionPane.YES_NO_OPTION,
			// JOptionPane.QUESTION_MESSAGE, null, new String[] {
			// "Yes, keep them", "No, remove them now" }, null);
			//
			// if (val == JOptionPane.NO_OPTION) {
			//
			// this.membraneOperations.clearAllLipidsFrom3DandModel(-1,
			// DataPool.getCurrentLayerID());
			// }

		}

		return true;
	}

	public boolean prepare() {

		CalculationManager.algo.setListenerAndPrepare(
				this.membraneOperations,
				this.basicoperations,
				this.databaseoperations,
				this);

		DataPool.getMembraneModel().prepare(CalculationManager.algo);

		final boolean returnvalue = CalculationManager.algo.buildAndShowGui();

		return returnvalue;

	}

	public void start() {

		final JButton stop = new JButton(" STOP ");
		stop.setFont(stop.getFont().deriveFont(Font.BOLD, 12f));

		stop.setToolTipText("Request algorithm to stop");

		final JCheckBox dockscreen = new JCheckBox();
		dockscreen.setToolTipText("Run in Background (affects performance)");
		dockscreen.setIcon(Cm2_IconDepot.getSpecifiedIcon("Screen_Docked"));
		// dockscreen.setSelectedIcon(Cm2_IconDepot
		// .getSpecifiedIcon("Screen_Docked"));
		dockscreen.setSelected(true);

		this.showStats
				.setToolTipText("Activate / Deactivate Statistics (affects performance)");
		this.showStats.setIcon(Cm2_IconDepot.getSpecifiedIcon("Stats_Off"));
		this.showStats.setSelectedIcon(Cm2_IconDepot
				.getSpecifiedIcon("Stats_On"));
		this.showStats.setSelected(true);
		
		stop.addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent e) {
			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseExited(MouseEvent e) {
			}

			public void mousePressed(MouseEvent e) {
				CalculationManager.this.stopRequested = true;
				CalculationManager.algo.stopAlgorithm();
			}

			public void mouseReleased(MouseEvent e) {
			}
			
		});

		dockscreen.addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent e) {
			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseExited(MouseEvent e) {
			}

			public void mousePressed(MouseEvent e) {
				if (!dockscreen.isSelected()) {
					CalculationManager.this.membraneview.toSystemTray();
					DataPool.J3DSettings.isInSystemTray = true;
				}
			}

			public void mouseReleased(MouseEvent e) {
			}
			
		});
		
		this.showStats.addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent e) {
			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseExited(MouseEvent e) {
			}

			public void mousePressed(MouseEvent e) {
				CalculationManager.this.updateMembraneInfo(true);
			}

			public void mouseReleased(MouseEvent e) {
			}
			
		});

		if (CalculationManager.algo.getRuntimeControlPanel() != null) {

			final JButton runtimeControls = new JButton();

			runtimeControls.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("CV_Settings"));
			runtimeControls.setToolTipText("Show Control Panel");
			runtimeControls.addMouseListener(new MouseListener() {

				public void mouseClicked(MouseEvent e) {
				}

				public void mouseEntered(MouseEvent e) {
				}

				public void mouseExited(MouseEvent e) {
				}

				public void mousePressed(MouseEvent e) {
					CalculationManager.this.membraneview.showRuntimeControls();
				}

				public void mouseReleased(MouseEvent e) {
				}
				
			});
			
			this.membraneview.setInAlgorithmMode(stop, dockscreen,
					this.showStats, runtimeControls, CalculationManager.algo
							.getRuntimeControlPanel());
		} else
			this.membraneview.setInAlgorithmMode(stop, dockscreen,
					this.showStats, null, null);

		 T = new CalculationThread();

		T.setPriority(CM2_CONSTANTS.THREADS.PRIORITY_LIPID_FILLING_ALGORITHMS);

		T.start();

	}

	long lastUpdated = 0;

	public void updateMembraneInfo(final boolean immediateUpdate) {

		this.statisticsview.setVisible(this.showStats.isSelected());
		this.propertiesview.setVisible(this.showStats.isSelected());

		if (!immediateUpdate
				&& (System.currentTimeMillis() - this.lastUpdated) < 3000)
			return;

		if (this.showStats.isSelected()) {

			// add the actual number of lipids to the time statistic
			long elapsedTimeMillis = System.currentTimeMillis()-
										((algo.getStartingTime()!=-1)?algo.getStartingTime():startingTime);
			timeStatistic.add(elapsedTimeMillis
					+" "
					+(algo.getAllLipids(MembranePackingAlgorithm.EXTRA).size()
					+algo.getAllLipids(MembranePackingAlgorithm.INTRA).size()));
			
			this.statisticsview.updateMembraneStatistics(DataPool
					.getMembraneModel(), -1);
			
			this.propertiesview.showMembraneInfo();

			this.lastUpdated = System.currentTimeMillis();
		}

	}
	
	public void returnFromSystemTray() {
		this.membraneview.returnFromSystemTray();
		DataPool.J3DSettings.isInSystemTray = false;
	} 

	public void sendToSystemTray() {
		this.membraneview.toSystemTray();
		DataPool.J3DSettings.isInSystemTray = true;
	}
	
	public class CalculationThread extends Thread {
		@Override
		public void run() {
			if (CalculationManager.algo.overrideExistingLipids())
				CalculationManager.this.membraneview.getMembrane3D()
						.clearLipids(-1, DataPool.getCurrentLayerID());

			CalculationManager.this.basicoperations.setAllEnabled(false);

			final Vector<Molecule_3D> invalidones = new Vector<Molecule_3D>();

			try {

				final Molecule_3D[] mols = DataPool.getMembraneModel()
						.getAllMoleculesInMembrane();
				for (int i = 0; i < mols.length; i++) {

					if (!mols[i].isPositionInAValidSpace())
						invalidones.addElement(mols[i]);
					mols[i].setPositionIsInAValidSpace(true);

				}

				// get the starting time for the protocol
				startingTime = System.currentTimeMillis(); 
				
				// minimize the computation right from the start if the option is selected
				if (DataPool.AlgorithmSettings.startInBackground) {
					CalculationManager.algo.setStartInBackgroundSelected(true);
					if (CalculationManager.algo.isStartInBackgroundAllowed()) {
						CalculationManager.this.membraneview.toSystemTray();
						DataPool.J3DSettings.isInSystemTray = true;
					}
				}
				
				// add the starting number of lipids to the time statistic
				long elapsedTimeMillisToStart = System.currentTimeMillis()-
												((algo.getStartingTime()!=-1)?algo.getStartingTime():startingTime);
				timeStatistic.add(elapsedTimeMillisToStart
												+" "
												+(algo.getAllLipids(MembranePackingAlgorithm.EXTRA).size()
												+algo.getAllLipids(MembranePackingAlgorithm.INTRA).size()));
				
				// start the algorithm
				CalculationManager.algo.fillWithLipids();
				
				try {
					SystemTray.getSystemTray().getTrayIcons()[SystemTray.getSystemTray().getTrayIcons().length-1]
							.displayMessage("Algorithm Completed!", "",
									TrayIcon.MessageType.INFO);
					SystemTray.getSystemTray().getTrayIcons()[SystemTray.getSystemTray().getTrayIcons().length-1]
							.setToolTip("Algorithm Completed!");
				} catch (Exception e) {

				}
				
				// check if the current algorithm provides a special starting time
				if (CalculationManager.algo.getStartingTime()!=-1)
					startingTime = CalculationManager.algo.getStartingTime(); 
				
				// get the termination time for the protocol and 
				// check if the current algorithm provides a special termination time
				long terminationTime = (CalculationManager.algo.getTerminationTime()!=-1)? 
										CalculationManager.algo.getTerminationTime():
										System.currentTimeMillis();
				// compute the passed time and the dates for the protocol
				long elapsedTimeMillis = terminationTime-startingTime; 
				
				// seems not to be needed, because each algorithm updates the state before termination
//				// add the final number of lipids to the time statistic
//				timeStatistic.add(elapsedTimeMillis
//									+" "
//									+(algo.getAllLipids(MembranePackingAlgorithm.EXTRA).size()
//									+algo.getAllLipids(MembranePackingAlgorithm.INTRA).size()));
				
				// 0d 1h 38m 2300s 2300203ms
				int elapsedTimeDay = (int)Math.floor(elapsedTimeMillis/(24*60*60*1000F));
				int elapsedTimeHour= (int)Math.floor(elapsedTimeMillis/(60*60*1000F)); 
				int elapsedTimeMin = (int)Math.floor(elapsedTimeMillis/(60*1000F));
				int elapsedTimeSec = (int)Math.floor(elapsedTimeMillis/1000F) - elapsedTimeMin*60; 
				elapsedTimeMin     = elapsedTimeMin - (elapsedTimeHour*60);
				elapsedTimeHour    = elapsedTimeHour - (elapsedTimeDay*24);
				
				Calendar cal = Calendar.getInstance();
				cal.setTimeInMillis(startingTime);
				java.util.Date startingDate = cal.getTime();
				cal.setTimeInMillis(terminationTime);
				java.util.Date finishingDate = cal.getTime();

				String timeProtocol = (	"MembraneEditor   : "+CM2_CONSTANTS.GENERAL.VERSION+"\n"+
										"Status           : "+(CalculationManager.this.stopRequested?"stopped":"completed")+"\n"+
										"UniqueRandom     : "+DataPool.AlgorithmSettings.uniqueRandom+(DataPool.AlgorithmSettings.uniqueRandom?
															" (Seed: "+DataPool.AlgorithmSettings.uniqueRandomSeed+")\n":"\n")+
										"Atom-Collision   : "+(DataPool.J3DSettings.atomCollisionCheck?"Min. Distance: "+DataPool.J3DSettings.minatomdistance+(DataPool.J3DSettings.atomDistanceAbsolute?" absolute":"")+" angstrom":"deactivated")+"\n"+
										"Shape-Collision  : "+(DataPool.J3DSettings.shapeCollisionCheck?
																	 (DataPool.J3DSettings.automaticDetailSettings?"Automatic Details":"Precision: Mod. "+DataPool.J3DSettings.precision_modifier+", Min. "+DataPool.J3DSettings.minprecision+", Verts. "+DataPool.J3DSettings.numberofvertices):"deactivated")+"\n"+
										"Layer ID         : "+(DataPool.getCurrentLayerID()==0?1:DataPool.getCurrentLayerID())+
															 "\n\n"+
										"Computer         : "+InetAddress.getLocalHost().getHostName()+" ("+System.getProperty("os.name")+")\n"+
										"Java/J3D Version : "+System.getProperty("java.version")+"/"+SimpleUniverse.getProperties().get("j3d.version")+"\n"+
										"Starting Date    : "+startingDate+"\n"+
										"Finishing Date   : "+finishingDate+"\n"+
										"Elapsed Time     : "+elapsedTimeDay+"d "+elapsedTimeHour+"h "+elapsedTimeMin+"m "+elapsedTimeSec+"s ["+elapsedTimeMillis+"ns]\n"
										);
				
				CalculationManager.this.showStats.setSelected(true);

				CalculationManager.this.basicoperations.setAllEnabled(true);
				CalculationManager.this.membraneview.setNormalMode();

				DataPool.setMembraneHasBeenEditedAgain(true);

				for (int i = 0; i < invalidones.size(); i++) {
					invalidones.get(i).setPositionIsInAValidSpace(false);

				}

				CalculationManager.this.membraneOperations.refreshViews(false,
						true, false, true, true, true);
				CalculationManager.this.membraneview.getMembrane3D()
						.setShowAxis(
								CalculationManager.this.membraneview
										.getMembrane3D().areAxisVisible());
				DataPool.getMembraneModel().setAlgorithmdescription(
						CalculationManager.algo.getShortInformation());
				DataPool.getMembraneModel().setAlgorithmname(
						CalculationManager.algo.getName());
				
				String newAlgoString = (CalculationManager.algo
										.getStringRepresentationOfSetupParameters()!=null?CalculationManager.algo
										.getStringRepresentationOfSetupParameters():"no algorithm specific information available") +
										"\n\n"+timeProtocol; 
				
				DataPool.getMembraneModel().addAlgorithmStringRepresentation(
												"<b>"+CalculationManager.algo.getName()+"</b><br><br>"+newAlgoString);
				
				CalculationManager.this.showStats.setSelected(true);
				CalculationManager.this.updateMembraneInfo(true);
				if (DataPool.AlgorithmSettings.showProtocol) {
					
					if (DataPool.J3DSettings.isInSystemTray) {
						while (true) {
							int optionChosen = 
								JOptionPane.showOptionDialog(
									null,
									"<html><code><b>"+DataPool.getMembraneModel().getAlgorithmName()+" Protocol</b><br><br>"+
									newAlgoString.replace("\n", "<br>").replace(" ", "&nbsp;")+
									"</code><br><br><b>Return to MembraneEditor?</b></html>",
									"Protocol | Return to MembraneEditor?", 
									JOptionPane.YES_NO_OPTION,
									JOptionPane.QUESTION_MESSAGE, null,
									new String[] { "Yes", "No", "Save Protocol" }, null);
							if (optionChosen == 0) {
								CalculationManager.this.membraneview.returnFromSystemTray();break;								
							}else if  (optionChosen == 2) {
								basicoperations.userChoseSaveTimeProtocol(timeStatistic);
							}else{
								break;
							}
						}
					} else {	
						while (true) {
							int optionChosen = 
								JOptionPane.showOptionDialog(
									null,
									"<html><code><b>"+DataPool.getMembraneModel().getAlgorithmName()+" Protocol</b><br><br>"+
									newAlgoString.replace("\n", "<br>").replace(" ", "&nbsp;")+
									"</code></html>",
									"Protocol", 
									JOptionPane.YES_NO_OPTION,
									JOptionPane.INFORMATION_MESSAGE, null,
									new String[] { "Okay", "Save Run Time Protocol (.xvg)" }, null);
							if  (optionChosen == 1) {
								basicoperations.userChoseSaveTimeProtocol(timeStatistic);
							}else{
								break;
							}
						}
					}
					
				}
				
				CalculationManager.this.membraneview.getMembrane3D().dockScreen(true);
				
				System.gc();
			} catch (final Exception e) {
				CalculationManager.this.showStats.setSelected(true);

				CalculationManager.this.basicoperations.setAllEnabled(true);
				CalculationManager.this.membraneview.setNormalMode();

				try {
					SystemTray.getSystemTray().getTrayIcons()[SystemTray.getSystemTray().getTrayIcons().length-1]
							.displayMessage("ERROR!", "",
									TrayIcon.MessageType.ERROR);
					SystemTray.getSystemTray().getTrayIcons()[SystemTray.getSystemTray().getTrayIcons().length-1]
							.setToolTip("Algorithm Aborted!");
				} catch (Exception e2) {

				}
				final int val = JOptionPane.showOptionDialog(
						CalculationManager.this.membraneview,
						"An Error occured while performing algorithm!\n\n"
								+ "Error Message:\n" + e.toString(), "ERROR",
						JOptionPane.YES_NO_OPTION, JOptionPane.ERROR_MESSAGE,
						null, new String[] { "Print Stack Trace", "Close" },
						null);

				if (val == JOptionPane.YES_NO_OPTION)
					e.printStackTrace();

				CalculationManager.this.membraneview.getMembrane3D()
						.dockScreen(true);

				CalculationManager.this.membraneview.getMembrane3D()
						.clearLipids(-1, DataPool.getCurrentLayerID());

				DataPool.setMembraneHasBeenEditedAgain(true);
				for (int i = 0; i < invalidones.size(); i++) {
					invalidones.get(i).setPositionIsInAValidSpace(false);

				}
				CalculationManager.this.membraneOperations.refreshViews(false,
						true, false, true, true, true);
				CalculationManager.this.membraneview.getMembrane3D()
						.setShowAxis(
								CalculationManager.this.membraneview
										.getMembrane3D().areAxisVisible());
				System.gc();

			}

			Molecule_3D[]mols=DataPool.getMembraneModel().getAllMoleculesInMembrane();
			for(int i=0;i<mols.length;i++) {
				mols[i].teporaryMolecule=null;
			}
			Vector<Lipid_3D> v=DataPool.getMembraneModel().getAllExampleLipidsExtra();
			for(int i=0;i<v.size();i++) {
				v.get(i).teporaryMolecule=null;
			}
			 v=DataPool.getMembraneModel().getAllExampleLipidsIntra();
			for(int i=0;i<v.size();i++) {
				v.get(i).teporaryMolecule=null;
			}
		}
	}

	public static MembranePackingAlgorithm getAlgo() {
		return CalculationManager.algo;
	}

}
