/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_0)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.pluginmanager;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Vector;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;


/**
 * 
 * This class parses the different information located in a 
 * MembranePackingAlgorithm.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class PluginIO {
	static File outfile = CM2_CONSTANTS.IO.PLUGIN_FILE;

	public static void write(final Vector<Plugin> output) throws Exception {
		if (!CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.DEVELOPER_EDITION)
			return;
		final BufferedWriter bw = new BufferedWriter(new FileWriter(
				PluginIO.outfile));

		bw
				.write("This file stores information about the imported plugin algorithms.");
		bw.write("Do not edit!");
		bw.newLine();
		bw.newLine();
		bw.newLine();

		for (int i = 0; i < output.size(); i++) {
			final Plugin plugin = output.get(i);

			if (plugin.getName() != null) {
				bw.write(PluginIO.makeString("Name", plugin.getName()));
				bw.newLine();
			}

			if (plugin.supportsMDs() != null) {
				bw.write(PluginIO.makeString("SupportsMicrodomains", ""
						+ plugin.supportsMDs().booleanValue()));
				bw.newLine();
			}
			
			if (plugin.supportsMDs() != null) {
				bw.write(PluginIO.makeString("SupportsAbsoluteValues", ""
						+ plugin.supportsAbsoluteValues().booleanValue()));
				bw.newLine();
			}

			if (plugin.overridesLipids() != null) {
				bw.write(PluginIO.makeString("OverridesLipids", ""
						+ plugin.overridesLipids().booleanValue()));
				bw.newLine();
			}

			if (plugin.getInformation() != null) {
				bw.write(PluginIO.makeString("Information", plugin
						.getInformation()));
				bw.newLine();
			}

			if (plugin.getOriginalFile() != null) {
				bw.write(PluginIO.makeString("File", plugin.getOriginalFile()
						.getAbsolutePath()));
				bw.newLine();
			}

			if (plugin.getState() != null) {
				bw.write(PluginIO.makeString("State", plugin.getState()));
				bw.newLine();
			}

			if (plugin.getType() != null) {
				bw.write(PluginIO.makeString("Type", plugin.getType()));
				bw.newLine();
			}

			bw.write(PluginIO.makeString("LastModified", ""
					+ plugin.getLastModified()));
			bw.newLine();

			bw.write("#/ENTRY#");
			bw.newLine();
		}

		bw.close();
	}

	private static String makeString(final String key, final String value) {
		return "<" + key + ">" + value;
	}

	public static Vector<Plugin> read() throws Exception {

		final Vector<Plugin> input = new Vector<Plugin>();
		if (!CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.DEVELOPER_EDITION)
			return input;
		final BufferedReader br = new BufferedReader(new FileReader(
				PluginIO.outfile));

		Plugin newpl = new Plugin();
		while (true) {

			final String s = br.readLine();
			if (s == null)
				break;

			if (s.equals("#/ENTRY#")) {

				input.add(newpl);
				newpl = new Plugin();
				continue;
			}

			if (s.startsWith("<")) {

				try {

					String key;
					String value;

					final String[] str = s.split(">");
					key = str[0].replaceAll("<", "").trim();
					value = str[1].trim();
					if (key == null || key.length() < 1)
						continue;
					if (value == null || value.length() < 1)
						continue;

					if (key.equals("Name")) {
						newpl.setName(value);
					} else if (key.equals("SupportsMicrodomains")) {
						newpl.setSupportsMDs(Boolean.parseBoolean(value));
					} else if (key.equals("SupportsAbsoluteValues")) {
						newpl.setSupportsAbsoluteValues(Boolean.parseBoolean(value));
					} else if (key.equals("OverridesLipids")) {
						newpl.setOverridesLipids(Boolean.parseBoolean(value));
					} else if (key.equals("Information")) {
						newpl.setInformation(value);
					} else if (key.equals("File")) {
						newpl.setOriginalFile(new File(value));
					} else if (key.equals("State")) {
						newpl.setState(value);
					} else if (key.equals("Type")) {
						newpl.setType(value);
					} else if (key.equals("LastModified")) {
						newpl.setLastModified(Long.parseLong(value));
					}

				} catch (final Exception e) {
					continue;
				}

			}
		}

		br.close();

		return input;

	}

}
